SystemOrganization addCategory: #'Balloon3D-Wonderland-Core'!
SystemOrganization addCategory: #'Balloon3D-Wonderland-Import'!
SystemOrganization addCategory: #'Balloon3D-Wonderland-Lights'!
SystemOrganization addCategory: #'Balloon3D-Wonderland-Mesh'!
SystemOrganization addCategory: #'Balloon3D-Wonderland-Misc'!
SystemOrganization addCategory: #'Balloon3D-Wonderland-Morphs'!
SystemOrganization addCategory: #'Balloon3D-Wonderland-Objects'!
SystemOrganization addCategory: #'Balloon3D-Wonderland-Time'!
SystemOrganization addCategory: #'Balloon3D-Wonderland-Undo'!

StringHolder subclass: #WonderlandActorBrowser
	instanceVariableNames: 'myWonderland myListMorph actorListIndex selectedActor actorViewer'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandActorBrowser commentStamp: '<historical>' prior: 0!
This class implements the actor browser.  The name of every actor in the Wonderland namespace is listed. The formatting is determined by the parent-child hierarchy.  Children of an actor are listed below and to the right of that actor.  The scene is at the root of the actor tree, and so is at the top left of the hierarchy.

Any actor listed in the actor browser can be addressed by that name in the script editor.

!

!WonderlandActorBrowser methodsFor: 'accessing' stamp: 'jsp 4/1/1999 16:57'!
getMorph
	"Return my morph"

	^ myListMorph.! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 11:51'!
destroy
	"Destroy the selected actor."

	selectedActor destroy.
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 12:14'!
becomeFirstClass
	"Make the selected actor first class"

	selectedActor becomeFirstClass.
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 11:52'!
standUp
	"Make the selected actor stand up"

	selectedActor standUp.
! !

!WonderlandActorBrowser methodsFor: 'actor list management' stamp: 'jsp 4/1/1999 16:48'!
actorListIndex
	"Return the index of the currently selected actor"

	^ actorListIndex.
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 11:57'!
stretch
	"Stretch the selected actor"

	selectedActor resizeLikeRubber: 1.5 dimension: topToBottom.
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 11:48'!
pointAt
	"Point the default camera at the selected object"

	myWonderland getDefaultCamera pointAt: selectedActor.
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 11:57'!
grow
	"Make the selected actor grow"

	selectedActor resize: 1.5.
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 12:01'!
turnAround
	"Make the selected actor turnAround"

	selectedActor turn: left turns: 1 duration: 2.
! !

!WonderlandActorBrowser class methodsFor: 'instance creation' stamp: 'jsp 4/1/1999 16:44'!
newFor: aWonderland
	"Create a new actor browser for the Wonderland"

	^ self new initializeWith: aWonderland.
! !

!WonderlandActorBrowser methodsFor: 'actor list functions' stamp: 'bf 5/9/2001 17:29'!
actorMenu: aMenu
	"Builds the menu to display when the user right clicks on an actor"

 	selectedActor ifNil: [ ^ aMenu ].

	(selectedActor isKindOf: WonderlandScene)
		ifTrue: [ ^ aMenu addList: { { 'Load actor'. #loadActor } } ].

	(selectedActor isFirstClass)
		ifTrue: [
			^ aMenu addList: {
								{'Point camera at'. #pointAt}.
								{'Turn around once'. #turnAround}.
								{'Become part'. #becomePart }.
								{'Stand up'. #standUp}.
								{'Grow'. #grow}.
								{'Shrink'. #shrink}.
								{'Squash'. #squash}.
								{'Stretch'. #stretch}.
								{'Destroy'. #destroy}
							  }.
				]
		ifFalse: [
			^ aMenu addList: {
								{'Point camera at'. #pointAt}.
								{'Turn around once'. #turnAround}.
								{'Become first class'. #becomeFirstClass }.
								{'Stand up'. #standUp}.
								{'Grow'. #grow}.
								{'Shrink'. #shrink}.
								{'Squash'. #squash}.
								{'Stretch'. #stretch}.
								{'Destroy'. #destroy}
							  }.
				].
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 11:57'!
squash
	"Squash the selected actor."

	selectedActor resizeLikeRubber: (1.0 / 1.5) dimension: topToBottom.
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'bf 5/9/2001 18:04'!
loadActor
	"Load an actor from file. Sent for WonderlandScenes"

	selectedActor loadActor.
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 11:57'!
shrink
	"Make the selected actor shrink"

	selectedActor resize: (1.0 / 1.5).
! !

!WonderlandActorBrowser methodsFor: 'actor menu functions' stamp: 'jsp 4/13/1999 12:13'!
becomePart
	"Make the selected actor a part"

	selectedActor becomePart.
! !

!WonderlandActorBrowser methodsFor: 'accessing' stamp: 'jsp 6/29/1999 01:44'!
getSelectedActor
	"Returns the currently selected actor in the actor browser."

	^ selectedActor.
! !

!WonderlandActorBrowser methodsFor: 'actor list management' stamp: 'jsp 4/1/1999 17:38'!
updateActorList
	"Updates the actor list."

	self changed: #actorList.

! !

!WonderlandActorBrowser methodsFor: 'actor list management' stamp: 'jsp 4/1/1999 17:53'!
actorList
	"Returns a list of actors in the scene"

	| scene |

	scene _ myWonderland getScene.

	^ { scene getName } , (scene appendChildrenNamesTo: '    ').
! !

!WonderlandActorBrowser methodsFor: 'actor list management' stamp: 'jsp 6/29/1999 01:55'!
actorListIndex: anInteger
	"Set the index of the currently selected actor."

	| scene childList |

	actorListIndex _ anInteger.
	myListMorph selectionIndex: anInteger.

	(anInteger = 0)
		ifTrue: [
					selectedActor _ nil.
				]
		ifFalse: [
					scene _ myWonderland getScene.
					childList _ scene getAllChildren.
					childList addFirst: scene.
					selectedActor _ childList at: anInteger.
				].

	actorViewer ifNotNil: [
		actorViewer setSelectedActor: selectedActor.
	].

	self contentsChanged.
! !

!WonderlandActorBrowser methodsFor: 'initialize-release' stamp: 'jsp 6/29/1999 01:50'!
initializeWith: aWonderland
	"Initialize this instance."

	myWonderland _ aWonderland.
	actorListIndex _ 0.
	selectedActor _ nil.
	actorViewer _ nil.

	myListMorph _ (PluggableListMorph on: self list: #actorList
						selected: #actorListIndex changeSelected: #actorListIndex:
						menu: #actorMenu: keystroke: #systemCatListKey:from:).
	myListMorph name: 'Actors'.
	myListMorph scrollBarOnLeft: false.
	myListMorph extent: 140@320.
	myListMorph color: (Color r: 0.627 g: 0.909 b: 0.972).
	myListMorph retractable: false.
! !

!WonderlandActorBrowser methodsFor: 'accessing' stamp: 'jsp 6/29/1999 01:52'!
setActorViewer: anActorViewer
	"Sets the WonderlandActorViewer to update when the user changes the selected object in the browser"

	actorViewer _ anActorViewer.
! !

Workspace subclass: #WonderlandQuickReference
	instanceVariableNames: 'myTextEditor'
	classVariableNames: 'HelpText'
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandQuickReference methodsFor: 'accessing' stamp: 'jsp 4/20/1999 12:10'!
getMorph
	"Returns the text editor morph"

	^ myTextEditor.
! !

!WonderlandQuickReference methodsFor: 'accessing' stamp: 'jsp 4/20/1999 12:12'!
contents
	"Return the help text"

	^ HelpText.
! !

!WonderlandQuickReference methodsFor: 'initialize-release' stamp: 'jsp 8/21/1999 16:34'!
initialize
	"Create a modified workspace as our script editor"

	super initialize.

	myTextEditor _ (PluggableTextMorph on: self text: #contents accept: #acceptContents:
			readSelection: nil menu: #codePaneMenu:shifted:).

	myTextEditor name: 'Quick Reference'.
	myTextEditor scrollBarOnLeft: false.
	myTextEditor retractable: false.
	myTextEditor extent: 500@350.
	myTextEditor color: (Color r: 0.815 g: 0.972 b: 0.878).
! !

!WonderlandQuickReference class methodsFor: 'class initialization' stamp: 'jsp 4/20/1999 13:52'!
initialize
	"Initialize the help text."

	HelpText _ 'This Quick Reference tab lists the basic commands and command combinations.

Names in every Wonderland:
	scene - the scene containing all the actors
	scheduler - the animation scheduler
	camera - the default camera
	cameraWindow - the morph the default camera renders into
	ground - the ground plane
	w - the Wonderland


WonderlandActors
=====================

This section lists the commands for actors broken down by category.

Movement
------------

Useful constants:
	direction: left, right, up, down, forward, back
	duration: rightNow, eachFrame
	style: gently, abruptly, beginGently, endGently
	position: asIs
	location: onTopOf, below, beneath, inFrontOf, inBackOf, behind, toLeftOf,
			toRightOf, onFloorOf, onCeilingOf

move: <direction>
move: <direction> distance: <number>
move: <direction> distance: <number>
move: <direction> distance: <number> duration: <number>
move: <direction> distance: <number> duration: <number> style: <style>
move: <direction> asSeenBy: <actor>
move: <direction> distance: <number> asSeenBy: <actor>
move: <direction> distance: <number> duration: <number> asSeenBy: <actor>
move: <direction> distance: <number> duration: <number> asSeenBy: <actor> style: <style>

move: <direction> speed: <number>
move: <direction> speed: <number> for: <number>
move: <direction> speed: <number> until: <condition>
move: <direction> speed: <number> asSeenBy: <actor>
move: <direction> speed: <number> asSeenBy: <actor> for: <number>
move: <direction> speed: <number> asSeenBy: <actor> until: <condition>

For moveTo, the position may be a { right. up. forward } triple or an <actor>. The triple values may be a number or asIs (ex. { asIs. 0. asIs } ).

moveTo: <position>
moveTo: <position> duration: <number>
moveTo: <position> duration: <number> style: <style>
moveTo: <position> asSeenBy: <actor>
moveTo: <position> duration: <number> asSeenBy: <actor>
moveTo: <position> duration: <number> asSeenBy: <actor> style: <style>

moveTo: <position> eachFrameFor: <number>
moveTo: <position> eachFrameUntil: <condition>
moveTo: <position> asSeenBy: <actor> eachFrameFor: <number>
moveTo: <position> asSeenBy: <actor> eachFrameUntil: <condition>

moveTo: <position> speed: <number>
moveTo: <position> speed: <number> asSeenBy: <actor> 

nudge: <direction>
nudge: <direction> distance: <number>
nudge: <direction> distance: <number> duration: <number>
nudge: <direction> distance: <number> duration: <number> style: <style>

place: <location> object: <actor>
place: <location> object: <actor> duration: <number>
place: <location> object: <actor> duration: <number> style: <style>
place: <location> object: <actor> eachFrameFor: <number>
place: <location> object: <actor> eachFrameUntil: <condition>



Turning
---------

Useful constants
	direction: left, right, up, down, forward, back
	style: gently, abruptly, beginGently, endGently
	duration: rightNow, eachFrame

alignWith: <actor>
alignWith: <actor> duration: <number>
alignWith: <actor> duration: <number> style: <style>

For the pointAt command, a target may be an <actor>, a { right. up. forward } trip, or a x@y pixel value.

pointAt: <target>
pointAt: <target> duration: <number>
pointAt: <target> duration: <number> style: <style>
pointAt: <target> duration: <number> eachFrameFor: <number>
pointAt: <target> duration: <number> eachFrameUntil: <condition>

roll: <direction>
roll: <direction> turns: <number> 
roll: <direction> turns: <number> duration: <number>
roll: <direction> turns: <number> duration: <number> style: <style>

roll: <direction> asSeenBy: <actor>
roll: <direction> turns: <number> asSeenBy: <actor>
roll: <direction> turns: <number> duration: <number> asSeenBy: <actor>
roll: <direction> turns: <number> duration: <number> asSeenBy: <actor> style: <style>

roll: <direction> turns: <number> speed: <number>
roll: <direction> turns: <number> speed: <number> asSeenBy: <actor>

roll: <direction> speed: <number>
roll: <direction> speed: <number> for: <number>
roll: <direction> speed: <number> until: <condition>
roll: <direction> speed: <number> asSeenBy: <actor>
roll: <direction> speed: <number> asSeenBy: <actor> for: <number>
roll: <direction> speed: <number> asSeenBy: <actor> until: <condition>

standUp
standUpWithDuration: <number>
standUpWithDuration: <number> style: <style>

turn: <direction>
turn: <direction> turns: <number> 
turn: <direction> turns: <number> duration: <number>
turn: <direction> turns: <number> duration: <number> style: <style>

turn: <direction> asSeenBy: <actor>
turn: <direction> turns: <number> asSeenBy: <actor>
turn: <direction> turns: <number> duration: <number> asSeenBy: <actor>
turn: <direction> turns: <number> duration: <number> asSeenBy: <actor> style: <style>

turn: <direction> turns: <number> speed: <number>
turn: <direction> turns: <number> speed: <number> asSeenBy: <actor>

turn: <direction> speed: <number>
turn: <direction> speed: <number> for: <number>
turn: <direction> speed: <number> until: <condition>
turn: <direction> speed: <number> asSeenBy: <actor>
turn: <direction> speed: <number> asSeenBy: <actor> for: <number>
turn: <direction> speed: <number> asSeenBy: <actor> until: <condition>

For turnTo, the orientation may be a rotation about the  { right. up. forward } axes or an <actor>. The triple values may be a number or asIs (ex. { asIs. 0. asIs } ).

turnTo: <orientation>
turnTo: <orientation> duration: <number>
turnTo: <orientation> duration: <number> style: <style>

turnTo: <orientation> asSeenBy: <actor>
turnTo: <orientation> duration: <number> asSeenBy: <actor>
turnTo: <orientation> duration: <number> asSeenBy: <actor> style: <style>

turnTo: <orientation> eachFrameFor: <number>
turnTo: <orientation> eachFrameUntil: <condition>
turnTo: <orientation> asSeenBy: <actor> eachFrameFor: <number>
turnTo: <orientation> asSeenBy: <actor> eachFrameUntil: <condition>

turnTo: <orientation> speed: <number>
turnTo: <orientation> speed: <number> asSeenBy: <actor>


Moving and Turning
------------------------

Useful constants:
	target: asIs
	duration: rightNow, eachFrame
	style: gently, abruptly, beginGently, endGently

The target for setPointOfView may be another actor or a { rpos. upos. fpos. rrot. urot. frot } sextuple.

setPointOfView: <target>
setPointOfView: <target> duration: <number>
setPointOfView: <target> duration: <number> style: <style>

setPointOfView: <target> asSeenBy: <actor>
setPointOfView: <target> duration: <number> asSeenBy: <actor>
setPointOfView: <target> duration: <number> asSeenBy: <actor> style: <style>

setPointOfView: <target> eachFrameFor: <number>
setPointOfView: <target> eachFrameUntil: <condition>
setPointOfView: <target> asSeenBy: <actor> eachFrameFor: <number>
setPointOfView: <target> asSeenBy: <actor> eachFrameUntil: <condition>

Resizing
---------

Useful constants:
	dimension: topToBottom, leftToRight, frontToBack


A resize amount may be a <number> for uniform scaling or a { right. up. forward } triple for non-uniform resizing.

resize: <amount>
resize: <amount> duration: <number>
resize: <amount> duration: <number> style: <style>
resize: <amount> eachFrameFor: <number>
resize: <amount> eachFrameUntil: <condition>

resizeLikeRubber: <amount> dimension: <dimension>
resizeLikeRubber: <amount> dimension: <dimension> duration: <number>
resizeLikeRubber: <amount> dimension: <dimension> duration: <number> style: <style>

The size for a setSize are the desired { leftToRight. toptoBottom. frontToBack } dimensions.

setSize: <size>
setSize: <size> duration: <number>
setSize: <size> duration: <number> style: <style>

Waiting
---------

wait
wait: <number>


Actor Hierarchy
-------------------

becomeChildOf: <actor>
becomePart
becomeFirstClass


Playing Sounds
------------------

playSound: ''soundfile.wav''


Changing Color
------------------

A color may be a named color (red, green, blue, orange, yellow, periwinkle, etc) or a { red. green. blue } triple.

setColor: <color>
setColor: <color> duration: <number>
setColor: <color> duration: <number> style: <style>

Changing Visibility
----------------------

hide
show

setVisibility: <number>
setVisibility: <number> duration: <number>
setVisibility: <number> duration: <number> style: <style>

Changing Texture
---------------------

setTexture: ''texturefile.bmp''


Animated State
-----------------

pause
resume
stop

Adding Reactions
--------------------

Useful constants:
	event types: keyPress, leftMouseDown, leftMouseUp, leftMouseClick,
			rightMouseDown, rightMouseUp, rightMouseClick

addResponse: <block> to: <eventType>
removeResponse: <response> to: <eventType>
respondWith: <block> to: <eventType>

Destruction
-------------

destroy
destroy: <duration>


Wonderlands
===============

Creating
----------

makeActor
makeActorFrom: ''actorFile''

makeCamera


Composing Animations
--------------------------

doTogether and doInOrder both accept an ordered collection of animations.

doTogether: { animations }
doInOrder: { animations }

'.
! !

Workspace subclass: #WonderlandScriptEditor
	instanceVariableNames: 'myTextEditor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandScriptEditor commentStamp: '<historical>' prior: 0!
This class implements the script editor for Wonderlands.  The script editor is a Wonderland that has been modified to use the Wonderland's namespace.
!

!WonderlandScriptEditor methodsFor: 'initialize-release' stamp: 'jsp 8/21/1999 16:34'!
initialize
	"Create a modified workspace as our script editor"

	super initialize.

	myTextEditor _ (PluggableTextMorph on: self text: #contents accept: #acceptContents:
			readSelection: nil menu: #codePaneMenu:shifted:).

	myTextEditor name: 'Script'.
	myTextEditor scrollBarOnLeft: false.
	myTextEditor retractable: false.
	myTextEditor extent: 500@350.
	myTextEditor color: (Color r: 0.815 g: 0.972 b: 0.878).
! !

!WonderlandScriptEditor methodsFor: 'accessing' stamp: 'jsp 4/1/1999 15:42'!
getMorph
	"Returns the text editor morph"

	^ myTextEditor.
! !

Object subclass: #Action
	instanceVariableNames: 'actionTask paused affectedObject lifetime stopCondition myScheduler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!Action commentStamp: '<historical>' prior: 0!
This class implements Actions for Wonderlands. An Action is some task that should be executed every frame either forever, until a specified amount of time has elapsed, or until a specified condition holds true.
!

!Action class methodsFor: 'initialize-release' stamp: 'jsp 3/9/1999 16:09'!
do: task eachframefor: time toObject: anObject inScheduler: scheduler
	"Creates a new Action that performs the specified task each frame for (time) seconds"

	| newAction |
	newAction _ Action new.

	newAction setTask: task.
	newAction setLifetime: (time + (scheduler getTime)) andCondition: [false].
	newAction setAffectedObject: anObject.
	newAction setScheduler: scheduler.

	scheduler addAction: newAction.

	^ newAction.
! !

!Action methodsFor: 'management' stamp: 'jsp 3/9/1999 16:20'!
setTask: task
	"Sets the task the Action should perform each frame"

	actionTask _ task.
	paused _ false.

! !

!Action methodsFor: 'accessing' stamp: 'jsp 3/9/1999 16:22'!
isPaused
	"Returns true if the action is paused"

	^ paused.
! !

!Action methodsFor: 'accessing' stamp: 'jsp 3/9/1999 16:21'!
resume
	"resume the action"

	paused _ false.
! !

!Action methodsFor: 'management' stamp: 'jsp 2/1/1999 16:20'!
setScheduler: scheduler
	"Sets the scheduler the Action is active in"

	myScheduler _ scheduler.
! !

!Action methodsFor: 'accessing' stamp: 'jsp 3/9/1999 16:21'!
pause
	"Pause the action"

	paused _ true.
! !

!Action methodsFor: 'management' stamp: 'jsp 3/9/1999 16:20'!
execute
	"Execute the Action's task"

	paused ifFalse: [ actionTask value ].

! !

!Action methodsFor: 'management' stamp: 'jsp 3/9/1999 16:08'!
setAffectedObject: anObject
	"Sets the object affected by the action"

	affectedObject _ anObject.
! !

!Action methodsFor: 'management' stamp: 'jsp 3/30/1999 11:50'!
stop
	"This method removes the Action from myScheduler's list of active actions"

	stopCondition _ [ true ].
	myScheduler removeAction: self.
! !

!Action class methodsFor: 'initialize-release' stamp: 'jsp 3/9/1999 16:10'!
do: task toObject: anObject inScheduler: scheduler
	"Creates a new Action that executes the specified task each frame"

	| newAction |
	newAction _ Action new.

	newAction setTask: task.
	newAction setLifetime: -1 andCondition: [false].
	newAction setAffectedObject: anObject.
	newAction setScheduler: scheduler.

	scheduler addAction: newAction.

	^ newAction.
! !

!Action methodsFor: 'accessing' stamp: 'jsp 2/1/1999 15:13'!
isDone
	"Returns true if the Action is done executing either because it's lifetime has expired or because the specified condition is true"

	(lifetime > 0) ifTrue: [^ (lifetime < (myScheduler getTime))]
				ifFalse: [^ (stopCondition value)].
	! !

!Action methodsFor: 'accessing' stamp: 'jsp 3/9/1999 16:08'!
getAffectedObject
	"Returns the object affected by the action"

	^ affectedObject.
! !

!Action class methodsFor: 'initialize-release' stamp: 'jsp 3/9/1999 16:09'!
do: task eachframeuntil: condition toObject: anObject inScheduler: scheduler
	"Creates a new Action that performs the specified task each frame until the specified condition holds true"

	| newAction |
	newAction _ Action new.

	newAction setTask: task.
	newAction setLifetime: -1 andCondition: condition.
	newAction setAffectedObject: anObject.
	newAction setScheduler: scheduler.

	scheduler addAction: newAction.

	^ newAction.
! !

!Action methodsFor: 'management' stamp: 'jsp 2/1/1999 11:44'!
setLifetime: howlong andCondition: condition
	"Sets how long the action should run, or the condition under which it should stop"

	lifetime _ howlong.
	stopCondition _ condition.
! !

Object subclass: #UndoParentChange
	instanceVariableNames: 'oldParent theActor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoParentChange commentStamp: '<historical>' prior: 0!
This class implements undo for reparenting operations.
!

!UndoParentChange methodsFor: 'initialization' stamp: 'jsp 4/5/1999 15:55'!
initializeWith: aParent for: anActor
	"Initialize the instance variables."

	oldParent _ aParent.
	theActor _ anActor.
! !

!UndoParentChange methodsFor: 'undoing' stamp: 'jsp 4/5/1999 15:56'!
undoIt
	"Undo the parent change"

	theActor reparentTo: oldParent.
! !

!UndoParentChange class methodsFor: 'instance creation' stamp: 'jsp 4/5/1999 15:56'!
newFor: anActor from: theParent
	"Create and initialize a new instance"

	^ super new initializeWith: theParent for: anActor.! !

Object subclass: #Scheduler
	instanceVariableNames: 'currentTime elapsedTime speed alarmList actionList animationList lastSystemTime isRunning schedulerProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!Scheduler commentStamp: '<historical>' prior: 0!
The Scheduler manages time for a Wonderland. As part of this responsibility, it manages active alarms, actions, and animations in the Wonderland.!

!Scheduler methodsFor: 'initialize-release' stamp: 'jsp 3/23/1999 15:08'!
initialize

	"Initializes the scheduler: sets Wonderland time to 0, creates empty lists for actions, alarms, and animations"

	currentTime _ 0.
	speed _ 1.
	isRunning _ true.
	alarmList _ OrderedCollection new.
	actionList _ OrderedCollection new.
	animationList _ OrderedCollection new.

	lastSystemTime _ Time millisecondClockValue / 1000.0.

	
! !

!Scheduler methodsFor: 'lists maintenance' stamp: 'jsp 1/29/1999 14:16'!
addAlarm: anAlarm
	"Add an alarm to the Scheduler's list of alarms"

	alarmList addLast: anAlarm.
! !

!Scheduler methodsFor: 'lists maintenance' stamp: 'jsp 1/29/1999 14:16'!
addAction: anAction
	"Add an action to the Scheduler's list of actions"

	actionList addLast: anAction.! !

!Scheduler methodsFor: 'ticking' stamp: 'jsp 2/8/1999 16:38'!
pause
	"Pause the scheduler. All alarms, actions, and animations pause and time stops passing in the Wonderland, but any active cameras continue to render."

	isRunning _ false.
! !

!Scheduler methodsFor: 'lists maintenance' stamp: 'jsp 1/29/1999 14:24'!
getAlarms
	"Returns a copy of the Scheduler's list of alarms"

	^ alarmList copy.
! !

!Scheduler methodsFor: 'ticking' stamp: 'jsp 3/10/1999 15:35'!
getFPS

	"Returns the instantaneous frames per second (1 / elapsedTime)"

	^ (1.0 / elapsedTime).
! !

!Scheduler methodsFor: 'lists maintenance' stamp: 'jsp 1/29/1999 14:16'!
addAnimation: anAnimation
	"Add an animation to the Scheduler's list of animations"

	animationList addLast: anAnimation.
! !

!Scheduler methodsFor: 'lists maintenance' stamp: 'jsp 1/29/1999 14:24'!
getActions
	"Returns a copy of the Scheduler's list of actions"

	^ actionList copy.
! !

!Scheduler methodsFor: 'ticking' stamp: 'jsp 2/1/1999 14:18'!
getTime

	"Returns the current scheduler time"

	^ currentTime.
! !

!Scheduler methodsFor: 'lists maintenance' stamp: 'jsp 1/29/1999 14:25'!
getAnimations
	"Returns a copy of the Scheduler's list of animations"

	^ animationList copy.
! !

!Scheduler methodsFor: 'ticking' stamp: 'jsp 2/1/1999 16:33'!
processActions
	"This method executes every active action and removes any that done"

	actionList do: [:action | action execute.
							(action isDone) ifTrue: [self removeAction: action]
				].
! !

!Scheduler methodsFor: 'ticking' stamp: 'ar 11/6/2000 23:31'!
processAlarms
	"This method checks all the alarms and processes any that have gone off"
	alarmList do: [:alarm | 
		((alarm checkTime) < currentTime) ifTrue:[
			self removeAlarm: alarm.
			alarm execute]]! !

!Scheduler methodsFor: 'animated object' stamp: 'jsp 3/9/1999 16:07'!
getActionsFor: anObject
	"Return the animations that list anObject as the object they're affecting"

	| aList |
	aList _ OrderedCollection new.

	actionList do: [:action | ((action getAffectedObject) = anObject)
								ifTrue: [ aList addLast: action ] ].

	^ aList.
! !

!Scheduler methodsFor: 'lists maintenance' stamp: 'jsp 1/29/1999 14:20'!
removeAnimation: anAnimation
	"Remove an animation from the Scheduler's list of animations"

	animationList remove: anAnimation ifAbsent: [].
! !

!Scheduler methodsFor: 'lists maintenance' stamp: 'jsp 1/29/1999 14:19'!
removeAction: anAction
	"Remove an action from the Scheduler's list of actions"

	actionList remove: anAction ifAbsent: [].
! !

!Scheduler methodsFor: 'initialize-release' stamp: 'jsp 3/30/1999 15:43'!
reset
	"Clears the animation, action, and alarm lists and resets the Wonderland time to 0"

	currentTime _ 0.
	speed _ 1.
	isRunning _ true.
	lastSystemTime _ (Time millisecondClockValue) / 1000.0.

	alarmList _ OrderedCollection new.
	actionList _ OrderedCollection new.
	animationList _ OrderedCollection new.! !

!Scheduler methodsFor: 'ticking' stamp: 'jsp 2/8/1999 16:38'!
resume
	"If the scheduler was paused, resume it."

	isRunning ifFalse: 	[ isRunning _ true.
						  lastSystemTime _ (Time millisecondClockValue) / 1000.0.
						].

! !

!Scheduler class methodsFor: 'initialize' stamp: 'jsp 2/3/1999 17:31'!
new
	"Creates a new Scheduler and make sure it gets initialized."

	| newScheduler |
	newScheduler _ super new.
	newScheduler initialize.
	^ newScheduler.
! !

!Scheduler methodsFor: 'ticking' stamp: 'jsp 3/8/1999 11:47'!
getElapsedTime

	"Returns the time that elapsed in the last Scheduler tick"

	^ elapsedTime.
! !

!Scheduler methodsFor: 'lists maintenance' stamp: 'jsp 1/29/1999 14:18'!
removeAlarm: anAlarm
	"Remove an alarm from the Scheduler's list of alarms"

	alarmList remove: anAlarm ifAbsent: [].
! !

!Scheduler methodsFor: 'animated object' stamp: 'jsp 3/9/1999 16:05'!
getAnimationsFor: anObject
	"Return the animations that list anObject as the object they're affecting"

	| animList |
	animList _ OrderedCollection new.

	animationList do: [:anim | ((anim getAnimatedObject) = anObject)
								ifTrue: [ animList addLast: anim ] ].

	^ animList.
! !

!Scheduler methodsFor: 'ticking' stamp: 'jsp 2/1/1999 14:25'!
incrementBy: amount
	"This method allows animations to be stepped through; the user pauses the Scheduler and then repeatedly issues incrementBy messages with the desired increment"

	currentTime _ currentTime + amount.
	lastSystemTime _ lastSystemTime + amount.
	
	self processActions.
	self processAlarms.
	self processAnimations.! !

!Scheduler methodsFor: 'ticking' stamp: 'jsp 2/3/1999 14:23'!
processAnimations
	"This method processes any active Animations and removes any that are completed."

	animationList do: [:anim | anim update: currentTime.
							(anim isDone) ifTrue: [self removeAnimation: anim]
				].
! !

!Scheduler methodsFor: 'ticking' stamp: 'jsp 2/1/1999 14:34'!
setSpeed: newSpeed
	"This method sets the speed for the Scheduler. 1 is a 1:1 mapping with clock time, 2 is a 2:1 mapping, etc."

	(speed > 0) ifTrue: [speed _ newSpeed] ifFalse: [self error: 'Scheduler speed must be greater than 0.'].

! !

!Scheduler methodsFor: 'ticking' stamp: 'ar 9/1/2000 10:24'!
tick
	"Figure out how much time has elapsed since the last Scheduler tick and update all our actions, alarms, and animations"
	| deltaTime |
	isRunning ifTrue: [
		deltaTime _ (Time millisecondClockValue / 1000.0) - lastSystemTime.
		"See if the clock rolled over or the scheduler has been put to sleep for too long"
		(deltaTime < 0.0 or:[deltaTime > 2.0]) ifTrue:[
			"adjust for extra time"
			lastSystemTime _ (Time millisecondClockValue - 1) / 1000.0.
			deltaTime _ 1.0 / 1000.0].
		elapsedTime _ deltaTime * speed.
		currentTime _ currentTime + elapsedTime.
		lastSystemTime _ lastSystemTime + elapsedTime.
	
		self processAlarms.
		self processAnimations.
		self processActions.
	].
! !

Object subclass: #Interpolateable
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Misc'!

Interpolateable class
	instanceVariableNames: 'value'!

!Interpolateable commentStamp: '<historical>' prior: 0!
The Interpolateable class is a wrapper for instances that don't have interpolation methods provided for them (like Numbers).
!

!Interpolateable methodsFor: 'accessing' stamp: 'jsp 2/9/1999 16:21'!
setValue: aValue
	"Set the base interpolateable value."

	value _ aValue.! !

!Interpolateable class methodsFor: 'instance creation' stamp: 'jsp 2/9/1999 16:20'!
value: aValue
	"Create and initialize a new interpolateable value."

	| newInterp |
	newInterp _ Interpolateable new.
	newInterp setValue: aValue.
	^ newInterp.
! !

!Interpolateable methodsFor: 'interpolating' stamp: 'jsp 2/9/1999 16:18'!
interpolateTo: end at: amountDone
	"Interpolate between the instance and end after the specified amount has been done (0 - 1)."

	^ value + ((end - value) * amountDone).! !

Object subclass: #UndoSizeChange
	instanceVariableNames: 'theActor originalSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoSizeChange commentStamp: '<historical>' prior: 0!
This class undoes the resizing of an actor.
!

!UndoSizeChange methodsFor: 'accessing' stamp: 'ar 11/7/1999 20:16'!
getActor
	^theActor! !

!UndoSizeChange methodsFor: 'undoing' stamp: 'jsp 3/8/1999 16:49'!
undoIt
	"Move the object back to its original position"

	| anim |

	anim _ theActor setSize: originalSize duration: 0.5.
	anim setUndoable: false.
! !

!UndoSizeChange class methodsFor: 'instance creation' stamp: 'jsp 3/19/1999 15:59'!
for: anActor from: size
	"Create a new UndoSizeChange instance using size as the original size"

	^ self new setActor: anActor andOriginalSize: size.
! !

!UndoSizeChange methodsFor: 'accessing' stamp: 'jsp 3/8/1999 16:48'!
setActor: anActor andOriginalSize: size
	"Set the original rotation to undo back to"

	theActor _ anActor.
	originalSize _ size.
! !

Object subclass: #Hierarchical
	instanceVariableNames: 'myParent myChildren'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Objects'!

!Hierarchical commentStamp: '<historical>' prior: 0!
The Hierarchical class provides the basic functionality for parent-child relationships (and changing these relationships) for both the WonderlandScene and any WonderlandActors in the Wonderland.
!

Hierarchical subclass: #WonderlandActor
	instanceVariableNames: 'myName myWonderland myMesh myTexture myMaterial myColor composite scaleMatrix hidden firstClass myReactions myProperties'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Objects'!

!WonderlandActor commentStamp: '<historical>' prior: 0!
The WonderlandActor class provides the basic behaviors for 3D objects.  These behaviors include (but are not limited to):

- move
- moveTo
- turn
- turnTo
- roll
- pointAt
- place
- resize

For more details, see the Wonderlands ActiveEssay (coming soon to a theater near you).
 !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 9/17/1999 14:23'!
setPivot: aVector
	"Modify the object to use the specified point as the pivot point."

	| matrix invMatrix b3dVector |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what pivot to use because ', msg.
			^ nil ].


	"Our parameters check out, so change the pivot"
	(aVector isKindOf: WonderlandActor)
		ifTrue: [ matrix _ aVector getMatrixFromRoot. ]
		ifFalse: [ b3dVector _ B3DVector3 x: (aVector at: 1) y: (aVector at: 2)
									z: (aVector at: 3).
				  matrix _ B3DMatrix4x4 identity translation: b3dVector ].

	"Add an undo action"
	b3dVector _ self getMatrixFromRoot translation.
	myWonderland getUndoStack push: (UndoAction new: [self setPivot: b3dVector ]).

	invMatrix _ (matrix inverseTransformation) composeWith: (self getMatrixFromRoot).

	myMesh ifNotNil: [ myMesh _ myMesh copy.
					    myMesh vertices: (myMesh vertices copy).
					    myMesh transformBy: invMatrix ].

	composite _ (myParent getMatrixToRoot) composeWith: matrix.

	(self getChildren) do: [:child | child setComposite:
										(invMatrix composeWith: (child getComposite)) ].

! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 15:19'!
setPointOfView: aVector duration: aDuration style: aStyle
	"Sets the object's position and orientation"

	| positionVector rotationVector |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyPointOfView: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what point of view to assign ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for changing the point of view for ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use for changing the point of view for ' , myName , ' because ', msg.
			^ nil ].

	"Our parameters check out, so built the animation"
	(aVector isKindOf: WonderlandActor)
		ifTrue: [
					positionVector _ aVector.
					rotationVector _ aVector.
				]
		ifFalse: [
					positionVector _ Array new: 3.
					positionVector at: 1 put: (aVector at: 1).
					positionVector at: 2 put: (aVector at: 2).
					positionVector at: 3 put: (aVector at: 3).

					rotationVector _ Array new: 3.
					rotationVector at: 1 put: (aVector at: 4).
					rotationVector at: 2 put: (aVector at: 5).
					rotationVector at: 3 put: (aVector at: 6).
				].

	^ myWonderland doTogether: { self moveTo: positionVector duration: aDuration style: aStyle.
								self turnTo: rotationVector duration: aDuration style: aStyle }.
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:38'!
color
	^myColor asColor! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/9/1999 15:56'!
wait
	"Creates an animation where the object merely waits for one second"

	| anim |

	anim _ RelativeAnimation new.

	anim object: self
			update: [:temp | temp ]
			getStartState: [ Interpolateable value: 0 ]
			getEndState: [ 1.0 ]
			getReverseState: [ 1.0 ]
			style: [:elapsed :total | WonderlandStyle abruptlyAfter: elapsed outOf: total]
			duration: 1.0
			undoable: true
			inWonderland: myWonderland.

	^ anim.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/22/1999 12:29'!
resizeLikeRubber: amount dimension: aDimension duration: aDuration style: aStyle
	"Resizes the object using a volume preserving resize"

	| x y z |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDimension: aDimension ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine along which dimension to resize ' , myName , '  because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: amount ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , '  because ', msg.
			^ nil ].

	(aDimension = leftToRight)
		ifTrue: [
					x _ amount.
					y _ 1 / (amount sqrt).
					z _ 1 / (amount sqrt).
				]
		ifFalse: [ (aDimension = topToBottom)
				ifTrue: [
							x _ 1 / (amount sqrt).
							y _ amount.
							z _ 1 / (amount sqrt).
						]
				ifFalse: [ (aDimension = frontToBack)
						ifTrue: [
									x _ 1 / (amount sqrt).
									y _ 1 / (amount sqrt).
									z _ amount.
								].
						].
				].

	^ self resizeTopToBottom: y leftToRight: x frontToBack: z duration: aDuration style: aStyle.

! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 11/7/1999 22:17'!
getProperty: aSymbol ifAbsent: aBlock
	myProperties == nil ifTrue:[^aBlock value].
	^myProperties at: aSymbol ifAbsent: aBlock! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/22/1999 12:42'!
resizeLikeRubber: amount dimension: aDimension duration: aDuration
	"Resizes the object using a volume preserving resize"

	| x y z |

	(aDuration = rightNow) ifTrue: [
		(aDimension = leftToRight)
			ifTrue: [
						x _ amount.
						y _ 1 / (amount sqrt).
						z _ 1 / (amount sqrt).
					]
			ifFalse: [ (aDimension = topToBottom)
					ifTrue: [
								x _ 1 / (amount sqrt).
								y _ amount.
								z _ 1 / (amount sqrt).
							]
					ifFalse: [ (aDimension = frontToBack)
							ifTrue: [
										x _ 1 / (amount sqrt).
										y _ 1 / (amount sqrt).
										z _ amount.
									].
							].
					].

			^ self resize: { x. y. z } duration: aDuration.
		].

	^ self resizeLikeRubber: amount dimension: aDimension duration: aDuration style: gently.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/5/1999 12:23'!
standUp
	"Rotate the object to align it along its parent's up vector"

	^ self turnTo: #(0 #asIs 0) duration: 1.0 style: gently.
! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 12/25/2001 16:02'!
hasOnlySketchCostumes
	"Workaround"
	^false! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:53'!
moveTo: aVector duration: aDuration asSeenBy: reference style: aStyle
	"Moves the object to the specified position in its parent's coordinate system using the specified style over the specified duration."

	| anim endStateFunc pos newComposite vectorMatrix |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for moving ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to move ' , myName , ' because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ AbsoluteAnimation new.

	endStateFunc _ [
		vectorMatrix _ B3DMatrix4x4 identity.
		pos _ self getPosition: reference.
		((aVector at: 1) = asIs) ifFalse: [ pos at: 1 put: (aVector at: 1) ].
		((aVector at: 2) = asIs) ifFalse: [ pos at: 2 put: (aVector at: 2) ].
		((aVector at: 3) = asIs) ifFalse: [ pos at: 3 put: (aVector at: 3) ].
		vectorMatrix translation: (B3DVector3 x: (pos at: 1) y: (pos at: 2) z: (pos at: 3)).
		newComposite _ (myParent getMatrixToRoot) composeWith: (reference getMatrixFromRoot).
		newComposite _ newComposite composeWith: vectorMatrix.
		newComposite translation.
					].


	anim object: self
			update: [:tPos | self setPositionVector: tPos]
			getStartState: [self getPositionVector]
			getEndState: endStateFunc
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 9/10/2000 02:21'!
getPointSize
	^self getProperty: #pointSize ifAbsent:[1].! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 9/10/2000 02:20'!
getPolygonMode
	^self getProperty: #polygonMode! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:51'!
roll: aDirection turns: numTurns speed: aSpeed
	"Roll the actor the specified number of turns in the specified direction, taking the specified duration and using the specified animation style."

	| anim aDuration updateFunc degrees lastAmount tMatrix |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: numTurns ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the number of turns to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to roll ' , myName , ' at because ', msg.
			^ nil ].

	aDuration _ (numTurns / aSpeed) abs.
	anim _ RelativeAnimation new.
	degrees _ 360 * numTurns.
	lastAmount _ 0.
	tMatrix _ B3DMatrix4x4 new.

	(aDirection = #left) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #right) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated. ]

		ifFalse: [ myWonderland reportErrorToUser: 'Squeak does not know how to roll ' , myName , ' ', (aDirection asString) , '.'.
				^ nil ]].
	
	anim object: self
			update: updateFunc
			getStartState: [ lastAmount _ 0. (Interpolateable value: 0) ]
			getEndState: [ degrees ]
			getReverseState: [ degrees negated ]
			style: abruptly
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 8/31/2000 20:01'!
getMeshAnimationParameter
	myMesh == nil ifTrue:[^0.0].
	^myMesh animationParameter! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/12/1999 10:13'!
loadMeshFromFile: meshFile
	"Load this object's mesh from the specified file"

	| dict words |

	dict _ myWonderland getSharedMeshDict.

	myMesh _ dict at: meshFile
				ifAbsent: [
							words _ (meshFile findTokens: #.).

							((words last) = 'vfb') ifTrue: [
								myMesh _ WonderlandMesh fromVFBFile: meshFile ].

							((words last) = 'obj') ifTrue: [
								myMesh _ WonderlandMesh fromOBJFile: meshFile ].

							dict at: meshFile put: myMesh.
						].
! !

!WonderlandActor methodsFor: 'morphic reactions' stamp: 'ar 10/10/2000 21:33'!
morphicMouseUp: event
	"Handle the given event"
	(event getMorphicEvent shiftPressed and:[self getUserPointOfView notNil])
		ifTrue:[^self]. "Reserved for navigation"
	(self getProperty: #userTransition) == true ifTrue:["Shift key went up"
		^self setProperty: #userTransition toValue: nil].
	myTexture isMorph ifTrue:[
		self dispatchEvent: event to: myTexture.
	].! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/8/2001 18:53'!
vocabularyDemanded
	"Answer the vocabulary that the receiver really would like to use in a Viewer"

	^ Vocabulary wonderlandVocabulary! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/10/1999 16:45'!
getPositionVector
	"Returns the current position as a vector."

	^ (composite translation).
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:20'!
move: aDirection speed: aSpeed asSeenBy: reference
	"Creates and returns an action that moves the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to move ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	"The parameters check out, so build the action"
	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveRightNow: aDirection
							distance: ((myWonderland getScheduler getElapsedTime) * aSpeed)
							asSeenBy: reference
							undoable: false ].
! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 11/7/1999 18:32'!
getFullBoundsFor: aCamera
	"Return the 2D bounds of the receiver and its children or nil if not visible"
	| box bounds |
	box _ self getBoundsFor: aCamera.
	myChildren do:[:child|
		bounds _ child getFullBoundsFor: aCamera.
		bounds == nil ifFalse:[
			box == nil ifTrue:[box _ bounds] ifFalse:[box _ box quickMerge: bounds].
		].
	].
	^box! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/24/1999 10:25'!
setMesh: aMesh
	"Sets the actor's mesh"

	myMesh _ aMesh.
! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 3/19/1999 14:45'!
getSize
	"Returns the scale factor of the actor"

	| anArray meshSize |

	meshSize _ self getBoundingBox extent.

	anArray _ Array new: 3.
	anArray at: 1 put: (meshSize x).
	anArray at: 2 put: (meshSize y).
	anArray at: 3 put: (meshSize z).

	^ anArray.
! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 2/24/1999 11:46'!
getAngles
	"Return the object's current orientation in its parent's coordinate system"

	| a3DVector anArray |

	anArray _ Array new: 3.
	a3DVector _ composite rotation.

	anArray at: 1 put: (a3DVector x).
	anArray at: 2 put: (a3DVector y).
	anArray at: 3 put: (a3DVector z).

	^ anArray.

	! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:46'!
roll: aDirection speed: aSpeed until: aCondition
	"Creates and returns an action that rolls the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to roll ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should roll ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self rollRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				undoable: false
						]
		   until: aCondition.

! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/18/1999 10:18'!
moveRightNow: aDirection distance: aDistance asSeenBy: reference undoable: isUndoable
	"Move this object in the specified direction instantaneously"

	| a3DVector aMatrix newComposite |

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoPositionChange for: self from: (self getPosition)).
						].

	[ a3DVector _ self makeVectorFromDistance: aDistance andDirection: aDirection. ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak does not know how to move ' , myName , ' in that direction.'.
			^ nil ].

	aMatrix _ B3DMatrix4x4 identity translation: a3DVector.

	newComposite _ (reference getMatrixToRoot) composeWith: (self getMatrixFromRoot).
	composite _ (((myParent getMatrixToRoot)
							composeWith: (reference getMatrixFromRoot))
							composeWith: aMatrix)
							composeWith: newComposite.


! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:50'!
roll: aDirection turns: numTurns duration: aDuration asSeenBy: reference style: aStyle
	"Roll the actor the specified number of turns in the specified direction, taking the specified duration and using the specified animation style."

	| anim updateFunc degrees lastAmount buildMatrix newMatrix tMatrix|

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: numTurns ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the number of turns to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for rolling ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not roll ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to roll ' , myName , ' because ', msg.
			^ nil ].

	"The parameters check out, so go ahead and build the animation"
	anim _ RelativeAnimation new.
	degrees _ 360 * numTurns.
	lastAmount _ 0.
	tMatrix _ B3DMatrix4x4 new.

	buildMatrix _ [ :rotMatrix | newMatrix _ reference getMatrixToRoot.
						newMatrix _ newMatrix composeWith: (self getMatrixFromRoot).
						newMatrix _ (((myParent getMatrixToRoot)
										composeWith: (reference getMatrixFromRoot))
										composeWith: rotMatrix)
										composeWith: newMatrix. ].

	(aDirection = #left) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #right) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated. ]

		ifFalse: [ myWonderland reportErrorToUser: 'Squeak does not know how to roll ' , myName , ' ', (aDirection asString) , '.'.
				^ nil ]].
	
	anim object: self
			update: updateFunc
			getStartState: [ lastAmount _ 0. (Interpolateable value: 0) ]
			getEndState: [ degrees ]
			getReverseState: [ degrees negated ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 17:27'!
place: aLocation object: aTarget duration: aDuration style: aStyle
	"Moves this object to one of several pre-determined locations relative to the target object."

	 | anim selfPos selfBBox selfLBB selfRUF objBBox objLBB objRUF pos |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyLocation: aLocation ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyActor: aTarget ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for placing ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to place ' , myName , ' because ', msg.
			^ nil ].

	"The parameters check out, so build the animation"
	selfBBox _ self getBoundingBox: aTarget.
	selfLBB _ selfBBox origin.
	selfRUF _ selfBBox corner.

	objBBox _ aTarget getBoundingBox.
	objLBB _ objBBox origin.
	objRUF _ objBBox corner.

	selfPos _ self getPosition: aTarget.

	((aLocation = onTopOf) or: [ aLocation = above])
	ifTrue: [
				pos _ { 0. (objRUF y) + ((selfPos at: 2) - (selfLBB y)). 0}
			]
 	ifFalse: [

	((aLocation = onBottomOf) or: [ (aLocation = beneath) or: [ aLocation = below ] ])
	ifTrue: [
				pos _ { 0. (objLBB y) - ((selfRUF y) - (selfPos at: 2)). 0}
			]
	ifFalse: [

	(aLocation = toLeftOf)
	ifTrue: [
				pos _ { (objLBB x) - ((selfRUF x) - (selfPos at: 1)). 0. 0 }
			]
	ifFalse: [

	(aLocation = toRightOf)
	ifTrue: [
				pos _ { (objRUF x) + ((selfPos at: 1) - (selfLBB x)). 0. 0 }
			]
	ifFalse: [

	(aLocation = inFrontOf)
	ifTrue: [
				pos _ { 0. 0. (objRUF z) + ((selfPos at: 3) - (selfLBB z)) }
			]
	ifFalse: [

	((aLocation = inBackOf) or: [ aLocation = behind])
	ifTrue: [
				pos _ { 0. 0. (objLBB z) - ((selfRUF z) - (selfPos at:3)) }
			]
	ifFalse: [

	(aLocation = onCeilingOf)
	ifTrue: [
				pos _ { 0. (objRUF y) - ((selfRUF y) - (selfPos at:2)). 0}
			]
	ifFalse: [

	(aLocation = onFloorOf)
	ifTrue: [
				pos _ { 0. (objLBB y) + ((selfPos at: 2) - (selfLBB y)). 0}
			]
	ifFalse: [ self error: 'Unrecognized target surface' ]]]]]]]].


	anim _	self moveTo: pos duration: aDuration asSeenBy: aTarget style: aStyle.

	^ anim.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/29/1999 14:39'!
pointAt: target eachFrameUntil: aCondition
	"Turns the object to point at the specified target each frame for the specified lifetime."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTargetOrPixel: target ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what to point ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should point ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self pointAtRightNow: target undoable: false ]
			until: aCondition.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:25'!
move: aDirection distance: aDistance duration: aDuration asSeenBy: reference
	"Move the actor the specified distance in the specified direction, taking the specified duration and using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyDirection: aDirection ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: aDistance ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the distance to move ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyReferenceFrame: reference ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
				^ nil ].

		self moveRightNow: aDirection distance: aDistance asSeenBy: reference undoable: true.
		^ self. ].

	^ (self move: aDirection distance: aDistance duration: aDuration asSeenBy: reference
			style: gently).

! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 9/10/2000 02:22'!
setPointSize: aNumber
	self setProperty: #pointSize toValue: aNumber.! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 4/9/1999 14:35'!
reparentTo: anObject
	"Make this actor a child of the specified object."

	| newComposite classList parentClass |

	classList _ myWonderland getActorClassList.

	"First break ties with the current parent"
	myParent removeChild: self.
	parentClass _ myParent class.

	(parentClass == WonderlandScene)
		ifTrue: [
				myWonderland getNamespace removeKey: myName ifAbsent: [].
				]
		ifFalse: [
				classList remove: parentClass.
				parentClass removeSelector: (myName asSymbol).
				parentClass removeInstVarName: myName.
				classList addLast: (myParent class).
				].

	"Figure out the new composite transformation matrix"
	newComposite _ anObject getMatrixToRoot.
	newComposite _ newComposite composeWith: (self getMatrixFromRoot).
	
	"Now build ties with the new parent"
	anObject addChild: self.
	myParent _ anObject.
	parentClass _ myParent class.

	(parentClass == WonderlandScene)
		ifTrue: [
				(myWonderland getNamespace at: myName ifAbsent: [ nil ])
					ifNotNil: [ myName _ myWonderland uniqueNameFrom: myName ].

				(myWonderland getNamespace at: myName put: self).
				]
		ifFalse: [
				myName _ parentClass uniqueNameFrom: myName.		
				classList remove: parentClass.
				myParent addInstanceVarNamed: myName withValue: self.
				(myParent class) compile: (myName , '
											^ ' , myName, '.').
				classList addLast: (myParent class).
				].

	composite _ newComposite.

	"Now update the actor browser"
	myWonderland getEditor updateActorBrowser.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:04'!
resizeTopToBottom: height leftToRight: width frontToBack: thickness duration: aDuration
	"Resize the object by the specified amount over the specified duration using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyNumber: height ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from top to bottom because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: width ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from left to right because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: thickness ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from front to back because ', msg.
				^ nil ].

		self resizeRightNow: (B3DVector3 x: width y: height z: thickness) undoable: true.
		^ self.
									].

	(aDuration = eachFrame) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyNumber: height ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from top to bottom because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: width ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from left to right because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: thickness ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from front to back because ', msg.
				^ nil ].

		myWonderland getUndoStack push: (UndoSizeChange for: self from: (self getSize)).

		^ self doEachFrame: [ self resizeRightNow: (B3DVector3 x: width y: height z: thickness)
									undoable: false ]
									].

	^ (self resizeTopToBottom: height leftToRight: width frontToBack: thickness
				duration: aDuration style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:22'!
turnTo: aVector eachFrameFor: aLifetime
	"Turns the object to the specified orientation in its parent's coordinate system each frame for aLifetime seconds."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should turn ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self turnToRightNow: aVector	undoable: false ]
			for: aLifetime.

! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/9/2001 22:39'!
assignIncrGetter: getterSelector setter: setterSelector amt: anIncrement
	self perform: setterSelector with:
		((self perform: getterSelector) + anIncrement)! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:59'!
turn: aDirection asSeenBy: reference
	"Turn the actor once around in the specified direction, taking 1 second and using the Gently animation style."

	^ (self turn: aDirection turns: 0.25 duration: 1.0 asSeenBy: reference
			style: gently).

! !

!WonderlandActor methodsFor: 'event handling' stamp: 'jsp 4/19/1999 17:13'!
onLeftMouseUp: event
	"The default response to left mouse up events"

	(event getCameraMorph) setEventFocus: nil.
	self respondWith: nil to: mouseMove.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:18'!
move: aDirection distance: aDistance duration: aDuration style: aStyle
	"Move the actor the specified distance in the specified direction, taking the specified duration and using the specified animation style."

	| anim a3DVector tmpVector tMatrix |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: aDistance ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the distance to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for moving ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to move ' , myName , ' because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ RelativeAnimation new.
	tmpVector _ (B3DVector3 x:0 y:0 z:0).
	tMatrix _ B3DMatrix4x4 identity.

	[ a3DVector _ self makeVectorFromDistance: aDistance andDirection: aDirection. ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak does not know how to move ' , myName , ' in that direction.'.
			^ nil ].

	anim object: self
			update: [:tPos | composite _ composite composeWith:
												(tMatrix translation: (tPos - tmpVector)).
							tmpVector _ tPos]
			getStartState: [tmpVector x:0; y:0; z:0.
								(B3DVector3 x:0 y:0 z:0) ]
			getEndState: [ a3DVector ]
			getReverseState: [ a3DVector negated ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.

! !

!WonderlandActor methodsFor: 'do-ing' stamp: 'jsp 3/30/1999 11:38'!
doEachFrame: aBlock for: aDuration
	"Build an action that causes the object to execute the block every frame for the specified amount of time"

	| newAction |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyZeroArgumentBlock: aBlock ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not make ' , myName , ' do what you asked because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long ' , myName , ' should do this because ', msg.
			^ nil ].

	newAction _ Action do: aBlock eachframefor: aDuration toObject: self
						inScheduler: (myWonderland getScheduler).
	(myWonderland getUndoStack) push:
			(UndoByStopping new: newAction in: (myWonderland getUndoStack)).

	^ newAction.
! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 11/7/1999 18:32'!
getBoundsFor: aCamera
	"Return the 2D bounds of the receiver as seen by the camera"
	^aCamera getBoundsForActor: self! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:45'!
setDepth: value
	self resize: {1.0. 1.0. value / self depth asFloat} duration: rightNow.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:07'!
moveTo: aVector speed: aSpeed asSeenBy: reference
	"Moves the object to the specified position in its parent's coordinate system using the specified style over the specified duration."

	^ self moveTo: aVector duration: (((self distanceTo: aVector) / aSpeed) abs)
			asSeenBy: reference style: abruptly.
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/9/2001 15:45'!
setGraphic: aGraphicalObject
	aGraphicalObject isForm ifTrue:[
		^self setTexturePointer: aGraphicalObject.
	].
	aGraphicalObject isMorph ifTrue:[
		^aGraphicalObject installAsWonderlandTextureOn: self.
	].! !

!WonderlandActor methodsFor: 'printing' stamp: 'ar 4/18/1999 00:49'!
printOn: aStream
	aStream 
		nextPutAll: self class name;
		nextPut:$<;
		print: myName;
		nextPut:$>.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:10'!
pointAt: target
	"Turns the object to point at the specified target over the specified duration using the Gently animation style."

	^ (self pointAt: target duration: 1.0
			style: gently).
! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 3/25/1999 14:59'!
getPositionInPicturePlane
	"Returns the actor's position in the picture plane of the default camera"

	| camera d x y z pos |

	camera _ myWonderland getDefaultCamera.

	pos _ self getPosition: camera.
	d _ (camera getFrustum) near.

	z _ (pos at: 3) abs.

	(z = 0) ifTrue: 	[
						x _ 0.
						y _ 0.
					]
			ifFalse: [
						x _ (d / z) * (pos at: 1).
						y _ (d / z) * (pos at: 2).
					].
					
	^ x @ y.
! !

!WonderlandActor methodsFor: 'animated state' stamp: 'jsp 3/15/1999 16:15'!
resume
	"Resume any actions and animations associated with this object"

	| scheduler |

	scheduler _ myWonderland getScheduler.

	(scheduler getAnimationsFor: self) do: [:anim | anim resume ].

	(scheduler getActionsFor: self) do: [:action | action resume ].

	myChildren do: [:child | (child isPart) ifTrue: [ child resume ] ].! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/9/2001 22:39'!
assignMultGetter: getterSelector setter: setterSelector amt: aMultiplier
	self perform: setterSelector with:
		((self perform: getterSelector) * aMultiplier)! !

!WonderlandActor methodsFor: 'drawing' stamp: 'ar 9/10/2000 02:24'!
drawMesh: aRenderer
	"Draw the mesh for this actor."
	| polyMode |
	myMaterial ifNotNil: [
			aRenderer pushMaterial.
			aRenderer material: myMaterial.
						].

	myTexture ifNotNil: [
			aRenderer pushTexture.
			aRenderer texture: myTexture.
						].
	aRenderer cullFace: self getBackfaceCulling.
	polyMode _ self getPolygonMode.
	aRenderer polygonMode: polyMode.
	polyMode == nil ifFalse:[
		aRenderer lineWidth: self getLineWidth.
		aRenderer pointSize: self getPointSize.
	].
	"Note: Using myMesh>>renderOn: here prevents meshes from being picked!!"
	myMesh ifNotNil: [ myMesh renderOn: aRenderer].

	myTexture ifNotNil: [ aRenderer popTexture ].

	myMaterial ifNotNil: [ aRenderer popMaterial ].
! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 9/16/1999 16:30'!
getTexturePointer
	^myTexture! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:09'!
move: aDirection
	"Move the actor one meter in the specified direction, taking 1 second and using the Gently animation style."

	^ (self move: aDirection distance: 1.0 duration: 1.0
			style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/29/1999 14:39'!
pointAt: target duration: aDuration
	"Turns the object to point at the specified target over the specified duration using the Gently animation style."


	(aDuration = rightNow) ifTrue: [
		"Check the target to make sure it's valid"
		[ WonderlandVerifier VerifyTargetOrPixel: target ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine what to point ' , myName , ' at because ', msg.
				^ nil ].
		self pointAtRightNow: target undoable: true.
		^ self. ].

	(aDuration = eachFrame) ifTrue: [
		"Check the target to make sure it's valid"
		[ WonderlandVerifier VerifyTargetOrPixel: target ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine what to point ' , myName , ' at because ', msg.
				^ nil ].

		myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

		^ self doEachFrame: [ self pointAtRightNow: target undoable: false ].
									].

	^ (self pointAt: target duration: aDuration style: gently).
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 16:42'!
setVisibilityRightNow: newVisibility undoable: isUndoable
	"Change this instance's visibility instantaneously"

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoVisibilityChange for: self
																 from: (self getVisibility)) ].

	self setColorVector: (B3DColor4 red: (myColor red) green: (myColor green)
						blue: (myColor blue) alpha: newVisibility).
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 4/8/1999 14:46'!
getChildNamed: childName
	"Returns the child with the specified name if it is a child of this object"

	| theChild nameParts |

	childName at: 1 put: ((childName at: 1) asLowercase).

	"First see if they're looking for me"
	(myName = childName) ifTrue: [ ^ self ].

	nameParts _ childName findTokens: ' '.
	nameParts do: [:name | name at: 1 put: ((name at: 1) asLowercase) ].

	((nameParts first) = myName) ifTrue: [ nameParts removeFirst ].
	theChild _ self.
	nameParts do: [:aName | theChild _ theChild perform: (aName asSymbol) ].
	^ theChild.! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:39'!
getY
	^(self getPointOfView at: 2) * 100! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 4/9/1999 14:09'!
turnTo: aVector speed: aSpeed 
	"Turns the object to the specified orientation in its parent's coordinate system at aSpeed turns per second."

	^ self turnTo: aVector duration: (((self angularDistanceTo: aVector) / (aSpeed * 360)) abs)
			style: abruptly.

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 17:30'!
place: aLocation object: aTarget eachFrameUntil: aCondition
	"Moves this object to one of several pre-determined locations relative to the target object each frame for the specified lifetime."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyLocation: aLocation ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyActor: aTarget ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine for how long it should place ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self placeRightNow: aLocation object: aTarget undoable: false ]
			until: aCondition.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/4/1999 11:52'!
alignWith: anObject duration: aDuration style: aStyle
	"This method sets the orientation of this instance to be the same as that of the specified object."

	^ self turnTo: anObject duration: aDuration style: aStyle.
! !

!WonderlandActor class methodsFor: 'unique name creation' stamp: 'jsp 4/8/1999 16:20'!
uniqueNameFrom: aName
	"If aName is not an instance variable of this class, returns aName.  Otherwise it returns a unique name based on aName that is not an instance var."

	| index |

	(self instVarNames includes: aName) ifFalse: [ ^ aName ].

	index _ 2.
	[ self instVarNames includes: (aName , (index asString)) ]
		whileTrue: [ index _ index + 1 ].

	^ aName , (index asString).
	! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 12:42'!
moveToRightNow: aVector asSeenBy: reference undoable: isUndoable
	"Move this object in the specified direction instantaneously"

	| pos aMatrix newComposite |

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoPositionChange for: self from: (self getPosition)).
						].

	aMatrix _ B3DMatrix4x4 identity.
	pos _ self getPosition: reference.
	((aVector at: 1) = asIs) ifFalse: [ pos at: 1 put: (aVector at: 1) ].
	((aVector at: 2) = asIs) ifFalse: [ pos at: 2 put: (aVector at: 2) ].
	((aVector at: 3) = asIs) ifFalse: [ pos at: 3 put: (aVector at: 3) ].
	aMatrix translation: (B3DVector3 x: (pos at: 1) y: (pos at: 2) z: (pos at: 3)).
	newComposite _ (myParent getMatrixToRoot) composeWith: (reference getMatrixFromRoot).
	newComposite _ newComposite composeWith: aMatrix.
		newComposite translation.
	self setPositionVector: (newComposite translation).

! !

!WonderlandActor methodsFor: 'morphic reactions' stamp: 'ar 10/10/2000 21:32'!
morphicMouseMove: event
	"Handle the given event"
	(event getMorphicEvent shiftPressed and:[self getUserPointOfView notNil])
		ifTrue:[^self]. "Reserved for navigation"
	(self getProperty: #userTransition) == true ifTrue:[^true]. "Shift key went up"
	myTexture isMorph ifTrue:[
		self dispatchEvent: event to: myTexture.
	].! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'ar 8/31/2000 19:59'!
animateMeshTo: param
	^ (self animateMeshTo: param duration: 1.0
			style: gently).
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:45'!
depth
	^self getBoundingBox extent z * 100! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 17:01'!
resizeRightNow: aVector undoable: isUndoable
	"Set this object's size instantaneously"

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoSizeChange for: self from: (self getSize)).
						].

	self scaleByVector: aVector.
! !

!WonderlandActor methodsFor: 'copying' stamp: 'tk 9/5/2001 09:44'!
veryDeepInner: deepCopier
	"Special code that handles user-added instance variables of a uniClass.
	Copy all of my instance variables.  Some need to be not copied at all, but shared.  This is special code for the dictionary.  See DeepCopier."
	| instVar weak subAss |

	super veryDeepInner: deepCopier.
	"my own instance variables are completely normal"
	myName _ myName veryDeepCopyWith: deepCopier.
	myWonderland _ myWonderland.		"don't make a new one"
	myMesh _ myMesh veryDeepCopyWith: deepCopier.
	myTexture _ myTexture veryDeepCopyWith: deepCopier.
	myMaterial _ myMaterial veryDeepCopyWith: deepCopier.
	myColor _ myColor veryDeepCopyWith: deepCopier.
	scaleMatrix _ scaleMatrix veryDeepCopyWith: deepCopier.
	hidden _ hidden veryDeepCopyWith: deepCopier.
	firstClass _ firstClass veryDeepCopyWith: deepCopier.
	myReactions _ myReactions veryDeepCopyWith: deepCopier.
	myProperties _ myProperties veryDeepCopyWith: deepCopier.	"may copy too deeply"

	WonderlandActor instSize + 1 to: self class instSize do: [:index |
		instVar _ self instVarAt: index.
		weak _ instVar isMorph | instVar isPlayerLike. 
		(subAss _ deepCopier references associationAt: instVar ifAbsent: [nil])
				"use association, not value, so nil is an exceptional value"
			ifNil: [weak ifFalse: [
					self instVarAt: index put: (instVar veryDeepCopyWith: deepCopier)]]
			ifNotNil: [self instVarAt: index put: subAss value].
		].
! !

!WonderlandActor methodsFor: 'playing sound' stamp: 'jsp 4/22/1999 11:26'!
playSound: soundFile
	"Create an animation that plays the sound and lasts the duration of the sound"

	| aSound extension |

	extension _ (soundFile findTokens: '.') last.

	(extension = 'wav')
		ifTrue: [ [ aSound _ SampledSound fromWaveFileNamed: soundFile ]
					ifError: [ :msg : rcvr | myWonderland reportErrorToUser:
										(soundFile asString) , ' is not a valid sound.'.
											^ nil ]
				]
		ifFalse: [ aSound _ SampledSound soundNamed: soundFile.
				  aSound ifNil: [ ^ nil ] ].

	^ myWonderland doTogether: {
				self do: [ aSound play ].
				self wait: (aSound duration)
								}.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:44'!
roll: aDirection speed: aSpeed asSeenBy: reference
	"Creates and returns an action that rolls the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to roll ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not roll ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self rollRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				asSeenBy: reference
				undoable: false
						].
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/22/1999 14:32'!
getMatrixToRoot
	"Returns the composite transformation matrix from this instance up to the root"

	^ ((myParent getMatrixFromRoot) composeWith: composite) inverseTransformation.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:16'!
move: aDirection distance: aDistance speed: aSpeed asSeenBy: reference
	"Move the actor the specified distance in the specified direction, taking the specified duration and using the specified animation style."

	| anim a3DVector aDuration tmpVector tMatrix newComposite |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: aDistance ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the distance to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to move ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	"The parameters check out, so build the animation"
	anim _ RelativeAnimation new.
	tmpVector _ (B3DVector3 x:0 y:0 z:0).
	tMatrix _ B3DMatrix4x4 identity.

	[ a3DVector _ self makeVectorFromDistance: aDistance andDirection: aDirection. ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak does not know how to move ' , myName , ' in that direction.'.
			^ nil ].

	aDuration _ (aDistance / aSpeed) abs.

	anim object: self
			update: [:tPos | newComposite _ (reference getMatrixToRoot)
										composeWith: (self getMatrixFromRoot).
						 composite _ (((myParent getMatrixToRoot)
										composeWith: (reference getMatrixFromRoot))
										composeWith:
										(tMatrix translation: (tPos - tmpVector)))
										composeWith: newComposite.
						 tmpVector _ tPos]
			getStartState: [tmpVector x:0; y:0; z:0.
								(B3DVector3 x:0 y:0 z:0) ]
			getEndState: [ a3DVector ]
			getReverseState: [ a3DVector negated ]
			style: abruptly
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'ar 5/9/2001 22:37'!
getAnimationIndex
	"Why does eToy system send #getXyz instead of #xyz???"
	^self animationIndex! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/9/2001 22:39'!
assignDecrGetter: getterSelector setter: setterSelector amt: aDecrement
	self perform: setterSelector with:
		((self perform: getterSelector) - aDecrement)! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:56'!
turn: aDirection turns: numTurns speed: aSpeed
	"Turn the actor the specified distance in the specified direction, taking the specified duration and using the specified animation style."

	| anim aDuration updateFunc degrees lastAmount tMatrix |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: numTurns ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how many turns to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to turn ' , myName , ' at because ', msg.
			^ nil ].

	aDuration _ (numTurns / aSpeed) abs.

	anim _ RelativeAnimation new.
	degrees _ 360 * numTurns.
	lastAmount _ 0.
	tMatrix _ B3DMatrix4x4 new.

	(aDirection = #right) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundY: (tAngle - lastAmount)).
							lastAmount _ tAngle] ]

		ifFalse: [(aDirection = #left) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundY: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated ]

		ifFalse: [(((aDirection = #up) or: [aDirection = #back]) or: [aDirection = #backward])
			ifTrue: [
				updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundX: (tAngle - lastAmount)).
							lastAmount _ tAngle].
				degrees _ degrees negated ]

		ifFalse: [((aDirection = #down) or: [aDirection = #forward]) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundX: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #ccw) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #cw) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated. ]

		ifFalse: [ self error: 'Unrecognized direction' ]]]]]].
	
	anim object: self
			update: updateFunc
			getStartState: [ lastAmount _ 0. (Interpolateable value: 0) ]
			getEndState: [ degrees ]
			getReverseState: [ degrees negated ]
			style: abruptly
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:48'!
roll: aDirection turns: numTurns duration: aDuration
	"Roll the actor the specified number of turns in the specified direction, taking the specified duration and using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyDirection: aDirection ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: numTurns ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the number of turns to roll ' , myName , ' because ', msg.
				^ nil ].

			self rollRightNow: aDirection numberOfTurns: numTurns undoable: true.
			^ self. ].

	^ (self roll: aDirection turns: numTurns duration: aDuration
			style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/8/1999 17:02'!
alignWith: anObject duration: aDuration
	"This method sets the orientation of this instance to be the same as that of the specified object."

	^ self turnTo: anObject duration: aDuration.
! !

!WonderlandActor methodsFor: 'do-ing' stamp: 'jsp 3/30/1999 11:38'!
doEachFrame: aBlock
	"Build an action that causes the object to execute the block every frame"

	| newAction |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyZeroArgumentBlock: aBlock ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not make ' , myName , ' do what you asked because ', msg.
			^ nil ].

	newAction _ Action do: aBlock toObject: self inScheduler: (myWonderland getScheduler).
	(myWonderland getUndoStack) push:
			(UndoByStopping new: newAction in: (myWonderland getUndoStack)).

	^ newAction.! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:40'!
setX: value
	^self setPointOfView: ((self getPointOfView) at: 1 put: value * 0.01; yourself) duration: rightNow! !

!WonderlandActor methodsFor: 'set property' stamp: 'ar 11/7/1999 22:19'!
setProperty: aSymbol toValue: anObject
	anObject == nil ifTrue:["Remove property"
		myProperties ifNotNil:[^myProperties removeKey: aSymbol ifAbsent:[nil]]].
	myProperties == nil ifTrue:[myProperties _ IdentityDictionary new].
	myProperties at: aSymbol put: anObject.! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 15:24'!
setPointOfView: aVector eachFrameFor: aLifetime
	"Sets the object's position and orientation"

	| positionVector rotationVector |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyPointOfView: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what point of view to assign ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long ' , myName , ' should keep this point of view because ', msg.
			^ nil ].

	(aVector isKindOf: WonderlandActor)
			ifTrue: [
						positionVector _ aVector.
						rotationVector _ aVector.
					]
			ifFalse: [
						positionVector _ Array new: 3.
						positionVector at: 1 put: (aVector at: 1).
						positionVector at: 2 put: (aVector at: 2).
						positionVector at: 3 put: (aVector at: 3).

						rotationVector _ Array new: 3.
						rotationVector at: 1 put: (aVector at: 4).
						rotationVector at: 2 put: (aVector at: 5).
						rotationVector at: 3 put: (aVector at: 6).
					].

	myWonderland getUndoStack push: (UndoPOVChange for: self
														from: (self getPointOfView)).

	^ self doEachFrame: [ 
							self moveToRightNow: positionVector undoable: false.
							self turnToRightNow: rotationVector undoable: false.
						]
			for: aLifetime.

! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:40'!
turnBy: angle
	"Note: This uses angle - Alice uses turns"
	self turn: right turns: (angle / 360.0) duration: rightNow.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:09'!
turn: aDirection speed: aSpeed asSeenBy: reference
	"Creates and returns an action that turn the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to turn ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self turnRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				asSeenBy: reference
				undoable: false
						].
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'ar 8/31/2000 19:59'!
animateMeshTo: param duration: aDuration style: aStyle
	"Turns the object to point at the specified target using the specified style over the specified duration."
	| anim |
	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyNumber: param ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what to animate to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use animating ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to animate ' , myName , ' because ', msg.
			^ nil ].

	"The parameters check out, so build the animation"
	anim _ AbsoluteAnimation new.

	anim object: self
			update: [:t | self setMeshAnimationParameter: t ]
			getStartState: [self getMeshAnimationParameter]
			getEndState: [ param ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:43'!
roll: aDirection speed: aSpeed
	"Creates and returns an action that rolls the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to roll ' , myName , ' at because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self rollRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				undoable: false
						].
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 9/10/2000 02:22'!
setPolygonMode: aSymbol
	self setProperty: #polygonMode toValue: aSymbol.! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 3/19/1999 14:41'!
getBoundingBox: reference
	"Returns the actor's bounding box"

	| RUF LDB min max cMatrix tMatrix tVector meshBBox |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the bounding box of ' , myName , ' relative to ', (reference asString) , ' because ', msg.
			^ nil ].

	meshBBox _ self getBoundingBox.
	LDB _ meshBBox origin.
	RUF _ meshBBox corner.

	cMatrix _ (reference getMatrixToRoot) composeWith: (self getMatrixFromRoot).
	tMatrix _ B3DMatrix4x4 identity.

	tMatrix _ cMatrix composeWith: (tMatrix translation: RUF).
	min _ tMatrix translation.
	max _ tMatrix translation.
				
	tMatrix setIdentity.
	tVector _ (cMatrix composeWith: (tMatrix translation: LDB)) translation.	
	min _ min min: tVector.
	max _ max max: tVector.

	"Right up behind"
	tVector _ B3DVector3 x: (RUF x) y: (RUF y) z: (LDB z).
	tMatrix setIdentity.
	tVector _ (cMatrix composeWith: (tMatrix translation: tVector)) translation.	
	min _ min min: tVector.
	max _ max max: tVector.
	
	"Left up behind"
	tVector _ B3DVector3 x: (LDB x) y: (RUF y) z: (LDB z).
	tMatrix setIdentity.
	tVector _ (cMatrix composeWith: (tMatrix translation: tVector)) translation.	
	min _ min min: tVector.
	max _ max max: tVector.

	"Left up front"
	tVector _ B3DVector3 x: (LDB x) y: (RUF y) z: (RUF z).
	tMatrix setIdentity.
	tVector _ (cMatrix composeWith: (tMatrix translation: tVector)) translation.	
	min _ min min: tVector.
	max _ max max: tVector.

	"Right down behind"
	tVector _ B3DVector3 x: (RUF x) y: (LDB y) z: (LDB z).
	tMatrix setIdentity.
	tVector _ (cMatrix composeWith: (tMatrix translation: tVector)) translation.	
	min _ min min: tVector.
	max _ max max: tVector.

	"Right down front"
	tVector _ B3DVector3 x: (RUF x) y: (LDB y) z: (RUF z).
	tMatrix setIdentity.
	tVector _ (cMatrix composeWith: (tMatrix translation: tVector)) translation.	
	min _ min min: tVector.
	max _ max max: tVector.

	"Left down front"
	tVector _ B3DVector3 x: (LDB x) y: (LDB y) z: (RUF z).
	tMatrix setIdentity.
	tVector _ (cMatrix composeWith: (tMatrix translation: tVector)) translation.	
	min _ min min: tVector.
	max _ max max: tVector.

	^ Rectangle origin: min corner: max.
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/9/2001 15:43'!
graphic
	| ff |
	ff _ self getTexturePointer.
	ff isForm ifTrue:[^ff].
	ff isMorph ifTrue:[^ff imageForm].
	^ScriptingSystem formAtKey: #Paint
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:50'!
roll: aDirection turns: numTurns duration: aDuration style: aStyle
	"Roll the actor the specified number of turns in the specified direction, taking the specified duration and using the specified animation style."

	| anim updateFunc degrees lastAmount tMatrix|

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: numTurns ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the number of turns to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for rolling ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to roll ' , myName , ' because ', msg.
			^ nil ].

	"The parameters check out, so go ahead and build the animation"
	anim _ RelativeAnimation new.
	degrees _ 360 * numTurns.
	lastAmount _ 0.
	tMatrix _ B3DMatrix4x4 new.

	(aDirection = #left) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #right) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated. ]

		ifFalse: [ myWonderland reportErrorToUser: 'Squeak does not know how to roll ' , myName , ' ', (aDirection asString) , '.'.
				^ nil ]].
	
	anim object: self
			update: updateFunc
			getStartState: [ lastAmount _ 0. (Interpolateable value: 0) ]
			getEndState: [ degrees ]
			getReverseState: [ degrees negated ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'event handling' stamp: 'ar 6/1/2001 10:15'!
onMouseMove: event
	"The default response to mouse motion events"

	| deltaX deltaY screenPos scenePos depth camera evt |

	deltaX _ event getCursorDelta x.
	deltaY _ event getCursorDelta y.
	camera _ event getCamera.
	evt _ event getMorphicEvent.
	(evt shiftPressed)
		ifTrue: [ (evt controlKeyPressed)
					ifTrue: [
						"If both shift and control are pressed, tumble the bunny"
							scenePos _ self getPosition.
							self moveToRightNow: { 0. 0. 0 } asSeenBy: (myWonderland getScene)
									undoable: false.
							self turnRightNow: down numberOfTurns: (deltaY * 0.01)
									asSeenBy: (myWonderland getScene) undoable: false.
							self turnRightNow: left numberOfTurns: (deltaX * 0.01)
									asSeenBy: (myWonderland getScene) undoable: false.
							self moveToRightNow: scenePos undoable: false.

							]

					ifFalse: [
						"If only shift is pressed move the actor up/down"
						screenPos _ (self getPositionInPixels: camera) + (0@deltaY).
						scenePos _ camera transformScreenPointToScenePoint: screenPos
												atDepthOf: self.
						self moveToRightNow: (scenePos) asSeenBy: (myWonderland getScene)
								undoable: false.
							].
				]
		ifFalse: [ (evt controlKeyPressed)
					ifTrue: [
							scenePos _ self getPosition.
							self moveToRightNow: { 0. 0. 0 } asSeenBy: (myWonderland getScene)
									undoable: false.
							self turnRightNow: left numberOfTurns: (deltaX * 0.01)
									asSeenBy: (myWonderland getScene) undoable: false.
							self moveToRightNow: scenePos undoable: false.
							]
					ifFalse: [
						"If no modifier keys are held down, move the actor forward-back
						and left-right"
						screenPos _ (self getPositionInPixels: camera) + (deltaX@0).
						depth _ ((self getPosition: camera) at: 3) - (deltaY * 0.01).
						scenePos _ camera transformScreenVectorToSceneVector:
							(B3DVector3 x: (screenPos x) y: (screenPos y) z: depth).

						self moveToRightNow: { scenePos at: 1. asIs. scenePos at: 3 }
								asSeenBy: (myWonderland getScene) undoable: false.
							].
				].
! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'ar 5/8/2001 19:41'!
hasUserDefinedScripts
	"We associate every script with the world"
	^false! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:48'!
moveTo: aVector duration: aDuration style: aStyle
	"Moves the object to the specified position in its parent's coordinate system using the specified style over the specified duration."

	| anim pos endStateFunc |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
			^ nil ].


	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for moving ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to move ' , myName , ' because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ AbsoluteAnimation new.

	(aVector isKindOf: WonderlandActor)
		ifTrue: [ endStateFunc _ [ pos _ aVector getPosition: myParent.
				 				   B3DVector3 x: (pos at: 1) y: (pos at: 2) z: (pos at: 3). ]
				]
		ifFalse: [ endStateFunc _ [
						pos _ self getPositionVector.
				  		((aVector at: 1) = asIs) ifFalse: [pos x: (aVector at: 1) ].
				  		((aVector at: 2) = asIs) ifFalse: [pos y: (aVector at: 2) ].
				  		((aVector at: 3) = asIs) ifFalse: [pos z: (aVector at: 3) ].
				  		pos.
								]
				].

	anim object: self
			update: [:tPos | self setPositionVector: tPos]
			getStartState: [self getPositionVector]
			getEndState: endStateFunc
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:21'!
turnTo: aVector AsSeenBy: reference eachFrameUntil: aCondition
	"Turns the object to the specified orientation in its parent's coordinate system each frame until the condition returns true."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should turn ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self turnToRightNow: aVector asSeenBy: reference undoable: false ]
			until: aCondition.

! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'ar 5/8/2001 19:42'!
updateAllViewersAndForceToShow: mumble
	"Ugh."! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:56'!
moveTo: aVector asSeenBy: reference eachFrameFor: aLifetime
	"Move the actor to a specified point each frame for the specified lifetime."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long ' , myName , ' should move to this position because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveToRightNow: aVector asSeenBy: reference undoable: false ]
			for: aLifetime.


! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:11'!
resizeTopToBottom: height leftToRight: width frontToBack: thickness
	"Resize the object by the specified amount over 1 second using the Gently animation style."

	^ (self resizeTopToBottom: height leftToRight: width frontToBack: thickness
				duration: 1.0
				style: gently).

! !

!WonderlandActor methodsFor: 'set property' stamp: 'ar 9/14/1999 20:02'!
setTexture: textureFile
	"Sets the object's texture"
	(myWonderland getUndoStack) push: (UndoTextureChange for: self from: myTexture).
	myTexture _ myWonderland makeTextureFrom: textureFile flipping: true.
	myChildren do: [:child | (child isPart) ifTrue: [ child setTexturePointer: myTexture ] ].
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 5/23/2001 23:28'!
scaleByVector: aVector
	"Scale the object by the given vector."

	| aMatrix |
	aMatrix _ B3DMatrix4x4 identity.

	"Scale my matrix"
	aMatrix scalingX: (aVector x) y: (aVector y) z: (aVector z).
	scaleMatrix _ aMatrix composeWith: scaleMatrix.

	"Now scale my parts"
	myChildren do: [:child | (child isPart) ifTrue: [
								child setPositionVector: (child getPositionVector) * aVector.
								child scaleByVector: aVector
												].
					].
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:57'!
turn: aDirection turns: numTurns duration: aDuration style: aStyle
	"Turn the actor the specified distance in the specified direction, taking the specified duration and using the specified animation style."

	| anim updateFunc degrees lastAmount tMatrix|

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: numTurns ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how many turns to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for turning ' , myName , ' because ', msg.
			^ nil ].


	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to turn ' , myName , ' because ', msg.
			^ nil ].

	"The parameters check out, so go ahead and build the animation"
	anim _ RelativeAnimation new.
	degrees _ 360 * numTurns.
	lastAmount _ 0.
	tMatrix _ B3DMatrix4x4 new.

	(aDirection = #right) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundY: (tAngle - lastAmount)).
							lastAmount _ tAngle] ]

		ifFalse: [(aDirection = #left) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundY: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated ]

		ifFalse: [(((aDirection = #up) or: [aDirection = #back]) or: [aDirection = #backward])
			ifTrue: [
				updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundX: (tAngle - lastAmount)).
							lastAmount _ tAngle].
				degrees _ degrees negated ]

		ifFalse: [((aDirection = #down) or: [aDirection = #forward]) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundX: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #ccw) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #cw) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. self rotateByMatrix:
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated. ]

		ifFalse: [ self error: 'Unrecognized direction' ]]]]]].
	
	anim object: self
			update: updateFunc
			getStartState: [ lastAmount _ 0. (Interpolateable value: 0) ]
			getEndState: [ degrees ]
			getReverseState: [ degrees negated ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:47'!
roll: aDirection turns: numTurns
	"Roll the actor the specified number of turns in the specified direction, taking 1 second and using the Gently animation style."

	^ (self roll: aDirection turns: numTurns duration: 1.0 style: gently).

! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 3/12/1999 15:22'!
getName
	"Returns the object's name"

	^ myName.
! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'ar 5/8/2001 19:41'!
emptyScript
	"Required to see an empty script slot"! !

!WonderlandActor methodsFor: 'drawing' stamp: 'ar 11/7/1999 18:46'!
renderOn: aRenderer
	"Draw the actor."

	"Save the old transformation matrix"
	aRenderer pushMatrix.

	"Modify the matrix using our composite matrix for position and orientation"
	aRenderer transformBy: composite.

	"Save the new transformation matrix"
	aRenderer pushMatrix.

	"Modify the matrix using our scale matrix - we do this seperately to avoid scaling space"
	aRenderer transformBy: scaleMatrix.

	"Draw our mesh if the object is not hidden"
	(hidden) ifFalse: [ self drawMesh: aRenderer ].

	"Remove the scaling matrix"
	aRenderer popMatrix.

	"Set the default texture for our children"
	myTexture ifNotNil:[aRenderer pushTexture; texture: myTexture].

	"Draw our children.
	Note: For correct picking it is important to use B3DRenderEngine>>render: here."
	myChildren do: [:child | aRenderer render: child].

	"Restore the old texture"
	myTexture ifNotNil:[aRenderer popTexture].

	"Restore the old matrix"
	aRenderer popMatrix.! !

!WonderlandActor methodsFor: 'initialize' stamp: 'ar 6/2/1999 14:16'!
initializeFor: aWonderland
	"Initialize the instance variables for the WonderlandActor"

	super initialize.

	myName _ 'Unnamed'.
	myWonderland _ aWonderland.
	myParent _ aWonderland getScene.
	myParent addChild: self.

	"Initialize our material"
	myMaterial _ B3DMaterial new.
	myMaterial ambientPart: Color white.
	myMaterial diffusePart: Color white.
	myMaterial specularPart: Color white.

	"Set up our default properties"
	myColor _ B3DColor4 r: 1.0 g: 1.0 b: 1.0 a: 1.0.
	composite _ B3DMatrix4x4 identity.
	scaleMatrix _ B3DMatrix4x4 identity.

	"Setup the default reactions"
	self initializeDefaultReactions.

	hidden _ false.
	firstClass _ true.
! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 3/22/1999 14:14'!
getWonderland
	"Return the wonderland this actor is a part of"

	^ myWonderland.
! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/26/2001 16:14'!
isPlayerLike
	"Return true if the receiver is a player-like object"
	^true! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:39'!
forwardBy: distance
	"Note: This assumes we're moving in centimeters - common Alice reference is meter!!"
	self move: forward distance: (distance * 0.01) duration: rightNow.! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/10/1999 16:46'!
setPositionVector: aVector
	"Instantaneously sets the actor's current position"

	composite translation: aVector.

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:08'!
turn: aDirection speed: aSpeed
	"Creates and returns an action that turn the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to turn ' , myName , ' at because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self turnRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				undoable: false
						].
! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'sw 8/3/2001 13:56'!
methodInterfacesForScriptsCategoryIn: aVocabulary
	"Needed for showing emptyScript"

	^ self methodInterfaceForEmptyScript! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 15:25'!
turnRightNow: aDirection numberOfTurns: numTurns asSeenBy: reference undoable: isUndoable
	"Turn the object instantaneously the specified number of turns in the specified direction"

	| tMatrix degrees  newMatrix |

	tMatrix _ B3DMatrix4x4 identity.
	degrees _ 360 * numTurns.

	(isUndoable = true) ifTrue: [
		myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).
						].

	(aDirection = right) ifTrue: [ tMatrix rotationAroundY: degrees ]
		ifFalse: [(aDirection = left) ifTrue: [ tMatrix rotationAroundY: (degrees negated) ]

		ifFalse: [(((aDirection = up) or: [aDirection = back]) or: [aDirection = backward])
			ifTrue: [ tMatrix rotationAroundX: (degrees negated) ]

		ifFalse: [((aDirection = down) or: [aDirection = forward]) ifTrue: [
			tMatrix rotationAroundX: degrees ]

		ifFalse: [(aDirection = ccw) ifTrue: [ tMatrix rotationAroundZ: degrees ]

		ifFalse: [(aDirection = cw) ifTrue: [ tMatrix rotationAroundZ: (degrees negated) ]

		ifFalse: [ self error: 'Unrecognized direction' ]]]]]].

	newMatrix _ reference getMatrixToRoot.
	newMatrix _ newMatrix composeWith: (self getMatrixFromRoot).
	composite _ (((myParent getMatrixToRoot) composeWith: (reference getMatrixFromRoot))
											composeWith: tMatrix)
											composeWith: newMatrix.

! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 2/10/1999 17:05'!
getPosition
	"Return the object's current position in its parent's coordinate system"

	| a3DVector anArray |

	anArray _ Array new: 3.
	a3DVector _ composite translation.

	anArray at: 1 put: (a3DVector x).
	anArray at: 2 put: (a3DVector y).
	anArray at: 3 put: (a3DVector z).

	^ anArray.

	! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/4/1999 15:45'!
nudge: aDirection
	"Nudge the actor one length in the specified direction, taking 1 second and using the Gently animation style."

	^ (self nudge: aDirection distance: 1.0 duration: 1.0 style: gently).

! !

!WonderlandActor methodsFor: 'testing' stamp: 'ar 11/8/1999 14:00'!
hasActiveTexture
	"Return true if the receiver has an active texture that wants events routed directly."
	^(self getProperty: #activeTexture) == true! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:10'!
turn: aDirection speed: aSpeed for: aLifetime
	"Creates and returns an action that turn the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to turn ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should turn ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self turnRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				undoable: false
						]
		   for: aLifetime.

! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/19/1999 16:07'!
setSizeRightNow: aVector undoable: isUndoable
	"Set this object's size instantaneously"

	| meshSize |

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoSizeChange for: self from: (self getSize)).
						].

	meshSize _ self getBoundingBox extent.

	"Now make sure we don't accidentally divide by zero"
	((meshSize x) = 0) ifTrue: [ aVector at: 1 put: 1.
							    meshSize x: 1 ].

	((meshSize y) = 0) ifTrue: [ aVector at: 2 put: 1.
							    meshSize y: 1 ].

	((meshSize z) = 0) ifTrue: [ aVector at: 3 put: 1.
							    meshSize z: 1 ].

	^ self scaleByVector: (B3DVector3 x: ((aVector at: 1) / (meshSize x))
									y: ((aVector at: 2) / (meshSize y))
									z: ((aVector at: 3) / (meshSize z)) ).


! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 11/8/1999 16:53'!
getLocalPosition: aVector
	"Return the local position of the given vector"
	| vv cc |
	vv _ B3DVector3 x: (aVector at: 1) y: (aVector at: 2) z: (aVector at: 3).
	cc _ composite composeWith: scaleMatrix.
	vv _ cc inverseTransformation localPointToGlobal: vv.
	^(Array with: vv x with: vv y with: vv z)! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/19/1999 15:21'!
setSize: aVector duration: time style: aStyle
	"Sets the object's position and orientation"

	| meshSize |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTriple: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what size to make ' , myName , ' because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	meshSize _ self getBoundingBox extent.

	"Now make sure we don't accidentally divide by zero"
	((meshSize x) = 0) ifTrue: [ aVector at: 1 put: 1.
							    meshSize x: 1 ].

	((meshSize y) = 0) ifTrue: [ aVector at: 2 put: 1.
							    meshSize y: 1 ].

	((meshSize z) = 0) ifTrue: [ aVector at: 3 put: 1.
							    meshSize z: 1 ].

	^ self resizeTopToBottom: (aVector at: 2) / (meshSize y)
			leftToRight: (aVector at: 1) / (meshSize x)
			frontToBack: (aVector at: 3) / (meshSize z)
			duration: time
			style: aStyle.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/24/1999 16:53'!
resize: amount eachFrameUntil: aCondition
	"Resize the object by the specified amount each frame until the condition returns true."

	| aVector |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyNumber: amount ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , '  because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should resize ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoSizeChange for: self from: (self getSize)).

	(amount isNumber)
		ifTrue: [ aVector _ (B3DVector3 x: amount y: amount z: amount) ]
		ifFalse: [ aVector _ (B3DVector3 x: (amount at: 1) y: (amount at: 2)
									 z: (amount at: 3)) ].

	^ self doEachFrame: [ self resizeRightNow: aVector undoable: false ]
			until: aCondition.


! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 2/9/1999 10:09'!
isFirstClass
	"Returns true if the object is a First Class object"

	^ firstClass.
! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 14:39'!
setColor: newColor
	"Set the actor's color"

	 ^ (self setColor: newColor duration: 1.0 style: gently).

! !

!WonderlandActor methodsFor: 'morphic reactions' stamp: 'ar 10/10/2000 22:04'!
dispatchEvent: event to: aMorph
	| evt delta |
	evt _ event getMorphicEvent.
	delta _ (myTexture mapPrimitiveVertex: event getVertex) - evt position.
	evt _ evt translatedBy: delta.
	evt resetHandlerFields.
	"Make sure my texture thinks it's in the world otherwise we might run into problems"
	aMorph isInWorld ifFalse:[aMorph privateOwner: event getCameraMorph].
	aMorph processEvent: evt.! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 11/7/1999 18:33'!
getFullBounds
	"Return the 2D bounds of the receiver and its children or nil if not visible"
	^self getFullBoundsFor: myWonderland getDefaultCamera.! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 11/8/1999 13:14'!
getUserPointOfView
	"Return the user defined point of view for this object"
	^self getProperty: #userPointOfView! !

!WonderlandActor methodsFor: 'morphic reactions' stamp: 'ar 10/10/2000 21:32'!
morphicMouseDown: event
	"Handle the given event.
	If a local viewpoint is defined go to this viewpoint."
	| vp |
	(event getMorphicEvent shiftPressed and:[(vp _ self getUserPointOfView) notNil])
		ifTrue:[event getCameraMorph getCamera 
					setPointOfView: vp
					duration: 2
					asSeenBy: self.
				"Set an additional flag to avoid inbetween shift key changes"
				^self setProperty: #userTransition toValue: true].
	myTexture isMorph ifTrue:[
		self dispatchEvent: event to: myTexture.
	].! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/22/1999 14:30'!
getMatrixFromRoot
	"Returns the composite transformation matrix from the root down to this instance"

	^ (myParent getMatrixFromRoot) composeWith: composite.
! !

!WonderlandActor methodsFor: 'event handling' stamp: 'jsp 4/19/1999 17:26'!
addResponse: aReaction to: evtType
	"Add a new response to the given event type"

	| newReaction currentReactions |

	(myReactions includesKey: evtType)
		ifFalse: [ myReactions at: evtType put: OrderedCollection new ].

	currentReactions _ myReactions at: evtType.

	newReaction _ WonderlandReaction new: aReaction.

	currentReactions addLast: newReaction.

	^ newReaction.
! !

!WonderlandActor methodsFor: 'animated state' stamp: 'jsp 3/15/1999 16:15'!
pause
	"Pause any actions and animations associated with this object"

	| scheduler |

	scheduler _ myWonderland getScheduler.

	(scheduler getAnimationsFor: self) do: [:anim | anim pause ].

	(scheduler getActionsFor: self) do: [:action | action pause ].

	myChildren do: [:child | (child isPart) ifTrue: [ child pause ] ].! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 8/15/2001 13:24'!
tileReferringToSelf
	"answer a tile that refers to the receiver"

	| aTile  nn tile |
	Preferences universalTiles ifTrue:
		[nn _ self externalName. 	"name it, if necessary, and put in References"
		(References includesKey: nn asSymbol) ifFalse: [
			 References at: nn asSymbol put: self].
		tile _ SyntaxMorph new parseNode: 
			(VariableNode new name: nn key: nn code: nil).
		tile layoutInset: 1; addMorph: (tile addString: nn special: false).
		tile color: (SyntaxMorph translateColor: #variable).
		tile extent: tile firstSubmorph extent + (2@2).
		^ tile].

	aTile _ TileMorph new setToReferTo: self.
	^ aTile! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'ar 5/10/2001 12:42'!
getDistanceToCamera
	"Why does eToy system send #getXyz instead of #xyz???"
	^self distanceToCamera! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'ar 5/9/2001 16:34'!
getGraphic
	"Why does eToy system send #getGraphic instead of #graphic???"
	^self graphic! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/5/1999 12:23'!
standUpWithDuration: duration style: aStyle
	"Rotate the object to align it along its parent's up vector"

	^ self turnTo: #(0 #asIs 0) duration: duration style: aStyle.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:56'!
moveTo: aVector eachFrameFor: aLifetime
	"Move the actor to a specified point each frame for the specified lifetime."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long ' , myName , ' should move to this position because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveToRightNow: aVector undoable: false ]
			for: aLifetime.


! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/4/1999 16:07'!
setSize: aVector
	"Sets the object's position and orientation"

	^ (self setSize: aVector duration: 1.0 style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:09'!
move: aDirection asSeenBy: reference
	"Move the actor one meter in the specified direction, taking 1 second and using the Gently animation style."

	^ (self move: aDirection distance: 1.0 duration: 1.0 asSeenBy: reference
			style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:19'!
turnTo: aVector duration: aDuration asSeenBy: reference
	"Turns the object to the specified orientation in its parent's coordinate system over the specified duration using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		[ WonderlandVerifier VerifyTarget: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyReferenceFrame: reference ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
				^ nil ].

		self turnToRightNow: aVector asSeenBy: reference undoable: true.
		^ self. ].

	(aDuration = eachFrame) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyTarget: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyReferenceFrame: reference ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
				^ nil ].

		myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

		^ self doEachFrame: [ self turnToRightNow: aVector asSeenBy: reference undoable: false ].
									].

	^ (self turnTo: aVector duration: aDuration asSeenBy: reference style: gently).
! !

!WonderlandActor methodsFor: 'event handling' stamp: 'jsp 4/19/1999 16:19'!
onLeftMouseDown: event
	"The default response to left mouse down events"

	(event getCameraMorph) setEventFocus: self.
	(myWonderland getUndoStack) push: (UndoPOVChange for: self from: (self getPointOfView)).

	self respondWith: [:anEvent | self onMouseMove: anEvent] to: mouseMove.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:46'!
roll: aDirection speed: aSpeed asSeenBy: reference until: aCondition
	"Creates and returns an action that rolls the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to roll ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not roll ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should roll ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self rollRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				asSeenBy: reference
				undoable: false
						]
		   until: aCondition.

! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 15:25'!
setPointOfView: aVector eachFrameUntil: aCondition
	"Sets the object's position and orientation"

	| positionVector rotationVector |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyPointOfView: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what point of view to assign ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine for how long ' , myName , ' should keep this point of view because ', msg.
			^ nil ].

	(aVector isKindOf: WonderlandActor)
			ifTrue: [
						positionVector _ aVector.
						rotationVector _ aVector.
					]
			ifFalse: [
						positionVector _ Array new: 3.
						positionVector at: 1 put: (aVector at: 1).
						positionVector at: 2 put: (aVector at: 2).
						positionVector at: 3 put: (aVector at: 3).

						rotationVector _ Array new: 3.
						rotationVector at: 1 put: (aVector at: 4).
						rotationVector at: 2 put: (aVector at: 5).
						rotationVector at: 3 put: (aVector at: 6).
					].

	myWonderland getUndoStack push: (UndoPOVChange for: self
														from: (self getPointOfView)).

	^ self doEachFrame: [ 
							self moveToRightNow: positionVector undoable: false.
							self turnToRightNow: rotationVector undoable: false.
						]
			until: aCondition.

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 4/9/1999 14:09'!
turnTo: aVector speed: aSpeed asSeenBy: reference
	"Turns the object to the specified orientation in the reference's coordinate system at aSpeed turns per second."

	^ self turnTo: aVector duration: (((self angularDistanceTo: aVector) / (aSpeed * 360)) abs)
				asSeenBy: reference style: abruptly.

! !

!WonderlandActor methodsFor: 'event handling' stamp: 'jsp 4/19/1999 13:29'!
removeResponse: aReaction to: evtType
	"Remove the response to the given event type"

	(myReactions includesKey: evtType)
		ifTrue: [ (myReactions at: evtType) remove: aReaction ].
! !

!WonderlandActor methodsFor: 'initialize' stamp: 'ar 6/2/1999 14:15'!
initializeDefaultReactions
	"Set up our default reactions"
	myReactions _ Dictionary new.
	self respondWith: [:event | self onLeftMouseDown: event] to: leftMouseDown.
	self respondWith: [:event | self onLeftMouseUp: event] to: leftMouseUp.
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 9/9/2000 23:53'!
setBackfaceCulling: aSymbol
	"Set backface culling. aSymbol must be either #cw, #ccw or nil"
	self setProperty: #backfaceCulling toValue: aSymbol.! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 8/31/2000 20:03'!
setMeshAnimationParameter: param
	myMesh == nil ifTrue:[^self].
	myMesh animationParameter: param! !

!WonderlandActor class methodsFor: 'instance creation' stamp: 'jsp 3/10/1999 16:14'!
createFor: aWonderland
	"Create a new instance for this Wonderland."

	^ super new initializeFor: aWonderland.
! !

!WonderlandActor methodsFor: 'enumerating' stamp: 'ar 6/2/1999 14:03'!
partsDo: aBlock
	"Evaluate aBlock with all elements of the receiver that are a part of it"
	myChildren do:[:child| child isPart ifTrue:[aBlock value: child]].! !

!WonderlandActor methodsFor: 'event handling' stamp: 'jsp 4/19/1999 14:30'!
respondWith: aReaction to: evtType
	"Set this actor's response to the given event type"

	| reaction collection |

	aReaction ifNil: [ myReactions removeKey: evtType ifAbsent: [].
					  ^ nil ].

	reaction _ WonderlandReaction new: aReaction.

	collection _ OrderedCollection new.
	collection addLast: reaction.

	myReactions at: evtType put: collection.

	^ reaction.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/25/1999 09:43'!
setRotationMatrix: aMatrix
	"Rotate the object by blamming the rotation component of its matrix with the given matrix."

	composite a11: (aMatrix a11).
	composite a12: (aMatrix a12).
	composite a13: (aMatrix a13).

	composite a21: (aMatrix a21).
	composite a22: (aMatrix a22).
	composite a23: (aMatrix a23).

	composite a31: (aMatrix a31).
	composite a32: (aMatrix a32).
	composite a33: (aMatrix a33).
! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 15:46'!
setPointOfView: aVector duration: aDuration asSeenBy: reference
	"Sets the object's position and orientation"

	| positionVector rotationVector |

	((aDuration = rightNow) or: [ aDuration = eachFrame ]) ifTrue: [
			[ WonderlandVerifier VerifyPointOfView: aVector ]
				ifError: [ :msg :rcvr |
					myWonderland reportErrorToUser: 'Squeak could not determine what point of view to assign ' , myName , ' because ', msg.
					^ nil ].

			( aVector isKindOf: WonderlandActor) ifTrue: [
					myWonderland reportErrorToUser: 'Squeak does not allow you to use an Actor to specify the point of view when using asSeenBy.'.
					^ nil ].

			[ WonderlandVerifier VerifyReferenceFrame: reference ]
				ifError: [ :msg :rcvr |
					myWonderland reportErrorToUser:
						'Squeak can not set the point of view for ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
					^ nil ].

			positionVector _ Array new: 3.
			positionVector at: 1 put: (aVector at: 1).
			positionVector at: 2 put: (aVector at: 2).
			positionVector at: 3 put: (aVector at: 3).

			rotationVector _ Array new: 3.
			rotationVector at: 1 put: (aVector at: 4).
			rotationVector at: 2 put: (aVector at: 5).
			rotationVector at: 3 put: (aVector at: 6).

			myWonderland getUndoStack push: (UndoPOVChange for: self
														from: (self getPointOfView)).

			(aDuration = eachFrame) ifTrue: [
				^ self doEachFrame: [ 
										self moveToRightNow: positionVector undoable: false.
										self turnToRightNow: rotationVector undoable: false.
									]
											].

			self moveToRightNow: positionVector asSeenBy: reference undoable: false.
			self turnToRightNow: rotationVector asSeenBy: reference undoable: false.
			^ self ].

	^ self setPointOfView: aVector duration: aDuration asSeenBy: reference style: gently.
! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'ar 5/26/2001 16:17'!
renameTo: aName
	| aPresenter putInViewer oldKey assoc |
	self setName: aName.
	putInViewer _ false.
	((aPresenter _ self presenter) isNil) ifFalse:
		[putInViewer _ aPresenter currentlyViewing: self.
		putInViewer ifTrue:
			[self viewerFlapTab hibernate]].  "empty it temporarily"

	"Fix References dictionary.  See restoreReferences to know why oldKey is 
		already aName, but oldName is the old name."
	oldKey _ References keyAtIdentityValue: self ifAbsent: [nil].
	oldKey ifNotNil:
		[assoc _ References associationAt: oldKey.
		oldKey = aName ifFalse: ["normal rename"
			assoc key: aName asSymbol.
			References rehash]].

	World allTileScriptingElements do: [:m | m bringUpToDate].

	putInViewer ifTrue: [aPresenter viewObject: self].	"recreate my viewer"
	oldKey ifNil: [^ aName].
	^ aName! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/18/1999 10:18'!
moveRightNow: aDirection distance: aDistance undoable: isUndoable
	"Move this object in the specified direction instantaneously"

	| a3DVector aMatrix |

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoPositionChange for: self from: (self getPosition)).
						].

	[ a3DVector _ self makeVectorFromDistance: aDistance andDirection: aDirection. ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak does not know how to move ' , myName , ' in that direction.'.
			^ nil ].

	aMatrix _ B3DMatrix4x4 identity translation: a3DVector.

	composite _ composite composeWith: aMatrix.


! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 15:27'!
setPointOfView: aVector duration: aDuration
	"Sets the object's position and orientation"

	| positionVector rotationVector |

	((aDuration = rightNow) or: [ aDuration = eachFrame ]) ifTrue: [
		[ WonderlandVerifier VerifyPointOfView: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine what point of view to assign ' , myName , ' because ', msg.
				^ nil ].

		(aVector isKindOf: WonderlandActor)
			ifTrue: [
						positionVector _ aVector.
						rotationVector _ aVector.
					]
			ifFalse: [
						positionVector _ Array new: 3.
						positionVector at: 1 put: (aVector at: 1).
						positionVector at: 2 put: (aVector at: 2).
						positionVector at: 3 put: (aVector at: 3).

						rotationVector _ Array new: 3.
						rotationVector at: 1 put: (aVector at: 4).
						rotationVector at: 2 put: (aVector at: 5).
						rotationVector at: 3 put: (aVector at: 6).
					].

			myWonderland getUndoStack push: (UndoPOVChange for: self
														from: (self getPointOfView)).

			(aDuration = eachFrame) ifTrue: [
				^ self doEachFrame: [ 
										self moveToRightNow: positionVector undoable: false.
										self turnToRightNow: rotationVector undoable: false.
									]
											].

			self moveToRightNow: positionVector undoable: false.
			self turnToRightNow: rotationVector undoable: false.
			^ self ].

	^ self setPointOfView: aVector duration: aDuration style: gently.
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/11/2001 13:20'!
setDistanceToCamera: distance
	| pos oldDistance oldPos aCamera |
	aCamera _ myWonderland getDefaultCamera.
	aCamera _ aCamera getChildren 
		detect:[:any| any getName = 'referencePoint']
		ifNone:[aCamera].
	oldPos _ self getPosition: aCamera.
	pos _ B3DVector3 x: oldPos first y: 0 z: oldPos third.
	oldDistance _ pos length.
	oldDistance abs < 1.0e-5 ifTrue:[
		pos _ 0@0@1.
		oldDistance _ 1].
	pos _ pos * (distance * 0.01 / oldDistance).
	self moveToRightNow: {pos x. oldPos second. pos z} asSeenBy: myWonderland getDefaultCamera undoable: false.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 17:00'!
nudge: aDirection distance: aDistance duration: aDuration 
	"Nudge the actor the specified object lengths in the specified direction, taking the specified duration and using the Gently animation style."

	|  size newDistance |
	
	(aDuration = rightNow) ifTrue: [
			size _ self getSize.

			((aDirection = #left) or: [ aDirection = #right ])
				ifTrue: [ newDistance _ aDistance * (size at: 1) ]
				ifFalse: [

			((aDirection = #up) or: [ aDirection = #down ])
				ifTrue: [ newDistance _ aDistance * (size at: 2) ]
				ifFalse: [

			((aDirection = #forward) or: [ (aDirection = #back) or: [aDirection = #backward]])
				ifTrue: [ newDistance _ aDistance * (size at: 3) ]
				ifFalse: [ 	myWonderland reportErrorToUser: 'Squeak could not determine which direction to nudge the object because ' , (aDirection asString) , 'is not a valid direction.'.
							^ nil.]]].


			^ self moveRightNow: aDirection distance: newDistance undoable: true ].

	^ self nudge: aDirection distance: aDistance duration: aDuration style: gently.
	

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:10'!
moveTo: aVector
	"Moves the object to the specified position in its parent's coordinate system over 1 second using the Gently animation style."

	^ (self moveTo: aVector duration: 1.0
			style: gently).
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 09:47'!
moveTo: aVector duration: aDuration asSeenBy: reference
	"Moves the object to the specified position in its parent's coordinate system over the specified duration using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyTarget: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyDuration: aDuration ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the duration to use for moving ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyReferenceFrame: reference ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
				^ nil ].

		self moveToRightNow: aVector asSeenBy: reference undoable: true.
		^ self ].

	(aDuration = eachFrame) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyTarget: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyDuration: aDuration ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the duration to use for moving ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyReferenceFrame: reference ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
				^ nil ].

		myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

		^ self doEachFrame: [ self moveToRightNow: aVector	asSeenBy: reference undoable: false ].
									].

	^ (self moveTo: aVector duration: aDuration asSeenBy: reference
			style: gently).
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/29/1999 14:39'!
pointAt: target eachFrameFor: aLifetime
	"Turns the object to point at the specified target each frame for the specified lifetime."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTargetOrPixel: target ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what to point ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should point ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self pointAtRightNow: target undoable: false ]
			for: aLifetime.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:07'!
moveTo: aVector speed: aSpeed 
	"Moves the object to the specified position in its parent's coordinate system using the specified style over the specified duration."

	^ self moveTo: aVector duration: (((self distanceTo: aVector) / aSpeed) abs) style: abruptly.

! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/8/2001 19:05'!
defaultNameStemForInstances
	"Answer a basis for names of default instances of the receiver."
	^myName ifNil:[super defaultNameStemForInstances].! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 9/10/2000 02:22'!
setLineWidth: aNumber
	self setProperty: #lineWidth toValue: aNumber.! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 7/8/1999 23:20'!
hasParts
	"Return true if the receiver has any children that are parts"

	self partsDo: [:child | ^ true ].

	^ false.
! !

!WonderlandActor methodsFor: 'initialize' stamp: 'ar 9/1/2000 05:22'!
initializeNoReactions
	myReactions _ Dictionary new.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/4/1999 15:50'!
nudge: aDirection distance: aDistance
	"Nudge the actor the specified object lengths in the specified direction, taking one second and using the Gently animation style."

	^ self nudge: aDirection distance: aDistance duration: 1.0 style: gently.
	

! !

!WonderlandActor methodsFor: 'animated state' stamp: 'jsp 3/9/1999 16:18'!
getAnimations
	"Return the animations currently affecting this object"

	^ (myWonderland getScheduler) getAnimationsFor: self.
! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 3/4/1999 15:35'!
becomePart
	"Make the actor a part."

	firstClass _ false.

	(myWonderland getUndoStack) push: (UndoAction new: [ firstClass _ true ]).
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:16'!
turnTo: aVector duration: aDuration
	"Turns the object to the specified orientation in its parent's coordinate system over the specified duration using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		[ WonderlandVerifier VerifyTarget: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
				^ nil ].

		self turnToRightNow: aVector undoable: true.
		^ self. ].

	(aDuration = eachFrame) ifTrue: [
		[ WonderlandVerifier VerifyTarget: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
				^ nil ].

		myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

		^ self doEachFrame: [ self turnToRightNow: aVector	undoable: false ].
									].

	^ (self turnTo: aVector duration: aDuration style: gently).
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/24/1999 16:47'!
resize: amount duration: aDuration style: aStyle
	"Resize the object by the specified amount over the specified duration using the specified animation style."

	"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyNumberOrTriple: amount ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , '  because ', msg.
				^ nil ].

	(amount isNumber)
		ifTrue: [ ^ (self resizeTopToBottom: amount leftToRight: amount frontToBack: amount
						duration: 1.0
						style: gently). ]
		ifFalse: [  ^ (self resizeTopToBottom: (amount at: 2)
						leftToRight: (amount at: 1)
						frontToBack: (amount at: 3)
						duration: 1.0
						style: gently). ].
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/11/2001 13:20'!
distanceToCamera
	| aCamera |
	aCamera _ myWonderland getDefaultCamera.
	aCamera _ aCamera getChildren 
		detect:[:any| any getName = 'referencePoint']
		ifNone:[aCamera].
	^(self distanceTo: aCamera) * 100! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:33'!
move: aDirection speed: aSpeed until: aCondition
	"Creates and returns an action that moves the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to move ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long ' , myName , ' should move because ', msg.
			^ nil ].

	"The parameters check out, so start the action"
	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveRightNow: aDirection
							distance: ((myWonderland getScheduler getElapsedTime) * aSpeed)
							undoable: false ]
		   until: aCondition.
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 5/23/2001 00:10'!
getScaleMatrix
	^scaleMatrix! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 3/19/1999 16:28'!
getFaceCount
	"Return the number of faces in this actor and its parts"

	| count |

	(myMesh notNil) ifTrue: [ count _ myMesh faces size ]
					 ifFalse: [ count _ 0 ].

	myChildren do: [:child | (child isPart) ifTrue: [ count _ count + (child getFaceCount) ] ].

	^ count.
! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/1/1999 14:15'!
setVisibility: newVisibility
	"Sets the current object's visibility"

	^ (self setVisibility: newVisibility duration: 1.0
			style: gently).
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:57'!
moveTo: aVector asSeenBy: reference eachFrameUntil: aCondition
	"Move the actor to a specified point each frame until the condition returns true."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine for how long ' , myName , ' should move to this position because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveToRightNow: aVector asSeenBy: reference undoable: false ]
			until: aCondition.


! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:22'!
turnTo: aVector AsSeenBy: reference eachFrameFor: aLifetime
	"Turns the object to the specified orientation in its parent's coordinate system each frame for the specified lifetime."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should turn ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self turnToRightNow: aVector asSeenBy: reference undoable: false ]
			for: aLifetime.

! !

WonderlandActor subclass: #WonderlandCamera
	instanceVariableNames: 'perspective viewMatrix myMorph drawSceneBackground bounds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Objects'!

!WonderlandCamera commentStamp: '<historical>' prior: 0!
The WonderlandCamera class extends the WonderlandActor functionality with the methods for specifying the viewing parameters.  Every view into the 3D world is assigned a camera, and each camera renders into a different morph.

WonderlandCamera also know how to transform points on the 2D desktop into the 3D wonderland.
!

!WonderlandCamera methodsFor: 'set property' stamp: 'ar 5/15/2001 14:03'!
setFogColor: aColor
	self setProperty: #fogColor toValue: aColor! !

!WonderlandCamera methodsFor: 'set property' stamp: 'jsp 2/11/1999 11:14'!
setNearClippingPlane: aFloat
	"Sets the camera's near clipping plane - anything closer than this won't be drawn."

	perspective nearDistance: aFloat.
! !

!WonderlandCamera class methodsFor: 'change logging' stamp: 'jsp 7/8/1999 23:12'!
wantsChangeSetLogging
	^ (self == WonderlandCamera).
! !

!WonderlandCamera methodsFor: 'get property' stamp: 'jsp 2/11/1999 11:14'!
getNearClippingPlane
	"Returns the camera's near clipping plane"

	^ perspective nearDistance.
! !

!WonderlandCamera methodsFor: 'get property' stamp: 'ar 5/15/2001 14:16'!
getFogColor
	^self getProperty: #fogColor ifAbsent:[Color black]! !

!WonderlandCamera methodsFor: 'get property' stamp: 'ar 5/15/2001 14:16'!
getFogType
	^self getProperty: #fogType ifAbsent:[#none]! !

!WonderlandCamera methodsFor: 'initialize-release' stamp: 'ar 4/25/1999 06:48'!
copySettingsFrom: aB3DCamera
	"Copy my settings from the given B3DCamera"
	| transformer |
	transformer _ B3DVertexTransformer new.
	transformer loadIdentity.
	transformer lookFrom: aB3DCamera position to: aB3DCamera target up: aB3DCamera up.
	self setComposite: transformer modelViewMatrix inverseTransformation.
	perspective _ aB3DCamera perspective.! !

!WonderlandCamera methodsFor: 'transforming' stamp: 'jsp 3/29/1999 11:00'!
transformScreenPointToScenePoint: aPoint atDepthOf: anActor
	"Converts a pixel point to a position in the 3D scene using the actor to scale the position of the point. Uses the actor's distance from the camera to determine the depth in the scene."

	| d  x y m scale frustum vector |

	frustum _ self getFrustum.

	d _ (anActor getPosition: self) at: 3.
	scale _ d / (frustum near).

	x _ ((aPoint x) - (myMorph center x)) / ((myMorph width) / 2) * (frustum right) * scale.
	y _ (((aPoint y) - (myMorph center y)) / ((myMorph height) / 2)) * (frustum bottom) * scale.

	m _ B3DMatrix4x4 identity.
	m translation: (B3DVector3 x: x y: y z: d).
	m _ (self getMatrixFromRoot) composeWith: m.

	vector _ (m translation).

	^ { vector at: 1. vector at: 2. vector at: 3 }.
! !

!WonderlandCamera methodsFor: 'set property' stamp: 'ar 5/15/2001 14:03'!
setFogRangeStart: aNumber
	self setProperty: #fogRangeStart toValue: aNumber! !

!WonderlandCamera methodsFor: 'get property' stamp: 'jsp 3/25/1999 15:58'!
getFieldOfView
	"Returns the camera's field of view."

	^ (perspective fieldOfView) * 2.
! !

!WonderlandCamera methodsFor: 'rendering' stamp: 'ar 6/2/1999 13:20'!
render: aRenderEngine pickingAt: aPointOrNil
	"Render one frame of the Wonderland using this camera.
	If aPointOrNil is not nil then return the top most object at this point.
	Note: If picking, no objects are actually drawn."
	^self render: aRenderEngine pickingAt: aPointOrNil withPrimitiveVertex: false! !

!WonderlandCamera methodsFor: 'initialize-release' stamp: 'jsp 3/22/1999 11:07'!
release
	"Destroy the renderer"

	myMorph delete.

! !

!WonderlandCamera methodsFor: 'set property' stamp: 'jsp 3/25/1999 15:58'!
setFieldOfView: aFloat
	"Sets the camera's field of view."

	perspective fieldOfView: (aFloat / 2).
! !

!WonderlandCamera methodsFor: 'set property' stamp: 'jsp 2/11/1999 11:12'!
setAspectRatio: aFloat
	"Sets the camera's aspect ratio"

	perspective aspectRatio: aFloat.
! !

!WonderlandCamera methodsFor: 'accessing' stamp: 'jsp 3/30/1999 16:28'!
resetPointOfView
	"Return the camera to it's 'home' position and orientation"

	^ self setPointOfView: { -1.5. 0.5. 2.6. 180. -30. 180}.
! !

!WonderlandCamera methodsFor: 'set property' stamp: 'jsp 4/23/1999 12:07'!
setName: newName
	"Sets the object's name"

	myName _ newName.
	myMorph name: myName.
! !

!WonderlandCamera methodsFor: 'get property' stamp: 'ar 5/15/2001 14:16'!
getFogRangeStart
	^self getProperty: #fogRangeStart ifAbsent:[0.0]! !

!WonderlandCamera methodsFor: 'private' stamp: 'jsp 3/25/1999 10:50'!
getPerspectiveMatrix
	"Returns the camera's perspective matrix"

	^ perspective asFrustum asPerspectiveMatrix.
! !

!WonderlandCamera methodsFor: 'primitive behaviors' stamp: ''!
destroy: t1 
	| t2 t3 t4 |
	t4 _ OrderedCollection new.
	t3 _ self getAllChildren.
	t3
		do: [:t5 | t5 isFirstClass
				ifTrue: [t4
						addLast: (UndoParentChange newFor: t5 from: t5 getParent).
					t5 reparentTo: self getParent]].
	self removeFromScene.
	myMorph delete.
	t2 _ myWonderland getCameraList.
	t2 remove: self.
	t4
		addLast: (UndoAction
				new: [myMorph openInWorld.
					myWonderland getScene addChild: self.
					t2 addLast: self]).
	myWonderland getUndoStack
		push: (UndoChangeList new setChangeList: t4)! !

!WonderlandCamera methodsFor: 'transforming' stamp: 'jsp 3/29/1999 10:58'!
transformScreenPointToScenePoint: aPoint
	"Converts a pixel point to a position in the 3D scene. Uses the near clipping plane distance for the depth in the scene."

	| d  x y m frustum vector |

	frustum _ self getFrustum.

	d _ frustum near.

	x _ ((aPoint x) - (myMorph center x)) / ((myMorph width) / 2) * (frustum right).
	y _ (((aPoint y) - (myMorph center y)) / ((myMorph height) / 2)) * (frustum bottom).

	m _ B3DMatrix4x4 identity.
	m translation: (B3DVector3 x: x y: y z: d).
	m _ (self getMatrixFromRoot) composeWith: m.

	vector _ (m translation).

	^ { vector at: 1. vector at: 2. vector at: 3 }.
! !

!WonderlandCamera methodsFor: 'rendering' stamp: 'ar 4/18/1999 00:10'!
render: aRenderer
	^self render: aRenderer pickingAt: nil.! !

!WonderlandCamera methodsFor: 'get property' stamp: 'ar 5/15/2001 14:16'!
getFogRangeEnd
	^self getProperty: #fogRangeEnd ifAbsent:[1.0]! !

!WonderlandCamera methodsFor: 'get property' stamp: 'jsp 2/11/1999 11:15'!
getFarClippingPlane
	"Returns the camera's far clipping plane"

	^ perspective farDistance.
! !

!WonderlandCamera methodsFor: 'accessing' stamp: 'ar 4/28/1999 23:48'!
drawSceneBackground
	^drawSceneBackground! !

!WonderlandCamera methodsFor: 'accessing' stamp: 'jsp 3/22/1999 16:19'!
turnBackgroundOff
	"Stop drawing the scene background"

	drawSceneBackground _ false.! !

!WonderlandCamera methodsFor: 'rendering' stamp: 'ar 5/15/2001 14:04'!
renderFogOn: aRenderer
	"Set fog parameters if requested"
	aRenderer
		fogType: self getFogType
		density: self getFogDensity
		rangeStart: self getFogRangeStart
		rangeEnd: self getFogRangeEnd
		color: self getFogColor.! !

WonderlandCamera subclass: #WonderlandStillCamera
	instanceVariableNames: 'focusObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Objects'!

!WonderlandStillCamera methodsFor: 'initialize-release' stamp: 'ar 5/10/2001 22:54'!
initializeFor: aWonderland
	"Initializes the camera."

	super initializeFor: aWonderland.

	"Set the camera's mesh and geometry"
	self setMesh: nil.
	self setTexturePointer: nil.

	"Delete the WonderlandCameraMorph our superclass created"
	myMorph delete.	

	"Create a WonderlandStillCameraMorph for the camera to render into"
	myMorph _ WonderlandStillCameraMorph new.
	myMorph initializeWithCamera: self.

	"Initially we aren't focusing on any particular object"
	focusObject _ nil.

	"Initially draw the scene background"
	drawSceneBackground _ true.
! !

!WonderlandStillCamera class methodsFor: 'change logging' stamp: ''!
acceptsLoggingOfCompilation
	^ self == WonderlandStillCamera! !

!WonderlandStillCamera methodsFor: 'rendering' stamp: 'ar 9/8/2000 16:52'!
render: aRenderEngine pickingAt: aPointOrNil withPrimitiveVertex: aBoolean
	"Override the parent method. A WonderlandStillCamera only renders the object that it's focusing on."

	| aRenderer pickedObject |
	aRenderer _ aRenderEngine. "A couple of things must be set before we can pick!!"

	aRenderer viewport: (myMorph bounds insetBy: 1@1).

	(drawSceneBackground)
		ifTrue: [ aRenderer clearViewport: (myWonderland getScene getColorObject). ]
		ifFalse: [ ].

	aRenderer clearDepthBuffer.
	aRenderer loadIdentity.

	focusObject ifNotNil: [

		"Add any existing lights to the renderer for this camera"
		(myWonderland getLights) do: [:light | aRenderer addLight: light ].

		"Calculate our view matrix by inverting the camera's composite matrix and hand it
		to the renderer - note that this will eventually have to walk up the tree"
		viewMatrix _ self getMatrixToRoot.
		aRenderer transformBy: viewMatrix.

		aRenderer perspective: perspective.

		"Initialize picking if necessary"
		aPointOrNil ifNotNil:[aRenderer _ aRenderer asPickerAt: aPointOrNil].

		(focusObject isKindOf: WonderlandScene)
			ifTrue: [ myWonderland renderWonderland: aRenderer ]
			ifFalse: [ aRenderer transformBy: (focusObject getParent getMatrixFromRoot).
					  focusObject renderOn: aRenderer ].

		"Fetch the picked object"
		aPointOrNil ifNotNil:[
			aBoolean
				ifTrue:[pickedObject _ aRenderer topMostObject -> aRenderer topMostVertex]
				ifFalse:[pickedObject _ aRenderer topMostObject]].

	].

	"Force the renderer to draw to the screen"
	aRenderer finish.

	^pickedObject "Will be nil if not picking".
! !

!WonderlandStillCamera methodsFor: 'private' stamp: 'jsp 6/29/1999 02:40'!
appearsInChildLists
	"Return false; we want to hide the still camera from the user"

	^ false.
! !

!WonderlandStillCamera class methodsFor: 'change logging' stamp: 'jsp 7/8/1999 23:15'!
wantsChangeSetLogging
	^ (self == WonderlandStillCamera).
! !

!WonderlandStillCamera methodsFor: 'assigning focus object' stamp: 'jsp 7/15/1999 15:13'!
setFocusObject: anActor
	"Assign the object in the Wonderland that the camera should focus on"

	| boundingBox origin corner center maxDimension distance frustum |

	focusObject _ anActor.

	anActor ifNotNil: [

			(anActor isKindOf: WonderlandActor)
				ifTrue: [ boundingBox _ anActor getBoundingBox.
						  origin _ boundingBox origin.
						  corner _ boundingBox corner.
						  center _ (origin + corner) / 2.0.

						  self turnToRightNow: { 0. 0. 0} asSeenBy: anActor undoable: false.
						  self turnRightNow: left numberOfTurns: 0.5 undoable: false.

						  maxDimension _ (((corner x - center x) max: (corner y - center y))
												max: (center x - origin x))
													max: (center y - origin y).

						  frustum _ self getFrustum.
						  distance _ ((frustum near) * maxDimension / (0.7 * (frustum right)))
										+ (corner z).

						  self moveToRightNow: { center x.
												  center y.
												  distance }
								asSeenBy: anActor undoable: false.

						  self moveRightNow: right distance: (maxDimension) undoable: false.
						  self moveRightNow: up distance: (maxDimension) undoable: false.
						  self turnRightNow: left numberOfTurns:
								((((maxDimension) arcTan: distance) radiansToDegrees) / 360.0)
								undoable: false.
						  self turnRightNow: down numberOfTurns:
								((((maxDimension) arcTan: distance) radiansToDegrees) / 360.0)
								undoable: false.
						]

				ifFalse: [ self moveToRightNow: { -1.5. 0.5. 2.6 } asSeenBy: anActor undoable: false.
						  self pointAtRightNow: { 0. 0. 0 } undoable: false.
						  self moveRightNow: back distance: 20 undoable: false ].

		].

	myMorph changed.
! !

!WonderlandCamera methodsFor: 'get property' stamp: 'jsp 2/11/1999 11:13'!
getAspectRatio
	"Returns the camera's current aspect ratio."

	^ perspective aspectRatio.
! !

!WonderlandCamera methodsFor: 'initialize-release' stamp: 'ar 3/4/2004 21:10'!
initializeFor: aWonderland
	"Initializes the camera."

	super initializeFor: aWonderland.

	"Set the camera's mesh and geometry"
	self setMesh: (WonderlandConstants default at: 'cameraMesh').
	self setTexturePointer: (WonderlandConstants default at: 'cameraTexture').
	
	"Set the camera initial position"
	composite translation: (B3DVector3 x: -1.5 y: 0.5 z: 2.6).
	self turnTo: #(180 -30 180) duration: #rightNow.

	"Initialize the camera viewing parameters"
	perspective _ B3DCameraPerspective new.
	self setFieldOfView: 40.0.
	self setAspectRatio: 1.0.
	self setNearClippingPlane: 0.1.
	self setFarClippingPlane: 10000.0.

	viewMatrix _ B3DMatrix4x4 new.

	"Create a WonderlandCameraMorph for the camera to render into"
	myMorph _ WonderlandCameraMorph new.
	myMorph initializeWithCamera: self.

	"Initially draw the scene background"
	drawSceneBackground _ true.
! !

!WonderlandCamera methodsFor: 'accessing' stamp: 'jsp 3/22/1999 14:25'!
getMorph
	"Returns the camera window morph that this camera renders into"

	^ myMorph.
! !

!WonderlandCamera methodsFor: 'private' stamp: 'jsp 3/25/1999 10:48'!
getFrustum
	"Returns the camera's frustum"

	^ perspective asFrustum.
! !

!WonderlandCamera methodsFor: 'rendering' stamp: 'ar 5/28/2000 12:24'!
pickAt: aPoint
	"Return the top object at the given point or nil"
	^self render: (B3DRenderEngine defaultForPlatformOn: nil) pickingAt: aPoint! !

!WonderlandCamera methodsFor: 'rendering' stamp: 'ar 9/9/2000 03:03'!
renderBounds
	"Compute 2D bounding boxes for all scene elements"
	| engine |
	engine _ (B3DRenderEngine defaultForPlatformOn: nil) asMapperEngine.
	self render: engine.
	bounds _ engine boundsMap.! !

!WonderlandCamera methodsFor: 'debug' stamp: 'ar 11/7/1999 18:39'!
debugDrawRectsOn: aCanvas
	bounds == nil ifTrue:[^self].
	bounds keysAndValuesDo:[:actor :rect|
		aCanvas frameRectangle: rect color: Color white.
	].! !

!WonderlandCamera methodsFor: 'set property' stamp: 'ar 11/7/1999 18:50'!
setBounds: aRectangle for: anActor
	"Record the 2D bounds for the given actor during a rendering pass"
	aRectangle == nil ifFalse:[bounds at: anActor put: (aRectangle intersect: myMorph bounds)].! !

!WonderlandCamera methodsFor: 'get property' stamp: 'ar 4/20/2001 03:32'!
getBoundsForActor: anActor
	"Return the 2D bounds of the actor as seen by the receiver"
	bounds == nil ifTrue:[self renderBounds].
	^bounds at: anActor ifAbsent:[nil]! !

!WonderlandCamera methodsFor: 'testing' stamp: 'ar 5/8/2001 20:38'!
isWonderlandCamera
	^true! !

!WonderlandCamera methodsFor: 'rendering' stamp: 'ar 5/28/2000 12:24'!
pickObjectAndVertexAt: aPoint
	"Return an association with the top object and the primitive vertex at the given point or nil"
	^self render: (B3DRenderEngine defaultForPlatformOn: nil) 
		pickingAt: aPoint
		withPrimitiveVertex: true.! !

!WonderlandCamera methodsFor: 'set property' stamp: 'ar 5/15/2001 14:03'!
setFogDensity: aNumber
	self setProperty: #fogDensity toValue: aNumber! !

!WonderlandCamera methodsFor: 'transforming' stamp: 'jsp 3/29/1999 10:58'!
transformScreenPointToScenePoint: aPoint using: anActor
	"Converts a pixel point to a position in the 3D scene using the actor to scale the position of the point. Uses the near clipping plane distance for the depth in the scene."

	| d  x y m scale frustum vector |

	frustum _ self getFrustum.

	d _ frustum near.
	scale _ ((anActor getPosition: self) at: 3) / d.

	x _ ((aPoint x) - (myMorph center x)) / ((myMorph width) / 2) * (frustum right) * scale.
	y _ (((aPoint y) - (myMorph center y)) / ((myMorph height) / 2)) * (frustum bottom) * scale.

	m _ B3DMatrix4x4 identity.
	m translation: (B3DVector3 x: x y: y z: d).
	m _ (self getMatrixFromRoot) composeWith: m.

	vector _ (m translation).

	^ { vector at: 1. vector at: 2. vector at: 3 }.
! !

!WonderlandCamera methodsFor: 'transforming' stamp: 'jsp 4/19/1999 15:30'!
transformScreenVectorToSceneVector: aVector
	"Converts a picture plane vector to a position in the 3D scene."

	| x y d m frustum scale vector |

	frustum _ self getFrustum.

	d _ aVector at: 3.
	scale _ d / (frustum near).

	x _ ((aVector x) - (myMorph center x)) / ((myMorph width) / 2) * (frustum right) * scale.
	y _ (((aVector y) - (myMorph center y)) / ((myMorph height) / 2)) * (frustum bottom) * scale.

	m _ B3DMatrix4x4 identity.
	m translation: (B3DVector3 x: x y: y z: d).
	m _ (self getMatrixFromRoot) composeWith: m.

	vector _ (m translation).

	^ { vector at: 1. vector at: 2. vector at: 3 }.
! !

!WonderlandCamera methodsFor: 'set property' stamp: 'ar 5/15/2001 14:03'!
setFogType: aSymbol
	self setProperty: #fogType toValue: aSymbol! !

!WonderlandCamera methodsFor: 'rendering' stamp: 'ar 5/15/2001 16:27'!
render: aRenderEngine pickingAt: aPointOrNil withPrimitiveVertex: aBoolean
	"Render one frame of the Wonderland using this camera.
	If aPointOrNil is not nil then pick the top most object at this point.
	For picking only: If aBoolean is true return an association 
	object -> B3DPrimitiveVertex; otherwise simply return the top most object.
	Note: If picking, no objects are actually drawn."
	| aRenderer pickedObject |
	aRenderer _ aRenderEngine. "A couple of things must be set before we can pick!!"
	bounds _ nil. "forget bounds"
	aRenderer viewport: myMorph bounds.

	(drawSceneBackground)
		ifTrue: [ aRenderer clearViewport: (myWonderland getScene) getColorObject. ]
		ifFalse: [ ].

	self renderFogOn: aRenderer.
	aRenderer clearDepthBuffer.
	aRenderer loadIdentity.

	"Add any existing lights to the renderer for this camera"
	(myWonderland getLights) do: [:light | aRenderer addLight: light ].

	"Calculate our view matrix by inverting the camera's composite matrix and hand it
	to the renderer - note that this will eventually have to walk up the tree"
	viewMatrix _ self getMatrixToRoot.
	aRenderer transformBy: viewMatrix.

	aRenderer perspective: perspective.

	"Initialize picking if necessary"
	aPointOrNil == nil
		ifFalse:[	"Make us a picker"
				aRenderer _ aRenderer asPickerAt: aPointOrNil].

	"Now render the scene"
	myWonderland renderWonderland: aRenderer.

	"Force the renderer to draw to the screen"
	aRenderer finish.

	"Fetch the picked object"
	aPointOrNil ifNotNil:[
		aBoolean
			ifTrue:[pickedObject _ aRenderer topMostObject -> aRenderer topMostVertex]
			ifFalse:[pickedObject _ aRenderer topMostObject]].

	^pickedObject "Will be nil if not picking"! !

!WonderlandCamera methodsFor: 'get property' stamp: 'ar 5/15/2001 14:16'!
getFogDensity
	^self getProperty: #fogDensity ifAbsent:[1.0]! !

!WonderlandCamera class methodsFor: 'change logging' stamp: ''!
acceptsLoggingOfCompilation
	^ self == WonderlandCamera! !

!WonderlandCamera methodsFor: 'set property' stamp: 'jsp 2/11/1999 11:15'!
setFarClippingPlane: aFloat
	"Sets the camera's far clipping plane - anything farther than this won't be drawn."

	perspective farDistance: aFloat.
! !

!WonderlandCamera methodsFor: 'set property' stamp: 'ar 5/15/2001 14:03'!
setFogRangeEnd: aNumber
	self setProperty: #fogRangeEnd toValue: aNumber! !

!WonderlandCamera methodsFor: 'accessing' stamp: 'jsp 3/22/1999 16:19'!
turnBackgroundOn
	"Resume drawing the scene background"

	drawSceneBackground _ true.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/4/1999 10:57'!
place: aLocation object: aTarget
	"Moves this object to one of several pre-determined locations relative to the target object."

	 ^ self place: aLocation object: aTarget duration: 1.0 style: gently.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 15:57'!
pointAtRightNow: aTarget undoable: isUndoable
	"Turn this object to the specified orientation instantaneously"

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoRotationChange for: self from: (self getAngles)).
						].

	self setRotationMatrix: (self getPointAtMatrix: aTarget).


! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:40'!
setWidth: value
	self resize: {value / self width asFloat. 1.0. 1.0} duration: rightNow.
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/9/2001 22:33'!
setAnimationIndex: aNumber
	self setMeshAnimationIndex: aNumber.! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:41'!
width
	^self getBoundingBox extent x * 100! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/18/1999 10:40'!
rollRightNow: aDirection numberOfTurns: numTurns undoable: isUndoable
	"Turn the object instantaneously the specified number of turns in the specified direction"

	| tMatrix degrees  |

	tMatrix _ B3DMatrix4x4 identity.
	degrees _ 360 * numTurns.

	(isUndoable = true) ifTrue: [
		myWonderland getUndoStack push: (UndoRotationChange for: self from: (self getAngles)).
						].

	(aDirection = left) ifTrue: [ tMatrix rotationAroundZ: degrees ]
		ifFalse: [(aDirection = right) ifTrue: [ tMatrix rotationAroundZ: (degrees negated) ]
									ifFalse: [ myWonderland reportErrorToUser: 'Squeak does not know how to roll ' , myName , ' ', (aDirection asString) , '.'.
											^ nil ]].

	self rotateByMatrix: tMatrix.
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:39'!
getZ
	^(self getPointOfView at: 3) * 100! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 4/9/1999 14:06'!
angularDistanceTo: aRotationVector
	"Returns the angular distance in degrees between the actor's current orientation and an (x, y, z) rotation vector"

	| m sum angles vector1 vector2 |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aRotationVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can only calculate the distance from ' , myName , ' to an actor or 3D point.'.
			^ nil ].

	"Our arguments check out, so determine the distance"
	m _ B3DMatrix4x4 identity.
	m rotation: (self getRotationVector).

	m _ m composeWith: (B3DMatrix4x4 identity translation: (B3DVector3 x:0 y:0 z:1)).
	vector1 _ m translation.

	m setIdentity.

	(aRotationVector isKindOf: WonderlandActor)
		ifTrue: [
				angles _ aRotationVector getAngles: myParent.
				m rotation: (B3DVector3 x: (angles at: 1) y: (angles at: 2) z: (angles at: 3)).
				]
		ifFalse: [
				m rotation: (B3DVector3 x: (aRotationVector at: 1) y: (aRotationVector at: 2)
										z: (aRotationVector at: 3)).
				].

	m _ m composeWith: (B3DMatrix4x4 identity translation: (B3DVector3 x:0 y:0 z:1)).
	vector2 _ m translation.

	sum _ ((vector1 x) * (vector2 x)) + ((vector1 y) * (vector2 y)) + ((vector1 z) * (vector2 z)).

	^ ((sum / ((vector1 length) * (vector2 length))) arcCos) radiansToDegrees.
! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'sw 9/13/2001 14:20'!
methodInterfaceForEmptyScript
	"Copied from Player"

	| anInterface |
	anInterface _ MethodInterface new.
	anInterface documentation: 'an empty script; drop on desktop to get a new empty script for this object.'.
	anInterface receiverType: #none.
	anInterface flagAttribute: #scripts.

	anInterface absorbTranslation: 
		(ElementTranslation new wording: #emptyScript helpMessage: 'an empty script; drop on desktop to get a new empty script for this object.' language: #English).

	anInterface selector: #emptyScript type: nil setter: nil.
	^ anInterface! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'ar 5/8/2001 20:48'!
newScriptorAround: aPhraseTileMorph
	"Fake, fake, fake."
	| someCamera |
	someCamera _ myWonderland getScene getAllChildren detect:[:any| any isWonderlandCamera].
	^someCamera getMorph assuredPlayer assureUniClass newScriptorAround: aPhraseTileMorph! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 14:59'!
turnRightNow: aDirection numberOfTurns: numTurns undoable: isUndoable
	"Turn the object instantaneously the specified number of turns in the specified direction"

	| tMatrix degrees  |

	tMatrix _ B3DMatrix4x4 identity.
	degrees _ 360 * numTurns.

	(isUndoable = true) ifTrue: [
		myWonderland getUndoStack push: (UndoRotationChange for: self from: (self getAngles)).
						].

	(aDirection = right) ifTrue: [ tMatrix rotationAroundY: degrees ]
		ifFalse: [(aDirection = left) ifTrue: [ tMatrix rotationAroundY: (degrees negated) ]

		ifFalse: [(((aDirection = up) or: [aDirection = back]) or: [aDirection = backward])
			ifTrue: [ tMatrix rotationAroundX: (degrees negated) ]

		ifFalse: [((aDirection = down) or: [aDirection = forward]) ifTrue: [
			tMatrix rotationAroundX: degrees ]

		ifFalse: [(aDirection = ccw) ifTrue: [ tMatrix rotationAroundZ: degrees ]

		ifFalse: [(aDirection = cw) ifTrue: [ tMatrix rotationAroundZ: (degrees negated) ]

		ifFalse: [ self error: 'Unrecognized direction' ]]]]]].

	self rotateByMatrix: tMatrix.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:17'!
move: aDirection distance: aDistance duration: aDuration asSeenBy: reference style: aStyle
	"Move the actor the specified distance in the specified direction, taking the specified duration and using the specified animation style."

	| anim a3DVector tmpVector tMatrix newComposite |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: aDistance ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the distance to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for moving ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to move ' , myName , ' because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ RelativeAnimation new.
	tmpVector _ (B3DVector3 x:0 y:0 z:0).
	tMatrix _ B3DMatrix4x4 identity.

	[ a3DVector _ self makeVectorFromDistance: aDistance andDirection: aDirection. ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak does not know how to move ' , myName , ' in that direction.'.
			^ nil ].

	anim object: self
			update: [:tPos | newComposite _ (reference getMatrixToRoot)
										composeWith: (self getMatrixFromRoot).
						 composite _ (((myParent getMatrixToRoot)
										composeWith: (reference getMatrixFromRoot))
										composeWith:
										(tMatrix translation: (tPos - tmpVector)))
										composeWith: newComposite.
						 tmpVector _ tPos]
			getStartState: [tmpVector x:0; y:0; z:0.
								(B3DVector3 x:0 y:0 z:0) ]
			getEndState: [ a3DVector ]
			getReverseState: [ a3DVector negated ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/24/1999 09:54'!
setRotationVector: aVector
	"Instantaneously blam the actor's rotation"

	composite rotation: aVector.
! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/26/2001 16:15'!
externalName
	^ self knownName ifNil: [self getName]! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:10'!
move: aDirection distance: aDistance 
	"Move the actor the specified distance in the specified direction, taking 1 second and using the Gently animation style."

	^ (self move: aDirection distance: aDistance duration: 1.0
			style: gently).

! !

!WonderlandActor methodsFor: 'testing' stamp: 'laza 3/21/2000 15:59'!
isHandmade
	^(self getProperty: #handmade) == true! !

!WonderlandActor methodsFor: 'testing' stamp: 'ar 5/8/2001 20:37'!
isWonderlandCamera
	^false! !

!WonderlandActor methodsFor: 'animated state' stamp: 'jsp 3/9/1999 16:18'!
getActions
	"Return the actions currently affecting this object"

	^ (myWonderland getScheduler) getActionsFor: self.
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 9/10/2000 00:24'!
getBackfaceCulling
	"Return backface culling - either #cw, #ccw or nil"
	^self getProperty: #backfaceCulling! !

!WonderlandActor methodsFor: 'set property' stamp: 'ar 11/8/1999 13:14'!
setUserPointOfView: aPOV
	"Return the user defined point of view for this object"
	^self setProperty: #userPointOfView toValue: aPOV! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/1/1999 13:27'!
getColorVector
	"Returns the actor's color/alpha vector"

	^ myColor.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 11:00'!
turn: aDirection turns: numTurns duration: aDuration
	"Turn the actor the specified distance in the specified direction, taking the specified duration and using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyDirection: aDirection ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: numTurns ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how many turns to turn ' , myName , ' because ', msg.
				^ nil ].

		self turnRightNow: aDirection numberOfTurns: numTurns undoable: true.
		^ self. ].

	^ (self turn: aDirection turns: numTurns duration: aDuration style: gently).

! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/18/1999 10:40'!
rollRightNow: aDirection numberOfTurns: numTurns asSeenBy: reference undoable: isUndoable
	"Turn the object instantaneously the specified number of turns in the specified direction"

	| tMatrix degrees  newMatrix |

	tMatrix _ B3DMatrix4x4 identity.
	degrees _ 360 * numTurns.

	(isUndoable = true) ifTrue: [
		myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).
						].

	(aDirection = left) ifTrue: [ tMatrix rotationAroundZ: degrees ]
	ifFalse: [(aDirection = right) ifTrue: [ tMatrix rotationAroundZ: (degrees negated) ]
								ifFalse: [ myWonderland reportErrorToUser: 'Squeak does not know how to roll ' , myName , ' ', (aDirection asString) , '.'.
										^ nil ]].

	newMatrix _ reference getMatrixToRoot.
	newMatrix _ newMatrix composeWith: (self getMatrixFromRoot).
	composite _ (((myParent getMatrixToRoot) composeWith: (reference getMatrixFromRoot))
											composeWith: tMatrix)
											composeWith: newMatrix.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/24/1999 11:11'!
getRotationVector
	"Returns the current rotation as a vector."

	^ (composite rotation).
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/9/2001 22:33'!
animationIndex
	^self getMeshAnimationIndex! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:58'!
turn: aDirection turns: numTurns duration: aDuration asSeenBy: reference style: aStyle
	"Turn the actor the specified numberOfTurns in the specified direction, taking the specified duration and using the specified animation style."

	| anim updateFunc degrees lastAmount buildMatrix newMatrix tMatrix|

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: numTurns ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how many turns to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for turning ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to turn ' , myName , ' because ', msg.
			^ nil ].

	"The parameters check out, so go ahead and build the animation"
	anim _ RelativeAnimation new.
	degrees _ 360 * numTurns.
	lastAmount _ 0.
	tMatrix _ B3DMatrix4x4 new.

	buildMatrix _ [ :rotMatrix | newMatrix _ reference getMatrixToRoot.
						newMatrix _ newMatrix composeWith: (self getMatrixFromRoot).
						newMatrix _ (((myParent getMatrixToRoot)
										composeWith: (reference getMatrixFromRoot))
										composeWith: rotMatrix)
										composeWith: newMatrix. ].

	(aDirection = #right) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundY: (tAngle - lastAmount)).
							lastAmount _ tAngle] ]

		ifFalse: [(aDirection = #left) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundY: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated ]

		ifFalse: [(((aDirection = #up) or: [aDirection = #back]) or: [aDirection = #backward])
			ifTrue: [
				updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundX: (tAngle - lastAmount)).
							lastAmount _ tAngle].
				degrees _ degrees negated ]

		ifFalse: [((aDirection = #down) or: [aDirection = #forward]) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundX: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #ccw) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #cw) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated. ]

		ifFalse: [ self error: 'Unrecognized direction' ]]]]]].
	
	anim object: self
			update: updateFunc
			getStartState: [ lastAmount _ 0. (Interpolateable value: 0) ]
			getEndState: [ degrees ]
			getReverseState: [ degrees negated ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:40'!
setY: value
	^self setPointOfView: ((self getPointOfView) at: 2 put: value * 0.01; yourself) duration: rightNow! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 09:47'!
moveTo: aVector duration: aDuration
	"Moves the object to the specified position in its parent's coordinate system over the specified duration using the Gently animation style."


	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyTarget: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyDuration: aDuration ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the duration to use for moving ' , myName , ' because ', msg.
				^ nil ].

		self moveToRightNow: aVector undoable: true.
		^ self. ].

	(aDuration = eachFrame) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyTarget: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyDuration: aDuration ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the duration to use for moving ' , myName , ' because ', msg.
				^ nil ].

		myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

		^ self doEachFrame: [ self moveToRightNow: aVector	undoable: false ].
									].

	^ (self moveTo: aVector duration: aDuration
			style: gently).
! !

!WonderlandActor methodsFor: 'do-ing' stamp: 'jsp 3/30/1999 11:39'!
doEachFrame: aBlock until: aCondition
	"Build an action that causes the object to execute the block every frame until the specified condition is true"

	| newAction |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyZeroArgumentBlock: aBlock ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not make ' , myName , ' do what you asked because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine for how long ' , myName , ' should do this because ', msg.
			^ nil ].

	newAction _ Action do: aBlock eachframeuntil: aCondition toObject: self
						inScheduler: (myWonderland getScheduler).
	(myWonderland getUndoStack) push:
			(UndoByStopping new: newAction in: (myWonderland getUndoStack)).

	^ newAction.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:29'!
move: aDirection speed: aSpeed for: aLifetime
	"Creates and returns an action that moves the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to move ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long ' , myName , ' should move because ', msg.
			^ nil ].

	"The parameters check out, so start the action"
	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveRightNow: aDirection
							distance: ((myWonderland getScheduler getElapsedTime) * aSpeed)
							undoable: false ]
		   for: aLifetime.
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:51'!
heading
	^(self getPointOfView at: 5) negated! !

!WonderlandActor methodsFor: 'do-ing' stamp: 'jsp 3/18/1999 17:07'!
do: aBlock
	"Build an animation that causes the object to execute the block"

	| anim |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyZeroArgumentBlock: aBlock ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not make ' , myName , ' do what you asked because ', msg.
			^ nil ].

	anim _ RelativeAnimation new.

	anim object: self
		update: [:tmp | aBlock value ]
		getStartState: [ Interpolateable value: 0 ]
		getEndState: [ 1 ]
		getReverseState: [ 1 ]
		style: abruptly
		duration: 0
		undoable: false
		inWonderland: myWonderland.

	^ anim.! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/23/1999 12:03'!
makeVectorFromDistance: aDistance andDirection: aDirection
	"Generates the vector the object should move along using the specified direction"

	| a3DVector |

	(aDirection = right) ifTrue: [a3DVector _ B3DVector3 x: aDistance y: 0 z: 0]
		ifFalse: [(aDirection = left) ifTrue: [a3DVector _ B3DVector3 x: -1 * aDistance y:0 z:0]
		ifFalse: [(aDirection = up) ifTrue: [a3DVector _ B3DVector3 x: 0 y: aDistance z:0]
		ifFalse: [(aDirection = down) ifTrue: [a3DVector _ B3DVector3 x: 0 y: -1 * aDistance z:0]
		ifFalse: [(aDirection = forward) ifTrue: [a3DVector _ B3DVector3 x: 0 y: 0 z: aDistance]
		ifFalse: [((aDirection = back) or: [aDirection = #backward])
					 ifTrue: [a3DVector _ B3DVector3 x: 0 y: 0 z: -1 * aDistance]
		ifFalse: [ self error: 'Unrecognized direction' ]]]]]].

	^ a3DVector.
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 9/10/2000 02:21'!
getLineWidth
	^self getProperty: #lineWidth ifAbsent:[1]! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 8/15/2001 13:25'!
tearOffTileForSelf
	| tiles |
	self currentHand attachMorph: (tiles _ self tileReferringToSelf).
	(tiles isKindOf: SyntaxMorph) ifTrue:
		[Preferences tileTranslucentDrag
			ifTrue: [tiles lookTranslucent]
			ifFalse: [tiles align: tiles topLeft 
			 			with: self currentHand position + tiles cursorBaseOffset]]
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 4/19/1999 11:24'!
destroy: aDuration
	"Implements the animated destroy of an actor.  This takes all the actors parts and spins them off in an arbitrary direction"

	| anim allAnims undoActions childList partsList |

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for destroying ' , myName , ' because ', msg.
			^ nil ].

	"The parameter checks out, so start the setup"
	undoActions _ OrderedCollection new.
	allAnims _ OrderedCollection new.
	childList _ self getAllChildren.
	partsList _ self getAllParts.
	partsList addFirst: self.

	"Do it for the top level object"

	"We need to do this for every part"
	childList do: [:child |
		(child isPart)
			ifTrue: [

				"Make sure our POV gets reset on undo"
				undoActions addFirst: (UndoPOVChange for: child from: (child getPointOfView)).
				
				"Create the animation for moving toward a random endpoint"
				anim _ child moveTo: { (-2 to: 2) atRandom. (-2 to: 2) atRandom.
							(-2 to: 2) atRandom} duration: aDuration
							asSeenBy: (myWonderland getScene) style: abruptly.

				anim stop.
				anim setUndoable: false.
				allAnims addLast: anim.

				"Create the animation for spinning this object"
				child turnToRightNow: { (0 to: 360) atRandom. (0 to: 360) atRandom.
							(0 to: 360) atRandom} undoable: false.

				anim _ child turn: left turns: aDuration speed: 1.
				anim stop.
				anim setUndoable: false.
				allAnims addLast: anim.
					]
			ifFalse: [
				"Make the non-part children point to this actor's parent"
				undoActions addLast: (UndoParentChange newFor: child from: (child getParent)).
				child reparentTo: (self getParent).
					].
					].

	"Add an undo action to put the objects back in the scene"	
	undoActions addFirst: (UndoAction new: [ (myWonderland getScene) addChild: self.
											 myWonderland getEditor updateActorBrowser ]).

	"Add the undo list to the stack"
	(myWonderland getUndoStack) push: (UndoChangeList new setChangeList: undoActions).

	"Create the animation to remove the parts from the scene and update the actor browser"
	anim _ self do: [ self removeFromScene.
					 myWonderland getEditor updateActorBrowser ].

	"Now start our parallel animation"
	anim _ myWonderland doInOrder: { myWonderland doTogether: allAnims. anim }.
	anim setUndoable: false.
! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 3/18/1999 16:57'!
getAngles: reference
	"Returns the object's orientation relative to a reference object"

	| newComposite anArray a3DVector |

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not determine the angles of ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	newComposite _ reference getMatrixToRoot.
	a3DVector _ (newComposite composeWith: (self getMatrixFromRoot)) rotation.

	anArray _ Array new: 3.
	anArray at: 1 put: (a3DVector x).
	anArray at: 2 put: (a3DVector y).
	anArray at: 3 put: (a3DVector z).

	^ anArray.
! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 3/25/1999 15:59'!
getPositionInPixels
	"Returns the actor's position in pixels in the Squeak window"

	| camera morph point x y frustum |

	camera _ myWonderland getDefaultCamera.
	morph _ camera getMorph.

	point _ self getPositionInPicturePlane.

	frustum _ camera getFrustum.

	((point x) < 0) ifTrue: [
			x _ (morph center x) - (((point x) / (frustum left)) * ((morph width) / 2))
						]
				ifFalse: [
			x _ (morph center x) + (((point x) / (frustum right)) * ((morph width) / 2))
						].

	((point y) < 0) ifTrue: [
			y _ (morph center y) + (((point y) / (frustum bottom)) * ((morph height) / 2))
						]
				ifFalse: [
			y _ (morph center y) - (((point y) / (frustum top)) * ((morph height) / 2))
						].

	^ (x @ y) rounded.
	! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 5/9/2001 22:34'!
getMeshAnimationIndex
	myMesh == nil ifTrue:[^1].
	^myMesh animationIndex! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 11/8/1999 16:54'!
getGlobalPosition: aVector
	"Return the global position of the given vector"
	| vv cc |
	vv _ B3DVector3 x: (aVector at: 1) y: (aVector at: 2) z: (aVector at: 3).
	cc _ composite composeWith: scaleMatrix.
	vv _ cc localPointToGlobal: vv.
	^(Array with: vv x with: vv y with: vv z)! !

!WonderlandActor class methodsFor: 'change logging' stamp: 'jsp 4/23/1999 11:50'!
acceptsLoggingOfCompilation

	^ self == WonderlandActor.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:21'!
turnTo: aVector eachFrameUntil: aCondition
	"Turns the object to the specified orientation in its parent's coordinate system each frame until the condition returns true."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should turn ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self turnToRightNow: aVector	undoable: false ]
			until: aCondition.

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/24/1999 16:51'!
resize: amount duration: aDuration
	"Resize the object by the specified amount over the specified duration using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyNumberOrTriple: amount ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , '  because ', msg.
				^ nil ].

		(amount isNumber)
			ifTrue: [ self resizeRightNow: (B3DVector3 x: amount y: amount z: amount)
							undoable: true. ]
			ifFalse: [ self resizeRightNow: (B3DVector3 x: (amount at: 1) y: (amount at: 2)
							z: (amount at: 3)) undoable: true. ].
		^ self.
									].

	(aDuration = eachFrame) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyNumberOrTriple: amount ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , '  because ', msg.
				^ nil ].

		myWonderland getUndoStack push: (UndoSizeChange for: self from: (self getSize)).

		(amount isNumber)
			ifTrue: [ ^ self doEachFrame: [ self resizeRightNow: (B3DVector3 x: amount y:
												amount z: amount) undoable: false ] ]
			ifFalse: [ ^ self doEachFrame: [ self resizeRightNow: (B3DVector3 x: (amount at: 1)
												y: (amount at: 2) z: (amount at: 3))
												undoable: true. ] ].
									].

	(amount isNumber)
		ifTrue: [ ^ (self resizeTopToBottom: amount leftToRight: amount frontToBack: amount
						duration: 1.0
						style: gently). ]
		ifFalse: [  ^ (self resizeTopToBottom: (amount at: 2)
						leftToRight: (amount at: 1)
						frontToBack: (amount at: 3)
						duration: 1.0
						style: gently). ].
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:45'!
roll: aDirection speed: aSpeed for: aLifetime
	"Creates and returns an action that rolls the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to roll ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should roll ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self rollRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				undoable: false
						]
		   for: aLifetime.

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:28'!
move: aDirection speed: aSpeed asSeenBy: reference for: aLifetime
	"Creates and returns an action that moves the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to move ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long ' , myName , ' should move because ', msg.
			^ nil ].

	"The parameters check out, so start the action"
	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveRightNow: aDirection
							distance: ((myWonderland getScheduler getElapsedTime) * aSpeed)
							asSeenBy: reference
							undoable: false ]
		   for: aLifetime.

! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/24/1999 16:15'!
getComposite
	"Returns the actor's composite matrix"

	^ composite.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/22/1999 12:42'!
resizeLikeRubber: amount dimension: aDimension
	"Resizes the object using a volume preserving resize"

	^ self resizeLikeRubber: amount dimension: aDimension duration: 1.0 style: gently.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 15:43'!
turnToRightNow: aVector undoable: isUndoable
	"Turn this object to the specified orientation instantaneously"

	| m angles |

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoRotationChange for: self from: (self getAngles)).
						].

	m _ B3DMatrix4x4 identity.

	(aVector isKindOf: WonderlandActor)
		ifTrue: [ angles _ aVector getAngles: myParent.
				  m rotation: (B3DVector3 x: (angles at: 1) y: (angles at: 2) z: (angles at: 3)).
				]
		ifFalse: [ angles _ self getRotationVector.
				  ((aVector at: 1) = asIs) ifFalse: [ angles x: (aVector at: 1) ].
				  ((aVector at: 2) = asIs) ifFalse: [ angles y: (aVector at: 2) ].
				  ((aVector at: 3) = asIs) ifFalse: [ angles z: (aVector at: 3) ].
				  m rotation: angles.
				].

	self setRotationMatrix: m.

! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 5/9/2001 22:34'!
setMeshAnimationIndex: index
	myMesh animationIndex: index.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 15:48'!
turnToRightNow: aVector asSeenBy: reference undoable: isUndoable
	"Turn this object to the specified orientation instantaneously"

	| m angles newComposite |

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoRotationChange for: self from: (self getAngles)).
						].

	m _ B3DMatrix4x4 identity.
	angles _ self getAngles: reference.
	((aVector at: 1) = asIs) ifFalse: [ angles at: 1 put: (aVector at: 1) ].
	((aVector at: 2) = asIs) ifFalse: [ angles at: 2 put: (aVector at: 2) ].
	((aVector at: 3) = asIs) ifFalse: [ angles at: 3 put: (aVector at: 3) ].
	m rotation: (B3DVector3 x: (angles at: 1) y: (angles at: 2) z: (angles at: 3)).
	newComposite _ (myParent getMatrixToRoot) composeWith: (reference getMatrixFromRoot).
	self setRotationMatrix: (newComposite composeWith: m).


! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:18'!
turnTo: aVector duration: aDuration asSeenBy: reference style: aStyle
	"Turns the object to the specified orientation in its parent's coordinate system using the specified style over the specified duration."

	| anim endStateFunc angles newComposite vectorMatrix |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the duration to use for turning ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to turn ' , myName , ' because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ AbsoluteAnimation new.

	vectorMatrix _ B3DMatrix4x4 identity.
	endStateFunc _ [
		vectorMatrix setIdentity.
		angles _ self getAngles: reference.
		((aVector at: 1) = asIs) ifFalse: [ angles at: 1 put: (aVector at: 1) ].
		((aVector at: 2) = asIs) ifFalse: [ angles at: 2 put: (aVector at: 2) ].
		((aVector at: 3) = asIs) ifFalse: [ angles at: 3 put: (aVector at: 3) ].
		vectorMatrix rotation: (B3DVector3 x: (angles at: 1) y: (angles at: 2) z: (angles at: 3)).
		newComposite _ (myParent getMatrixToRoot) composeWith: (reference getMatrixFromRoot).
		newComposite _ newComposite composeWith: vectorMatrix.
		newComposite asQuaternion.
					].

	anim object: self
			update: [:tQuat | self setRotationMatrix: (tQuat asMatrix4x4) ]
			getStartState: [ composite asQuaternion ]
			getEndState: endStateFunc
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:07'!
move: aDirection distance: aDistance asSeenBy: reference
	"Move the actor the specified distance in the specified direction, taking 1 second and using the Gently animation style."

	^ (self move: aDirection distance: aDistance duration: 1.0 asSeenBy: reference
			style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:11'!
turn: aDirection speed: aSpeed until: aCondition
	"Creates and returns an action that turn the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to turn ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should turn ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self turnRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				undoable: false
						]
		   until: aCondition.

! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 8/20/1999 00:56'!
getMesh
	^myMesh! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:07'!
resizeTopToBottom: height leftToRight: width frontToBack: thickness eachFrameUntil: aCondition
	"Resize the object by the specified amount each frame until the condition is true."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyNumber: height ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from top to bottom because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: width ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from left to right because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: thickness ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from front to back because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should resize ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoSizeChange for: self from: (self getSize)).

	^ self doEachFrame: [ self resizeRightNow: (B3DVector3 x: width y: height z: thickness)
									undoable: false ]
			until: aCondition.
! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 13:58'!
setColor: newColor duration: time
	"Set the actor's color"

	(time = rightNow) ifTrue: [
		[ WonderlandVerifier VerifyColor: newColor ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine what color to make ' , myName , ' because ', msg.
				^ nil ].

		self setColorRightNow: newColor undoable: true.
		^ self. ].

	^ (self setColor: newColor duration:time style: gently)

! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/10/1999 16:46'!
rotateByMatrix: aMatrix
	"Rotate the object by composing its rotation matrix with the given matrix."

	composite _ composite composeWith: aMatrix.! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 11/7/1999 22:33'!
getProperty: aSymbol
	^self getProperty: aSymbol ifAbsent:[nil]! !

!WonderlandActor methodsFor: 'accessing' stamp: 'ar 5/23/2001 23:26'!
getBoundingBox
	"Returns the actor's bounding box"

	| size x y z nOrigin nCorner childBBox cOrigin cCorner meshBBox mOrigin mCorner |

	(myMesh notNil)
		ifTrue: [ meshBBox _ myMesh boundingBox.
true ifTrue:[
				cOrigin _ scaleMatrix localPointToGlobal: meshBBox origin.
				cCorner _ scaleMatrix localPointToGlobal: meshBBox corner.
				nOrigin _ cOrigin min: cCorner.
				nCorner _ cOrigin max: cCorner.
] ifFalse:[
				 size _ self getSizeVector.
				 mOrigin _ meshBBox origin.
				 mCorner _ meshBBox corner.

				 x _ (mOrigin x) * (size x).
				 y _ (mOrigin y) * (size y).
				 z _ (mOrigin z) * (size z).
				 nOrigin _ B3DVector3 x: x y: y z: z.

				 x _ (mCorner x) * (size x).
				 y _ (mCorner y) * (size y).
				 z _ (mCorner z) * (size z).
				 nCorner _ B3DVector3 x: x y: y z: z.
].
				]
		ifFalse: [ nOrigin _ (B3DVector3 x:0 y:0 z:0).
				  nCorner _ (B3DVector3 x:0 y:0 z:0) ].


	myChildren do: [:child | 
			(child isPart) ifTrue: [
									childBBox _ child getBoundingBox: self.
				 					cOrigin _ childBBox origin.
				 					cCorner _ childBBox corner.
									nOrigin _ nOrigin min: cOrigin.
									nCorner _ nCorner max: cCorner.
								 ].
					].

	^ Rectangle origin: nOrigin corner: nCorner.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 4/5/1999 14:55'!
removeFromScene
	"Remove this actor from the scene."

	"Break ties with the current parent"
	myParent removeChild: self.

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:49'!
roll: aDirection turns: numTurns duration: aDuration asSeenBy: reference
	"Roll the actor the specified number of turns in the specified direction, taking the specified duration and using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyDirection: aDirection ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: numTurns ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the number of turns to roll ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyReferenceFrame: reference ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak can not roll ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
				^ nil ].

			self rollRightNow: aDirection numberOfTurns: numTurns
						asSeenBy: reference undoable: true.
			^ self. ].

	^ (self roll: aDirection turns: numTurns duration: aDuration asSeenBy: reference
			style: gently).

! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:51'!
setHeading: heading
	self setPointOfView: ((self getPointOfView) at: 5 put: heading negated; yourself) duration: rightNow.! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/10/2001 16:18'!
graphicForViewerTab
	"When a Viewer is open on the receiver, its tab needs some graphic to show to the user.  Answer a form or a morph to serve that purpose.  A generic image is used for arbitrary objects, but note my reimplementors"
	| aCamera image thumb |
	aCamera _ WonderlandStillCamera createFor: myWonderland.
	aCamera turnBackgroundOff.
	aCamera setFocusObject: self.
	thumb _ aCamera getMorph imageForm.
	aCamera getMorph delete.
	aCamera removeFromScene. "silently"
	thumb _ thumb trimBordersOfColor: Color transparent.
	image _ Form extent: (thumb extent x max: thumb extent y) asPoint depth: thumb depth.
	image fillColor: myWonderland getScene getColorObject asColor.
	thumb displayOn: image at: (image extent - thumb extent // 2) rule: Form paint.
	^image! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/4/1999 11:53'!
alignWith: anObject
	"This method sets the orientation of this instance to be the same as that of the specified object."

	^ self turnTo: anObject duration: 1.0 style: gently.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:18'!
move: aDirection speed: aSpeed
	"Creates and returns an action that moves the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to move ' , myName , ' at because ', msg.
			^ nil ].

	"The parameters check out, so go ahead and start the action"
	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveRightNow: aDirection
							distance: ((myWonderland getScheduler getElapsedTime) * aSpeed)
							undoable: false ].
! !

!WonderlandActor class methodsFor: 'unique name creation' stamp: 'tk 12/5/2001 14:51'!
baseUniclass
	"Answer the uniclass that new instances should be instances of; this protocol allows for individual cards of a background to have their own class"

	| curr |
	curr _ self.
	[curr theNonMetaClass superclass name endsWithDigit]
		whileTrue:
			[curr _ curr superclass].
	^ curr

"CardPlayer100 baseUniclass 
CardPlayer100X baseUniclass
"! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:07'!
resizeTopToBottom: height leftToRight: width frontToBack: thickness eachFrameFor: aLifetime
	"Resize the object by the specified amount each frame for the specified lifetime."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyNumber: height ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from top to bottom because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: width ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from left to right because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: thickness ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from front to back because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should resize ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoSizeChange for: self from: (self getSize)).

	^ self doEachFrame: [ self resizeRightNow: (B3DVector3 x: width y: height z: thickness)
									undoable: false ]
			for: aLifetime.
! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/19/1999 14:01'!
show
	"Tell the object that it should draw itself each frame"

	(myWonderland getUndoStack) push: (UndoShowHide undoShowFor: self).

	self setHidden: false.
! !

!WonderlandActor methodsFor: 'morphic reactions' stamp: 'ar 10/10/2000 21:33'!
morphicKeyPress: event
	"Handle the given event"
	myTexture isMorph ifTrue:[self dispatchEvent: event to: myTexture].! !

!WonderlandActor methodsFor: 'set property' stamp: 'ar 4/24/1999 05:25'!
setMaterial: aMaterial
	myMaterial _ aMaterial! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 4/5/1999 14:33'!
destroy
	"Destroy the object over 1 second"

	self destroy: 1.0.
! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/3/1999 17:45'!
setPointOfView: aVector asSeenBy: reference
	"Sets the object's position and orientation"

	^ self setPointOfView: aVector duration: 1.0 asSeenBy: reference style: gently.

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:58'!
turn: aDirection
	"Turn the actor once around in the specified direction, taking 1 second and using the Gently animation style."

	^ (self turn: aDirection turns: 0.25 duration: 1.0 style: gently).

! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:39'!
height
	^self getBoundingBox extent y * 100! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 4/24/1999 05:25'!
getMaterial
	^myMaterial! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:48'!
roll: aDirection turns: numTurns asSeenBy: reference
	"Roll the actor the specified number of turns in the specified direction, taking 1 second and using the Gently animation style."

	^ (self roll: aDirection turns: numTurns duration: 1.0 asSeenBy: reference
			style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 17:30'!
place: aLocation object: aTarget eachFrameFor: aLifetime 
	"Moves this object to one of several pre-determined locations relative to the target object each frame for the specified lifetime."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyLocation: aLocation ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyActor: aTarget ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine for how long it should place ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self placeRightNow: aLocation object: aTarget undoable: false ]
			for: aLifetime.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/24/1999 14:34'!
getSizeVector
	"Returns the object's current size as a vector"

	^ (B3DVector3 x: (scaleMatrix a11) y: (scaleMatrix a22) z: (scaleMatrix a33)).
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 11:01'!
turn: aDirection turns: numTurns
	"Turn the actor the specified number of turns in the specified direction, taking one second and using the Gently animation style."

	^ (self turn: aDirection turns: numTurns duration: 1.0 style: gently).

! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 2/11/1999 10:12'!
isHidden
	"Returns true if the object is hidden."

	^ hidden.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:59'!
turn: aDirection turns: numTurns duration: aDuration asSeenBy: reference
	"Turn the actor the specified distance in the specified direction, taking the specified duration and using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyDirection: aDirection ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: numTurns ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how many turns to turn ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyReferenceFrame: reference ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
				^ nil ].

		self turnRightNow: aDirection numberOfTurns: numTurns
								asSeenBy: reference undoable: true.
		^ self. ].

	^ (self turn: aDirection turns: numTurns duration: aDuration asSeenBy: reference
			style: gently).

! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 15:46'!
setPointOfView: aVector duration: aDuration asSeenBy: reference style: aStyle
	"Sets the object's position and orientation"

	| positionVector rotationVector |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyPointOfView: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what point of view to assign ' , myName , ' because ', msg.
			^ nil ].

	( aVector isKindOf: WonderlandActor) ifTrue: [
			myWonderland reportErrorToUser: 'Squeak does not allow you to use an Actor to specify the point of view when using asSeenBy.'.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for changing the point of view for ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not set the point of view for ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use for changing the point of view for ' , myName , ' because ', msg.
			^ nil ].

	positionVector _ Array new: 3.
	positionVector at: 1 put: (aVector at: 1).
	positionVector at: 2 put: (aVector at: 2).
	positionVector at: 3 put: (aVector at: 3).

	rotationVector _ Array new: 3.
	rotationVector at: 1 put: (aVector at: 4).
	rotationVector at: 2 put: (aVector at: 5).
	rotationVector at: 3 put: (aVector at: 6).

	^ myWonderland doTogether: { self moveTo: positionVector duration: aDuration
												asSeenBy: reference style: aStyle.
								  self turnTo: rotationVector duration: aDuration
												asSeenBy: reference style: aStyle }.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:51'!
wait: time
	"Creates an animation where the object merely waits for the specified amount of time"

	| anim |

	[ WonderlandVerifier VerifyDuration: time ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine how long ' , myName , ' should wait because ', msg.
			^ nil ].

	anim _ RelativeAnimation new.

	anim object: self
			update: [:temp | temp ]
			getStartState: [ Interpolateable value: 0 ]
			getEndState: [ time ]
			getReverseState: [ time ]
			style: [:elapsed :total | WonderlandStyle abruptlyAfter: elapsed outOf: total]
			duration: time
			undoable: true
			inWonderland: myWonderland.

	^ anim.! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:39'!
getX
	^(self getPointOfView at: 1) * 100! !

!WonderlandActor methodsFor: 'animated state' stamp: 'jsp 3/15/1999 16:15'!
stop
	"Stop any actions and animations associated with this object"

	| scheduler |

	scheduler _ myWonderland getScheduler.

	(scheduler getAnimationsFor: self) do: [:anim | anim stop ].

	(scheduler getActionsFor: self) do: [:action | action stop ].

	myChildren do: [:child | (child isPart) ifTrue: [ child stop ] ].! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 3/18/1999 16:57'!
getPointOfView: reference
	"Returns the object's position and orientation"

	| newComposite anArray a3DVector |

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can determine the point of view of ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	newComposite _ reference getMatrixToRoot.
	newComposite _ newComposite composeWith: (self getMatrixFromRoot).

	a3DVector _ newComposite translation.

	anArray _ Array new: 6.
	anArray at: 1 put: (a3DVector x).
	anArray at: 2 put: (a3DVector y).
	anArray at: 3 put: (a3DVector z).

	a3DVector _ newComposite rotation.
	anArray at: 4 put: (a3DVector x).
	anArray at: 5 put: (a3DVector y).
	anArray at: 6 put: (a3DVector z).

	^ anArray.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:11'!
turn: aDirection speed: aSpeed asSeenBy: reference until: aCondition
	"Creates and returns an action that turn the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to turn ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should turn ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self turnRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				asSeenBy: reference
				undoable: false
						]
		   until: aCondition.

! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/19/1999 13:55'!
setHidden: aBoolean
	"Set the object's hidden status (including its parts)"

	hidden _ aBoolean.

	"Set the show/hide status of our parts"
	myChildren do: [:child | (child isPart) ifTrue: [child setHidden: aBoolean] ].
! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 2/9/1999 10:10'!
isPart
	"Returns true if the object is a Part."

	^ (firstClass not).
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:16'!
turnTo: aVector duration: aDuration style: aStyle
	"Turns the object to the specified orientation in its parent's coordinate system using the specified style over the specified duration."

	| anim endStateFunc m angles |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to turn ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the duration to use for turning ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to turn ' , myName , ' because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ AbsoluteAnimation new.

	m _ B3DMatrix4x4 identity.

	(aVector isKindOf: WonderlandActor)
		ifTrue: [ endStateFunc _ [
						m setIdentity.
						angles _ aVector getAngles: myParent.
						m rotation: (B3DVector3 x: (angles at: 1) y: (angles at: 2) z: (angles at: 3)).
						m asQuaternion.
								 ]
				]
		ifFalse: [ endStateFunc _ [
						m setIdentity.
						angles _ self getRotationVector.
						((aVector at: 1) = asIs) ifFalse: [ angles x: (aVector at: 1) ].
						((aVector at: 2) = asIs) ifFalse: [ angles y: (aVector at: 2) ].
						((aVector at: 3) = asIs) ifFalse: [ angles z: (aVector at: 3) ].
						m rotation: angles.
						m asQuaternion.
								 ]
				].


	anim object: self
			update: [:tQuat | self setRotationMatrix: (tQuat asMatrix4x4) ]
			getStartState: [ composite asQuaternion ]
			getEndState: endStateFunc
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/29/1999 14:44'!
getPointAtMatrix: target
	"create a matrix such that the actor looks at the target using the upDirection"
	| aVector m1 m2 |

	m1 _ B3DMatrix4x4 identity.
	m2 _ B3DMatrix4x4 identity.

	"Calculate the vector to look along"
	(target isKindOf: WonderlandActor)
		ifTrue: [ aVector _ (target getMatrixFromRoot translation) -
								(self getMatrixFromRoot translation) ]
		ifFalse: [ (target isKindOf: Point)
					ifTrue: [
							aVector _ ((myWonderland getDefaultCamera)
								transformScreenPointToScenePoint: target atDepthOf: self).
							aVector _ B3DVector3 x: (aVector at: 1) y: (aVector at: 2)
												z: (aVector at: 3).
							aVector _ aVector - (self getMatrixFromRoot translation).
							]
					ifFalse: [
							aVector _ (B3DVector3 x: (target at: 1) y: (target at: 2)
									z: (target at: 3)) - (self getMatrixFromRoot translation).
							].
				].


	"Calculate the rotation around the z-axis"
	((aVector z) = 0) ifTrue: [ ((aVector x ) > 0) ifTrue: [ m1 rotationAroundY: 90 ]
												ifFalse: [ ((aVector x) = 0) 
													ifTrue: [ m1 rotationAroundY: 0 ]
													ifFalse: [ m1 rotationAroundY: -90 ] ]
							]
					ifFalse: [ ((aVector z) > 0)
									ifTrue: [ m1 rotationAroundY:
										(((aVector x) / (aVector z)) arcTan radiansToDegrees) ]
									ifFalse: [ m1 rotationAroundY: (180 +
										(((aVector x) / (aVector z)) arcTan radiansToDegrees)) ]
							].

	((aVector length) = 0) ifFalse: [
		m2 rotationAroundX: (((aVector y) / (aVector length)) arcSin radiansToDegrees negated) ].

	^ (myParent getMatrixToRoot) composeWith: (m1 composeWith: m2).
! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 4/19/1999 15:18'!
getPositionInPicturePlane: aCamera
	"Returns the actor's position in the picture plane of the given camera"

	| d x y z pos |

	pos _ self getPosition: aCamera.
	d _ (aCamera getFrustum) near.

	z _ (pos at: 3) abs.

	(z = 0) ifTrue: 	[
						x _ 0.
						y _ 0.
					]
			ifFalse: [
						x _ (d / z) * (pos at: 1).
						y _ (d / z) * (pos at: 2).
					].
					
	^ x @ y.
! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 15:46'!
setPointOfView: aVector asSeenBy: reference eachFrameFor: aLifetime
	"Sets the object's position and orientation"

	| positionVector rotationVector |

	[ WonderlandVerifier VerifyPointOfView: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what point of view to assign ' , myName , ' because ', msg.
			^ nil ].

	( aVector isKindOf: WonderlandActor) ifTrue: [
			myWonderland reportErrorToUser: 'Squeak does not allow you to use an Actor to specify the point of view when using asSeenBy.'.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not set the point of view for ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long ' , myName , ' should keep this point of view because ', msg.
			^ nil ].

	positionVector _ Array new: 3.
	positionVector at: 1 put: (aVector at: 1).
	positionVector at: 2 put: (aVector at: 2).
	positionVector at: 3 put: (aVector at: 3).

	rotationVector _ Array new: 3.
	rotationVector at: 1 put: (aVector at: 4).
	rotationVector at: 2 put: (aVector at: 5).
	rotationVector at: 3 put: (aVector at: 6).

	myWonderland getUndoStack push: (UndoPOVChange for: self
														from: (self getPointOfView)).

	^ self doEachFrame: [ 
							self moveToRightNow: positionVector undoable: false.
							self turnToRightNow: rotationVector undoable: false.
						]
			for: aLifetime.

! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 3/18/1999 16:58'!
getPosition: reference
	"Returns the object's position relative to a reference object"

	| newComposite anArray a3DVector |

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not determine the position of ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	newComposite _ reference getMatrixToRoot.
	a3DVector _ (newComposite composeWith: (self getMatrixFromRoot)) translation.

	anArray _ Array new: 3.
	anArray at: 1 put: (a3DVector x).
	anArray at: 2 put: (a3DVector y).
	anArray at: 3 put: (a3DVector z).

	^ anArray.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:12'!
turnTo: aVector
	"Turns the object to the specified orientation in its parent's coordinate system over one second using the Gently animation style."

	^ (self turnTo: aVector duration: 1.0
			style: gently).
! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/3/1999 17:44'!
setPointOfView: aVector
	"Sets the object's position and orientation"

	^ self setPointOfView: aVector duration: 1.0 style: gently.
! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:45'!
setHeight: value
	self resize: {1.0. value / self height asFloat. 1.0} duration: rightNow.
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 5/10/2001 20:22'!
postCopy
	| props |
	myName _ myName copy.
	myMaterial _ myMaterial copy.
	myColor _ myColor copy.
	myReactions _ myReactions copy.
	myMesh _ myMesh shallowCopy.
	composite _ composite copy.
	scaleMatrix _ scaleMatrix copy.
	props _ myProperties.
	myProperties _ nil.
	props = nil ifFalse:[
		props keysAndValuesDo:[:k :v| self setProperty: k toValue: v]].! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 2/24/1999 12:45'!
getPointOfView
	"Returns the object's position and orientation"

	| anArray a3DVector |

	a3DVector _ composite translation.

	anArray _ Array new: 6.
	anArray at: 1 put: (a3DVector x).
	anArray at: 2 put: (a3DVector y).
	anArray at: 3 put: (a3DVector z).

	a3DVector _ composite rotation.
	anArray at: 4 put: (a3DVector x).
	anArray at: 5 put: (a3DVector y).
	anArray at: 6 put: (a3DVector z).

	^ anArray.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:50'!
roll: aDirection asSeenBy: reference
	"Roll the actor 1/4 turn in the specified direction, taking 1 second and using the Gently animation style."

	^ (self roll: aDirection turns: 0.25 duration: 1.0 asSeenBy: reference
			style: gently).

! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 16:02'!
placeRightNow: aLocation object: aTarget undoable: isUndoable
	"Instantaneously move the object to the specified place."

	 | selfPos selfBBox selfLBB selfRUF objBBox objLBB objRUF pos |

	selfBBox _ self getBoundingBox: aTarget.
	selfLBB _ selfBBox origin.
	selfRUF _ selfBBox corner.

	objBBox _ aTarget getBoundingBox.
	objLBB _ objBBox origin.
	objRUF _ objBBox corner.

	selfPos _ self getPosition: aTarget.

	((aLocation = onTopOf) or: [ aLocation = above])
	ifTrue: [
				pos _ { 0. (objRUF y) + ((selfPos at: 2) - (selfLBB y)). 0}
			]
 	ifFalse: [

	((aLocation = onBottomOf) or: [ (aLocation = beneath) or: [ aLocation = below ] ])
	ifTrue: [
				pos _ { 0. (objLBB y) - ((selfRUF y) - (selfPos at: 2)). 0}
			]
	ifFalse: [

	(aLocation = toLeftOf)
	ifTrue: [
				pos _ { (objLBB x) - ((selfRUF x) - (selfPos at: 1)). 0. 0 }
			]
	ifFalse: [

	(aLocation = toRightOf)
	ifTrue: [
				pos _ { (objRUF x) + ((selfPos at: 1) - (selfLBB x)). 0. 0 }
			]
	ifFalse: [

	(aLocation = inFrontOf)
	ifTrue: [
				pos _ { 0. 0. (objRUF z) + ((selfPos at: 3) - (selfLBB z)) }
			]
	ifFalse: [

	((aLocation = inBackOf) or: [ aLocation = behind])
	ifTrue: [
				pos _ { 0. 0. (objLBB z) - ((selfRUF z) - (selfPos at:3)) }
			]
	ifFalse: [

	(aLocation = onCeilingOf)
	ifTrue: [
				pos _ { 0. (objRUF y) - ((selfRUF y) - (selfPos at:2)). 0}
			]
	ifFalse: [

	(aLocation = onFloorOf)
	ifTrue: [
				pos _ { 0. (objLBB y) + ((selfPos at: 2) - (selfLBB y)). 0}
			]
	ifFalse: [ self error: 'Unrecognized target surface' ]]]]]]]].


	self moveToRightNow: pos asSeenBy: aTarget undoable: isUndoable.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:12'!
turnTo: aVector asSeenBy: reference
	"Turns the object to the specified orientation in its parent's coordinate system over one second using the Gently animation style."

	^ (self turnTo: aVector duration: 1.0 asSeenBy: reference
			style: gently).
! !

!WonderlandActor methodsFor: 'parent-child' stamp: 'jsp 4/7/1999 16:06'!
becomeChildOf: anObject
	"Update the undo stack and make this actor a child of the specified object."

	| oldParent |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyReferenceFrame: anObject ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can only make ' , myName , ' a child of other actors or the scene, and ', msg.
			^ nil ].

	(self == anObject) ifTrue: [
		myWonderland reportErrorToUser: 'Squeak can not make ' , myName , ' a child of itself.'.
		^ nil ].
		
	oldParent _ myParent.

	(myWonderland getUndoStack) push: (UndoAction new: [self reparentTo: oldParent]).

	self reparentTo: anObject.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/12/1999 16:38'!
setComposite: newComposite
	"Sets the actor's composite matrix"

	composite _ newComposite.
! !

!WonderlandActor class methodsFor: 'unique name creation' stamp: 'tk 5/7/1999 15:14'!
isSystemDefined
	^ name endsWithDigit not! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:57'!
moveTo: aVector eachFrameUntil: aCondition
	"Move the actor to a specified point each frame until the condition returns true."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to move ' , myName , ' to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine for how long ' , myName , ' should move to this position because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveToRightNow: aVector undoable: false ]
			until: aCondition.


! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 16:33'!
setColorRightNow: newColor undoable: isUndoable
	"Change this instance's color instantaneously"

	| aColor |

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoColorChange for: self from: (self getColor)) ].

	((newColor at: 1) isNumber)
				ifTrue: [ aColor _ B3DColor4 red: (newColor at: 1) green: (newColor at: 2)
										blue: (newColor at: 3) alpha: (myColor alpha) ]
				ifFalse: [ aColor _ Color perform: newColor.
						  aColor _ B3DColor4 red: (aColor red) green: (aColor green)
										blue: (aColor blue) alpha: (myColor alpha) ].

	self setColorVector: aColor.
! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 6/2/1999 15:31'!
getActiveTexture
	^myTexture isMorph
		ifTrue:[myTexture]
		ifFalse:[nil]! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:38'!
color: aColor
	self setColor: {aColor red. aColor green. aColor blue} duration: rightNow.! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 15:46'!
setPointOfView: aVector asSeenBy: reference eachFrameUntil: aCondition
	"Sets the object's position and orientation"

	| positionVector rotationVector |

	[ WonderlandVerifier VerifyPointOfView: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what point of view to assign ' , myName , ' because ', msg.
			^ nil ].

	( aVector isKindOf: WonderlandActor) ifTrue: [
			myWonderland reportErrorToUser: 'Squeak does not allow you to use an Actor to specify the point of view when using asSeenBy.'.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not set the point of view for ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine for how long ' , myName , ' should keep this point of view because ', msg.
			^ nil ].

	positionVector _ Array new: 3.
	positionVector at: 1 put: (aVector at: 1).
	positionVector at: 2 put: (aVector at: 2).
	positionVector at: 3 put: (aVector at: 3).

	rotationVector _ Array new: 3.
	rotationVector at: 1 put: (aVector at: 4).
	rotationVector at: 2 put: (aVector at: 5).
	rotationVector at: 3 put: (aVector at: 6).

	myWonderland getUndoStack push: (UndoPOVChange for: self
														from: (self getPointOfView)).

	^ self doEachFrame: [ 
							self moveToRightNow: positionVector undoable: false.
							self turnToRightNow: rotationVector undoable: false.
						]
			until: aCondition.

! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 3/17/1999 10:44'!
asString
	"Returns the name of the object."

	^ myName.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:50'!
roll: aDirection
	"Roll the actor a quarter turn in the specified direction, taking 1 second and using the Gently animation style."

	^ (self roll: aDirection turns: 0.25 duration: 1.0 style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/24/1999 16:46'!
resize: amount
	"Resize the object by the specified amount over 1 second using the Gently animation style."

	"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyNumberOrTriple: amount ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , '  because ', msg.
				^ nil ].

	(amount isNumber)
		ifTrue: [ ^ (self resizeTopToBottom: amount leftToRight: amount frontToBack: amount
						duration: 1.0
						style: gently). ]
		ifFalse: [  ^ (self resizeTopToBottom: (amount at: 2)
						leftToRight: (amount at: 1)
						frontToBack: (amount at: 3)
						duration: 1.0
						style: gently). ].


! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 4/22/1999 17:52'!
getPositionInPixels: camera
	"Returns the actor's position in pixels in the Squeak window"

	| morph point x y frustum |

	morph _ camera getMorph.

	point _ self getPositionInPicturePlane: camera.

	frustum _ camera getFrustum.

	((point x) < 0) ifTrue: [
			x _ (morph center x) - (((point x) / (frustum left)) * ((morph width) / 2))
						]
				ifFalse: [
			x _ (morph center x) + (((point x) / (frustum right)) * ((morph width) / 2))
						].

	((point y) < 0) ifTrue: [
			y _ (morph center y) + (((point y) / (frustum bottom)) * ((morph height) / 2))
						]
				ifFalse: [
			y _ (morph center y) - (((point y) / (frustum top)) * ((morph height) / 2))
						].

	^ (x @ y) rounded.
	! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/29/1999 14:39'!
pointAt: target duration: aDuration style: aStyle
	"Turns the object to point at the specified target using the specified style over the specified duration."

	| anim |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTargetOrPixel: target ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what to point ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for pointing ' , myName , ' at ' , (target asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to point ' , myName , ' because ', msg.
			^ nil ].

	"The parameters check out, so build the animation"
	anim _ AbsoluteAnimation new.

	anim object: self
			update: [:tQuat | self setRotationMatrix: (tQuat asMatrix4x4) ]
			getStartState: [ composite asQuaternion ]
			getEndState: [ (self getPointAtMatrix: target) asQuaternion ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 3/1/1999 11:50'!
getVisibility
	"Returns the actor's visibility"

	^ ((myMaterial ambientPart) alpha)
! !

!WonderlandActor methodsFor: 'copying' stamp: 'tk 9/4/2001 21:46'!
copyUniClassWith: deepCopier
	"my class is a subclass of WonderlandActor.  Return another class just like my class."
	| newCls |
	newCls _ self class officialClass 
		newUniqueClassInstVars: self class instanceVariablesString 
		classInstVars: self class class instanceVariablesString.
	newCls copyMethodDictionaryFrom: self class.
	newCls class copyMethodDictionaryFrom: self class class.
	newCls copyAddedStateFrom: self class.  "All class inst vars, if any"
	^ newCls
! !

!WonderlandActor methodsFor: 'get property' stamp: 'jsp 3/1/1999 13:26'!
getColor
	"Return the actor's color"

	| tmpArray  |
	tmpArray _ Array new:3.

	tmpArray at:1 put: (myColor red).
	tmpArray at:2 put: (myColor green).
	tmpArray at:3 put: (myColor blue).

	^ tmpArray.! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 4/23/1999 15:01'!
setColorVector: aColor
	"Instaneously sets the object's color/alpha vector and copies it down the object tree for all objects that are parts of this object"

	"Set our color"
	myColor _ aColor.
	myMaterial ambientPart: myColor.
	myMaterial diffusePart: myColor.
	myMaterial specularPart: myColor.

	"Set the color of our parts"
	myChildren do: [:child | (child isPart) ifTrue: [child setColorVector: aColor] ].
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 11/8/1999 01:07'!
adjustToTextureIfNecessary
	| texExtent texRatio myExtent myRatio |
	"Adjust the receiver to the texture extent if necessary"
	(self getProperty: #adjustToTexture) == true ifFalse:[^self]. "Don't adjust"
	texExtent _ self getTexturePointer extent.
	texRatio _ texExtent x / texExtent y.
	myExtent _ self getBoundingBox extent.
	myRatio _ myExtent x / myExtent y.
	self resize: (Array with: texRatio / myRatio with: 1.0 with: 1.0) duration: 2! !

!WonderlandActor methodsFor: 'event handling' stamp: 'jsp 4/19/1999 13:44'!
getReactionsTo: eventType
	"Return this actor's reactions to the given event type.  Because only first class actors react to mouse events, if this actor is a part pass this message up to its parent object."

	(self isFirstClass)
		ifTrue: [ (myReactions includesKey: eventType)
					ifTrue: [^ myReactions at: eventType ]
					ifFalse: [ ^ nil ].
				]
		ifFalse: [ ^ myParent getReactionsTo: eventType ].
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:24'!
move: aDirection distance: aDistance duration: aDuration
	"Move the actor the specified distance in the specified direction, taking the specified duration and using the Gently animation style."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyDirection: aDirection ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyNumber: aDistance ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine the distance to move ' , myName , ' because ', msg.
				^ nil ].

		self moveRightNow: aDirection distance: aDistance undoable: true.
		^ self. ].

	^ (self move: aDirection distance: aDistance duration: aDuration style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 11:10'!
turn: aDirection speed: aSpeed asSeenBy: reference for: aLifetime
	"Creates and returns an action that turn the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to turn ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should turn ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self turnRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				asSeenBy: reference
				undoable: false
						]
		   for: aLifetime.

! !

!WonderlandActor methodsFor: 'enumerating' stamp: 'ar 6/2/1999 14:04'!
allPartsDo: aBlock
	"Recursively evaluate aBlock with all elements of the receiver that are a part of its parent"
	self partsDo:[:child|
		aBlock value: child.
		child allPartsDo: aBlock]! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 3/8/1999 12:39'!
moveToRightNow: aVector undoable: isUndoable
	"Move this object in the specified direction instantaneously"

	| pos |

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoPositionChange for: self from: (self getPosition)).
						].

	(aVector isKindOf: WonderlandActor)
		ifTrue: [ pos _ aVector getPositionVector ]
		ifFalse: [ pos _ self getPositionVector.
				  ((aVector at: 1) = asIs) ifFalse: [pos x: (aVector at: 1) ].
				  ((aVector at: 2) = asIs) ifFalse: [pos y: (aVector at: 2) ].
				  ((aVector at: 3) = asIs) ifFalse: [pos z: (aVector at: 3) ].
				].

	self setPositionVector: pos.

! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 4/5/1999 15:10'!
getAllParts
	"Return all of this actor's parts"

	| parts |

	parts _ OrderedCollection new.

	myChildren do: [:child | (child isPart) ifTrue: [
						parts addLast: child.
						parts _ parts , (child getAllParts).
												].
					].

	^ parts.

	! !

!WonderlandActor class methodsFor: 'change logging' stamp: 'jsp 4/23/1999 11:50'!
wantsChangeSetLogging

	^ self == WonderlandActor.


! !

!WonderlandActor methodsFor: 'eToy-fake' stamp: 'ar 5/26/2001 16:11'!
viewerFlapTab
	"If a viewer in a flap exists for me, return it."

	(self currentWorld)
		submorphsDo: [:mm |
			(mm isKindOf: ViewerFlapTab)
				ifTrue:
					[mm scriptedPlayer == self
						ifTrue: [^mm]]].
	^nil! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 3/4/1999 15:35'!
becomeFirstClass
	"Make the actor a first class object."

	firstClass _ true.

	(myWonderland getUndoStack) push: (UndoAction new: [ firstClass _ false ]).
! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/19/1999 14:00'!
hide
	"Tell the object that it should not draw itself each frame"

	(myWonderland getUndoStack) push: (UndoShowHide undoHideFor: self).

	self setHidden: true.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 17:01'!
nudge: aDirection distance: aDistance duration: aDuration style: aStyle
	"Nudge the actor the specified object lengths in the specified direction, taking the specified duration and using the specified animation style."

	|  size newDistance |
	
	size _ self getSize.

	((aDirection = #left) or: [ aDirection = #right ])
		ifTrue: [ newDistance _ aDistance * (size at: 1) ]
		ifFalse: [

	((aDirection = #up) or: [ aDirection = #down ])
		ifTrue: [ newDistance _ aDistance * (size at: 2) ]
		ifFalse: [

	((aDirection = #forward) or: [ (aDirection = #back) or: [aDirection = #backward]])
		ifTrue: [ newDistance _ aDistance * (size at: 3) ]
		ifFalse: [ 	myWonderland reportErrorToUser: 'Squeak could not determine which direction to nudge ' , myName, ' because ' , (aDirection asString) , ' is not a valid direction.'.
					^ nil.]]].

	^ self move: aDirection distance: newDistance duration: aDuration style: aStyle.
! !

!WonderlandActor methodsFor: 'set property' stamp: 'bf 3/16/2000 19:00'!
setVisibility: newVisibility duration: time style: aStyle
	"Sets the current object's visibility"

	| anim |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier Verify0To1Number: newVisibility ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what visibility to make ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: time ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for changing the visibility of ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use for changing the visibility of ' , myName , ' because ', msg.
			^ nil ].

	anim _ AbsoluteAnimation new.
	anim object: self
			update: [:tColor | self setColorVector: tColor]
			getStartState: [self getColorVector]
			getEndState: [B3DColor4 red: (myColor red) green: (myColor green)
						blue: (myColor blue) alpha: newVisibility]
			style: aStyle
			duration: time
			undoable: true
			inWonderland: myWonderland.


	^ anim.
! !

!WonderlandActor methodsFor: 'accessing' stamp: 'jsp 3/18/1999 16:43'!
distanceTo: aVector
	"Returns the distance from the object to the specified point in the parent's coordinate system"

	| pos |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyTarget: aVector ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can only calculate the distance from ' , myName , ' to an actor or 3D point.'.
			^ nil ].

	(aVector isKindOf: WonderlandActor)
		ifTrue: [ pos _ aVector getPosition: self.
				 ^ ((pos at: 1) squared + (pos at: 2) squared + (pos at: 3) squared) sqrt ]
		ifFalse: [ pos _ self getPositionVector.
				  ((aVector at: 1) = asIs) ifTrue: [ aVector at: 1 put: (pos x) ].
				  ((aVector at: 2) = asIs) ifTrue: [ aVector at: 2 put: (pos y) ].
				  ((aVector at: 3) = asIs) ifTrue: [ aVector at: 3 put: (pos z) ].
				  ^ ((self getPositionVector) - aVector) length ].
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:51'!
roll: aDirection turns: numTurns speed: aSpeed asSeenBy: reference
	"Roll the actor the specified number of turns in the specified direction, taking the specified duration and using the specified animation style."

	| anim aDuration updateFunc degrees lastAmount buildMatrix newMatrix tMatrix|

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: numTurns ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the number of turns to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to roll ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not roll ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	anim _ RelativeAnimation new.
	aDuration _ (numTurns / aSpeed) abs.
	degrees _ 360 * numTurns.
	lastAmount _ 0.
	tMatrix _ B3DMatrix4x4 new.

	buildMatrix _ [ :rotMatrix | newMatrix _ reference getMatrixToRoot.
						newMatrix _ newMatrix composeWith: (self getMatrixFromRoot).
						newMatrix _ (((myParent getMatrixToRoot)
										composeWith: (reference getMatrixFromRoot))
										composeWith: rotMatrix)
										composeWith: newMatrix. ].

	(aDirection = #left) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #right) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated. ]

		ifFalse: [ myWonderland reportErrorToUser: 'Squeak does not know how to roll ' , myName , ' ', (aDirection asString) , '.'.
				^ nil ]].
	
	anim object: self
			update: updateFunc
			getStartState: [ lastAmount _ 0. (Interpolateable value: 0) ]
			getEndState: [ degrees ]
			getReverseState: [ degrees negated ]
			style: abruptly
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/17/1999 16:31'!
move: aDirection speed: aSpeed asSeenBy: reference until: aCondition
	"Creates and returns an action that moves the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to move ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak can not move ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyCondition: aCondition ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long ' , myName , ' should move because ', msg.
			^ nil ].

	"The parameters check out, so start the action"
	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [ self moveRightNow: aDirection
							distance: ((myWonderland getScheduler getElapsedTime) * aSpeed)
							asSeenBy: reference
							undoable: false ]
		   until: aCondition.

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 11:00'!
turn: aDirection turns: numTurns asSeenBy: reference
	"Turn the actor the specified number of turns in the specified direction, taking one second and using the Gently animation style."

	^ (self turn: aDirection turns: numTurns duration: 1.0 asSeenBy: reference
			style: gently).

! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 15:07'!
setSize: aVector duration: time
	"Sets the object's position and orientation"

	(time = rightNow) ifTrue: [
		[ WonderlandVerifier VerifyTriple: aVector ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine what size to make ' , myName , ' because ', msg.
				^ nil ].

		self setSizeRightNow: aVector undoable: true.
		^ self. ].

	^ (self setSize: aVector duration:time style: gently).

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/19/1999 10:56'!
turn: aDirection turns: numTurns speed: aSpeed asSeenBy: reference
	"Turn the actor the specified numberOfTurns in the specified direction, taking the specified duration and using the specified animation style."

	| anim aDuration updateFunc degrees lastAmount buildMatrix newMatrix tMatrix|

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: numTurns ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how many turns to turn ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to turn ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not turn ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	aDuration _ (numTurns / aSpeed) abs.
	anim _ RelativeAnimation new.
	degrees _ 360 * numTurns.
	lastAmount _ 0.
	tMatrix _ B3DMatrix4x4 new.

	buildMatrix _ [ :rotMatrix | newMatrix _ reference getMatrixToRoot.
						newMatrix _ newMatrix composeWith: (self getMatrixFromRoot).
						newMatrix _ (((myParent getMatrixToRoot)
										composeWith: (reference getMatrixFromRoot))
										composeWith: rotMatrix)
										composeWith: newMatrix. ].

	(aDirection = #right) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundY: (tAngle - lastAmount)).
							lastAmount _ tAngle] ]

		ifFalse: [(aDirection = #left) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundY: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated ]

		ifFalse: [(((aDirection = #up) or: [aDirection = #back]) or: [aDirection = #backward])
			ifTrue: [
				updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundX: (tAngle - lastAmount)).
							lastAmount _ tAngle].
				degrees _ degrees negated ]

		ifFalse: [((aDirection = #down) or: [aDirection = #forward]) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundX: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #ccw) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle]]

		ifFalse: [(aDirection = #cw) ifTrue: [
			updateFunc _ [:tAngle | tMatrix setIdentity. composite _ buildMatrix value: 
										(tMatrix rotationAroundZ: (tAngle - lastAmount)).
							lastAmount _ tAngle].
			degrees _ degrees negated. ]

		ifFalse: [ self error: 'Unrecognized direction' ]]]]]].
	
	anim object: self
			update: updateFunc
			getStartState: [ lastAmount _ 0. (Interpolateable value: 0) ]
			getEndState: [ degrees ]
			getReverseState: [ degrees negated ]
			style: abruptly
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'ar 8/31/2000 20:01'!
animateMeshTo: param duration: aDuration
	^ (self animateMeshTo: param duration: aDuration style: gently).
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 8/19/1999 11:40'!
hasMesh
	^myMesh notNil! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/18/1999 13:58'!
setColor: newColor duration: time style: aStyle
	"Set the actor's color"

	| anim aColor |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyColor: newColor ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what color to make ' , myName , ' because ', msg.
			^ nil ].


	[ WonderlandVerifier VerifyDuration: time ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for changing the color of ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use for changing the color of ' , myName , ' because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	((newColor at: 1) isNumber)
				ifTrue: [ aColor _ B3DColor4 red: (newColor at: 1) green: (newColor at: 2)
										blue: (newColor at: 3) alpha: (myColor alpha) ]
				ifFalse: [ aColor _ Color perform: newColor.
						  aColor _ B3DColor4 red: (aColor red) green: (aColor green)
										blue: (aColor blue) alpha: (myColor alpha) ].

	anim _ AbsoluteAnimation new.
	anim object: self
			update: [:tColor | self setColorVector: tColor]
			getStartState: [self getColorVector]
			getEndState: [ aColor ]
			style: aStyle
			duration: time
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/9/2001 22:39'!
assignGetter: getterSelector setter: setterSelector amt: amt
	self perform: setterSelector with: amt! !

!WonderlandActor methodsFor: 'copying' stamp: 'tk 9/4/2001 22:22'!
veryDeepFixupWith: deepCopier
	| old |
	"Any uniClass inst var may have been weakly copied.  If they were in the tree being copied, fix them up, otherwise point to the originals."

super veryDeepFixupWith: deepCopier.
myWonderland _ deepCopier references at: myWonderland ifAbsent: [myWonderland].
"myWonderland getNamespace at: myName put: self." 	"register me"
	"That does not register it, and the name structure is messed up by green-handle copy"

WonderlandActor instSize + 1 to: self class instSize do:
	[:ii | old _ self instVarAt: ii.
	self instVarAt: ii put: (deepCopier references at: old ifAbsent: [old])].
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:45'!
roll: aDirection speed: aSpeed asSeenBy: reference for: aLifetime
	"Creates and returns an action that rolls the object in the specified direction at the specified speed."

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to roll ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to roll ' , myName , ' at because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyReferenceFrame: reference ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak can not roll ' , myName , ' relative to ' , (reference asString) , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should roll ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

	^ self doEachFrame: [
				self rollRightNow: aDirection
				numberOfTurns: ((myWonderland getScheduler getElapsedTime) * aSpeed)
				asSeenBy: reference
				undoable: false
						]
		   for: aLifetime.

! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/1/1999 14:10'!
moveTo: aVector asSeenBy: reference
	"Moves the object to the specified position in its parent's coordinate system over 1 second using the Gently animation style."

	^ (self moveTo: aVector duration: 1.0 asSeenBy: reference
			style: gently).
! !

!WonderlandActor methodsFor: 'eToy-support' stamp: 'ar 5/16/2001 01:40'!
updateThresholdForGraphicInViewerTab
	"When a Viewer is open on the receiver, its tab needs some graphic to show to the user. Computing this graphic can take quite some time so we want to make the update frequency depending on how long it takes to compute the thumbnail. The threshold returned by this method defines that the viewer will update at most every 'threshold * timeItTakesToDraw' milliseconds. Thus, if the time for computing the receiver's thumbnail is 200 msecs and the the threshold is 10, the viewer will update at most every two seconds."
	^100 "update almost never; computing thumbnails of 3D objects is very expensive"! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/8/1999 15:15'!
standUpWithDuration: duration
	"Rotate the object to align it along its parent's up vector"

	^ self turnTo: #(0 #asIs 0) duration: duration.
! !

!WonderlandActor methodsFor: 'private' stamp: 'jsp 2/11/1999 08:57'!
scaleByMatrix: aMatrix
	"Scale the object by composing its scale matrix with the given matrix."

	scaleMatrix _ scaleMatrix composeWith: aMatrix.! !

!WonderlandActor methodsFor: 'get property' stamp: 'ar 11/7/1999 18:31'!
getBounds
	"Return the 2D bounds of the receiver as seen by the default camera"
	^self getBoundsFor: myWonderland getDefaultCamera.! !

!WonderlandActor methodsFor: 'set property' stamp: 'bf 3/16/2000 19:00'!
setVisibility: newVisibility duration: time
	"Sets the current object's visibility"

	(time = rightNow) ifTrue: [
		[ WonderlandVerifier Verify0To1Number: newVisibility ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine what visibility to make ' , myName , ' because ', msg.
				^ nil ].

		self setVisibilityRightNow: newVisibility undoable: true.
		^ self. ].

	^ (self setVisibility: newVisibility duration: time
			style: gently).

! !

!WonderlandActor methodsFor: 'eToy-basic' stamp: 'ar 5/8/2001 19:40'!
setZ: value
	^self setPointOfView: ((self getPointOfView) at: 3 put: value * 0.01; yourself) duration: rightNow! !

!WonderlandActor methodsFor: 'set property' stamp: 'jsp 3/12/1999 15:22'!
setName: newName
	"Sets the object's name"

	myName _ newName.
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/24/1999 16:53'!
resize: amount eachFrameFor: aLifetime
	"Resize the object by the specified amount each frame for the specified lifetime."

	| aVector |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyNumberOrTriple: amount ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , '  because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNonNegativeNumber: aLifetime ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how long it should resize ' , myName , ' because ', msg.
			^ nil ].

	myWonderland getUndoStack push: (UndoSizeChange for: self from: (self getSize)).

	(amount isNumber)
		ifTrue: [ aVector _ (B3DVector3 x: amount y: amount z: amount) ]
		ifFalse: [ aVector _ (B3DVector3 x: (amount at: 1) y: (amount at: 2)
									 z: (amount at: 3)) ].

	^ self doEachFrame: [ self resizeRightNow: aVector undoable: false ]
			for: aLifetime.


! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:03'!
resizeTopToBottom: height leftToRight: width frontToBack: thickness duration: aDuration style: aStyle
	"Resize the object by the specified amount over the specified duration."

	| anim tmpVector |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyNumber: height ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from top to bottom because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: width ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from left to right because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: thickness ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine how much to resize ' , myName , ' from front to back because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for resizing ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to resize ' , myName , ' because ', msg.
			^ nil ].

	"The parameters check out, so build the animation"
	anim _ RelativeAnimation new.
	tmpVector _ B3DVector3 x: 1.0 y: 1.0 z: 1.0.

	anim object: self
			update: [:tScale | self scaleByVector: (tScale / tmpVector).
							tmpVector _ tScale]
			getStartState: [ tmpVector x: 1.0; y: 1.0; z: 1.0.
									(B3DVector3 x: 1.0 y: 1.0 z: 1.0) ]
			getEndState: [ B3DVector3 x: width y: height z: thickness ]
			getReverseState: [ B3DVector3 x: (1 / width) y: (1 / height) z: (1 / thickness) ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandActor methodsFor: 'private' stamp: 'ar 6/2/1999 14:05'!
setTexturePointer: texture
	"Sets the object's texture"
	myTexture _ texture.
	self partsDo:[:part| part setTexturePointer: texture].
! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 09:48'!
place: aLocation object: aTarget duration: aDuration 
	"Moves this object to one of several pre-determined locations relative to the target object."

	(aDuration = rightNow) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyLocation: aLocation ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyActor: aTarget ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
				^ nil ].

		self placeRightNow: aLocation object: aTarget undoable: true.
		^ self. ].

	(aDuration = eachFrame) ifTrue: [
		"Check our arguments to make sure they're valid"
		[ WonderlandVerifier VerifyLocation: aLocation ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
				^ nil ].

		[ WonderlandVerifier VerifyActor: aTarget ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine where to place ' , myName , ' because ', msg.
				^ nil ].

		myWonderland getUndoStack push: (UndoPOVChange for: self from: (self getPointOfView)).

		^ self doEachFrame: [ self placeRightNow: aLocation object: aTarget undoable: false ].
									].

	 ^ self place: aLocation object: aTarget duration: aDuration style: gently.
! !

!WonderlandActor methodsFor: 'set property' stamp: 'ar 5/27/1999 16:49'!
setTextureFromUser
	"Set the texture from the user"
	| form |
	form _ Form fromUser.
	self setTexturePointer: form asTexture.! !

!WonderlandActor methodsFor: 'primitive behaviors' stamp: 'jsp 3/18/1999 10:18'!
move: aDirection distance: aDistance speed: aSpeed
	"Move the actor the specified distance in the specified direction, taking the specified duration and using the specified animation style."

	| anim a3DVector tmpVector tMatrix aDuration |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: aDistance ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the distance to move ' , myName , ' because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the speed to move ' , myName , ' at because ', msg.
			^ nil ].

	"The parameters check out, so build the animation"
	anim _ RelativeAnimation new.
	tmpVector _ (B3DVector3 x:0 y:0 z:0).
	tMatrix _ B3DMatrix4x4 identity.

	aDuration _ (aDistance / aSpeed) abs.

	[ a3DVector _ self makeVectorFromDistance: aDistance andDirection: aDirection. ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak does not know how to move ' , myName , ' in that direction.'.
			^ nil ].

	anim object: self
			update: [:tPos | composite _ composite composeWith:
												(tMatrix translation: (tPos - tmpVector)).
							tmpVector _ tPos]
			getStartState: [tmpVector x:0; y:0; z:0.
								(B3DVector3 x:0 y:0 z:0) ]
			getEndState: [ a3DVector ]
			getReverseState: [ a3DVector negated ]
			style: abruptly
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.

! !

!Hierarchical methodsFor: 'private' stamp: 'jsp 2/22/1999 12:10'!
removeChild: aChild
	"Remove an object from this instance's list of children"

	myChildren remove: aChild ifAbsent: [].
! !

!Hierarchical methodsFor: 'parent-child' stamp: 'jsp 2/22/1999 12:10'!
setParent: anObject
	"Set this instance's parent"

	myParent _ anObject.
! !

!Hierarchical methodsFor: 'private' stamp: 'jsp 6/29/1999 02:37'!
appendChildrenNamesTo: prefix
	"Return the object's children's names, each appended to the prefix."

	| nameList |

	nameList _ OrderedCollection new.
	myChildren do: [:child |
						(child appearsInChildLists) ifTrue: [
							nameList addLast: (prefix , (child getName)).
							nameList _ nameList , (child appendChildrenNamesTo: (prefix , '    '))].
					].
	^ nameList.
! !

!Hierarchical methodsFor: 'parent-child' stamp: 'jsp 2/22/1999 12:08'!
getParent
	"Return the object's parent."

	^ myParent.
! !

!Hierarchical methodsFor: 'initialization' stamp: 'ar 5/10/2001 17:17'!
uniqueNameFrom: aString
	"Return a unique name for the string in the context of the receiver's name space"
	| index aName myNamespace |
	aName _ aString.
	myNamespace _ self getChildren asSet collect:[:each| each getName].
	(myNamespace includes: aName)
			ifFalse: [ ^ aName ].
	aName endsWithDigit ifTrue:[
		index _ aName findLast:[:ch| ch isDigit].
		aName _ (aName copyFrom: 1 to: index-1).
	].
	index _ 1.
	[ myNamespace includes: (aName , (index asString)) ]
		whileTrue: [ index _ index + 1 ].
	^ aName , (index asString).
! !

!Hierarchical methodsFor: 'parent-child' stamp: 'jsp 2/22/1999 13:54'!
getChildren
	"Return the object's children."

	^ (myChildren copy).
! !

!Hierarchical methodsFor: 'parent-child' stamp: 'jsp 6/29/1999 02:43'!
getAllChildren
	"Return all of this instance's children"

	| children |

	children _ OrderedCollection new.

	myChildren do: [:child | 
		(child appearsInChildLists)
			ifTrue: [
						children addLast: child.
						children _ children , (child getAllChildren).
					].
	].

	^ children.! !

!Hierarchical methodsFor: 'parent-child' stamp: 'jsp 4/1/1999 17:13'!
getChildrenNames
	"Return the object's children."

	^ myChildren collect: [: child | child asString ].
! !

!Hierarchical methodsFor: 'private' stamp: 'jsp 6/29/1999 02:36'!
appearsInChildLists
	"Returns true if this object should appear in the actor browser"

	^ true.! !

!Hierarchical methodsFor: 'initialization' stamp: 'jsp 2/22/1999 12:08'!
initialize
	"Initialize this instance"

	myChildren _ OrderedCollection new.
! !

!Hierarchical methodsFor: 'parent-child' stamp: 'ar 9/17/1999 12:28'!
withAllChildrenDo: aBlock
	"Execute aBlock for all children of and the receiver itself"
	aBlock value: self.
	myChildren do:[:child|
		child withAllChildrenDo: aBlock.
	].! !

!Hierarchical methodsFor: 'parent-child' stamp: 'ar 9/17/1999 12:28'!
allChildrenDo: aBlock
	"Execute aBlock for all children of the receiver"
	myChildren do:[:child|
		aBlock value: child.
		child allChildrenDo: aBlock].! !

!Hierarchical class methodsFor: 'instance creation' stamp: 'jsp 2/22/1999 12:06'!
new
	"Create and initialize a new instance."

	^ super new initialize.
! !

!Hierarchical methodsFor: 'private' stamp: 'jsp 2/23/1999 16:31'!
addChild: aChild
	"Add an object to this instance's list of children"

	((myChildren identityIndexOf: aChild) = 0) ifTrue: [ myChildren addLast: aChild ].
! !

Hierarchical subclass: #WonderlandScene
	instanceVariableNames: 'myWonderland myColor'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Objects'!

!WonderlandScene commentStamp: '<historical>' prior: 0!
Every Wonderland contains an instance of the WonderlandScene.  The scene is at the top of the object hierarchy: all objects are (directly or indirectly) children of the scene.  The scene also defines the global coordinate system for the 3D world.
!

!WonderlandScene methodsFor: 'set property' stamp: 'jsp 3/22/1999 16:06'!
setColor: newColor duration: time style: aStyle
	"Set the actor's color"

	| anim aColor |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyColor: newColor ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine what color to make the scene because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: time ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for changing the color of the scene because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use for changing the color of the scene because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	((newColor at: 1) isNumber)
				ifTrue: [ aColor _ B3DColor4 red: (newColor at: 1) green: (newColor at: 2)
										blue: (newColor at: 3) alpha: 1.0 ]
				ifFalse: [ aColor _ Color perform: newColor.
						  aColor _ B3DColor4 red: (aColor red) green: (aColor green)
										blue: (aColor blue) alpha: 1.0 ].

	anim _ AbsoluteAnimation new.
	anim object: self
			update: [:tColor | self setColorVector: tColor]
			getStartState: [self getColorVector]
			getEndState: [ aColor ]
			style: aStyle
			duration: time
			undoable: true
			inWonderland: myWonderland.


	^ anim.
! !

!WonderlandScene methodsFor: 'private' stamp: 'jsp 2/22/1999 14:39'!
getMatrixFromRoot
	"Returns the identity matrix"

	^ (B3DMatrix4x4 identity).
! !

!WonderlandScene methodsFor: 'private' stamp: 'jsp 2/23/1999 15:36'!
setColorVector: aColor
	"Set the scene's background color instantaneously"

	myColor _ aColor.
! !

!WonderlandScene methodsFor: 'private' stamp: 'jsp 3/22/1999 16:05'!
setColorRightNow: newColor undoable: isUndoable
	"Change this instance's color instantaneously"

	| aColor |

	(isUndoable) ifTrue: [
		myWonderland getUndoStack push: (UndoColorChange for: self from: (self getColor)) ].

	((newColor at: 1) isNumber)
				ifTrue: [ aColor _ B3DColor4 red: (newColor at: 1) green: (newColor at: 2)
										blue: (newColor at: 3) alpha: (1.0) ]
				ifFalse: [ aColor _ Color perform: newColor.
						  aColor _ B3DColor4 red: (aColor red) green: (aColor green)
										blue: (aColor blue) alpha: (1.0) ].

	self setColorVector: aColor.
! !

!WonderlandScene methodsFor: 'set property' stamp: 'jsp 3/1/1999 14:15'!
setColor: newColor
	"Set the scene's background color"

	^ (self setColor: newColor duration: 1.0
			style: gently)

! !

!WonderlandScene methodsFor: 'private' stamp: 'jsp 2/22/1999 14:39'!
getMatrixToRoot
	"Returns the identity matrix"

	^ (B3DMatrix4x4 identity).
! !

!WonderlandScene methodsFor: 'get property' stamp: 'jsp 3/22/1999 10:55'!
getColor
	"Return the scene's background color"

	| tmpArray |

	myColor ifNil: [ ^ nil ].

	tmpArray _ Array new:3.
	
	tmpArray at:1 put: (myColor red).
	tmpArray at:2 put: (myColor green).
	tmpArray at:3 put: (myColor blue).

	^ tmpArray.! !

!WonderlandScene methodsFor: 'accessing' stamp: 'bf 5/9/2001 18:05'!
loadActor
	^ myWonderland loadActor! !

!WonderlandScene methodsFor: 'private' stamp: 'jsp 3/22/1999 10:16'!
getColorObject
	"Returns the scene's background color object"

	^ myColor.

! !

!WonderlandScene methodsFor: 'drawing' stamp: 'ar 4/18/1999 00:27'!
renderOn: aRenderer
	"Tell the Scene's children to draw themselves"
	"Note: For correct picking, it is important to use B3DRenderEngine>>render: here."
	myChildren do: [:child | aRenderer render: child ].
! !

!WonderlandScene methodsFor: 'accessing' stamp: 'jsp 4/1/1999 17:26'!
asString
	"Return a string name for the scene"

	^ 'scene'.

! !

!WonderlandScene methodsFor: 'initialization' stamp: 'jsp 3/15/1999 15:27'!
initializeFor: aWonderland
	"Initialize the Wonderland Scene."

	super initialize.

	"Set our properties"
	myWonderland _ aWonderland.
	myColor _ B3DColor4 r: 0.4 g: 0.6 b: 0.85 a: 1.
! !

!WonderlandScene methodsFor: 'event-handling' stamp: 'jsp 4/19/1999 13:43'!
getReactionsTo: evtType

	^ nil.
! !

!WonderlandScene methodsFor: 'set property' stamp: 'jsp 3/22/1999 16:06'!
setColor: newColor duration: time
	"Set the scene's background color"

	(time = rightNow) ifTrue: [
		[ WonderlandVerifier VerifyColor: newColor ]
			ifError: [ :msg :rcvr |
				myWonderland reportErrorToUser: 'Squeak could not determine what color to make the scene because ', msg.
				^ nil ].
	
		^ self setColorRightNow: newColor undoable: true ].

	^ (self setColor: newColor duration:time style: gently)

! !

!WonderlandScene methodsFor: 'accessing' stamp: 'jsp 4/1/1999 17:26'!
getName
	"Return the scene's name"

	^ 'scene'.
! !

!WonderlandScene class methodsFor: 'instance creation' stamp: 'jsp 3/10/1999 16:10'!
newFor: aWonderland
	"Initialize the new instance for a Wonderland before returning it."

	^ super new initializeFor: aWonderland.
! !

!WonderlandScene methodsFor: 'private' stamp: 'jsp 2/23/1999 15:40'!
getColorVector
	"Return the scene's background color"

	^ myColor.

! !

Object subclass: #UndoChangeList
	instanceVariableNames: 'changeList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoChangeList commentStamp: '<historical>' prior: 0!
This class wraps a list of multiple undo actions, and tells them to undo all at the same time.
!

!UndoChangeList methodsFor: 'initialization' stamp: 'jsp 4/5/1999 15:18'!
initialize
	"Initialize this method by creating an empty change list."

	changeList _ OrderedCollection new.
! !

!UndoChangeList methodsFor: 'undoing' stamp: 'jsp 4/5/1999 15:20'!
undoIt
	"Undo by undoing all the changes in the change list"

	changeList do: [:undoable | undoable undoIt ].
! !

!UndoChangeList methodsFor: 'adding changes' stamp: 'jsp 4/5/1999 15:19'!
add: someUndo
	"Add an undoable to the change list"

	changeList addLast: someUndo.
! !

!UndoChangeList methodsFor: 'initialization' stamp: 'jsp 4/5/1999 15:20'!
setChangeList: aCollection

	changeList _ aCollection.
! !

!UndoChangeList class methodsFor: 'instance creation' stamp: 'jsp 4/5/1999 15:18'!
new

	^ super new initialize.
! !

Object subclass: #UndoAnimation
	instanceVariableNames: 'wrappedAnimation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoAnimation commentStamp: '<historical>' prior: 0!
Undo an Animation.  Currently Wonderlands undo animations by running them in reverse with a compressed duration.
!

!UndoAnimation class methodsFor: 'instance creation' stamp: 'jsp 2/16/1999 12:46'!
new: anAnimation
	"Create a wrapper for undoing an animation"

	| newUndo |

	newUndo _ UndoAnimation new.
	newUndo setAnimation: anAnimation.
	^ newUndo.! !

!UndoAnimation methodsFor: 'accessing' stamp: 'jsp 2/16/1999 12:45'!
setAnimation: anAnimation
	"Set wrapped animation."

	wrappedAnimation _ anAnimation.
! !

!UndoAnimation methodsFor: 'undoing' stamp: 'jsp 2/16/1999 12:45'!
undoIt
	"Undo by running the wrapped animation."

	wrappedAnimation start.
! !

Object subclass: #AbstractAnimation
	instanceVariableNames: 'startTime endTime duration state direction loopCount undoable myScheduler myWonderland pausedInterval animatedObject'
	classVariableNames: 'Forward Reverse Infinity Finished Stopped Waiting Running Paused'
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!AbstractAnimation commentStamp: '<historical>' prior: 0!
This class implements the basic functionality of Animations for Wonderlands. All animations pass through 4 specific stages:

Waiting - this is the state animations are in when they are just started, before they run their prologue (perform any tasks they need to do before the animation actually starts)

Running - this is the state animations are in when they are actually running

Stopped - this is the state animations are in after they stop running but before they execute their prologue

Finished - this is the state animations are in after they finish their epilogue (perform any tasks they need to do after the animation completes).
!

!AbstractAnimation methodsFor: 'accessing' stamp: 'jsp 2/26/1999 12:01'!
isLooping
	"Returns true if the animation is looping"

	^ ( loopCount > 1) or: [ loopCount = Infinity ].
! !

!AbstractAnimation methodsFor: 'accessing' stamp: 'jsp 2/26/1999 12:42'!
setLoopCount: count
	"Sets the animation's current loop count"

	loopCount _ count.

! !

!AbstractAnimation methodsFor: 'accessing' stamp: 'jsp 2/26/1999 12:41'!
getLoopCount
	"Returns the animation's current loop count"

	^ loopCount.
! !

!AbstractAnimation methodsFor: 'private' stamp: 'jsp 2/26/1999 14:21'!
scaleDuration: scaleAmount
	"Scales the animation's duration by the specified amount"

	duration _ duration * scaleAmount.
! !

!AbstractAnimation class methodsFor: 'class initialization' stamp: 'jsp 3/24/1999 11:01'!
initialize
	"Initialize the class variables"

	Waiting _ 1.
	Running _ 2.
	Paused _ 3.
	Finished _ 4.
	Stopped _ 5.

	Forward _ 0.
	Reverse _ 1.

	Infinity _ -1.
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 3/3/1999 12:06'!
epilogue: currentTime
	"This method does any work that needs to be done after an interation of the animation finishes."

	(loopCount = Infinity) ifTrue:
				[state _ Waiting]
	ifFalse:
				[
					loopCount _ loopCount - 1.

					(loopCount > 0) ifTrue: [ state _ Waiting ]
									ifFalse: [state _ Stopped.
											 loopCount _ 1 ].
				].

! !

!AbstractAnimation methodsFor: 'accessing' stamp: 'jsp 2/3/1999 14:23'!
isDone
	"Returns true if the animation is running"

	^ (state = Stopped).
! !

AbstractAnimation subclass: #CompositeAnimation
	instanceVariableNames: 'children childLoopCounts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!CompositeAnimation commentStamp: '<historical>' prior: 0!
This class extends the basic Animation class for composite animations (animations built of other animations). It adds an instance variable for the component child animations, and a method for adding them to this collection.
!

!CompositeAnimation methodsFor: 'reversing' stamp: 'jsp 2/26/1999 14:41'!
reverseDirection
	"Changes the direction an animation runs in (forward or in reverse)"

	super reverseDirection.

	children do: [:child | child reverseDirection ].
! !

CompositeAnimation subclass: #ParallelAnimation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!ParallelAnimation commentStamp: '<historical>' prior: 0!
This class extends CompositeAnimations for composite animations (an animation built of 2 or more animations) that all run in parallel.
!

!ParallelAnimation methodsFor: 'management' stamp: 'jsp 2/26/1999 15:28'!
prologue: currentTime
	"This method performs any actions that need to be done before the parallel animation starts."

	super prologue: currentTime.

	undoable ifTrue: [
						(myWonderland getUndoStack)
								push: (UndoAnimation new: (self makeUndoVersion)).
					].

	children do: [:child | child prologue: currentTime. ].

	(direction = Reverse) ifTrue: [ children do: [:child | child pause ] ].
! !

!ParallelAnimation methodsFor: 'initialization' stamp: 'jsp 2/26/1999 15:36'!
children: childCollection undoable: canUndo inWonderland: aWonderland
	"This method initializes the Animation with all the information that it needs to run."

	| childDuration |
	undoable _ canUndo.
	myWonderland _ aWonderland.
	myScheduler _ aWonderland getScheduler.
	loopCount _ 1.
	direction _ Forward.

	super initialize.

	"Add children"
	childCollection do: [:child | self append: child. child setUndoable: false ].

	"Calculate duration"
	duration _ 0.
	children do: [:child | childDuration _ ((child getDuration) * (child getLoopCount)).
							(childDuration > duration) ifTrue: [ duration _ childDuration. ]. ].

	state _ Waiting.
	myScheduler addAnimation: self.! !

!ParallelAnimation methodsFor: 'copying' stamp: 'jsp 4/9/1999 14:21'!
reversed
	"Creates a reversed version of an animation"

	| anim copyOfChildren copyOfChild i |

	anim _ ParallelAnimation new.

	i _ 1.
	copyOfChildren _ OrderedCollection new.
	children do: [:child | copyOfChild _ child copy.
						 copyOfChild setLoopCount: (childLoopCounts at: i).
						 copyOfChildren addLast: copyOfChild.
						 i _ i + 1 ].

	anim children: copyOfChildren
		undoable: true
		inWonderland: myWonderland.

	(direction = Reverse) ifTrue: [ anim setDirection: Reverse ].
	anim reverseDirection.

	anim stop.

	^ anim.
! !

!ParallelAnimation methodsFor: 'copying' stamp: 'jsp 2/26/1999 14:57'!
makeUndoVersion
	"Creates the undo version of an animation"

	| anim copyOfChildren copyOfChild i |

	anim _ ParallelAnimation new.

	i _ 1.
	copyOfChildren _ OrderedCollection new.
	children do: [:child | copyOfChild _ child copy.
						 copyOfChild setLoopCount: (childLoopCounts at: i).
						 copyOfChildren addLast: copyOfChild.
						 i _ i + 1 ].

	anim children: copyOfChildren
		undoable: false
		inWonderland: myWonderland.

	(direction = Reverse) ifTrue: [ anim setDirection: Reverse ].
	anim reverseDirection.

	"Undoing a parallel animation takes 1 second"	
	anim scaleDuration: (1 / duration).
	anim stop.

	^ anim.
! !

!ParallelAnimation methodsFor: 'copying' stamp: 'jsp 3/3/1999 12:31'!
copy
	"Creates the undo version of an animation"

	| anim copyOfChildren copyOfChild i |

	anim _ ParallelAnimation new.

	i _ 1.
	copyOfChildren _ OrderedCollection new.
	children do: [:child | copyOfChild _ child copy.
						 copyOfChild setLoopCount: (childLoopCounts at: i).
						 copyOfChildren addLast: copyOfChild.
						 i _ i + 1 ].

	anim children: copyOfChildren
		undoable: false
		inWonderland: myWonderland.

	(direction = Reverse) ifTrue: [ anim setDirection: Reverse ].

	^ anim.
! !

!ParallelAnimation methodsFor: 'management' stamp: 'jsp 2/26/1999 15:33'!
update: currentTime
	"This method updates the parallel animation based on the current time, which mainly consists of updating all the component animations."

	| index |

	(state = Waiting) ifTrue: [self prologue: currentTime].

	(state = Running) ifTrue: [
			state _ Finished.

			"Update every child.  If any one of them is not Stopped set our state back to Running"

			index _ 1.
			children do: [: child |  ((child getState) = Running) ifTrue: [child update: currentTime].
					((child getState) = Finished) ifTrue: [child epilogue: currentTime].
					((child getState) = Waiting) ifTrue: [child prologue: currentTime].
					((child getState) = Stopped) ifFalse: [state _ Running].

					((direction = Reverse) and: [ ((child getState) = Paused) ]) ifTrue: [
						((currentTime - startTime) >
								(duration - ((child getDuration) * (childLoopCounts at: index))))
									ifTrue: [ child resume ] ].
					index _ index + 1.
						].

							  ].

	(state = Finished) ifTrue: [self epilogue: currentTime].
! !

!CompositeAnimation methodsFor: 'private' stamp: 'jsp 2/26/1999 14:35'!
getComponents
	"Return the component animations of the sequential animation"

	^ children.
! !

CompositeAnimation subclass: #SequentialAnimation
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!SequentialAnimation commentStamp: '<historical>' prior: 0!
This class extends CompositeAnimation for composite animations (an animation built of 2 or more animations) that run one after another in sequence.
!

!SequentialAnimation methodsFor: 'copying' stamp: 'jsp 4/9/1999 14:21'!
reversed
	"Creates a reversed version of an animation"

	| anim copyOfChildren copyOfChild i |

	anim _ SequentialAnimation new.

	i _ 1.
	copyOfChildren _ OrderedCollection new.
	children do: [:child | copyOfChild _ child copy.
						 copyOfChild setLoopCount: (childLoopCounts at: i).
						 copyOfChildren addLast: copyOfChild.
						 i _ i + 1 ].

	anim children: copyOfChildren
		undoable: true
		inWonderland: myWonderland.

	(direction = Reverse) ifTrue: [ anim setDirection: Reverse ].
	anim reverseDirection.

	anim stop.

	^ anim.
! !

!SequentialAnimation methodsFor: 'management' stamp: 'jsp 2/26/1999 12:27'!
prologue: currentTime
	"Overrides the Animation prologue: message to set the index of the current animation to 1"

	super prologue: currentTime.

	undoable ifTrue: [
						(myWonderland getUndoStack)
								push: (UndoAnimation new: (self makeUndoVersion)).
					].

	index _ 1.
	((children size) > 0) ifTrue: [(children at: index) prologue: currentTime.]
						ifFalse: [ state _ Finished. ].
! !

!SequentialAnimation methodsFor: 'copying' stamp: 'jsp 2/26/1999 14:30'!
makeUndoVersion
	"Creates the undo version of an animation"

	| anim copyOfChildren copyOfChild i |

	anim _ SequentialAnimation new.

	i _ 1.
	copyOfChildren _ OrderedCollection new.
	children do: [:child | copyOfChild _ child copy.
						 copyOfChild setLoopCount: (childLoopCounts at: i).
						 copyOfChildren addLast: copyOfChild.
						 i _ i + 1 ].

	anim children: copyOfChildren
		undoable: false
		inWonderland: myWonderland.

	(direction = Reverse) ifTrue: [ anim setDirection: Reverse ].
	anim reverseDirection.

	"Undoing a sequential animation takes 2 seconds"	
	anim scaleDuration: (2 / duration).
	anim stop.

	^ anim.
! !

!SequentialAnimation methodsFor: 'copying' stamp: 'jsp 3/3/1999 12:30'!
copy
	"Creates a copy of the animation"

	| anim copyOfChildren copyOfChild i |

	anim _ SequentialAnimation new.

	i _ 1.
	copyOfChildren _ OrderedCollection new.
	children do: [:child | copyOfChild _ child copy.
						 copyOfChild setLoopCount: (childLoopCounts at: i).
						 copyOfChildren addLast: copyOfChild.
						 i _ i + 1 ].

	anim children: copyOfChildren
		undoable: false
		inWonderland: myWonderland.

	(direction = Reverse) ifTrue: [ anim setDirection: Reverse ].

	^ anim.
! !

!SequentialAnimation methodsFor: 'initialization' stamp: 'jsp 2/26/1999 14:27'!
children: childCollection undoable: canUndo inWonderland: aWonderland
	"This method initializes the Animation with all the information that it needs run."

	undoable _ canUndo.
	myWonderland _ aWonderland.
	myScheduler _ aWonderland getScheduler.
	loopCount _ 1.
	direction _ Forward.

	super initialize.

	"Add children"
	childCollection do: [:child | self append: child. child setUndoable: false ].

	"Calculate duration"
	duration _ 0.
	children do: [:child | duration _ duration + ((child getDuration) * (child getLoopCount)) ].

	state _ Waiting.
	myScheduler addAnimation: self.
! !

!SequentialAnimation methodsFor: 'management' stamp: 'jsp 2/14/1999 18:00'!
update: currentTime
	"This method updates the sequential animation based on the current time."

	| child notDone |

	"Do the prologue to this animation"
	(state = Waiting) ifTrue: [self prologue: currentTime].

	"Finish right away if there are no component animations"
	((children size) = 0) ifTrue: [ state _ Finished].

	"Update the current animation segment.  When it finishes, move on to the next non-zero duration animation. If there aren't any more, then the sequential animation is done."
	(state = Running) ifTrue: [
				child _ children at: index.
				child update: currentTime.
				notDone _ (child getState) = Stopped.
				[ notDone ] whileTrue: [
						index _ index + 1.
						(index <= (children size)) ifTrue: [ child _ children at: index.
														  child prologue: currentTime.
														  child update: currentTime.
														  notDone _ (child getState) = Stopped.
														]
												ifFalse: [ state _ Finished.
														  notDone _ false.
														].
											]

							  ].

	"After the animation finishes run the epilogue."
	(state = Finished) ifTrue: [self epilogue: currentTime].
! !

!SequentialAnimation methodsFor: 'reversing' stamp: 'jsp 3/12/1999 15:09'!
reverseDirection
	"Changes the direction an animation runs in (forward or in reverse)"

	super reverseDirection.

	children _ children reversed.
	childLoopCounts _ childLoopCounts reversed.


! !

!CompositeAnimation methodsFor: 'management' stamp: 'jsp 2/26/1999 13:55'!
prologue: currentTime
	"comment stating purpose of message"

	| index |

	super prologue: currentTime.

	index _ 1.
	children do: [:child | child setLoopCount: (childLoopCounts at: index).
						 index _ index + 1. ].
! !

!CompositeAnimation methodsFor: 'initialization' stamp: 'jsp 2/26/1999 12:39'!
initialize
	"Creates an empty OrderedCollection for the child animations."

	children _ OrderedCollection new.
	childLoopCounts _ OrderedCollection new.
! !

!CompositeAnimation methodsFor: 'initialization' stamp: 'jsp 2/26/1999 13:53'!
append: child
	"Adds a child animation to the composite animation."

	child stop.
	children addLast: child.
	childLoopCounts addLast: (child getLoopCount).


! !

!CompositeAnimation class methodsFor: 'initialization' stamp: 'jsp 2/11/1999 14:52'!
new
	"Creates a new Composite animation and makes sure it gets initialized."

	^ super new initialize.
! !

!CompositeAnimation methodsFor: 'private' stamp: 'jsp 2/26/1999 14:28'!
scaleDuration: scaleAmount
	"Scales the animation's duration by the specified amount"

	| i |

	super scaleDuration: scaleAmount.

	i _ 1.
	children do: [:child | child scaleDuration: (scaleAmount / (childLoopCounts at: i)).
						 i _ i + 1 ].
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 3/24/1999 15:48'!
loop
	"This method causes an animation to loop forever."

	loopCount _ Infinity.

	(state = Stopped) ifTrue: [ state _ Waiting.
							  myScheduler addAnimation: self. ].
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 3/3/1999 12:36'!
looped: numberOfTimes
	"This method creates a copy of an animation and loops it for the specified number of times."

	| anim |

	anim _ self copy.
	anim setLoopCount: numberOfTimes.

	^ anim.
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 2/26/1999 15:21'!
resume
	"This method resumes a paused animation"

	(state = Paused) ifTrue:
		[
			state _ Running.
			startTime _ (myScheduler getTime) - pausedInterval.
			endTime _ startTime + duration.
		]
	ifFalse: [(state = Stopped) ifTrue:
				[
					state _ Waiting.
					myScheduler addAnimation: self.
				].
			]


! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 2/3/1999 14:50'!
stopLooping
	"This method causes the animation to stop looping; the current interation of the animation completes before the animation stops."

	loopCount _ 1.
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 2/3/1999 15:00'!
start
	"This method starts an existing animation"

	state _ Waiting.
	loopCount _ 1.
	myScheduler addAnimation: self.
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 3/3/1999 12:38'!
copy

	self subclassResponsibility.
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 2/3/1999 14:25'!
stop
	"This method changes the state of an animation to stopped.  If it is currently active, the Scheduler will remove it from the list of active animations."

	state _ Stopped.
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 2/3/1999 14:53'!
update: currentTime
	"Updates the animation using the current Wonderland time"

	(state = Waiting) ifTrue: [self prologue: currentTime].

	(state = Running) ifTrue: [].

	(state = Finished) ifTrue: [self epilogue: currentTime].
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 2/3/1999 15:10'!
loop: numberOfTimes
	"This method causes an animation to loop for the specified number of times."

	loopCount _ numberOfTimes.

	(state = Stopped) ifTrue: [ state _ Waiting.
							  myScheduler addAnimation: self. ].
! !

AbstractAnimation subclass: #Animation
	instanceVariableNames: 'startState endState proportionDone getStartStateFunction getEndStateFunction updateFunction styleFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!Animation commentStamp: '<historical>' prior: 0!
The Animation class extends the AbstractAnimation class with methods designed for simple (non-composite) animations.
!

!Animation methodsFor: 'initialization' stamp: 'jsp 3/9/1999 15:48'!
object: anObject update: func getStartState: startFunc getEndState: endFunc style: styleFunc duration: time undoable: canUndo inWonderland: aWonderland
	"This method initializes the Animation with all the information that it needs run."

	animatedObject _ anObject.
	updateFunction _ func.
	styleFunction _ styleFunc.
	getStartStateFunction _ startFunc.
	getEndStateFunction _ endFunc.
	duration _ time.
	undoable _ canUndo.
	myScheduler _ aWonderland getScheduler.
	myWonderland _ aWonderland.
	loopCount _ 1.
	direction _ Forward.

	state _ Waiting.
	myScheduler addAnimation: self.! !

Animation subclass: #RelativeAnimation
	instanceVariableNames: 'getReverseStateFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!RelativeAnimation commentStamp: '<historical>' prior: 0!
A relative animation is any animation where the final state is relative to the initial state (for example, moving forward 1 meter). To reverse these animations we essentially use the same initial state and just reverse the vector to the final state (ex. we move back one meter).
!

!RelativeAnimation methodsFor: 'copying' stamp: 'jsp 3/9/1999 15:54'!
copy
	"Creates a copy of the animation"

	| anim |

	anim _ RelativeAnimation new.

	anim object: animatedObject
		update: updateFunction
		getStartState: getStartStateFunction
		getEndState: getEndStateFunction
		getReverseState: getReverseStateFunction 
		style: styleFunction
		duration: duration
		undoable: undoable
		inWonderland: myWonderland.

	(direction = Forward) ifFalse: [ anim reverseDirection ].

	^ anim.
! !

!RelativeAnimation methodsFor: 'initialization' stamp: 'jsp 3/9/1999 15:50'!
object: anObject update: func getStartState: startFunc getEndState: endFunc getReverseState: reverseFunc style: styleFunc duration: time undoable: canUndo inWonderland: aWonderland
	"This method initializes the Animation with all the information that it needs to run."

	getReverseStateFunction _ reverseFunc.

	super object: anObject update: func getStartState: startFunc getEndState: endFunc style: styleFunc duration: time undoable: canUndo inWonderland: aWonderland.
! !

!RelativeAnimation methodsFor: 'copying' stamp: 'jsp 3/9/1999 15:54'!
makeUndoVersion
	"Creates the undo version of an animation"

	| anim |

	anim _ RelativeAnimation new.

	anim object: animatedObject
		update: updateFunction
		getStartState: getStartStateFunction
		getEndState: getEndStateFunction
		getReverseState: getReverseStateFunction 
		style: styleFunction
		duration: 0.5
		undoable: false
		inWonderland: myWonderland.

	anim stop.
	(direction = Forward) ifTrue: [ anim reverseDirection ].

	^ anim.
! !

!RelativeAnimation methodsFor: 'copying' stamp: 'jsp 4/9/1999 14:21'!
reversed
	"Creates a reversed version of an animation"

	| anim |

	anim _ RelativeAnimation new.

	anim object: animatedObject
		update: updateFunction
		getStartState: getStartStateFunction
		getEndState: getEndStateFunction
		getReverseState: getReverseStateFunction 
		style: styleFunction
		duration: duration
		undoable: true
		inWonderland: myWonderland.

	anim stop.
	(direction = Forward) ifTrue: [ anim reverseDirection ].

	^ anim.
! !

!RelativeAnimation methodsFor: 'management' stamp: 'jsp 2/16/1999 16:36'!
prologue: currentTime
	"Extends the AbstractAnimation prologue by saving the start state of the animation."

	undoable ifTrue: [
						(myWonderland getUndoStack)
								push: (UndoAnimation new: (self makeUndoVersion)).
					].

	(direction = Forward) ifTrue: [
									startState _ getStartStateFunction value.
									endState _ getEndStateFunction value.
								]
						ifFalse: [
									startState _ getStartStateFunction value.
									endState _ getReverseStateFunction value.
								].


	super prologue: currentTime.
! !

Animation subclass: #AbsoluteAnimation
	instanceVariableNames: 'lastStartState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!AbsoluteAnimation commentStamp: '<historical>' prior: 0!
An AbsoluteAnimation is any animation where the final state of the animation is always the same. Every time this animation runs we store the initial state, so that when the animation is reversed and run we can determine what that end point should be.
!

!AbsoluteAnimation methodsFor: 'copying' stamp: 'jsp 4/9/1999 14:22'!
reversed
	"Creates a reversed version of an animation"

	| anim |

	anim _ AbsoluteAnimation new.

	anim object: animatedObject
		update: updateFunction
		getStartState: getStartStateFunction
		getEndState: getEndStateFunction
		style: styleFunction
		duration: duration
		undoable: true
		inWonderland: myWonderland.

	anim stop.
	(direction = Forward) ifTrue: [ anim reverseDirection ].

	^ anim.
! !

!AbsoluteAnimation methodsFor: 'management' stamp: 'jsp 2/16/1999 16:36'!
prologue: currentTime
	"Extends the AbstractAnimation prologue by saving the start state of the animation."

	undoable ifTrue: [
						(myWonderland getUndoStack)
								push: (UndoAnimation new: (self makeUndoVersion)).
					].

	(direction = Forward) ifTrue: [
									startState _ getStartStateFunction value.
									lastStartState _ startState.
									endState _ getEndStateFunction value.
								]
						ifFalse: [
									startState _ getStartStateFunction value.
									endState _ lastStartState.
								].


	super prologue: currentTime.
! !

!AbsoluteAnimation methodsFor: 'copying' stamp: 'jsp 3/9/1999 15:49'!
makeUndoVersion
	"Creates the undo version of an animation"

	| anim |

	anim _ AbsoluteAnimation new.

	anim object: animatedObject
		update: updateFunction
		getStartState: getStartStateFunction
		getEndState: getEndStateFunction
		style: styleFunction
		duration: 0.5
		undoable: false
		inWonderland: myWonderland.

	anim stop.
	(direction = Forward) ifTrue: [ anim reverseDirection ].

	^ anim.
! !

!AbsoluteAnimation methodsFor: 'copying' stamp: 'jsp 3/9/1999 15:49'!
copy
	"Creates a copy of the animation"

	| anim |

	anim _ AbsoluteAnimation new.

	anim object: animatedObject
		update: updateFunction
		getStartState: getStartStateFunction
		getEndState: getEndStateFunction
		style: styleFunction
		duration: duration
		undoable: undoable
		inWonderland: myWonderland.

	(direction = Forward) ifFalse: [ anim reverseDirection ].

	^ anim.
! !

!AbsoluteAnimation methodsFor: 'initialization' stamp: 'jsp 3/9/1999 15:49'!
object: anObject update: func getStartState: startFunc getEndState: endFunc style: styleFunc duration: time undoable: canUndo inWonderland: aWonderland
	"This method initializes the Animation with all the information that it needs to run."

	lastStartState _ startFunc value.

	super object: anObject update: func getStartState: startFunc getEndState: endFunc style: styleFunc duration: time undoable: canUndo inWonderland: aWonderland.
! !

!Animation methodsFor: 'management' stamp: 'jsp 3/3/1999 12:02'!
update: currentTime
	"Updates the animation using the current Wonderland time"

	| newState |

	(state = Waiting) ifTrue: [self prologue: currentTime].

	(state = Running) ifTrue: [
				proportionDone _ styleFunction value: (currentTime - startTime) value: duration.
				newState _ startState interpolateTo: endState at: proportionDone.
				updateFunction value: newState.
				(currentTime >= endTime) ifTrue: [ state _ Finished. ].
							  ].

	(state = Finished) ifTrue: [self epilogue: currentTime].! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 2/3/1999 14:43'!
pause
	"This method pauses an active Animation."

	(state = Running) ifTrue: [ state _ Paused. 
							    pausedInterval _ (myScheduler getTime) - startTime.].
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 3/3/1999 12:37'!
looped
	"This method creates a copy of an animation and loops it forever."

	| anim |

	anim _ self copy.
	anim setLoopCount: Infinity.

	^ anim.
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 2/3/1999 14:47'!
getDuration
	"This method returns the duration of the animation."

	^ duration.
! !

!AbstractAnimation methodsFor: 'reversing' stamp: 'jsp 2/15/1999 10:28'!
reverseDirection
	"Changes the direction an animation runs in (forward or in reverse)"

	(direction = Forward) ifTrue: [ direction _ Reverse ]
						 ifFalse: [ direction _ Forward ].
! !

!AbstractAnimation methodsFor: 'accessing' stamp: 'jsp 2/26/1999 12:23'!
setUndoable: aBoolean
	"Sets the animation's undoable property"

	undoable _ aBoolean.
! !

!AbstractAnimation methodsFor: 'management' stamp: 'jsp 2/3/1999 14:56'!
prologue: currentTime
	"This method does any work that needs to be done before the animation starts, including possibly adding the current state to the undo stack."

	"Undo stack stuff here"
	undoable ifTrue: [].

	startTime _ currentTime.
	endTime _ startTime + duration.
	state _ Running.
! !

!AbstractAnimation methodsFor: 'accessing' stamp: 'jsp 3/9/1999 15:45'!
getAnimatedObject
	"Return the object that this animation affects"

	^ animatedObject.
! !

!AbstractAnimation methodsFor: 'private' stamp: 'jsp 2/26/1999 14:17'!
setDirection: aDirection
	"Sets the animation's direction variable"

	direction _ aDirection.
! !

!AbstractAnimation methodsFor: 'accessing' stamp: 'jsp 2/4/1999 10:22'!
getState
	"Returns the current state of the animation."

	^ state.
! !

Object subclass: #WonderlandStyle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Misc'!

!WonderlandStyle commentStamp: '<historical>' prior: 0!
The WonderlandStyle class implements different animation styles.  The four styles this class provides are gently (slow-in, slow-out), beginGently (slow-in), endGently (slow-out), and abruptly (linear).

Animations use these styles through blocks of the form:

[ :elapsed :duration | WonderlandStyle beginGentlyAfter: elapsed outOf: duration ]

The intention is that using this mechanism the user can specify arbitrary styles using blocks and pass them to animations.
!

!WonderlandStyle class methodsFor: 'generate parameters' stamp: 'jsp 3/1/1999 17:04'!
beginGentlyAfter: elapsed outOf: duration
	"Returns the proportion done for an animation using the BeginGently style after elapsed seconds (out of duration seconds)."

	(elapsed >= duration) ifTrue: [ ^ 1.0 ].

	^ (self gently: (elapsed / duration) lowerBound: 0.99 upperBound: 0.999).
! !

!WonderlandStyle class methodsFor: 'instance creation' stamp: 'jsp 2/4/1999 14:58'!
new
	"Users shouldn't create any instances of this class, so this method throws an error if they try."

	self error: 'This class exists to generate style parameters, so it should not be instatiated.'
! !

!WonderlandStyle class methodsFor: 'generate parameters' stamp: 'jsp 3/1/1999 17:04'!
abruptlyAfter: elapsed outOf: duration
	"Returns the proportion done for an Abrupt animation style after elapsed seconds (out of duration seconds) have elapsed."

	(elapsed >= duration) ifTrue: [ ^ 1.0 ].

	^ (elapsed / duration).
! !

!WonderlandStyle class methodsFor: 'class initialization' stamp: 'ar 3/4/2004 20:41'!
initialize	"WonderlandStyle initialize"
	"Add the style constants gently, beginGently, endGently, and abruptly to the WonderlandConstants dictionary"

	| dict |

	dict _ WonderlandConstants default.
	dict at: 'gently' put:
			[:elapsed :duration | WonderlandStyle gentlyAfter: elapsed outOf: duration ].
	dict at: 'beginGently' put:
			[:elapsed :duration | WonderlandStyle beginGentlyAfter: elapsed outOf: duration ].
	dict at: 'endGently' put:
			[:elapsed :duration | WonderlandStyle endGentlyAfter: elapsed outOf: duration ].
	dict at: 'abruptly' put:
			[:elapsed :duration | WonderlandStyle abruptlyAfter: elapsed outOf: duration ].
! !

!WonderlandStyle class methodsFor: 'generate parameters' stamp: 'jsp 3/1/1999 17:05'!
gentlyAfter: elapsed outOf: duration
	"Returns the proportion done for an animation using the Gently style after elapsed seconds (out of duration seconds)."

	(elapsed >= duration) ifTrue: [ ^ 1.0 ].

	^ (self gently: (elapsed / duration) lowerBound: 0.3 upperBound: 0.8).
! !

!WonderlandStyle class methodsFor: 'private' stamp: 'jsp 2/4/1999 15:22'!
gently: x lowerBound: A upperBound: B
	"This method converts a linear proportion done to a slow in - slow out proportion. If x is less than A then the animation is in the slow in part, while if x is greater than B then the animation is in the slow out part."

	| y a3 b3 c3 m b2 |
 
 	(x < A) ifTrue: [ y _ ((B - 1)/(A *  ((B * B) - (A * B) + A - 1))) * x * x ]
		    ifFalse: [ (x > B) ifTrue: [
										a3 _ 1 / ((B * B) - (A * B) + A - 1).
										b3 _ -2 * a3.
										c3 _ 1 + a3.
  										y _ (a3 * x * x) + (b3 * x) + c3.
									]
							ifFalse: [
										m _ 2 * (B - 1) / ((B * B) - (A * B) + A - 1).
										b2 _ (0 - m) * A / 2.
										y _ m * x + b2.
									].
  				    ].

	^ y.
! !

!WonderlandStyle class methodsFor: 'generate parameters' stamp: 'jsp 3/1/1999 17:04'!
endGentlyAfter: elapsed outOf: duration
	"Returns the proportion done for an animation using the EndGently style after elapsed seconds (out of duration seconds)."

	(elapsed >= duration) ifTrue: [ ^ 1.0 ].

	^ (self gently: (elapsed / duration) lowerBound: 0.001 upperBound: 0.01).
! !

!WonderlandStyle class methodsFor: 'generate parameters' stamp: 'jsp 2/4/1999 15:08'!
oneAfter: elapsed outOf: duration
	"Returns the proportion of Animation done using an instantaneous style - this is also 1.0."

	^ 1.0.
! !

Object subclass: #WonderlandEvent
	instanceVariableNames: 'actor camera cursorPoint cursorDelta key morph vertex morphicEvent'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Misc'!

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:56'!
setKey: aKey

	key _ aKey.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'ar 6/2/1999 14:32'!
getMorphicEvent
	^morphicEvent! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:55'!
setCursorPoint: aPoint

	cursorPoint _ aPoint.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:53'!
setCameraMorph: aMorph
	"Set the camera window the user clicked in"

	morph _ aMorph.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:54'!
getActor

	^ actor.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:54'!
getCameraMorph
	"Return the camera window the user clicked in"

	^ morph.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:55'!
getCursorPoint

	^ cursorPoint.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'ar 6/2/1999 14:32'!
setMorphicEvent: aMorphicEvent
	morphicEvent _ aMorphicEvent! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:54'!
getCamera

	^ camera.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'ar 6/2/1999 13:23'!
getVertex
	"Return the primitive vertex from picking operations"
	^vertex! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:54'!
setActor: anActor

	actor _ anActor.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 14:37'!
getCursorDelta

	^ cursorDelta.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:53'!
setCamera: aCamera

	camera _ aCamera.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 14:37'!
setCursorDelta: aPoint

	cursorDelta _ aPoint.
! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'ar 6/2/1999 13:23'!
setVertex: aB3DPrimitiveVertex
	"Set the primitive vertex from picking operations"
	vertex _ aB3DPrimitiveVertex.! !

!WonderlandEvent methodsFor: 'accessing' stamp: 'jsp 4/19/1999 13:56'!
getKey

	^ key.
! !

Object subclass: #UndoTextureChange
	instanceVariableNames: 'theActor originalTexture'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoTextureChange commentStamp: '<historical>' prior: 0!
This class undoes an actor texture change.
!

!UndoTextureChange class methodsFor: 'instance creation' stamp: 'jsp 3/19/1999 15:59'!
for: anActor from: oldTexture
	"Wrap an actor and its old texture so that we can undo the texture change"

	^ (self new) setActor: anActor andTexture: oldTexture.
! !

!UndoTextureChange methodsFor: 'undoing' stamp: 'jsp 3/19/1999 16:14'!
undoIt
	"Restore the object's old texture"

	theActor setTexturePointer: originalTexture.
! !

!UndoTextureChange methodsFor: 'accessing' stamp: 'jsp 3/19/1999 15:57'!
setActor: anActor andTexture: aTexture
	"Store the old texture and the actor to restore it to"

	theActor _ anActor.
	originalTexture _ aTexture.
! !

Object subclass: #WonderlandVerifier
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Misc'!

!WonderlandVerifier commentStamp: '<historical>' prior: 0!
This class implements methods for verifying the parameters passed to methods.  The intention is that each parameter is verified, and if it's not of the expected type then an error message is constructed and displayed to the user. This provides a gentler method of informing the user of errors than throwing them into the debugger.
!

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/17/1999 15:41'!
VerifyDuration: parameter
	"If the parameter is a valid duration this method returns true, otherwise it throws an exception"

	((( parameter = rightNow) or: [ parameter = eachFrame ])
		or: [ (parameter isNumber) and: [ parameter >= 0 ]	])
					ifTrue: [ ^ true ]
					ifFalse: [ self error: (parameter asString) , ' is not a valid duration. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/17/1999 15:23'!
VerifyNumber: parameter
	"If the parameter is a valid number this method returns true, otherwise it throws an exception"

	(parameter isNumber)
					ifTrue: [ ^ true ]
					ifFalse: [ self error: (parameter asString) , ' is not a valid number. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/18/1999 13:45'!
VerifyCondition: parameter
	"If the parameter is a valid style (BlockContext that takes 1 parameter and returns a boolean) this method returns true, otherwise it throws an exception"

	| result | 

	[ result _ ((parameter value) isKindOf: Boolean) ] 
		ifError: [ :msg :rcvr |
					self error: 'your condition must be a block that returns true or false.' ].

	(result) ifTrue: [ ^ true ]
			ifFalse: [ self error: 'your condition does not return true or false. '].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/18/1999 14:59'!
VerifyColor: parameter
	"If the parameter is a valid color (triple or named color) this method returns true, otherwise it throws an exception"

	(parameter isKindOf: SequenceableCollection)
		ifTrue: [
			(((((parameter size) = 3)
				and: [ ((parameter at: 1) isNumber) and:
							[ ((parameter at: 1) >= 0) and: [ ((parameter at: 1) <= 1) ] ] ] )
				and: [ ((parameter at: 2) isNumber) and:
 							[ ((parameter at: 2) >= 0) and: [ ((parameter at: 2) <= 1) ] ] ] )
				and: [ ((parameter at: 3) isNumber) and:
							[ ((parameter at: 3) >= 0) and: [ ((parameter at: 3) <= 1) ] ] ] )
					ifTrue: [ ^ true ]
					ifFalse: [ [ Color perform: parameter. ]
								ifError: [ :msg : rcvr | self error: (parameter asString) ,
												' is not a valid color. '].
							  ^ true.
							].
  				]
		ifFalse: [ self error: (parameter asString) , ' is not a valid color. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/18/1999 17:05'!
VerifyZeroArgumentBlock: parameter
	"Return true if the parameter is a zero argument block, otherwise throw an exception"

	((parameter class) = BlockContext)
		ifTrue: [ ((parameter numArgs) = 0)
						ifTrue: [ ^ true ]
						ifFalse: [ self error: ' the block can not require arguments.' ]
				]
		ifFalse: [ self error: ' you need to supply a block that does not require arguments.' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/22/1999 12:25'!
VerifyDimension: parameter
	"If the parameter is a valid dimension this method returns true, otherwise it throws an exception"

	(((parameter = leftToRight) or: [ parameter = topToBottom ]) or: [ parameter = frontToBack ])
			ifTrue: [ ^ true ]
			ifFalse: [ self error: (parameter asString) , ' is not a valid dimension. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/18/1999 15:16'!
VerifyPointOfView: parameter
	"If the parameter is a valid point of view (sextuple or kind of WonderlandActor) this method returns true, otherwise it throws an exception"

	((parameter isKindOf: WonderlandActor)
		or: 	[ (((((((parameter isKindOf: SequenceableCollection)
				and: [ (parameter size) = 6 ] )
				and: [ ((parameter at: 1) isNumber) or: [ (parameter at: 1) = asIs ] ] )
				and: [ ((parameter at: 2) isNumber) or: [ (parameter at: 2) = asIs ] ] )
				and: [ ((parameter at: 3) isNumber) or: [ (parameter at: 3) = asIs ] ] )
 				and: [ ((parameter at: 4) isNumber) or: [ (parameter at: 4) = asIs ] ] )
 				and: [ ((parameter at: 5) isNumber) or: [ (parameter at: 5) = asIs ] ] )
  				and: [ ((parameter at: 6) isNumber) or: [ (parameter at: 6) = asIs ] ]
  			])
				ifTrue: [ ^ true ]
				ifFalse: [ self error: (parameter asString) , ' is not a valid point of view. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/17/1999 16:24'!
VerifyNonNegativeNumber: parameter
	"If the parameter is a valid number this method returns true, otherwise it throws an exception"

	((parameter isNumber) and: [ parameter >= 0 ])
			ifTrue: [ ^ true ]
			ifFalse: [ self error: (parameter asString) , ' is not a number greater than or equal to zero. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/18/1999 14:53'!
Verify0To1Number: parameter
	"If the parameter is a valid number this method returns true, otherwise it throws an exception"

	(((parameter isNumber) and: [ parameter >= 0 ]) and: [ parameter <= 1 ])
					ifTrue: [ ^ true ]
					ifFalse: [ self error: (parameter asString) , ' is not a number between 0 and 1. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/17/1999 15:41'!
VerifyLocation: parameter
	"If the parameter is a valid location this method returns true, otherwise it throws an exception"

	((((((((((parameter = toLeftOf) or: [ parameter = toRightOf ]) or: [ parameter = onTopOf ])
			or: [ parameter = onBottomOf ]) or: [ parameter = inFrontOf ])
			or: [ parameter = inBackOf ]) or: [ parameter = behind ])
			or: [ parameter = below ]) or: [ parameter = onFloorOf ])
			or: [ parameter = onCeilingOf ])
					ifTrue: [ ^ true ]
					ifFalse: [ self error: (parameter asString) , ' is not a valid location. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/29/1999 14:39'!
VerifyTargetOrPixel: parameter
	"If the parameter is a valid target (triple or kind of WonderlandActor) or a pixel coordinate this method returns true, otherwise it throws an exception"

	(((parameter isKindOf: WonderlandActor) or: [ parameter isKindOf: Point ])
		or: 	[ ((((parameter isKindOf: SequenceableCollection)
				and: [ (parameter size) = 3 ] )
				and: [ ((parameter at: 1) isNumber) or: [ (parameter at: 1) = asIs ] ] )
				and: [ ((parameter at: 2) isNumber) or: [ (parameter at: 2) = asIs ] ] )
 				and: [ ((parameter at: 3) isNumber) or: [ (parameter at: 3) = asIs ] ]
  			])
				ifTrue: [ ^ true ]
				ifFalse: [ self error: (parameter asString) , ' is not a valid target. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/18/1999 13:47'!
VerifyStyle: parameter
	"If the parameter is a valid style (BlockContext that takes 2 parameters) this method returns true, otherwise it throws an exception"

	| result |

	[ result _ ((parameter numArgs) = 2) ]
		ifError: [ :msg :rcvr |
					self error: 'your style was not a block that takes 2 parameters and returns a value between 0 and 1.' ].

	(result) ifTrue: [ ^ true ]
			ifFalse: [ self error: 'your style function did not take exactly 2 parameters. ' ].! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/17/1999 15:41'!
VerifyDirection: parameter
	"If the parameter is a valid direction this method returns true, otherwise it throws an exception"

	(((((((((parameter = left) or: [ parameter = right ]) or: [ parameter = up ])
			or: [ parameter = down ]) or: [ parameter = forward ])
			or: [ parameter = back ]) or: [ parameter = backward ])
			or: [ parameter = ccw ]) or: [ parameter = cw ])
					ifTrue: [ ^ true ]
					ifFalse: [ self error: (parameter asString) , ' is not a valid direction. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/17/1999 15:41'!
VerifyTarget: parameter
	"If the parameter is a valid target (triple or kind of WonderlandActor) this method returns true, otherwise it throws an exception"

	((parameter isKindOf: WonderlandActor)
		or: 	[ ((((parameter isKindOf: SequenceableCollection)
				and: [ (parameter size) = 3 ] )
				and: [ ((parameter at: 1) isNumber) or: [ (parameter at: 1) = asIs ] ] )
				and: [ ((parameter at: 2) isNumber) or: [ (parameter at: 2) = asIs ] ] )
 				and: [ ((parameter at: 3) isNumber) or: [ (parameter at: 3) = asIs ] ]
  			])
				ifTrue: [ ^ true ]
				ifFalse: [ self error: (parameter asString) , ' is not a valid target. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/18/1999 15:05'!
VerifyTriple: parameter
	"If the parameter is a valid triple  this method returns true, otherwise it throws an exception"

	(((((parameter isKindOf: SequenceableCollection)
			and: [ (parameter size) = 3 ] )
			and: [ ((parameter at: 1) isNumber) ] )
			and: [ ((parameter at: 2) isNumber) ] )
 			and: [ ((parameter at: 3) isNumber) ] )
 				ifTrue: [ ^ true ]
				ifFalse: [ self error: (parameter asString) , ' is not a collection with 3 numbers. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/18/1999 15:43'!
VerifyReferenceFrame: parameter
	"If the parameter is a valid WonderlandActor (or subclass) or WonderlandScene this method returns true, otherwise it throws an exception"

	((parameter isKindOf: WonderlandActor) or: [ parameter isKindOf: WonderlandScene])
		ifTrue: [ ^ true ]
		ifFalse: [ self error: (parameter asString) , ' does not have an orientation.' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/22/1999 16:25'!
VerifyPoint: parameter
	"If the parameter is a valid point this method returns true, otherwise it throws an exception"

	(parameter isKindOf: Point)
		ifTrue: [ ^ true ]
		ifFalse: [ self error: (parameter asString) , ' is not a valid point. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 4/29/1999 15:11'!
VerifyLight: parameter
	"If the parameter is a valid type of light this method returns true, otherwise it throws an exception"

	((((parameter = ambient) or: [parameter = positional]) or: [parameter = directional])
			or: [parameter = spotlight])
		ifTrue: [ ^ true ]
		ifFalse: [ self error: (parameter asString) , ' is not a valid type of light. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/17/1999 15:59'!
VerifyPositiveNumber: parameter
	"If the parameter is a valid number this method returns true, otherwise it throws an exception"

	((parameter isNumber) and: [ parameter > 0 ])
			ifTrue: [ ^ true ]
			ifFalse: [ self error: (parameter asString) , ' is not a number greater than zero. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/24/1999 16:43'!
VerifyNumberOrTriple: parameter
	"If the parameter is a number or a valid triple this method returns true, otherwise it throws an exception"

	((parameter isNumber) or: [(((((parameter isKindOf: SequenceableCollection)
			and: [ (parameter size) = 3 ] )
			and: [ ((parameter at: 1) isNumber) ] )
			and: [ ((parameter at: 2) isNumber) ] )
 			and: [ ((parameter at: 3) isNumber) ] ) ])
 				ifTrue: [ ^ true ]
				ifFalse: [ self error: (parameter asString) , ' is not a number or triple. ' ].
! !

!WonderlandVerifier class methodsFor: 'parameter parsing' stamp: 'jsp 3/17/1999 15:23'!
VerifyActor: parameter
	"If the parameter is a valid WonderlandActor (or subclass) this method returns true, otherwise it throws an exception"

	(parameter isKindOf: WonderlandActor)
		ifTrue: [ ^ true ]
		ifFalse: [ self error: (parameter asString) , ' is not a valid Wonderland Actor. ' ].
! !

Object subclass: #UndoPOVChange
	instanceVariableNames: 'theActor originalPOV'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoPOVChange commentStamp: '<historical>' prior: 0!
This class undoes a point of view change to an object.
!

!UndoPOVChange methodsFor: 'undoing' stamp: 'jsp 3/8/1999 14:25'!
undoIt
	"Set the object's POV back to its original value"

	| anim |

	anim _ theActor setPointOfView: originalPOV duration: 0.5.
	anim setUndoable: false.
! !

!UndoPOVChange methodsFor: 'accessing' stamp: 'ar 11/7/1999 20:15'!
getActor
	^theActor! !

!UndoPOVChange class methodsFor: 'instance creation' stamp: 'jsp 3/19/1999 16:00'!
for: anActor from: pov
	"Create a new UndoPOVChange instance using pov as the original point of view"

	^ self new setActor: anActor andOriginalPOV: pov.
! !

!UndoPOVChange methodsFor: 'accessing' stamp: 'jsp 3/8/1999 14:24'!
setActor: anActor andOriginalPOV: pov
	"Set the original POV to undo back to"

	theActor _ anActor.
	originalPOV _ pov.! !

Object subclass: #UndoAction
	instanceVariableNames: 'wrappedAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoAction commentStamp: '<historical>' prior: 0!
Undo the effects of the wrapped Action.
!

!UndoAction methodsFor: 'accessing' stamp: 'jsp 2/23/1999 14:23'!
setAction: anAction
	"Specify the undo action to wrap."

	wrappedAction _ anAction.
! !

!UndoAction methodsFor: 'accessing' stamp: 'di 3/10/2000 08:40'!
rectifyActorClassesInBlocks  "UndoAction allInstancesDo: [:ua | ua rectifyActorClassesInBlocks]"
	"Rectify obsolete class refs held onto by UndoActions"

	| ctxt obs |
	ctxt _ wrappedAction home.
	1 to: ctxt size do:
		[:i | obs _ ctxt at: i.
		((obs isKindOf: Behavior) and: [obs isObsolete])
			ifTrue: [ctxt at: i put: obs nonObsoleteClass]]! !

!UndoAction methodsFor: 'undoing' stamp: 'jsp 2/23/1999 14:21'!
undoIt
	"Undo the wrapped action"

	wrappedAction value.
! !

!UndoAction class methodsFor: 'instance creation' stamp: 'jsp 2/23/1999 14:33'!
new: anAction
	"Create a wrapper for undoing an animation"

	| newUndo |

	newUndo _ UndoAction new.
	newUndo setAction: anAction.
	^ newUndo.! !

Object subclass: #UndoVisibilityChange
	instanceVariableNames: 'theActor originalVisibility'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoVisibilityChange commentStamp: '<historical>' prior: 0!
This class undoes a change in an actor's visibility.
!

!UndoVisibilityChange methodsFor: 'accessing' stamp: 'jsp 3/8/1999 16:36'!
setActor: anActor andOriginalVisibility: aVisibility
	"Set the original visibility to undo back to"

	theActor _ anActor.
	originalVisibility _ aVisibility.! !

!UndoVisibilityChange methodsFor: 'undoing' stamp: 'jsp 3/8/1999 16:37'!
undoIt
	"Set the object's visibility back to its original value"

	| anim |

	anim _ theActor setVisibility: originalVisibility duration: 0.5.
	anim setUndoable: false.
! !

!UndoVisibilityChange class methodsFor: 'instance creation' stamp: 'jsp 3/8/1999 16:38'!
for: anActor from: aVisibility
	"Create a new UndoVisibilityChange instance using aVisibility as the original visibility"

	^ super new setActor: anActor andOriginalVisibility: aVisibility.
! !

Object subclass: #UndoByStopping
	instanceVariableNames: 'stoppableItem myUndoStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoByStopping commentStamp: '<historical>' prior: 0!
This class undoes some action by stopping it.  If the action is already stopped then the next undo action is popped off the stack and evaluated.
!

!UndoByStopping methodsFor: 'undoing' stamp: 'jsp 3/30/1999 11:40'!
undoIt
	"Undo by stopping the stoppable item"

	(stoppableItem isDone) ifTrue: [ myUndoStack popAndUndo ]
						   ifFalse: [ stoppableItem stop ].

! !

!UndoByStopping class methodsFor: 'instance creation' stamp: 'jsp 3/30/1999 11:37'!
new: stopable in: aStack
	"Create a new instance and initialize it"

	^ self new initializeWith: stopable in: aStack.
! !

!UndoByStopping methodsFor: 'initialization' stamp: 'jsp 3/30/1999 11:40'!
initializeWith: anActionorAnimation in: aStack
	"Initialize this instance"

	stoppableItem _ anActionorAnimation.
	myUndoStack _ aStack.
! !

Object subclass: #Alarm
	instanceVariableNames: 'alarmTask alarmTime myScheduler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Time'!

!Alarm commentStamp: '<historical>' prior: 0!
This class implements the alarms for Wonderlands. The user specifies the time the alarm should go off (either in a certain amount of time or at a specific moment) and the task the system should execute when the alarm goes off.
!

!Alarm methodsFor: 'management' stamp: 'jsp 2/1/1999 10:58'!
setTask: task
	"Specifies the task the alarm executes when it goes off"

	alarmTask _ task.
! !

!Alarm class methodsFor: 'intialize-release' stamp: 'jsp 2/8/1999 16:06'!
do: task in: waitTime inScheduler: scheduler
	"This sets an alarm that will expire in waitTime seconds and execute the specified task"

	| newAlarm |
	newAlarm _ Alarm new.

	newAlarm setTask: task.
	newAlarm setTime: waitTime + (scheduler getTime).
	newAlarm setScheduler: scheduler.

	scheduler addAlarm: newAlarm.

	^ newAlarm.
! !

!Alarm class methodsFor: 'intialize-release' stamp: 'jsp 2/8/1999 16:07'!
do: task at: executeTime inScheduler: scheduler
	"Creates an alarm that does the specified task at the specified time"

	| newAlarm |
	newAlarm _ Alarm new.

	newAlarm setTime: executeTime.
	newAlarm setTask: task.
	newAlarm setScheduler: scheduler.

	scheduler addAlarm: newAlarm.

	^ newAlarm.! !

!Alarm methodsFor: 'management' stamp: 'jsp 2/1/1999 10:59'!
setTime: time
	"Specifies the time the alarm goes off"

	alarmTime _ time.
! !

!Alarm methodsFor: 'management' stamp: 'jsp 2/1/1999 16:33'!
stop
	"This method removes the Alarm from myScheduler's list of active Alarms"

	myScheduler removeAlarm: self.
! !

!Alarm methodsFor: 'accessing' stamp: 'jsp 2/1/1999 14:50'!
checkTime
	"Returns the time the alarm is set to go off at"

	^ alarmTime.
! !

!Alarm methodsFor: 'management' stamp: 'jsp 2/1/1999 12:12'!
setScheduler: scheduler
	"Set the Scheduler that manages this Alarm"

	myScheduler _ scheduler.
! !

!Alarm methodsFor: 'management' stamp: 'jsp 1/29/1999 14:49'!
execute
	"Execute the appointed task because it's the appointed hour"

	alarmTask value.
! !

Object subclass: #UndoColorChange
	instanceVariableNames: 'theActor originalColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoColorChange commentStamp: '<historical>' prior: 0!
This class does a color change to an object.
!

!UndoColorChange methodsFor: 'accessing' stamp: 'ar 11/7/1999 21:27'!
getActor
	^theActor! !

!UndoColorChange methodsFor: 'undoing' stamp: 'jsp 3/8/1999 16:31'!
undoIt
	"Set the object's color back to its original value"

	| anim |

	anim _ theActor setColor: originalColor duration: 0.5.
	anim setUndoable: false.
! !

!UndoColorChange class methodsFor: 'instance creation' stamp: 'jsp 3/19/1999 16:00'!
for: anActor from: aColor
	"Create a new UndoColorChange instance using aColor as the original color"

	^ self new setActor: anActor andOriginalColor: aColor.
! !

!UndoColorChange methodsFor: 'accessing' stamp: 'jsp 3/8/1999 16:31'!
setActor: anActor andOriginalColor: aColor
	"Set the original color to undo back to"

	theActor _ anActor.
	originalColor _ aColor.! !

Object subclass: #UndoShowHide
	instanceVariableNames: 'theActor undoingHide'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoShowHide commentStamp: '<historical>' prior: 0!
This class shows a hidden object and hides a shown actor.
!

!UndoShowHide methodsFor: 'undoing' stamp: 'jsp 3/19/1999 13:56'!
undoIt
	"Undo the show or hide event"

	(undoingHide) 	ifTrue: [ theActor setHidden: false ]
					ifFalse: [ theActor setHidden: true ].
! !

!UndoShowHide class methodsFor: 'instance creation' stamp: 'jsp 3/19/1999 15:59'!
undoShowFor: anActor
	"Create an undo event that will show the actor"

	^ (self new) setActorForShow: anActor.
! !

!UndoShowHide methodsFor: 'accessing' stamp: 'jsp 3/19/1999 13:58'!
setActorForShow: anActor
	"Set this undo event to undo a show (hide the actor)."

	theActor _ anActor.
	undoingHide _ false.
! !

!UndoShowHide class methodsFor: 'instance creation' stamp: 'jsp 3/19/1999 15:59'!
undoHideFor: anActor
	"Create an undo event that will show the actor"

	^ (self new) setActorForHide: anActor.
! !

!UndoShowHide methodsFor: 'accessing' stamp: 'jsp 3/19/1999 13:58'!
setActorForHide: anActor
	"Set this undo event to undo a hide (show the actor)."

	theActor _ anActor.
	undoingHide _ true.
! !

Object subclass: #UndoPositionChange
	instanceVariableNames: 'theActor originalPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoPositionChange commentStamp: '<historical>' prior: 0!
This class undoes actor motion.
!

!UndoPositionChange class methodsFor: 'instance creation' stamp: 'jsp 3/19/1999 16:00'!
for: anActor from: position
	"Create a new UndoPositionChange instance using position as the original position"

	^ self new setActor: anActor andOriginalPosition: position.
! !

!UndoPositionChange methodsFor: 'accessing' stamp: 'jsp 3/8/1999 11:19'!
setActor: anActor andOriginalPosition: position
	"Set the original position to undo back to"

	theActor _ anActor.
	originalPosition _ position.
! !

!UndoPositionChange methodsFor: 'undoing' stamp: 'jsp 3/8/1999 11:21'!
undoIt
	"Move the object back to its original position"

	| anim |

	anim _ theActor moveTo: originalPosition duration: 0.5.
	anim setUndoable: false.
! !

Object subclass: #UndoRotationChange
	instanceVariableNames: 'theActor originalRotation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Undo'!

!UndoRotationChange commentStamp: '<historical>' prior: 0!
This class undoes actor rotation.
!

!UndoRotationChange methodsFor: 'accessing' stamp: 'jsp 3/8/1999 11:23'!
setActor: anActor andOriginalRotation: rotation
	"Set the original rotation to undo back to"

	theActor _ anActor.
	originalRotation _ rotation.
! !

!UndoRotationChange methodsFor: 'undoing' stamp: 'jsp 3/8/1999 11:24'!
undoIt
	"Move the object back to its original position"

	| anim |

	anim _ theActor turnTo: originalRotation duration: 0.5.
	anim setUndoable: false.
! !

!UndoRotationChange class methodsFor: 'instance creation' stamp: 'jsp 3/19/1999 15:59'!
for: anActor from: rotation
	"Create a new UndoRotationChange instance using rotation as the original rotation"

	^ self new setActor: anActor andOriginalRotation: rotation.
! !

Object subclass: #WonderlandUndoStack
	instanceVariableNames: 'theStack stackIsOpen maxStackDepth'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Undo'!

!WonderlandUndoStack commentStamp: '<historical>' prior: 0!
This class implements the undo stack for Wonderlands.  Every user action is added to the undo stack, and the user can then undo each action one at a time. This stack is intended to be "infinite" so that the user can always roll back to a safe state.  In practice the stack is actually depth limited (to 100 actions), but the user can adjust this depth using the setMaxDepth: message.

!

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'jsp 2/14/1999 18:38'!
pop
	"Takes the top undo action off the stack."

	| lastItem |

	(theStack isEmpty) ifTrue: [lastItem _ nil]
					   ifFalse: [lastItem _ theStack removeLast].

	^ lastItem.
! !

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'jsp 3/24/1999 15:54'!
setMaxDepth: aNumber
	"Sets the current maximum depth of the undo stack"

	maxStackDepth _ aNumber.

! !

!WonderlandUndoStack methodsFor: 'reset' stamp: 'jsp 3/30/1999 15:44'!
reset
	"Reset the undo stack."

	theStack _ OrderedCollection new.
	stackIsOpen _ true.! !

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'jsp 3/8/1999 12:18'!
isOpen
	"Returns true if the undo stack is open"

	^ stackIsOpen.
! !

!WonderlandUndoStack methodsFor: 'initialization' stamp: 'ar 3/4/2004 20:37'!
initialize
	"Initializes the undo stack."

	theStack _ OrderedCollection new.
	stackIsOpen _ true.
	maxStackDepth _ (WonderlandConstants default at: 'maxUndoDepth').
! !

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'jsp 2/16/1999 16:36'!
popAndUndo
	"Takes the top undo action off the stack and does it (undoing the original action)."

	| lastItem |

	(theStack isEmpty) ifTrue: [lastItem _ nil]
					   ifFalse: [lastItem _ (theStack removeLast) undoIt].

	^ lastItem.
! !

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'jsp 3/8/1999 12:16'!
closeStack
	"Close the stack to prevent undoable items from being pushed onto it."

	stackIsOpen _ false.
! !

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'jsp 3/8/1999 12:16'!
openStack
	"Open the stack to allow undoable items to be pushed onto it."

	stackIsOpen _ true.
! !

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'jsp 3/24/1999 15:54'!
getMaxDepth
	"Returns the current maximum depth of the undo stack"

	^ maxStackDepth.
! !

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'jsp 2/14/1999 18:36'!
isEmpty
	"Returns true if the undo stack is empty"

	^ (theStack size) = 0.
! !

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'ar 11/7/1999 20:13'!
top
	"Return the top undo action off the stack."

	| lastItem |

	(theStack isEmpty) ifTrue: [lastItem _ nil]
					   ifFalse: [lastItem _ theStack last].

	^ lastItem.
! !

!WonderlandUndoStack methodsFor: 'accessing' stamp: 'jsp 3/24/1999 15:55'!
push: anAction
	"Pushes an undo action on the stack."

	(stackIsOpen) ifTrue: [ theStack addLast: anAction ].

	((theStack size) > maxStackDepth) ifTrue: [ theStack removeFirst ].
! !

!WonderlandUndoStack class methodsFor: 'instance creation' stamp: 'jsp 2/14/1999 18:41'!
new
	"Create a new Wonderland undo stack and initializes it."

	^ super new initialize.
! !

Object subclass: #Wonderland
	instanceVariableNames: 'myScheduler myNamespace sharedMeshDict sharedTextureDict myUndoStack defaultCamera cameraList lightList sceneObject scriptEditor actorClassList'
	classVariableNames: 'ActorPrototypeClasses'
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Core'!

!Wonderland commentStamp: '<historical>' prior: 0!
What is Squeak-Alice?

Alice (http://www.alice.org) is an authoring tool designed to allow people without programming experience to build interactive 3D worlds. The Stage 3 Research Group (Randy Pausch director) started the Alice project at the University of Virginia, and is continuing their work at Carnegie Mellon University (http://www.cs.cmu.edu/~stage). Squeak-Alice brings the ideas and the ideals of Alice to Squeak, with the goal of introducing even more people to 3D graphics and allowing them to author in the medium.

Wonderlands are the basic building blocks of Squeak-Alice; a Wonderland is an interactive 3D world.

To create a Wonderland, evaluate:

Wonderland new

This creates a script editor, a control panel, and a window into the 3D world. From here you're ready to start building your world.

Jeff Pierce
jpierce@cs.cmu.edu
!

!Wonderland methodsFor: 'initialize-reset-release' stamp: 'di 3/10/2000 08:30'!
rectifyActorClassList  "Wonderland allInstancesDo: [:w | w rectifyActorClassList]"
	"Attempt to restore the actorClassList by replacing obsolete classes by current classes of the same name"

	actorClassList _ actorClassList collect:
		[:obs |
		(obs name beginsWith: 'AnObsolete')
			ifTrue: [obs nonObsoleteClass]
			ifFalse: [obs]]! !

!Wonderland methodsFor: 'creating' stamp: 'jsp 4/1/1999 16:21'!
makeScriptEditor
	"Create a new script editor for this Wonderland"

	scriptEditor _ WonderlandEditor newFor: self.
! !

!Wonderland methodsFor: 'composite animations' stamp: 'jsp 2/14/1999 18:14'!
doTogether: animations
	"Create a new ParallelAnimation instance and pass it the child animations that make it up."

	| newPA |
	newPA _ ParallelAnimation new.

	newPA children: animations undoable: true inWonderland: self.

	^ newPA.
! !

!Wonderland class methodsFor: 'class initialization' stamp: 'ar 3/4/2004 21:09'!
initialize
	"Initialize the WonderlandClass by creating the ActorPrototypeClasses collection"
	ActorPrototypeClasses _ Dictionary new.
	self initializeDefaultObjects.
	FileList registerFileReader: self.
	 (TheWorldMenu respondsTo: #registerOpenCommand:)
         ifTrue: [TheWorldMenu registerOpenCommand: {'Wonderland 3D'. {self. #new}}].
! !

!Wonderland methodsFor: 'accessing' stamp: 'jsp 2/4/1999 17:18'!
getScheduler
	"Returns the Wonderland's scheduler."

	^ myScheduler.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 8/31/2000 17:15'!
makeActorFromVRML: filename named: actorName
	"Creates a new actor using the specification from the given file"
	| baseActor parserClass builderClass scene |

	parserClass _ Smalltalk at: #VRMLNodeParser ifAbsent:[
		self reportErrorToUser:'No VRML parser found'.
		^self].
	builderClass _ Smalltalk at: #VRMLWonderlandBuilder ifAbsent:[
		self reportErrorToUser:'No VRML to Alice converter found'.
		^self].
	myUndoStack closeStack.
	baseActor _ self makeBaseActorFrom: filename.
	baseActor setName: (self uniqueNameFrom: actorName).
	parserClass == nil
		ifFalse:[scene _ parserClass parseFileNamed: filename].
	(builderClass == nil or:[scene == nil])
		ifFalse:[(builderClass on: scene) 
					baseActor: baseActor;
					buildActorsFor: self].
	myUndoStack openStack.

	"Ensure that the new actor's name is unique"
	myNamespace at: baseActor getName put: baseActor.
	scriptEditor updateActorBrowser.

	"Add an undo item to undo the creation of this object"
	myUndoStack push: (UndoAction new: [ baseActor removeFromScene.
											myNamespace removeKey: baseActor getName ifAbsent: [].
											scriptEditor updateActorBrowser.  ] ).

	^ baseActor.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 4/24/1999 05:39'!
fixNameFrom: aString
	"Fix the name to be a valid Smalltalk name (e.g., so that we can compile it as an inst var and accessor message)"

	| aName |
	aName _ aString select:[:c| c isAlphaNumeric].
	aName isEmpty ifTrue:[aName _ 'unkwown'].
	aName first isUppercase ifTrue:[aName _ aName first asLowercase asString, (aName copyFrom: 2 to: aName size)].
	aName first isLetter ifFalse:[aName _ 'a', aName].
	^aName! !

!Wonderland methodsFor: 'creating' stamp: 'ar 9/14/1999 20:02'!
makeTextureFrom: filename
	"Create a texture from the given file name"
	^self makeTextureFrom: filename flipping: false! !

!Wonderland class methodsFor: 'actor prototype mgmt' stamp: 'di 3/10/2000 08:49'!
rectifyActorPrototypeClasses  "Wonderland rectifyActorPrototypeClasses"
	"Rectify any obsolete prototype class references"

	ActorPrototypeClasses keysDo:
		[:key | (ActorPrototypeClasses at: key) isObsolete ifTrue:
				[ActorPrototypeClasses at: key
					put: (ActorPrototypeClasses at: key) nonObsoleteClass]]
! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:10'!
compileDefaultObjects
	"Wonderland compileDefaultObjects"
	| stream encoded snd |
	#(
		(groundMesh		'groundMesh.vfb')
		(cameraMesh		'cameraMesh.vfb')
		(lightMesh		'lightMesh.vfb')
	) do:[:spec|
		stream := (ByteArray new: 1000) writeStream.
		(WonderlandConstants default at: spec first) storeOnVFBStream: stream.
		encoded := (Base64MimeConverter mimeEncode: stream contents readStream) contents.
		Wonderland class compile: spec first,'	"Wonderland ', spec first,'"
	^WonderlandMesh new createFromVFBStream: (self decode: ''', encoded,''')'
	classified: 'default objects'
	].
	#(
		(groundTexture		'groundTexture.bmp')
		(cameraTexture		'cameraTexture.bmp')
		(lightTexture			'lightTexture.bmp')
		(camControlsBMP		'camControlsBMP.bmp')
	) do:[:spec|
		stream := (ByteArray new: 1000) writeStream.
		BMPReadWriter putForm: (WonderlandConstants default at: spec first) onStream: stream.
		encoded := (Base64MimeConverter mimeEncode: stream contents readStream) contents.
		Wonderland class compile: spec first,'	"Wonderland ', spec first,'"
	^Form fromBinaryStream: (self decode: ''', encoded,''')'
	classified: 'default objects'
	].

	#(
		(errorSound		'errorSound.wav')
	) do:[:spec|
		stream := (ByteArray new: 1000) writeStream.
		(WonderlandConstants default at: spec first) storeWAVSamplesOn: stream.
		"Okay, storing WAVs is just completely screwed up, so I do it twice to get rid of the problems."
		snd := SampledSound fromWaveStream: stream contents readStream.
		stream := (ByteArray new: 1000) writeStream.
		snd storeWAVSamplesOn: stream.
		encoded := (Base64MimeConverter mimeEncode: stream contents readStream) contents.
		Wonderland class compile: spec first,'	"Wonderland ', spec first,'"
	^SampledSound fromWaveStream: (self decode: ''', encoded,''')'
	classified: 'default objects'
	].
! !

!Wonderland methodsFor: 'drawing' stamp: 'jsp 2/8/1999 15:40'!
renderWonderland: aRenderer
	"Tell all the objects in the Wonderland to render themselves."

	sceneObject renderOn: aRenderer.

! !

!Wonderland methodsFor: 'undo' stamp: 'jsp 2/14/1999 18:43'!
undo
	"Undo the last action the user performed in the Wonderland.  This pulls a block context off the animation stack and executes it."

	myUndoStack popAndUndo.

	! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:05'!
decode: base64
	^(Base64MimeConverter mimeDecodeToBytes: base64 readStream) contents readStream! !

!Wonderland methodsFor: 'accessing' stamp: 'ar 11/16/2002 22:42'!
getScene
	"Returns the current scene object."

	^ sceneObject ifNil:[sceneObject _ WonderlandScene newFor: self].! !

!Wonderland methodsFor: 'creating' stamp: 'ar 4/24/1999 03:52'!
protoClassFor: aFilename
	| localName protoClass |
	localName _ FileDirectory localNameFor: aFilename.
	(ActorPrototypeClasses includesKey: localName)
			ifTrue: [ protoClass _ ActorPrototypeClasses at: localName ]
			ifFalse: ["Make a new prototype class for this model"
					protoClass _ (WonderlandActor newUniqueClassInstVars: '' classInstVars: '').
					ActorPrototypeClasses at: localName put: protoClass].
	^protoClass! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:05'!
lightMesh	"Wonderland lightMesh"
	^WonderlandMesh new createFromVFBStream: (self decode: 'AQAAADMAAABck8k8pwPZPa50LjkyHaI+xqhPP+4KnT3jxCs/JemqPsfXnjwjCoc9Gdx7PSl3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')! !

!Wonderland methodsFor: 'creating' stamp: 'ar 5/10/2001 22:53'!
makeCameraNamed: aString
	"Add a new camera to the Wonderland"
	| newClass newCamera name windowName |
	newClass _ WonderlandCamera newUniqueClassInstVars: '' classInstVars: ''.
	newCamera _ newClass createFor: self.
	newCamera getMorph openInWorld.
	actorClassList addLast: newClass.

	name _ self uniqueNameFrom: aString.
	newCamera setName: name.
	myNamespace at: name put: newCamera.

	windowName _ self uniqueNameFrom: aString,'Window'.
	myNamespace at: windowName put: (newCamera getMorph).

	cameraList addLast: newCamera.
	scriptEditor updateActorBrowser.

	"Add an undo action to remove this camera"
	myUndoStack push: (UndoAction new: [  cameraList remove: newCamera.
											newCamera removeFromScene.
											myNamespace removeKey: name ifAbsent: [].
											myNamespace removeKey: windowName.
											newCamera release.
											scriptEditor updateActorBrowser ]).

	^ newCamera.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 5/27/1999 16:13'!
makeActorNamed: aString
	"Creates a new actor without any geometry"

	| newClass newActor name |

	newClass _ WonderlandActor newUniqueClassInstVars: '' classInstVars: ''.
	newActor _ (newClass createFor: self).
	actorClassList addLast: newClass.

	scriptEditor ifNotNil: [ 
				name _ self uniqueNameFrom: aString.
				newActor setName: name.
				myNamespace at: name put: newActor.
				scriptEditor updateActorBrowser.
						].

	"Add an undo item to undo the creation of this object"
	myUndoStack push: (UndoAction new: [ newActor removeFromScene.
											myNamespace removeKey: name ifAbsent: [].
						 					scriptEditor updateActorBrowser. ]).

	^ newActor.
! !

!Wonderland methodsFor: 'creating' stamp: 'bf 5/9/2001 18:11'!
loadActor
	| strm filename |
	"Why does this open the file? Hrmpf."
	strm _ FileList2 modalFileSelectorForSuffixes: #('mdl' 'wrl' '3ds').
	strm ifNil: [^nil].
	filename _ strm name.
	strm close.
	(filename endsWith: 'mdl')
		ifTrue: [^ self makeActorFrom: filename].
	(filename endsWith: 'wrl')
		ifTrue: [^ self makeActorFromVRML: filename].
	(filename endsWith: '3ds')
		ifTrue: [^ self makeActorFrom3DS: filename].
	self error: 'Huh?'! !

!Wonderland methodsFor: 'accessing' stamp: 'jsp 4/8/1999 15:58'!
getActorClassList
	"Return the list of actor classes"

	^ actorClassList.
! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:08'!
initializeDefaultObjects
	"Wonderland initializeDefaultObjects"
	#(	groundMesh cameraMesh lightMesh 
		groundTexture cameraTexture lightTexture 
		errorSound camControlsBMP) do:[:each| 
			WonderlandConstants default at: each put: (self perform: each).
		].! !

!Wonderland methodsFor: 'initialize-reset-release' stamp: 'ar 3/4/2004 21:09'!
initialize
	"This method initializes the Wonderland."

	| ground |

	"Initialize the list of actor UniClasses"
	actorClassList _ OrderedCollection new.

	"Initialize the shared mesh and texture directories"
	sharedMeshDict _ Dictionary new.
	sharedTextureDict _ Dictionary new.

	"Initialize this Wonderland's shared namespace"
	myNamespace _ WonderlandConstants default copy.

	"Create the Wonderland's scheduler"
	myScheduler _ Scheduler new.
	myNamespace at: 'scheduler' put: myScheduler.

	"Create the undo stack for this Wonderland."
	myUndoStack _ WonderlandUndoStack new.

	"The scene object is the root of the object tree - all objects in the Wonderland are children (directly or indirectly) of the scene. "
	sceneObject _ WonderlandScene newFor: self.
	myNamespace at: 'scene' put: sceneObject.

	"Create a script editor for this Wonderland"
	self makeScriptEditor.

	"Create some default objects"
	ground _ self makeActor.
	ground setMesh: (WonderlandConstants default at: 'groundMesh').
	ground setTexturePointer: (WonderlandConstants default at: 'groundTexture').
	ground becomePart.
	ground setColor: {0.2902. 0.8000. 0.0000} duration: rightNow.
	ground setName: 'ground'.
	myNamespace at: 'ground' put: ground.

	"Initialize the light list and create a default light"
	lightList _ OrderedCollection new.
	self makeLight.
	lightList last moveTo: {0. 2. 0} duration: rightNow.
	
	"Create the default camera"
	cameraList _ OrderedCollection new.
	defaultCamera _ self makeCamera.

	"Clean out the Undo stack - we don't want people undoing the init stuff"
	myUndoStack reset.

	"Throw this Wonderland into the shared namespace"
	myNamespace at: 'w' put: self.

! !

!Wonderland methodsFor: 'accessing' stamp: 'jsp 4/9/1999 15:34'!
getCameraList

	^ cameraList.
! !

!Wonderland methodsFor: 'accessing' stamp: 'ar 11/16/2002 22:44'!
getSharedMeshDict
	"Return the shared mesh dictionary"

	^ sharedMeshDict ifNil:[sharedMeshDict := Dictionary new].! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:05'!
groundMesh	"Wonderland groundMesh"
	^WonderlandMesh new createFromVFBStream: (self decode: 'AQAAAAQAAAAAAPpDAAAAAAAA+kMAAAAAAACAPwAAAAAAAAAAAAAAAAAA+kMAAAAAAAD6wwAA
AAAAAIA/AAAAAAAAAAAAAMhCAAD6wwAAAAAAAPrDAAAAAAAAgD8AAAAAAADIQgAAyEIAAPrD
AAAAAAAA+kMAAAAAAACAPwAAAAAAAMhCAAAAAAIAAAAAAAAAAAAAAAMAAAAAAAAAAQAAAAIA
AAADAAAAAAAAAAIAAAADAAAA')! !

!Wonderland class methodsFor: 'actor prototype mgmt' stamp: 'ar 11/16/2002 19:20'!
removeActorPrototypesFromSystem
	"Clean out all the actor prototypes - this involves removing those classes from the Smalltalk dictionary"

	WonderlandActor withAllSubclassesDo:[:each|
		each isSystemDefined ifFalse:[each removeFromSystem].
	].
	ActorPrototypeClasses _ Dictionary new.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 8/19/1999 16:23'!
makeLight: lightType
	"Create a light of the specified type and add it to the Wonderland"
	^self makeLight: lightType named: nil! !

!Wonderland methodsFor: 'creating' stamp: 'jsp 7/8/1999 23:26'!
makeActor
	"Creates a new actor without any geometry"

	^ self makeActorNamed: 'unnamed'.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 8/19/1999 16:23'!
makeLight: lightType named: aString
	"Create a light of the specified type and add it to the Wonderland"

	| newClass theLight name |

	"Make sure the user gave us a type of light"
	[ WonderlandVerifier VerifyLight: lightType ]
		ifError: [ :msg :rcvr |
			self reportErrorToUser:
				'Squeak could not determine the type of light to create because ', msg.
			^ nil ].

	"The user gave us a valid type type, so proceed"
	(lightType = ambient)
		ifTrue: [ newClass _ WonderlandAmbientLight newUniqueClassInstVars: ''
									classInstVars: ''.
					theLight _ newClass createFor: self. ]
		ifFalse: [ (lightType = positional)
			ifTrue: [ newClass _ WonderlandPositionalLight newUniqueClassInstVars: ''
										classInstVars: ''.
					theLight _ newClass createFor: self. ]
			ifFalse: [ (lightType = directional)
				ifTrue: [ newClass _ WonderlandDirectionalLight newUniqueClassInstVars: ''
										classInstVars: ''.
						theLight _ newClass createFor: self. ]
				ifFalse: [ newClass _ WonderlandSpotLight newUniqueClassInstVars: ''
										classInstVars: ''.
						theLight _ newClass createFor: self. ]
					]
				].

	actorClassList addLast: newClass.
	name _ self uniqueNameFrom: (aString ifNil:['light']).
	theLight setName: name.
	myNamespace at: name put: theLight.
	scriptEditor updateActorBrowser.

	lightList addLast: theLight.

	^ theLight.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 5/27/1999 16:17'!
createSimpleActor: aMesh named: aString parent: parentActor
	| newActor |
	newActor _ self makeActorNamed: aString.
	aMesh ifNotNil:[newActor setMesh: aMesh].
	parentActor ifNotNil:[
		newActor reparentTo: parentActor.
		newActor becomePart].
	^newActor! !

!Wonderland class methodsFor: 'class initialization' stamp: 'ar 7/27/2003 23:12'!
unload

	FileList unregisterFileReader: self.
	(TheWorldMenu respondsTo: #registerOpenCommand:) ifTrue: 
		[TheWorldMenu unregisterOpenCommand: 'Wonderland 3D'].
! !

!Wonderland class methodsFor: 'fileIn/Out' stamp: 'sd 2/1/2002 22:11'!
services

	^ Array 
		with: self serviceOpenModelInEditor
		with: self serviceOpenInWonderland
	! !

!Wonderland methodsFor: 'creating' stamp: 'sw 9/26/2002 01:03'!
makePlaneNamed: aString
	"Create a simple plane"
	| tex vertices faces mesh |
	tex _ B3DTexture2Array new: 4.
	tex at: 1 put: (0@0); at: 2 put: (1@0); at: 3 put: (1@1); at: 4 put: (0@1).
	vertices _ B3DVector3Array new: 4.
	vertices  at: 1 put: -1@1@-1; at: 2 put: 1@1@-1; at: 3 put: 1@-1@-1; at: 4 put: -1@-1@-1.
	vertices do:[:vtx| vtx *= 0.5].
	faces _ B3DIndexedTriangleArray new: 2.
	faces at: 1 put: (B3DIndexedTriangle with: 1 with: 2 with: 3).
	faces at: 2 put: (B3DIndexedTriangle with: 3 with: 4 with: 1).
	mesh _ B3DIndexedTriangleMesh new.
	mesh vertices: vertices.
	mesh faces: faces.
	mesh texCoords: tex.
	^ self createSimpleActor: mesh named: aString parent: nil! !

!Wonderland methodsFor: 'accessing' stamp: 'ar 11/16/2002 22:44'!
getSharedTextureDict
	"Return the shared texture dictionary"

	^ sharedTextureDict ifNil:[sharedTextureDict _ Dictionary new].! !

!Wonderland methodsFor: 'initialize-reset-release' stamp: 'ar 11/5/2000 16:58'!
prepareToBeSaved
	"We are about to be written out. Release everything we don't want to carry around"
	myUndoStack reset.
	sharedMeshDict _ Dictionary new.
	sharedTextureDict _ Dictionary new.
! !

!Wonderland methodsFor: 'accessing' stamp: 'jsp 3/10/1999 16:41'!
getLights
	"Return the list of lights in the Wonderland"

	^ lightList.
! !

!Wonderland class methodsFor: 'fileIn/Out' stamp: 'SD 11/11/2001 14:16'!
openModelIntoAlice: fullName
	"If a Wonderland exists, load this model into it as an actor.  
	If it doesn't, make one first"

	| alice |
	Smalltalk isMorphic 
		ifFalse: [^self error: 'Only works in Morphic -  sorry!!'].
	alice _ World submorphs 
				detect: [:m | m isKindOf: WonderlandEditor] 
				ifNone: [nil].
	alice isNil 
		ifTrue: [alice := Wonderland new] 
		ifFalse: [alice := alice getWonderland].
	alice makeActorFrom: fullName.

	! !

!Wonderland methodsFor: 'creating' stamp: 'ar 5/10/2001 16:58'!
uniqueNameFrom: aString
	"If aName is unique to this wonderland's namespace, returns that name. Otherwise creates a unique variant and returns that."

	| index aName |
	aName _ self fixNameFrom: aString.
	(myNamespace includesKey: aName)
			ifFalse: [ ^ aName ].
	aName endsWithDigit ifTrue:[
		index _ aName findLast:[:ch| ch isDigit].
		aName _ (aName copyFrom: 1 to: index-1).
	].
	index _ 1.
	[ myNamespace includesKey: (aName , (index asString)) ]
		whileTrue: [ index _ index + 1 ].
	^ aName , (index asString).
! !

!Wonderland methodsFor: 'accessing' stamp: 'jsp 4/1/1999 16:17'!
getNamespace
	"Return the dictionary that comprises this Wonderland's namespace"

	^ myNamespace.
! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:05'!
cameraMesh	"Wonderland cameraMesh"
	^WonderlandMesh new createFromVFBStream: (self decode: 'AQAAAIoAAADAr7E9BfrkvahWb76wqio/sKoqv5+qqj4vFwE/JT3KO7OvsT0F+uS9aakQv+IF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')! !

!Wonderland methodsFor: 'creating' stamp: 'ar 3/14/2002 22:18'!
makeActorFrom: filename
	"Creates a new actor using the specification from the given file"

	| aFile words line startSubstr index parent name texture meshFile matrix baseActor newActor protoClass actorClass fileVersion pos |
	myUndoStack closeStack.

	words _ (filename findTokens: #.).
	((words last) = 'mdl') ifFalse: [^self].
	aFile _ (CrLfFileStream readOnlyFileNamed: filename) ascii.

	"First see if we need to create a prototype class for this model"
	(ActorPrototypeClasses includesKey: (aFile localName))
		ifTrue: [ protoClass _ ActorPrototypeClasses at: (aFile localName) ]
		ifFalse: ["Make a new prototype class for this model"
				protoClass _ (WonderlandActor newUniqueClassInstVars: ''
															classInstVars: '').
				ActorPrototypeClasses at: (aFile localName) put: protoClass].

	"Check what version this mdl file is"
	line _ aFile upTo: (Character cr).
	line _ aFile upTo: (Character cr).
	line _ aFile upTo: (Character cr).

	((line truncateTo: 7) = 'version')
		ifTrue: [ fileVersion _ 1 ]
		ifFalse: [ fileVersion _ 0 ].

	[ line _ aFile upTo: (Character cr).
	aFile atEnd] whileFalse:[
		words _ line findTokens: '='.
		"See if we're creating a new object"
		(((words size) > 1) and: [ ((words at: 2) beginsWith: ' _MakeObject')
			or: [ (words at: 2) beginsWith: ' Alice.MakeObject' ] ]) ifTrue: [
			(fileVersion = 0) ifTrue: [
				words _ line findTokens: #,.
				parent _ (words at: 2) withBlanksTrimmed.
				name _ (((words at: 3) withBlanksTrimmed) findBetweenSubStrs: '"') at: 1.
			] ifFalse: [
				name _ (words at: 1) truncateTo: (((words at: 1) size) - 1).
				parent _ ((words at: 3) findTokens: #,) at: 1.
			].

			"Now pull in the texture to use"
			startSubstr _ name , '.SetTexture'.
			pos _ aFile position.
			texture := nil.
			[aFile atEnd or:[texture notNil]] whileFalse:[
				line _ aFile upTo: (Character cr).
				(line beginsWith: startSubstr) ifTrue:[
					texture _ (line findBetweenSubStrs: '"') at: 2.
					texture _ (aFile directory pathName), FileDirectory slash, texture]].
			texture ifNil:[aFile position: pos].

			"Read the composite matrix to use"
			startSubstr _ name , '._SetLocalTransformation'.
			pos _ aFile position.
			matrix := nil.
			[aFile atEnd or:[matrix notNil]] whileFalse:[
				line _ aFile upTo: (Character cr).
				(line beginsWith: startSubstr) ifTrue:[
					matrix _ B3DMatrix4x4 new.
					words _ line findBetweenSubStrs: ',()'.
					words removeAllSuchThat: [:str | str = ' '].
					index _ words size.
					4 to: 1 by: -1 do: [:i | 
						4 to: 1 by: -1 do: [:j | matrix at: i at: j put: 
									((words at: index) withBlanksTrimmed) asNumber.
									   		index _ index - 1. ]. ].

					1 to: 4 do: [:i | index _ matrix at: i at: 4.
								   matrix at: i at: 4 put: (matrix at: 4 at: i).
								   matrix at: 4 at: i put: index. ].

					matrix a14: (matrix a14 negated).
				].
			].
			matrix ifNil:[aFile position: pos].

			"Read the mesh file to use"
			startSubstr _ 'LoadGeometry'.
			pos _ aFile position.
			meshFile := nil.
			[aFile atEnd or:[meshFile notNil]] whileFalse:[
				line _ aFile upTo: (Character cr).
				(line beginsWith: startSubstr) ifTrue:[
					meshFile _ (line findBetweenSubStrs: '"') at: 2.
					meshFile _ (aFile directory pathName), FileDirectory slash, meshFile.
				].
			].
			meshFile ifNil:[aFile position: pos].

			"Now build the actor name"
			words _ name findTokens: '.'.
			name _ words last.
			name at: 1 put: ((name at: 1) asLowercase).

			"Now build the parent name"
			parent _ parent copyReplaceAll: '.' with: ' '.

			"Now create the object"
			(parent = 'None') ifTrue: [
				actorClass _ protoClass newUniqueClassInstVars: '' classInstVars: ''.
				baseActor _ actorClass createFor: self.
				actorClassList addLast: actorClass.
				baseActor setName: name.
				texture ifNotNil:[baseActor setTexture: texture].
				meshFile ifNotNil:[baseActor loadMeshFromFile: meshFile].
				matrix ifNotNil:[baseActor setComposite: matrix].
				"end base actor creation"
			] ifFalse: [
				actorClass _ WonderlandActor newUniqueClassInstVars: ''
											classInstVars: ''.
				newActor _ actorClass createFor: self.
				actorClassList addLast: actorClass.
				newActor setName: name.
				parent _ (baseActor getChildNamed: parent).
				newActor reparentTo: parent.
				newActor becomePart.
				texture ifNotNil:[newActor setTexture: texture].
				meshFile ifNotNil:[newActor loadMeshFromFile: meshFile].
				matrix ifNotNil:[newActor setComposite: matrix].
				"end new actor with parent"
			].
		"end MakeObject parsing"
		].

	]. "end file parsing"

	aFile close.
	myUndoStack openStack.

	"Ensure that the new actor's name is unique"
	name _ self uniqueNameFrom: (baseActor getName).
	baseActor setName: name.
	myNamespace at: name put: baseActor.
	scriptEditor updateActorBrowser.

	"Add an undo item to undo the creation of this object"
	myUndoStack push: (UndoAction new: 
		[ baseActor removeFromScene.
		myNamespace removeKey: name ifAbsent: [].
		scriptEditor updateActorBrowser.  ] ).
	^ baseActor.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 5/10/2001 17:18'!
duplicateActor: anActor
	"Creates a new actor without any geometry"
	| newActor |
	newActor _ anActor clone postCopy.
	anActor getParent == nil ifTrue:[
		newActor setName: (self uniqueNameFrom: newActor getName).
		myNamespace at: newActor getName put: newActor.
	] ifFalse:[
		newActor setName: (anActor getParent uniqueNameFrom: newActor getName).
		anActor getParent addChild: newActor.
		newActor setParent: anActor getParent.
	].

	scriptEditor updateActorBrowser.

	"Add an undo item to undo the creation of this object"
	myUndoStack push: (UndoAction new: [ newActor removeFromScene.
						 					scriptEditor updateActorBrowser. ]).

	^ newActor.
! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:05'!
camControlsBMP	"Wonderland camControlsBMP"
	^Form fromBinaryStream: (self decode: 'Qk1mKgAAAAAAADYAAAAoAAAAOwAAADwAAAABABgAAAAAADAqAADwCgAA8AoAAAAAAAAAAAAA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')! !

!Wonderland methodsFor: 'creating' stamp: 'ar 9/14/1999 23:41'!
makeCameraNamed: aString parent: parentActor
	"Add a new camera to the Wonderland"
	| newCamera |
	newCamera _ self makeCameraNamed: aString.
	parentActor == nil ifFalse:[
		newCamera reparentTo: parentActor.
		newCamera becomePart].
	^newCamera! !

!Wonderland methodsFor: 'creating' stamp: 'ar 9/14/1999 19:40'!
makeBaseActorFrom: filename
	| protoClass actorClass baseActor name |
	"First see if we need to create a prototype class for this model"
	protoClass _ self protoClassFor: filename.
	"Base actor creation"
	actorClass _ protoClass newUniqueClassInstVars: '' classInstVars: ''.
	baseActor _ actorClass createFor: self.
	actorClassList addLast: actorClass.
	name _ FileDirectory localNameFor: filename.
	(name findString: FileDirectory dot) = 0 ifFalse:[
		name _ name copyFrom: 1 to: (name findString: FileDirectory dot)-1].
	baseActor setName: (self uniqueNameFrom: name).
	^baseActor! !

!Wonderland methodsFor: 'composite animations' stamp: 'jsp 2/14/1999 18:14'!
doInOrder: animations
	"Create a new SequentialAnimation instance and pass it the child animations that make it up."

	| newSA |
	newSA _ SequentialAnimation new.

	newSA children: animations undoable: true inWonderland: self.

	^ newSA.
! !

!Wonderland class methodsFor: 'fileIn/Out' stamp: 'sd 2/6/2002 21:37'!
fileReaderServicesForFile: fullName suffix: suffix

	^(suffix = 'wrl') | (suffix = '*') 
		ifTrue: [ Array with: self serviceOpenInWonderland]
		ifFalse: [(suffix = 'mdl') | (suffix = '*')
					ifTrue: [ Array with: self serviceOpenModelInEditor]
					ifFalse: [#()]]
! !

!Wonderland class methodsFor: 'accessing' stamp: 'ar 3/4/2004 21:10'!
wonderlandConstants
	^WonderlandConstants default ! !

!Wonderland methodsFor: 'creating' stamp: 'ar 11/5/2000 16:59'!
makeTextureFrom: filename flipping: aBool
	"Create a texture from the given file name and flip it in y if requested"
	| tex |
	filename == nil ifTrue:[^nil].
	^self getSharedTextureDict at: filename ifAbsentPut:[
		tex _ (Form fromFileNamed: filename) asTexture.
		aBool ifTrue:[tex flipVertically].
		tex wrap: true.
		tex interpolate: true.
		tex envMode: 0.
	].
! !

!Wonderland methodsFor: 'error handling' stamp: 'jsp 3/24/1999 10:15'!
reportErrorToUser: errorString
	"When any object in a Wonderland discovers an error it creates an error report and then calls this method to display the error to the user."

	| errWin tm |

	errWin _ SystemWindowWithButton labelled: 'Ooops'.
	errWin openInWorldExtent: 400@100.
	errWin color: (Color white).

	tm _ TextMorph new.
	tm initialize.
	errWin addMorph: tm.

	tm color: (Color red).
	tm contents: errorString wrappedTo: 380.
	tm position: ((errWin position) + (10@20)).
	tm lock.

	errWin height: (tm height) + 30.

	errorSound play.



! !

!Wonderland methodsFor: 'initialize-reset-release' stamp: 'jsp 4/23/1999 12:11'!
release
	"This method cleans up the Wonderland."

	cameraList do: [:camera | camera release].

	scriptEditor delete.

	"Clean up any uniclasses we created"
	actorClassList do: [:aClass | aClass removeFromSystem ].
! !

!Wonderland class methodsFor: 'fileIn/Out' stamp: 'hg 8/3/2000 16:21'!
openVRMLFile: fullName
	
	Wonderland new makeActorFromVRML: fullName.
! !

!Wonderland methodsFor: 'creating' stamp: 'jsp 4/29/1999 10:55'!
makeLight
	"Create a positional light and add it to the Wonderland"

	^ self makeLight: positional.
! !

!Wonderland methodsFor: 'accessing' stamp: 'jsp 2/5/1999 16:30'!
getDefaultCamera
	"Returns a handle on the Wonderland's default camera."

	^ defaultCamera.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 8/31/2000 18:40'!
makeActorFromVRML: filename
	"Creates a new actor using the specification from the given file"
	| baseActor parserClass builderClass scene |

	parserClass _ Smalltalk at: #VRMLNodeParser ifAbsent:[
		self reportErrorToUser:'No VRML parser found'.
		^self].
	builderClass _ Smalltalk at: #VRMLWonderlandBuilder ifAbsent:[
		self reportErrorToUser:'No VRML to Alice converter found'.
		^self].
	myUndoStack closeStack.
	baseActor _ self makeBaseActorFrom: filename.
	parserClass == nil
		ifFalse:[scene _ parserClass parseFileNamed: filename].
	(builderClass == nil or:[scene == nil])
		ifFalse:[(builderClass on: scene) 
					baseActor: baseActor;
					buildActorsFor: self].
	scene _ nil.
	myUndoStack openStack.

	"Ensure that the new actor's name is unique"
	myNamespace at: baseActor getName put: baseActor.
	scriptEditor updateActorBrowser.

	"Add an undo item to undo the creation of this object"
	myUndoStack push: (UndoAction new: [ baseActor removeFromScene.
											myNamespace removeKey: baseActor getName ifAbsent: [].
											scriptEditor updateActorBrowser.  ] ).

	^ baseActor.
! !

!Wonderland methodsFor: 'accessing' stamp: 'jsp 4/1/1999 17:36'!
getEditor
	"Return the editor for this wonderland"

	^ scriptEditor.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 4/24/1999 04:48'!
createHierarchyFrom3DS: globals
	| keyframes objects hierarchy name parentID parentName parent |
	hierarchy _ Dictionary new.
	keyframes _ globals at: #keyframes ifAbsent:[^hierarchy].
	objects _ keyframes at: #objects ifAbsent:[^hierarchy].
	objects do:[:obj|
		name _ obj at: #name.
		parentID _ obj at: #hierarchy.
		parent _ objects at: parentID ifAbsent:[nil].
		parent == nil ifFalse:[
			parentName _ parent at: #name.
			name = parentName
				ifFalse:[hierarchy at: name put: parentName]].
	].
	^hierarchy! !

!Wonderland methodsFor: 'creating' stamp: 'ar 11/16/2002 22:44'!
makeActorFromPath: filePath
	"Creates a new actor using the specification from the given file"
	| aFile words line startSubstr index parent name texture meshFile matrix baseActor newActor protoClass actorClass fileVersion pos filename |
	filename _ filePath inject: FileDirectory default pathName into:[:fp :p| fp, FileDirectory slash, p].
	aFile _ (CrLfFileStream readOnlyFileNamed: filename) ascii.

	"First see if we need to create a prototype class for this model"
	protoClass _ WonderlandActor.

	"Check what version this mdl file is"
	line _ aFile upTo: (Character cr).
	line _ aFile upTo: (Character cr).
	line _ aFile upTo: (Character cr).

	((line truncateTo: 7) = 'version')
		ifTrue: [ fileVersion _ 1 ]
		ifFalse: [ fileVersion _ 0 ].

	[ line _ aFile upTo: (Character cr).
	aFile atEnd] whileFalse:[
		words _ line findTokens: '='.
		"See if we're creating a new object"
		(((words size) > 1) and: [ ((words at: 2) beginsWith: ' _MakeObject')
			or: [ (words at: 2) beginsWith: ' Alice.MakeObject' ] ]) ifTrue: [
			(fileVersion = 0) ifTrue: [
				words _ line findTokens: #,.
				parent _ (words at: 2) withBlanksTrimmed.
				name _ (((words at: 3) withBlanksTrimmed) findBetweenSubStrs: '"') at: 1.
			] ifFalse: [
				name _ (words at: 1) truncateTo: (((words at: 1) size) - 1).
				parent _ ((words at: 3) findTokens: #,) at: 1.
			].

			"Now pull in the texture to use"
			startSubstr _ name , '.SetTexture'.
			pos _ aFile position.
			texture := nil.
			[aFile atEnd or:[texture notNil]] whileFalse:[
				line _ aFile upTo: (Character cr).
				(line beginsWith: startSubstr) ifTrue:[
					texture _ (line findBetweenSubStrs: '"') at: 2.
					texture _ (aFile directory pathName), FileDirectory slash, texture]].
			texture ifNil:[aFile position: pos].

			"Read the composite matrix to use"
			startSubstr _ name , '._SetLocalTransformation'.
			pos _ aFile position.
			matrix := nil.
			[aFile atEnd or:[matrix notNil]] whileFalse:[
				line _ aFile upTo: (Character cr).
				(line beginsWith: startSubstr) ifTrue:[
					matrix _ B3DMatrix4x4 new.
					words _ line findBetweenSubStrs: ',()'.
					words removeAllSuchThat: [:str | str = ' '].
					index _ words size.
					4 to: 1 by: -1 do: [:i | 
						4 to: 1 by: -1 do: [:j | matrix at: i at: j put: 
									((words at: index) withBlanksTrimmed) asNumber.
									   		index _ index - 1. ]. ].

					1 to: 4 do: [:i | index _ matrix at: i at: 4.
								   matrix at: i at: 4 put: (matrix at: 4 at: i).
								   matrix at: 4 at: i put: index. ].

					matrix a14: (matrix a14 negated).
				].
			].
			matrix ifNil:[aFile position: pos].

			"Read the mesh file to use"
			startSubstr _ 'LoadGeometry'.
			pos _ aFile position.
			meshFile := nil.
			[aFile atEnd or:[meshFile notNil]] whileFalse:[
				line _ aFile upTo: (Character cr).
				(line beginsWith: startSubstr) ifTrue:[
					meshFile _ (line findBetweenSubStrs: '"') at: 2.
					meshFile _ (aFile directory pathName), FileDirectory slash, meshFile.
				].
			].
			meshFile ifNil:[aFile position: pos].

			"Now build the actor name"
			words _ name findTokens: '.'.
			name _ words last.
			name at: 1 put: ((name at: 1) asLowercase).

			"Now build the parent name"
			parent _ parent copyReplaceAll: '.' with: ' '.

			"Now create the object"
			(parent = 'None') ifTrue: [
				actorClass _ protoClass newUniqueClassInstVars: '' classInstVars: ''.
				baseActor _ actorClass createFor: self.
				baseActor setName: name.
				texture ifNotNil:[baseActor setTexture: texture].
				meshFile ifNotNil:[baseActor loadMeshFromFile: meshFile].
				matrix ifNotNil:[baseActor setComposite: matrix].
				"end base actor creation"
			] ifFalse: [
				actorClass _ WonderlandActor newUniqueClassInstVars: ''
											classInstVars: ''.
				newActor _ actorClass createFor: self.
				newActor setName: name.
				parent _ (baseActor getChildNamed: parent).
				newActor reparentTo: parent.
				newActor becomePart.
				texture ifNotNil:[newActor setTexture: texture].
				meshFile ifNotNil:[newActor loadMeshFromFile: meshFile].
				matrix ifNotNil:[newActor setComposite: matrix].
				"end new actor with parent"
			].
		"end MakeObject parsing"
		].

	]. "end file parsing"

	aFile close.
	^ baseActor.! !

!Wonderland methodsFor: 'initialize-reset-release' stamp: 'ar 3/4/2004 21:10'!
reset
	"Reset this Wonderland"

	| ground |

	"Reset the scheduler"
	myScheduler reset.

	"Reset the shared mesh and texture directories"
	sharedMeshDict _ Dictionary new.
	sharedTextureDict _ Dictionary new.

	"Reset the list of actor uniclasses"
	actorClassList do: [:aClass | aClass removeFromSystem ].
	actorClassList _ OrderedCollection new.

	"Initialize this Wonderland's shared namespace"
	myNamespace _ WonderlandConstants default copy.

	"Rebuild the namespace"
	myNamespace at: 'scheduler' put: myScheduler.
	myNamespace at: 'w' put: self.

	"Create a new scene"
	sceneObject _ WonderlandScene newFor: self.
	myNamespace at: 'scene' put: sceneObject.

	"Recreate the default objects"
	ground _ self makeActor.
	ground setMesh: (WonderlandConstants default at: 'groundMesh').
	ground setTexturePointer: (WonderlandConstants default at: 'groundTexture').
	ground becomePart.
	ground setName: 'ground'.
	ground setColor: {0.2902. 0.8000. 0.0000} duration: rightNow.
	myNamespace at: 'ground' put: ground.

	"Re-initialize the light list and create a default light"
	lightList _ OrderedCollection new.
	self makeLight.
	lightList last moveTo: {0. 2. 0} duration: rightNow.

	"Wipe out the existing cameras"
	cameraList do: [:camera | camera release].
	
	"Recreate the default camera"
	cameraList _ OrderedCollection new.
	defaultCamera _ self makeCamera.

	"Reset the script editor's namespace"
	scriptEditor resetNamespace.

	"Update the actor browser"
	scriptEditor updateActorBrowser.

	"Reset the undo stack"
	myUndoStack reset.


! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:05'!
lightTexture	"Wonderland lightTexture"
	^Form fromBinaryStream: (self decode: 'Qk02MAAAAAAAADYAAAAoAAAAQAAAAEAAAAABABgAAAAAAAAwAADwCgAA8AoAAAAAAAAAAAAA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')! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:05'!
errorSound	"Wonderland errorSound"
	^SampledSound fromWaveStream: (self decode: 'UklGRnA8AABXQVZFZm10IBAAAAABAAEAIlYAAESsAAAEABAAZGF0YUw8AAAGAgYCBgKgAKAA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')! !

!Wonderland methodsFor: 'initialize-reset-release' stamp: 'ar 11/5/2000 16:58'!
objectForDataStream: refStrm
	self prepareToBeSaved.		"Amen"
	^ self
! !

!Wonderland methodsFor: 'accessing' stamp: 'ar 11/16/2002 22:43'!
getUndoStack
	"Returns the Wonderland's undo stack."

	^ myUndoStack ifNil:[myUndoStack _ WonderlandUndoStack new].! !

!Wonderland class methodsFor: 'fileIn/Out' stamp: 'sw 2/17/2002 02:48'!
serviceOpenModelInEditor
	"Answer a service for opening an alice model in an editor"

	^ SimpleServiceEntry 
		provider: self 
		label: 'open model in editor'
		selector: #openModelIntoAlice:
		description: 'open model in editor'
		buttonLabel: 'open'! !

!Wonderland class methodsFor: 'instance creation' stamp: 'di 4/20/1999 12:03'!
new   "Wonderland new" 
	"Create and initialize a new Wonderland."
	B3DPrimitiveEngine isAvailable ifFalse: [
	(self confirm: 'WARNING: This Squeak does not have real 3D support.
Opening a Wonderland will EXTREMELY time consuming.
Are you sure you want to do this?
(NO is probably the right answer :-)') ifFalse: [^ self]].
	Display depth < 8 ifTrue:
		[(self confirm: 'The display depth should be set to at least 8 bit.
Shall I do this now for you?') ifTrue: [Display newDepth: 8]].
	^ super new initialize
! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:05'!
groundTexture	"Wonderland groundTexture"
	^Form fromBinaryStream: (self decode: 'Qk02MAAAAAAAADYAAAAoAAAAQAAAAEAAAAABABgAAAAAAAAwAADwCgAA8AoAAAAAAAAAAAAA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')! !

!Wonderland class methodsFor: 'default objects' stamp: 'ar 3/4/2004 21:05'!
cameraTexture	"Wonderland cameraTexture"
	^Form fromBinaryStream: (self decode: 'Qk02MAAAAAAAADYAAAAoAAAAQAAAAEAAAAABABgAAAAAAAAwAADwCgAA8AoAAAAAAAAAAAAA
ICAgGBgYGBgYGBgYGBgYGBgYGBgYEBAQGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgY
ICAgICAgICAgICAgGBgYICAgGBgYKCgoODg4SEhISEhISEhISEhIUFBQSEhIUFBQSEhIUFBQ
SEhIUFBQSEhIUFBQUFBQUFBQUFBQUFBQUFBQSEhIQEBAaGhoiIiIaGhocHBwYGBgiIiIgICA
eHh4gICAgICAgICAgICAgICAcHBwgICAgICAgICAGBgYICAgGBgYICAgGBgYGBgYICAgGBgY
GBgYGBgYGBgYGBgYGBgYGBgYICAgGBgYICAgICAgICAgICAgICAgICAgKCgoICAgICAgICAg
ODg4QEBAQEBAQEBAQEBASEhIQEBASEhISEhISEhIQEBASEhIQEBASEhIQEBASEhISEhIQEBA
QEBAQEBAQEBAQEBAeHh4WFhYWFhYUFBQiIiIeHh4aGhoeHh4eHh4eHh4eHh4eHh4YGBgeHh4
cHBwcHBwGBgYEBAQSEhIGBgYICAgGBgYGBgYGBgYGBgYGBgYICAgGBgYGBgYGBgYGBgYICAg
ICAgICAgICAgKCgoICAgGBgYQEBAICAgICAgKCgoMDAwMDAwMDAwODg4MDAwODg4MDAwODg4
ODg4ODg4QEBAQEBAODg4ODg4MDAwMDAwKCgoKCgoMDAwMDAwKCgoKCgocHBwQEBAMDAwODg4
aGhoaGhoUFBQYGBgYGBgYGBgYGBgYGBgUFBQYGBgYGBgYGBgICAgICAgGBgYICAgGBgYGBgY
GBgYGBgYICAgGBgYGBgYICAgGBgYMDAwMDAwODg4QEBAQEBAKCgoICAgMDAwKCgoKCgoKCgo
KCgoKCgoMDAwKCgoGBgYGBgYGBgYICAgICAgKCgoMDAwODg4ODg4MDAwMDAwICAgICAgGBgY
ICAgGBgYGBgYKCgoKCgoKCgoUFBQMDAwICAgICAgWFhYWFhYQEBAUFBQWFhYUFBQUFBQUFBQ
SEhIWFhYUFBQUFBQICAgGBgYICAgGBgYICAgGBgYICAgICAgGBgYICAgGBgYICAgGBgYKCgo
GBgYGBgYEBAQODg4ICAgMDAwKCgoMDAwKCgoKCgoKCgoKCgoICAgGBgYGBgYGBgYGBgYEBAQ
GBgYGBgYICAgKCgoMDAwMDAwKCgoICAgGBgYGBgYEBAQGBgYGBgYGBgYMDAwKCgoQEBAICAg
EBAQICAgSEhISEhIODg4UFBQSEhISEhISEhISEhIODg4SEhISEhISEhIICAgGBgYICAgGBgY
ICAgICAgICAgICAgICAgICAgGBgYICAgGBgYKCgoGBgYGBgYGBgYODg4KCgoKCgoMDAwKCgo
MDAwKCgoICAgKCgoGBgYGBgYEBAQGBgYEBAQGBgYGBgYEBAQICAgICAgICAgICAgICAgGBgY
GBgYEBAQGBgYEBAQGBgYGBgYICAgMDAwODg4ICAgEBAQEBAQQEBAQEBAMDAwODg4ODg4MDAw
MDAwODg4KCgoMDAwODg4MDAwICAgGBgYICAgGBgYICAgICAgICAgICAgICAgICAgGBgYICAg
GBgYICAgGBgYGBgYGBgYODg4KCgoKCgoMDAwKCgoKCgoICAgICAgKCgoICAgEBAQGBgYGBgY
GBgYGBgYGBgYGBgYICAgICAgICAgICAgGBgYGBgYEBAQEBAQEBAQGBgYEBAQGBgYGBgYKCgo
ODg4ICAgCAgIEBAQMDAwKCgoICAgKCgoKCgoICAgKCgoICAgGBgYKCgoKCgoKCgoICAgGBgY
ICAgGBgYICAgICAgICAgICAgICAgICAgICAgGBgYGBgYKCgoMDAwMDAwMDAwODg4KCgoMDAw
KCgoICAgICAgICAgEBAQICAgICAgGBgYGBgYEBAQEBAQEBAQGBgYGBgYICAgKCgoKCgoICAg
ICAgICAgGBgYGBgYGBgYEBAQEBAQGBgYGBgYKCgoODg4GBgYCAgIEBAQICAgICAgGBgYICAg
GBgYICAgGBgYICAgGBgYICAgICAgICAgICAgGBgYICAgGBgYICAgGBgYICAgICAgGBgYICAg
GBgYICAgICAgICAgICAgICAgKCgoKCgoKCgoKCgoKCgoGBgYGBgYEBAQEBAQGBgYICAgKCgo
GBgYGBgYGBgYGBgYICAgKCgoMDAwKCgoMDAwMDAwMDAwMDAwKCgoKCgoICAgICAgICAgKCgo
KCgoICAgICAgEBAQCAgIEBAQGBgYICAgGBgYICAgICAgKCgoKCgoKCgoICAgKCgoKCgoKCgo
ICAgICAgGBgYICAgGBgYICAgICAgICAgICAgGBgYICAgICAgICAgICAgKCgoICAgKCgoKCgo
KCgoKCgoKCgoICAgGBgYEBAQEBAQICAgODg4QEBAQEBAQEBAQEBAQEBAQEBAQEBAODg4QEBA
ODg4QEBAQEBAQEBAQEBAQEBAQEBAQEBAODg4QEBAMDAwKCgoGBgYGBgYEBAQEBAQEBAQGBgY
GBgYICAgKCgoMDAwODg4ODg4ODg4QEBAQEBAODg4ICAgGBgYICAgICAgICAgICAgICAgICAg
ICAgGBgYICAgICAgICAgICAgICAgICAgKCgoKCgoKCgoKCgoICAgICAgGBgYEBAQEBAQKCgo
ODg4SEhISEhISEhISEhISEhISEhISEhISEhIUFBQSEhIUFBQSEhIUFBQUFBQUFBQUFBQSEhI
UFBQSEhIODg4GBgYEBAQCAgIEBAQCAgICAgIAQAACAgICAgICAgICAgICAgICAgIAQAAAQAA
AQAAUFBQICAgICAgICAgGBgYICAgICAgICAgICAgICAgICAgGBgYICAgGBgYICAgICAgICAg
KCgoICAgKCgoKCgoICAgGBgYICAgGBgYEBAQGBgYKCgoMDAwODg4ODg4ODg4QEBAODg4QEBA
ODg4ODg4QEBAQEBASEhISEhIQEBAQEBAODg4ODg4ODg4QEBAICAgEBAQCAgICAgIAQAACAgI
AQAACAgICAgICAgICAgICAgICAgIAQAAAQAAAQAAAQAASEhIICAgGBgYICAgGBgYICAgGBgY
ICAgGBgYICAgGBgYICAgGBgYICAgICAgICAgICAgICAgICAgKCgoICAgICAgICAgICAgGBgY
GBgYEBAQICAgGBgYGBgYICAgICAgICAgICAgICAgICAgICAgKCgoICAgKCgoICAgICAgICAg
KCgoICAgICAgKCgoICAgGBgYGBgYEBAQEBAQGBgYGBgYICAgICAgKCgoMDAwODg4MDAwSEhI
WFhYYGBgaGhoSEhIICAgICAgGBgYICAgGBgYICAgICAgGBgYICAgICAgGBgYICAgICAgICAg
ICAgGBgYICAgICAgICAgICAgICAgICAgICAgGBgYGBgYGBgYEBAQEBAQCAgICAgIEBAQEBAQ
EBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQGBgYICAgICAgGBgYEBAQEBAQEBAQ
GBgYGBgYGBgYGBgYICAgKCgoMDAwMDAwODg4SEhISEhIWFhYYGBgSEhIICAgGBgYICAgGBgY
ICAgGBgYGBgYGBgYICAgGBgYGBgYICAgICAgGBgYGBgYICAgICAgICAgICAgICAgICAgICAg
GBgYGBgYGBgYGBgYEBAQEBAQCAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgI
CAgICAgICAgIEBAQEBAQICAgEBAQEBAQEBAQEBAQGBgYGBgYGBgYICAgICAgICAgKCgoMDAw
ODg4QEBASEhIWFhYYGBgSEhIGBgYICAgGBgYICAgGBgYGBgYGBgYICAgGBgYGBgYICAgGBgY
ICAgGBgYICAgGBgYICAgICAgGBgYICAgICAgGBgYICAgGBgYGBgYGBgYEBAQEBAQAQAAEBAQ
EBAQEBAQEBAQEBAQEBAQEBAQCAgIEBAQCAgIEBAQCAgIEBAQCAgIEBAQEBAQEBAQGBgYEBAQ
EBAQGBgYEBAQGBgYGBgYGBgYICAgICAgKCgoMDAwODg4QEBAQEBAUFBQWFhYQEBAICAgEBAQ
QEBAGBgYICAgICAgGBgYICAgGBgYICAgGBgYICAgGBgYGBgYGBgYICAgGBgYGBgYICAgGBgY
GBgYEBAQQEBAGBgYGBgYGBgYEBAQEBAQAQAAEBAQCAgIEBAQEBAQEBAQEBAQEBAQEBAQEBAQ
EBAQEBAQEBAQCAgIEBAQEBAQCAgIEBAQEBAQGBgYGBgYGBgYGBgYGBgYICAgGBgYICAgGBgY
ICAgICAgICAgICAgICAgKCgoMDAwODg4GBgYGBgYGBgYGBgYICAgGBgYGBgYGBgYGBgYICAg
GBgYGBgYGBgYICAgGBgYGBgYICAgGBgYGBgYGBgYGBgYGBgYGBgYEBAQEBAQGBgYEBAQEBAQ
AQAAEBAQEBAQEBAQCAgIEBAQCAgIEBAQCAgIEBAQCAgIEBAQCAgIEBAQEBAQCAgIEBAQEBAQ
GBgYGBgYICAgGBgYICAgICAgICAgICAgICAgICAgICAgICAgICAgGBgYICAgICAgICAgKCgo
GBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYEBAQGBgYEBAQGBgY
GBgYGBgYGBgYEBAQGBgYEBAQEBAQGBgYEBAQEBAQAQAACAgIEBAQCAgIEBAQEBAQEBAQEBAQ
EBAQEBAQEBAQCAgIEBAQEBAQCAgIEBAQEBAQEBAQGBgYGBgYICAgICAgMDAwKCgoMDAwMDAw
MDAwMDAwICAgICAgICAgGBgYICAgGBgYICAgICAgGBgYGBgYICAgICAgICAgICAgICAgICAg
ICAgICAgICAgGBgYICAgGBgYICAgGBgYICAgICAgGBgYICAgGBgYICAgGBgYGBgYGBgYGBgY
EBAQEBAQAQAAEBAQEBAQEBAQCAgIEBAQCAgIEBAQCAgIEBAQCAgIEBAQEBAQCAgIEBAQEBAQ
EBAQGBgYGBgYICAgMDAwMDAwQEBAODg4ODg4ODg4ODg4MDAwMDAwMDAwKCgoICAgICAgGBgY
ICAgGBgYICAgEBAQQEBAICAgICAgICAgICAgICAgICAgKCgoICAgKCgoICAgICAgICAgICAg
ICAgICAgICAgICAgICAgCAgIQEBAICAgGBgYGBgYEBAQEBAQAQAACAgIEBAQCAgIEBAQEBAQ
EBAQEBAQEBAQEBAQEBAQEBAQCAgIEBAQEBAQCAgIGBgYGBgYICAgODg4QEBASEhIODg4KCgo
ICAgICAgICAgKCgoKCgoMDAwMDAwKCgoICAgGBgYICAgGBgYICAgGBgYICAgKCgoICAgKCgo
KCgoKCgoKCgoKCgoKCgoKCgoKCgoICAgKCgoICAgICAgICAgICAgICAgICAgICAgICAgICAg
GBgYGBgYEBAQEBAQEBAQEBAQEBAQEBAQEBAQCAgIEBAQCAgIEBAQCAgIEBAQCAgIEBAQEBAQ
CAgIEBAQGBgYICAgODg4QEBASEhIMDAwICAgCAgIEBAQCAgIEBAQEBAQGBgYKCgoMDAwKCgo
ICAgICAgGBgYICAgICAgGBgYKCgoICAgKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgo
ICAgICAgICAgICAgICAgICAgICAgICAgICAgGBgYICAgGBgYGBgYGBgYGBgYEBAQCAgIEBAQ
CAgIEBAQEBAQEBAQCAgIEBAQEBAQEBAQEBAQEBAQEBAQEBAQICAgMDAwQEBAQEBAMDAwEBAQ
CAgICAgICAgICAgIGBgYGBgYICAgICAgICAgKCgoKCgoICAgGBgYICAgGBgYICAgICAgICAg
KCgoKCgoKCgoKCgoKCgoKCgoICAgKCgoKCgoKCgoKCgoICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgGBgYICAgMDAwMDAwMDAwMDAwMDAwMDAwMDAwKCgoKCgoICAgGBgYEBAQEBAQ
EBAQEBAQGBgYICAgKCgoODg4QEBAKCgoEBAQAQAACAgIEAgICAgIEBAQMDAwUFBQWFhYQEBA
ICAgICAgKCgoICAgICAgGBgYICAgGBgYICAgKCgoICAgICAgICAgKCgoICAgGBgYGBgYICAg
ICAgKCgoICAgKCgoICAgICAgICAgICAgICAgICAgICAgICAgGBgYAQAAAQAAAQAAAQAAAQAA
AQAAQEBAQEBAODg4ODg4KCgoICAgEBAQEBAQEBAQCAgIEBAQEBAQICAgMDAwQEBAMDAwGBgY
AQAACAgIEAgIIBAQEAgIGBgYUFBQcHBwgICAYGBgKCgoEBAQICAgKCgoICAgICAgGBgYICAg
ICAgICAgGBgYGBgYEBAQGBgYEBAQEBAQEBAQGBgYGBgYKCgoKCgoKCgoKCgoICAgICAgICAg
ICAgICAgICAgICAgGBgYCAgIICAgICAgICAgGBgYGBgYQEBAMDAwMDAwKCgoGBgYEBAQAQAA
AQAAAQAAAQAAAQAAEBAQKCgoMDAwODg4ICAgCAgICAgIEAgIIBgYIBAQCAgIICAgSEhIeHh4
gICAYGBgKCgoCAgIGBgYMDAwKCgoICAgGBgYICAgGBgYGBgYEBAQCAgICAgICAgICAgICAgI
EBAQGBgYICAgKCgoKCgoKCgoKCgoKCgoICAgICAgICAgGBgYICAgGBgYGBgYODg4YGBgaGho
YGBgYGBgUFBQGBgYKCgoKCgoICAgGBgYEBAQAQAAAQAAAQAAAQAAAQAAEBAQICAgMDAwODg4
GBgYCAgIEAgIKBgYIBgYEBAQCAgIGBgYODg4UFBQUFBQQEBAGBAQCAgIGBgYMDAwKCgoGBgY
GBgYGBgYEBAQGBgYCAgIEBAQCAgICAgICAgICAgIEBAQKCgoKCgoMDAwKCgoKCgoKCgoKCgo
ICAgICAgGBgYGBgYGBgYGBgYGBgYODg4UFBQUFBQUFBQUFBQSEhIEBAQKCgoICAgICAgGBgY
EBAQAQAAAQAAAQAAAQAAAQAAEBAQGBgYMDAwMDAwGBgYAQAAGBAQIBAQGBAQCAgIEBAQEBAQ
EBAQICAgGBgYGBAQEBAQCAgIIBgYMDAwKCgoICAgGBgYGBgYEBAQEBAQEBAQCAgICAgICAgI
CAgIAQAAEBAQGBgYMDAwKCgoKCgoKCgoKCgoKCgoICAgICAgICAgGBgYGBgYGBgYGBgYODg4
QEBAQEBAQEBAQEBAMDAwEBAQICAgKCgoICAgICAgGBgYAQAAAQAAAQAAAQAAAQAAEBAQGBgY
KCgoMDAwGBgYCAgIGBAQIBAQEBAQCAgIEBAQEBAQEBAQGBgYODg4KCAgIBAQGBAQKCgoODg4
MDAwGBgYICAgGBgYGBgYEBAQGBgYCAgICAgICAgICAgICAgICAgIICAgMDAwKCgoMDAwKCgo
KCgoKCgoICAgICAgICAgGBgYGBgYGBgYEBAQODg4ODg4ODg4ODg4ODg4MDAwGBgYKCgoICAg
ICAgGBgYICAgGBgYGBgYEBAQGBgYGBgYGBgYGBgYICAgMDAwICAgEBAQEAgIGBAQEBAQEAgI
EBAQEBAQCAgIODg4YGBgSEBAGBAQICAgODg4QEBAMDAwICAgGBgYICAgGBgYGBgYEBAQEBAQ
CAgICAgICAgICAgIEBAQICAgKCgoKCgoKCgoKCgoKCgoKCgoICAgICAgGBgYGBgYGBgYGBgY
GBgYKCgoKCgoKCgoKCgoKCgoICAgCAgIGBgYGBgYGBgYICAgGBgYICAgGBgYGBgYGBgYGBgY
ICAgGBgYICAgKCgoKCgoGBgYEAgIEAgICAgICAgIEBAQCAgICAgIICAgSEBAKCAgGBgYODg4
UFBQQEBAKCgoGBgYICAgGBgYICAgGBgYGBgYEBAQCAgIEBAQEBAQEBAQGBgYGBgYICAgICAg
ICAgKCgoKCgoKCgoICAgICAgICAgICAgICAgGBgYGBgYICAgICAgICAgICAgICAgGBgYAQAA
GBgYGBgYGBgYGBgYICAgGBgYGBgYGBgYICAgGBgYICAgGBgYICAgICAgKCgoKCgoGBgYAQAA
CAgICAgICAgICAgIAQAACAgIEAgICAgIKCgoUFBQUFBQODg4ICAgICAgGBgYICAgICAgICAg
ICAgGBgYGBgYGBgYGBgYICAgGBgYICAgGBgYICAgICAgKCgoICAgKCgoICAgKCgoICAgICAg
ICAgICAgICAgGBgYGBgYGBgYGBgYGBgYEBAQAQAAGBgYGBgYGBgYGBgYICAgGBgYICAgGBgY
ICAgGBgYICAgICAgICAgICAgKCgoMDAwKCgoEBAQEBAQCAgICAgICAgIAQAACAgICAgIICAg
QEBASEhIODg4KCgoICAgGBgYICAgGBgYICAgICAgICAgICAgICAgKCgoICAgMDAwKCgoKCgo
KCgoICAgKCgoKCgoICAgKCgoICAgICAgICAgICAgICAgICAgICAgEBAQGBgYGBgYGBgYGBgY
EBAQAQAAEBAQEBAQGBgYICAgICAgICAgICAgICAgICAgICAgICAgKCgoICAgICAgICAgMDAw
MDAwMDAwICAgEBAQCAgICAgICAgIGBgYICAgODg4QEBAODg4KCgoICAgGBgYICAgGBgYICAg
ICAgICAgKCgoICAgKCgoICAgKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoICAgKCgoICAgICAg
ICAgICAgICAgGBgYICAgEBAQGBgYEBAQEBAQEBAQCAgIAQAAEBAQEBAQGBgYGBgYICAgICAg
ICAgICAgICAgICAgKCgoICAgKCgoKCgoICAgKCgoMDAwODg4MDAwMDAwICAgKCgoKCgoMDAw
MDAwQEBAODg4MDAwICAgICAgGBgYICAgICAgCAgIQEBAICAgICAgICAgICAgICAgKCgoICAg
KCgoKCgoICAgKCgoICAgKCgoICAgICAgICAgICAgICAgCAgIQEBAICAgGBgYGBgYEBAQCAgI
CAgICAgIEBAQAQAAEBAQGBgYGBgYICAgICAgICAgICAgICAgICAgKCgoKCgoKCgoKCgoKCgo
KCgoKCgoMDAwMDAwMDAwMDAwODg4MDAwMDAwODg4ODg4MDAwMDAwKCgoICAgGBgYICAgGBgY
GBgYGBgYICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgICAgGBgYICAgGBgYGBgYGBgYEBAQCAgICAgICAgICAgIICAgAQAAGBgYICAgICAgICAg
ICAgICAgICAgKCgoKCgoKCgoKCgoKCgoMDAwMDAwMDAwMDAwMDAwMDAwKCgoMDAwKCgoMDAw
KCgoMDAwKCgoKCgoEBAQEBAQCAgIEBAQCAgIICAgICAgGBgYICAgICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgGBgYICAgGBgYGBgYGBgYGBgYGBgYGBgYICAgEBAQ
EBAQEBAQEBAQEBAQICAgAQAAICAgICAgICAgICAgKCgoKCgoKCgoKCgoKCgoKCgoMDAwMDAw
MDAwMDAwODg4ODg4ODg4ODg4ODg4MDAwMDAwMDAwMDAwKCgoKCgoKCgoCAgICAgICAgIEBAQ
CAgIICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgGBgYGBgYGBgYGBgYICAgICAgAQAAICAgICAg
ICAgKCgoKCgoKCgoKCgoKCgoMDAwMDAwMDAwMDAwODg4MDAwODg4ODg4ODg4QEBAODg4ODg4
ODg4MDAwMDAwMDAwMDAwKCgoEBAQCAgIEBAQEBAQEBAQGBgYICAgICAgICAgGBgYICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgGBgYICAgICAgICAgICAgICAg
ICAgICAgGBgYICAgICAgICAgGBgYICAgICAgICAgKCgoKCgoKCgoMDAwKCgoKCgoMDAwMDAw
ODg4ODg4ODg4QEBAODg4QEBAQEBAQEBAQEBAODg4ODg4ODg4MDAwMDAwKCgoKCgoCAgICAgI
EBAQCAgIEBAQICAgGBgYGBgYGBgYICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgGBgYICAgICAgICAgGBgYICAgICAgICAgGBgYICAgGBgYGBgYGBgYICAgICAg
ICAgKCgoKCgoKCgoKCgoKCgoKCgoKCgoMDAwODg4ODg4ODg4ODg4ODg4ODg4QEBAQEBAQEBA
QEBAODg4QEBAODg4MDAwMDAwMDAwKCgoCAgIEBAQCAgIEBAQEBAQICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgKCgoKCgoKCgoKCgoKCgoMDAwMDAw
ODg4QEBAODg4MDAwQEBAQEBASEhISEhISEhISEhIQEBAODg4QEBAODg4QEBAODg4MDAwMDAw
KCgoKCgoKCgoKCgoICAgKCgoICAgKCgoKCgoICAgKCgoICAgKCgoKCgoICAgKCgoKCgoKCgo
ICAgKCgoICAgKCgoICAgKCgoICAgKCgoKCgoKCgoICAgKCgoICAgKCgoICAgICAgICAgICAg
ICAgKCgoKCgoKCgoKCgoKCgoKCgoMDAwMDAwODg4ODg4ODg4QEBAQEBAUFhYWGBoWGBgWGBo
YGBgYGhoSEhIQEBAODg4QEBAODg4ODg4MDAwMDAwMDAwKCgoMDAwKCgoICAgICAgICAgKCgo
ICAgKCgoKCgoKCgoKCgoKCgoICAgKCgoICAgKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgo
KCgoKCgoKCgoKCgoKCgoKCgoICAgKCgoICAgICAgICAgICAgKCgoKCgoKCgoKCgoKCgoMDAw
MDAwMDAwODg4QEBAQEBAWGBgUFhYQEBAQEBAQEBAQEBAQEBAYGhwWGBgUFhYODg4ODg4MDAw
QEBAMDAwMDAwKCgoKCgoKCgoKCgoKCgoICAgKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgo
KCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoICAgKCgoICAg
ICAgICAgICAgKCgoKCgoKCgoKCgoKCgoKCgoMDAwMDAwODg4ODg4QEBAUFhgSEhIQEBAQEBA
QEBAQEBAODg4ODg4ODg4QEBAWGBoQEBAODg4ODg4ODg4ODg4MDAwMDAwMDAwKCgoKCgoICAg
KCgoKCgoKCgoKCgoMDAwKCgoMDAwKCgoKCgoKCgoKCgoMDAwKCgoKCgoMDAwKCgoKCgoKCgo
KCgoKCgoKCgoKCgoKCgoKCgoICAgKCgoKCgoICAgKCgoICAgKCgoICAgKCgoKCgoKCgoKCgo
KCgoMDAwMDAwMDAwQEBAUFBQQEBAODg4QEBAODg4ODg4ODg4ODg4ODg4ODg4ODg4QEBAWGBo
ODg4MDAwODg4MDAwMDAwMDAwKCgoKCgoKCgoKCgoKCgoKCgoMDAwKCgoMDAwKCgoMDAwKCgo
KCgoKCgoKCgoKCgoMDAwKCgoMDAwKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoICAgKCgo
ICAgICAgICAgICAgICAgICAgKCgoKCgoKCgoKCgoKCgoMDAwKCgoMDAwWFhYODg4MDAwQEBA
MDAwODg4ODg4MDAwODg4MDAwODg4MDAwODg4QEBAQEBAODg4MDAwMDAwMDAwMDAwKCgoKCgo
KCgoKCgoKCgoKCgoMDAwKCgoMDAwKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgo
KCgoKCgoKCgoKCgoKCgoKCgoKCgoICAgKCgoICAgICAgKCgoICAgICAgICAgKCgoICAgKCgo
KCgoKCgoKCgoKCgoMDAwUFBQQEBAODg4ODg4ODg4ODg4ODg4ODg4ODg4MDAwODg4MDAwMDAw
MDAwODg4SEhIODg4ODg4MDAwMDAwKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoSEhIQEBASFBQ
UFhYUGBgWGBgaHB4aHB4WGBgUFhgSEhISEhIKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgo
KCgoICAgKCgoICAgKCgoICAgKCgoICAgKCgoICAgKCgoKCgoKCgoMDAwMDAwQEBAODg4ODg4
MDAwODg4MDAwODg4ODg4MDAwMDAwMDAwMDAwMDAwODg4MDAwODg4SEhIMDAwMDAwMDAwMDAw
ICAgKCgoKCgoKCgoICAgKCgoICAgODg4ODg4ODg4QEhQUFhYYGhwYGhoUFBQSEhIQEBAQEBA
ODg4KCgoKCgoKCgoKCgoKCgoKCgoKCgoICAgKCgoICAgKCgoICAgKCgoICAgICAgICAgKCgo
ICAgKCgoKCgoKCgoKCgoKCgoODg4ODg4MDAwMDAwMDAwMDAwODg4MDAwMDAwMDAwKCgoMDAw
MDAwMDAwMDAwMDAwQEBAODg4MDAwKCgoMDAwKCgoKCgoKCgoKCgoICAgICAgKCgoKCgoICAg
KCgoKCgoMDAwMDAwMDAwQEhIMDAwMDAwKCgoKCgoICAgKCgoKCgoKCgoKCgoKCgoKCgoKCgo
KCgoKCgoICAgICAgKCgoICAgICAgICAgKCgoICAgKCgoICAgKCgoICAgKCgoKCgoKCgoMDAw
MDAwKCgoMDAwODg4ODg4MDAwMDAwMDAwMDAwMDAwMDAwKCgoMDAwMDAwMDAwKCgoKCgoKCgo
KCgoMDAwKCgoKCgoICAgKCgoKCgoKCgoKCgoGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgY
GBgYGBgYICAgKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoICAgICAgKCgoICAgICAgKCgoICAg
ICAgICAgICAgKCgoICAgKCgoICAgKCgoMDAwODg4MDAwMDAwMDAwKCgoMDAwMDAwMDAwMDAw
MDAwMDAwKCgoMDAwKCgoMDAwKCgoKCgoICAgKCgoKCgoKCgoKCgoKCgoICAgICAgICAgICAg
ICAgICAgICAgKCgoICAgKCgoICAgKCgoICAgKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgo
KCgoKCgoKCgoICAgKCgoICAgICAgKCgoICAgICAgKCgoICAgKCgoICAgICAgKCgoICAgKCgo
ICAgKCgoMDAwMDAwKCgoMDAwMDAwMDAwKCgoMDAwMDAwKCgoMDAwKCgoKCgoKCgoMDAwICAg
KCgoICAgKCgoKCgoICAgKCgoICAgKCgoICAgICAgKCgoICAgKCgoICAgQEBAICAgICAgKCgo
ICAgQEBAKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoKCgoICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgKCgoKCgoKCgoKCgoICAgKCgoKCgoMDAwKCgoKCgoKCgoKCgo
KCgoKCgoMDAwKCgoKCgoKCgoMDAwKCgoKCgoICAgKCgoKCgoKCgoKCgoKCgoICAgKCgoICAg
ICAgICAgICAgICAgICAgICAgGBgYICAgICAgKCgoICAgGBgYICAgKCgoKCgoKCgoKCgoKCgo
KCgoICAgKCgoICAgKCgoICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgKCgoICAgICAgICAgKCgoKCgoKCgoICAgKCgoKCgoKCgoKCgoKCgoKCgoMDAwKCgoKCgo
ICAgKCgoKCgoKCgoKCgoICAgICAgKCgoICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgKCgoICAgKCgoICAgKCgoICAgKCgoICAgKCgoKCgoKCgoICAgKCgoICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgKCgoICAgKCgoKCgoICAgGBgYKCgoICAgKCgo
KCgoKCgoKCgoICAgKCgoKCgoKCgoKCgoKCgoICAgICAgICAgKCgoKCgoKCgoKCgoICAgKCgo
ICAgICAgICAgICAgICAgICAgKCgoICAgICAgICAgICAgICAgICAgICAgICAgICAgKCgoICAg
ICAgKCgoICAgKCgoICAgICAgICAgICAgeHh4UFhYQEhIQEBAQEBAQEBAGBgYICAgICAgICAg
ICAgICAgKCgoICAgKCgoICAgKCgoGBgYGBgYKCgoKCgoICAgKCgoICAgKCgoICAgKCgoKCgo
ICAgICAgICAgKCgoKCgoKCgoICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgKCgoICAgICAgICAgICAgICAgICAgICAg
QEhIICAgICAgICAgICAgSFBQGBgYGBgYICAgICAgICAgICAgICAgICAgICAgKCgoICAgICAg
GBgYGBgYICAgKCgoICAgKCgoKCgoKCgoICAgGBgYGBgYICAgICAgICAgKCgoICAgKCgoICAg
ICAgICAgICAgICAgICAgICAgICAgICAgKCgoICAgKCgoICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgQEBAICAgKCgoKCgoKCgoUFhYGBgYICAg
ICAgGBgYICAgICAgICAgICAgKCgoICAgICAgICAgICAgGBgYGBgYICAgICAgICAgICAgICAg
GBgYICAgICAgGBgYGBgYICAgICAgKCgoICAgKCgoICAgKCgoICAgICAgICAgICAgICAgICAg
ICAgICAgICAgICAgKCgoICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgICAgODg4QEBAQEBAWFhgYGhoeHh4GBgYICAgGBgYICAgICAgGBgYICAgICAgICAgKCgo
KCgoKCgoICAgICAgICAgGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgGBgYGBgYEBAQGBgYEBAQGBgYEBAQ
GBgYICAgGBgYICAgGBgYICAgGBgYICAgICAgICAgICAgICAgICAgICAgICAgGBgYICAgGBgY
GBgYGBgYICAgGBgYGBgYICAgGBgYICAgICAgICAgKCgoICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgGBgYICAgICAgICAg
ICAgICAgICAgGBgYICAgGBgYICAgGBgYGBgYICAgGBgYICAgGBgYICAgGBgYICAgGBgYICAg
GBgYICAgICAgKCgoKCgoICAgGBgYICAgGBgYICAgICAgICAgGBgYICAgICAgICAgICAgICAg
KCgoICAgICAgICAgICAgICAgICAgICAgGBgYICAgGBgYICAgGBgYICAgGBgYICAgICAgGBgY
ICAgGBgYICAgGBgYICAgGBgYICAgGBgYICAgGBgYICAgGBgYICAgGBgYICAgGBgYICAgGBgY
ICAgGBgYICAgGBgYGBgYICAgGBgYICAgGBgYICAgICAgICAgICAgICAgICAgICAgICAgICAg
KCgoKCgoICAgKCgoICAgKCgoICAgKCgoKCgoICAgICAgICAgICAgICAgICAgICAgICAgGBgY
GBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYGBgYEBAQ
GBgYEBAQGBgYGBgYGBgYGBgYGBgYGBgYEBAQGBgYEBAQGBgYEBAQGBgYGBgYGBgYICAgGBgY
ICAgGBgYICAgGBgYICAgICAgICAgKCgoICAgICAgICAgICAgICAgICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgGBgYICAg')! !

!Wonderland methodsFor: 'creating' stamp: 'ar 5/10/2001 22:53'!
makeActorFrom3DS: filename
	"Creates a new actor using the specification from the given file"

	| parent name baseActor newActor actorClass scene globals hierarchy meshSize materials material textureName cameras newCamera |

	myUndoStack closeStack.

	scene _ ThreeDSParser parseFileNamed: filename.
	scene ifNil:[^nil].
	globals _ scene at: #globals.
	materials _ globals at: #materials ifAbsent:[Dictionary new].
	"The remaining objects are the actual named meshes,"
	scene removeKey: #globals.
	hierarchy _ self createHierarchyFrom3DS: globals.

	baseActor _ self makeBaseActorFrom: filename.

	"Now create the actors"
	scene associationsDo:[:assoc|
		name _ assoc key.
		actorClass _ WonderlandActor newUniqueClassInstVars: '' classInstVars: ''.
		newActor _ actorClass createFor: self.
		actorClassList addLast: actorClass.
		newActor setName: (self fixNameFrom: name).
		"newActor setTexture: texture."
		newActor setMesh: (B3DSTriangleMesh from3DS: assoc value).
		material _ materials at: (assoc value at: #triList) last ifAbsent:[nil].
		(material isKindOf: Association) ifTrue:[
			"Note: In this case the name of the texture is the key"
			textureName _ material key.
			material _ material value].
		newActor setMaterial: material.
		"newActor setComposite: matrix."
		assoc value: newActor.
	].

	"Create hierarchy"
	scene associationsDo:[:assoc|
		name _ assoc key.
		newActor _ assoc value.
		newActor setName: (self uniqueNameFrom: name).
		parent _ hierarchy at: name ifAbsent:[nil].
		parent == nil ifFalse:[parent _ scene at: parent ifAbsent:[nil]].
		parent == nil ifTrue:[parent _ baseActor].
		newActor reparentTo: parent.
		newActor becomePart.
	].

	"Create the cameras"
	cameras _ globals at: #cameras.
	cameras associationsDo:[:assoc|
		name _ assoc key.
		newCamera _ WonderlandCamera createFor: self.
		newCamera getMorph openInWorld.
		newCamera setName: (self fixNameFrom: name).
		newCamera copySettingsFrom: assoc value.
		newCamera reparentTo: baseActor.
		newCamera becomePart.
	].

	meshSize _ baseActor getBoundingBox extent length.
	meshSize > 100.0 ifTrue:[
		(self inform:'This actor is huge!!
You should rescale it to a reasonable size.')].
	meshSize < 0.01 ifTrue:[
		self inform:'This actor is tiny!!
You should rescale it to a reasonable size.'].
		
	myUndoStack openStack.

	"Ensure that the new actor's name is unique"
	myNamespace at: baseActor getName put: baseActor.
	scriptEditor updateActorBrowser.

	"Add an undo item to undo the creation of this object"
	myUndoStack push: (UndoAction new: [ baseActor removeFromScene.
											myNamespace removeKey: baseActor getName ifAbsent: [].
											scriptEditor updateActorBrowser.  ] ).

	^ baseActor.
! !

!Wonderland methodsFor: 'creating' stamp: 'ar 9/14/1999 23:40'!
makeCamera
	"Add a new camera to the Wonderland"
	^self makeCameraNamed:'camera'! !

!Wonderland class methodsFor: 'fileIn/Out' stamp: 'sw 2/17/2002 02:49'!
serviceOpenInWonderland
	"Answer a service for opening a file in Wonderland"

	^ SimpleServiceEntry 
			provider: self 
			label: 'open in Wonderland'
			selector: #openVRMLFile:
			description: 'open in Wonderland' 
			buttonLabel: 'open'! !

Object subclass: #WonderlandReaction
	instanceVariableNames: 'reaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Misc'!

!WonderlandReaction class methodsFor: 'instance creation' stamp: 'jsp 4/19/1999 14:21'!
new: aReaction
	"Create a new WonderlandReaction instance.  This instance basically wraps a block that specifies what the given object should do"

	^ self new initializeWith: aReaction.
! !

!WonderlandReaction methodsFor: 'initialization' stamp: 'jsp 4/19/1999 13:15'!
initializeWith: aReaction
	"Set the wrapped reaction"

	reaction _ aReaction.
! !

!WonderlandReaction methodsFor: 'react' stamp: 'jsp 4/19/1999 14:20'!
reactTo: event
	"Activate the reaction with the event."

	reaction value: event.
! !

B3DIndexedTriangleMesh subclass: #WonderlandMesh
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Mesh'!

!WonderlandMesh commentStamp: '<historical>' prior: 0!
This class extends the basic Balloon 3D mesh class with the ability to parse OBJ and VFB (used with Alice MDL files) files.
!

!WonderlandMesh class methodsFor: 'instanceCreation' stamp: 'jsp 3/11/1999 15:03'!
fromVFBFile: filename
	"Reads and creates a mesh from an OBJ file"

	^ (WonderlandMesh new) createFromVFBFile: filename.
! !

!WonderlandMesh class methodsFor: 'instanceCreation' stamp: 'jsp 2/17/1999 15:25'!
fromOBJFile: filename
	"Reads and creates a mesh from an OBJ file"

	^ (WonderlandMesh new) createFromOBJFile: filename.


				! !

!WonderlandMesh methodsFor: 'initialize' stamp: 'ar 11/16/2002 23:30'!
storeOnVFBStream: aStream
	"Read in a mesh from the vfb file"
	| bytes  write |
	"Read in the version number"
	bytes := ByteArray new: 4.
	"version"
	bytes unsignedLongAt: 1 put: 1 bigEndian: false.
	aStream nextPutAll: bytes.

	"vCount"
	bytes unsignedLongAt: 1 put: vertices size bigEndian: false.
	aStream nextPutAll: bytes.

	"Store the vertices"
	1 to: vertices size do: [:i |
		write := [:val|
			bytes unsignedLongAt: 1 put: val asIEEE32BitWord bigEndian: false.
			aStream nextPutAll: bytes.
		].
		write value: (vertices at: i) x.
		write value: (vertices at: i) y.
		write value: (vertices at: i) z.

		write value: (vtxNormals at: i) x.
		write value: (vtxNormals at: i) y.
		write value: (vtxNormals at: i) z.

		write value: (vtxTexCoords at: i) u.
		write value: (vtxTexCoords at: i) v.
	].


	"fCount"
	bytes unsignedLongAt: 1 put: faces size bigEndian: false.
	aStream nextPutAll: bytes.

	"faceDataCount"
	bytes unsignedLongAt: 1 put: 0 bigEndian: false.
	aStream nextPutAll: bytes.

	"verticesPerFace"
	bytes unsignedLongAt: 1 put: 0 bigEndian: false.
	aStream nextPutAll: bytes.

	1 to: faces size do:[:i|
		bytes unsignedLongAt: 1 put: 3 bigEndian: false.
		aStream nextPutAll: bytes.
		bytes unsignedLongAt: 1 put: (faces at: i) p1Index-1  bigEndian: false.
		aStream nextPutAll: bytes.
		bytes unsignedLongAt: 1 put: (faces at: i) p2Index-1  bigEndian: false.
		aStream nextPutAll: bytes.
		bytes unsignedLongAt: 1 put: (faces at: i) p3Index-1  bigEndian: false.
		aStream nextPutAll: bytes.
	].
! !

!WonderlandMesh methodsFor: 'initialize' stamp: 'ar 11/16/2002 22:54'!
createFromVFBFile: filename
	"Read in a mesh from the vfb file"

	| aFile |
	aFile _ (StandardFileStream readOnlyFileNamed: filename) binary.
	self createFromVFBStream: aFile.
	aFile close.
! !

!WonderlandMesh methodsFor: 'initialize' stamp: 'jsp 3/11/1999 10:31'!
createFromOBJFile: filename
	"Read in a mesh from the obj file"

	| aFile pos line words oldWords triple pt1 pt2 pt3 vCount fCount index |
	
	aFile _ (CrLfFileStream readOnlyFileNamed: filename) ascii.
	line _ aFile upTo: (Character cr).
	words _ line findTokens: ' '.
	
	vCount _ 0.

	"Count the vertices"
	[ (words at: 1) = 'v' ] whileTrue: [ vCount _ vCount + 1.
									   line _ aFile upTo: (Character cr).
									   words _ line findTokens: ' ' ].

	"Now reset the file and grab the actual data"
	aFile position: 0.

	"Create the vertex array"
	vertices _ B3DVector3Array new: vCount.

	"Read in the vertices"
	line _ aFile upTo: (Character cr).
	words _ line findTokens: ' '.

	index _ 1.
	[ (words at: 1) = 'v' ] whileTrue: [ vertices at: index put:
													(B3DVector3 x: ((words at: 2) asNumber)
																y: ((words at: 3) asNumber)
																z: ((words at: 4) asNumber)).
									   index _ index + 1.
									   line _ aFile upTo: (Character cr).
									   words _ line findTokens: ' ' ].

	"Read in the texture coordinates"
	index _ 1.
	vtxTexCoords _ B3DTexture2Array new: vCount.
	[ (words at: 1) = 'vt' ] whileTrue: [ vtxTexCoords at: index
													put: (B3DVector2 u: ((words at: 2) asNumber)
																	v: ((words at: 3) asNumber)).
										index _ index + 1.
										line _ aFile upTo: (Character cr).
	  									words _ line findTokens: ' '.
									].

	"Read in the normals"
	index _ 1.
	vtxNormals _ B3DVector3Array new: vCount.
	[ (words at: 1) = 'vn' ] whileTrue: [ vtxNormals at: index
											put: (B3DVector3 x: ((words at: 2) asNumber)
															 y: ((words at: 3) asNumber)
															 z: ((words at: 4) asNumber)).
										index _ index + 1.
										line _ aFile upTo: (Character cr).
	  									words _ line findTokens: ' '.
 									].

	pos _ aFile position.
	oldWords _ words.

	"Count the faces"
	fCount _ 0.
	[ (words size) > 0 ] whileTrue: [((words at: 1) = 'f') ifTrue: [ fCount _ fCount + 1 ].
										line _ aFile upTo: (Character cr).
	  									words _ line findTokens: ' '. ].

	"Create the faces array"
	faces _ B3DIndexedTriangleArray new: fCount.

	aFile position: pos.
	words _ oldWords.

	index _ 1.
	"Read in the faces"
	[ (words size) > 0 ] whileTrue: [((words at: 1) = 'f') ifTrue: [ 
					triple _ (words at: 2) findTokens: '/'.
					pt1 _ (triple at: 1) asNumber.

					triple _ (words at: 3) findTokens: '/'.
					pt2 _ (triple at: 1) asNumber.

					triple _ (words at: 4) findTokens: '/'.
					pt3 _ (triple at: 1) asNumber.

					faces at: index put: (B3DIndexedTriangle with: pt1 with: pt2 with: pt3).
					index _ index + 1.
										].
					line _ aFile upTo: (Character cr).
	  				words _ line findTokens: ' '.
				].

	aFile close.
! !

!WonderlandMesh methodsFor: 'initialize' stamp: 'ar 11/16/2002 22:53'!
createFromVFBStream: aFile
	"Read in a mesh from the vfb file"

	| w x y z bytes version vCount fCount verticesPerFace  |
	
	"Read in the version number"
	bytes _ aFile next: 4.
	version _ bytes unsignedLongAt: 1 bigEndian: false.

	(version = 1) ifTrue: [
							"Read in the number of vertices"
							bytes _ aFile next: 4.
							vCount _ bytes unsignedLongAt: 1 bigEndian: false.
						]
				ifFalse: [
							vCount _ version.
							version _ 0.
						].

	(vCount > 0) ifTrue: [
		vertices _ B3DVector3Array new: vCount.
		vtxNormals _ B3DVector3Array new: vCount.
		vtxTexCoords _ B3DTexture2Array new: vCount.

		"Read in the vertices"
		1 to: vCount do: [:i | bytes _ aFile next: 32.
						x _ Float fromIEEE32Bit: (bytes unsignedLongAt: 1 bigEndian: false).
						y _ Float fromIEEE32Bit: (bytes unsignedLongAt: 5 bigEndian: false).
						z _ Float fromIEEE32Bit: (bytes unsignedLongAt: 9 bigEndian: false).
						vertices at: i put: (B3DVector3 x: (x negated) y: y z: z).

						x _ Float fromIEEE32Bit: (bytes unsignedLongAt: 13 bigEndian: false).
						y _ Float fromIEEE32Bit: (bytes unsignedLongAt: 17 bigEndian: false).
						z _ Float fromIEEE32Bit: (bytes unsignedLongAt: 21 bigEndian: false).
						vtxNormals at: i put: (B3DVector3 x: (x negated) y: y z: z).

						x _ Float fromIEEE32Bit: (bytes unsignedLongAt: 25 bigEndian: false).
						y _ Float fromIEEE32Bit: (bytes unsignedLongAt: 29 bigEndian: false).
						vtxTexCoords at: i put: (B3DVector2 u: x v: y).

						].

		"Read in the number of faces"
		bytes _ aFile next: 4.
		fCount _ bytes unsignedLongAt: 1 bigEndian: false.

		(fCount > 0) ifTrue: [
				"Read past the faceDataCount value"
				aFile next: 4.

				(version = 0) ifTrue: [ verticesPerFace _ 0. ]
							ifFalse: [
										bytes _ aFile next: 4.
										verticesPerFace _ bytes unsignedLongAt: 1 bigEndian: false.
									].

				(verticesPerFace = 0) ifTrue: [
							faces _ B3DIndexedTriangleArray new: fCount.

							1 to: fCount do: [: i |
								bytes _ aFile next: 4.
								w _ bytes unsignedLongAt: 1 bigEndian: false.

								(w = 3) ifTrue: [
									bytes _ aFile next: 4.
									x _ (bytes unsignedLongAt: 1 bigEndian: false) + 1.

									bytes _ aFile next: 4.
									y _ (bytes unsignedLongAt: 1 bigEndian: false) + 1.

									bytes _ aFile next: 4.
									z _ (bytes unsignedLongAt: 1 bigEndian: false) + 1.

									faces at: i put: (B3DIndexedTriangle with: x with: y with: z).
 												]
										ifFalse: [ 1 to: w do: [: j | aFile next: 4] ].

											].
																		].

							].  " (end if fCount > 0) "
						]	" (end if vCount > 0) "
			ifFalse: [ ^ nil ].! !

VRMLEnumerator subclass: #VRMLWonderlandBuilder
	instanceVariableNames: 'myWonderland scene attributes baseActor currentActor defaultCamera progress'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Import'!

!VRMLWonderlandBuilder commentStamp: '<historical>' prior: 0!
This class builds a Wonderland representation from a given scene.!

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 9/9/1999 12:29'!
createActorFor: aMesh suffix: aString
	^self createActorFor: aMesh
			name: (attributes at: #currentName ifAbsent:['unnamed'])
			suffix: aString! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:04'!
doTransform: node
	| attr c r s sr t xform oldActor |
	attr _ node attributeValueNamed: 'center'.
	attr notNil ifTrue:[c _ self positionFrom: attr].
	attr _ node attributeValueNamed: 'rotation'.
	attr notNil ifTrue:[r _ self rotationFrom: attr].
	attr _ node attributeValueNamed: 'scale'.
	attr notNil ifTrue:[s _ self scaleFrom: attr].
	attr _ node attributeValueNamed: 'scaleOrientation'.
	attr notNil ifTrue:[sr _ self rotationFrom: attr].
	attr _ node attributeValueNamed: 'translation'.
	attr notNil ifTrue:[t _ self positionFrom: attr].
	xform _ B3DMatrix4x4 identity.
	xform translation: t+c.
	xform _ xform composedWithLocal: r asMatrix4x4.
	xform _ xform composedWithLocal: sr asMatrix4x4.
	xform _ xform composedWithLocal: (B3DMatrix4x4 identity scaling: s).
	xform _ xform composedWithLocal: sr negated asMatrix4x4.
	xform _ xform composedWithLocal: (B3DMatrix4x4 identity translation: c negated).
	oldActor _ currentActor.
	currentActor _ self createActorFor: nil.
	currentActor setComposite: xform.
	super doTransform: node.
	currentActor _ oldActor.! !

!VRMLWonderlandBuilder class methodsFor: 'instance creation' stamp: 'ar 8/18/1999 19:46'!
on: aVRMLScene
	^self new on: aVRMLScene! !

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 9/16/1999 23:23'!
createActorFor: aMesh name: aString suffix: suffix
	| actor |
	actor _ myWonderland createSimpleActor: aMesh 
					named: aString, suffix
					parent: currentActor.
	"actor setMaterial: (attributes at: #currentMaterial ifAbsent:[nil])."
	aMesh == nil ifFalse:[attributes at: #currentShape put: actor].
	^actor! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:03'!
doNormal: node
	| attr points |
	(attr _ node attributeValueNamed: 'vector') notNil
		ifTrue:[points _ self mfVec3fFrom: attr].
	points == nil
		ifTrue:[attributes removeKey: #currentNormals]
		ifFalse:[attributes at: #currentNormals put: points].! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:00'!
doLOD: node
	| levels |
	levels _ (node attributeValueNamed: 'level').
	levels size > 0 ifTrue:[
		levels last doWith: self.
	].! !

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 9/14/1999 23:57'!
baseActor: aWonderlandActor
	baseActor _ aWonderlandActor.
	self currentActor: baseActor.! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 18:58'!
doColor: node
	| colors |
	colors _ (node attributeValueNamed: 'color').
	attributes at: #currentColors put: (colors collect:[:c| c asB3DColor])! !

!VRMLWonderlandBuilder methodsFor: 'initialize' stamp: 'ar 9/14/1999 23:33'!
initializeFor: aWonderland
	myWonderland _ aWonderland.
	defaultCamera _ aWonderland getDefaultCamera.! !

!VRMLWonderlandBuilder methodsFor: 'building meshes' stamp: 'ar 5/4/2000 19:02'!
assignIndexedNormals: node in: mesh
	| attr normals nrmlIndex faces vertices |
	attributes at: #currentNormals put: nil.
	(attr _ node attributeValueNamed: 'normal') notNil
		ifTrue:[attr doWith: self].
	normals _ attributes at: #currentNormals.
	normals == nil ifTrue:[^self].
	attr _ (node attributeValueNamed: 'normalIndex').
	attr size = 0 ifTrue:[attr _ (node attributeValueNamed: 'coordIndex')].
	nrmlIndex _ ReadStream on: attr.
	faces _ mesh faces.
	1 to: faces size do:[:i|
		vertices _ (faces at: i) vertices.
		[nrmlIndex next = -1] whileTrue.
		nrmlIndex skip: -1.
		1 to: vertices size do:[:index|
			(vertices at: index) normal: (normals at: nrmlIndex next + 1).
		].
	].
! !

!VRMLWonderlandBuilder methodsFor: 'converting' stamp: 'ar 7/14/2000 10:14'!
rotationFrom: attr
	"Fix up coord systems.
	FIXME: attr should always be a B3DRotation but for now
	the defaults are arrays. How ugly..."
	attr class == Array
		ifTrue:[^B3DRotation radiansAngle: (attr at: 4) negated axis: 
			(B3DVector3 x: (attr at: 1) y: (attr at: 2) z: (attr at: 3))]
		ifFalse:[^attr negated]! !

!VRMLWonderlandBuilder methodsFor: 'converting' stamp: 'ar 8/31/2000 19:13'!
mfVec3fFromInterpolator: node
	| key value len values vv keySize |
	key _ node attributeValueNamed: 'key'.
	value _ node attributeValueNamed: 'keyValue'.
	keySize _ key size.
	values _ Array new: keySize.
	len _ value size // keySize.
	1 to: values size do:[:i|
		vv _ B3DVector3Array new: len.
		vv replaceFrom: 1 to: len with: value startingAt: (i-1 * len)+1.
		values at: i put: (self mfVec3fFrom: vv).
	].
	^values! !

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 9/14/1999 19:43'!
currentActor
	^currentActor! !

!VRMLWonderlandBuilder methodsFor: 'converting' stamp: 'ar 9/15/1999 16:16'!
mfVec2fFrom: anArray
	"Convert the given array into a B3DVector2Array"
	| pts |
	pts _ WriteStream on: (B3DTexture2Array new: anArray size).
	anArray do:[:spec|
		pts nextPut: (B3DVector2 u: (spec at: 1) v: (spec at: 2)).
	].
	^pts contents! !

!VRMLWonderlandBuilder methodsFor: 'building meshes' stamp: 'ar 5/4/2000 19:01'!
assignIndexedColors: node in: mesh
	| attr colors colorIndex faces vertices |
	attributes at: #currentColors put: nil.
	(attr _ node attributeValueNamed: 'color') notNil
		ifTrue:[attr doWith: self].
	colors _ attributes at: #currentColors.
	(colors == nil or:[colors size = 0]) ifTrue:[^self].
	attr _ (node attributeValueNamed: 'colorIndex').
	attr size = 0 ifTrue:[attr _ (node attributeValueNamed:'coordIndex')].
	colorIndex _ ReadStream on: attr.
	faces _ mesh faces.
	1 to: faces size do:[:i|
		vertices _ (faces at: i) vertices.
		[colorIndex next = -1] whileTrue.
		colorIndex skip: -1.
		1 to: vertices size do:[:index|
			(vertices at: index) color: (colors at: colorIndex next + 1).
		].
	].
! !

!VRMLWonderlandBuilder methodsFor: 'building meshes' stamp: 'ar 5/4/2000 19:02'!
assignIndexedTexCoords: node in: mesh
	| attr faces vertices texCoords texIndex |
	attributes at: #currentTexCoords put: nil.
	(attr _ node attributeValueNamed: 'texCoord') notNil
		ifTrue:[attr doWith: self].
	texCoords _ attributes at: #currentTexCoords.
	texCoords == nil ifTrue:[^self].
	attr _ (node attributeValueNamed: 'texCoordIndex').
	attr size = 0 ifTrue:[attr _ (node attributeValueNamed:'coordIndex')].
	texIndex _ ReadStream on: attr.
	faces _ mesh faces.
	1 to: faces size do:[:i|
		vertices _ (faces at: i) vertices.
		[texIndex next = -1] whileTrue.
		texIndex skip: -1.
		1 to: vertices size do:[:index|
			(vertices at: index) texCoord: (texCoords at: texIndex next + 1).
		].
	].
! !

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 9/16/1999 19:52'!
createActorFor: aMesh defaultName: aString
	^self createActorFor: aMesh
			name: (attributes at: #currentName ifAbsent:[aString])
			suffix: ''! !

!VRMLWonderlandBuilder methodsFor: 'enumerating' stamp: 'ar 9/15/1999 16:29'!
doNamedNode: node
	| lastName |
	progress value:'Creating ', node name,' ...'.
	lastName _ attributes at: #currentName ifAbsent:[nil].
	attributes at: #currentName put: node name.
	super doNamedNode: node.
	lastName
		ifNil:[attributes removeKey: #currentName]
		ifNotNil:[attributes at: #currentName put: lastName].! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:01'!
doSphere: node
	| radius sphere |
	radius _ (node attributeValueNamed:'radius').
	sphere _ B3DIndexedMesh vrml97Sphere.
	self createActorFor: sphere defaultName:'sphere'.
	currentActor resizeRightNow: radius @ radius @ radius undoable: false.
! !

!VRMLWonderlandBuilder methodsFor: 'converting' stamp: 'ar 9/15/1999 18:05'!
scaleFrom: attr
	"Fix up coord systems"
	^B3DVector3 x: (attr at: 1) y: (attr at: 2) z: (attr at: 3).
! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:01'!
doSwitch: node
	| choices which |
	choices _ (node attributeValueNamed:'choices').
	which _ (node attributeValueNamed: 'whichChoice') + 1.
	(which > 0 and:[which <= choices size])
		ifTrue:[(choices at: which) doWith: self].! !

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 9/14/1999 19:39'!
buildActorsFor: aWonderland
	"Using the given VRML scene build a set of actors for Wonderland"
	self initializeFor: aWonderland.
	self buildActors.! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'rww 10/2/2001 07:25'!
doCoordinate: node
	| attr points interpolator |
	(attr _ node attributeValueNamed: 'point') notNil ifTrue:[
		points _ self mfVec3fFrom: attr.
		(scene targetNodes includes: node) ifTrue:[
			interpolator _ nil.
			scene sourceNodes do:[:n|
				n actionsDo:[:set|
					set do:[:msg| msg receiver == node ifTrue:[interpolator _ n]]]].
			interpolator _ self mfVec3fFromInterpolator: interpolator.
		].
	].
	points == nil
		ifTrue:[attributes removeKey: #currentPoints]
		ifFalse:[attributes at: #currentPoints put: points].
	interpolator == nil
		ifTrue:[attributes removeKey: #interpolatorPoints ifAbsent:[]]
		ifFalse:[attributes at: #interpolatorPoints put: interpolator].
! !

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 9/15/1999 16:29'!
buildActors
	"Using the given VRML scene build a set of actors for Wonderland"
	attributes _ Dictionary new.
	Utilities informUserDuring:[:bar| 
		progress _ bar.
		scene doWith: self].! !

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 9/14/1999 19:43'!
currentActor: aWonderlandActor
	currentActor _ aWonderlandActor! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 18:58'!
doCone: node
	| radius height doSide doBottom cone |
	radius _ (node attributeValueNamed:'bottomRadius').
	height _ (node attributeValueNamed:'height').
	doSide _ (node attributeValueNamed:'side').
	doBottom _ (node attributeValueNamed:'bottom').
	cone _ B3DIndexedMesh vrml97Cone: doSide bottom: doBottom.
	self createActorFor: cone defaultName:'cone'.
	currentActor resizeRightNow: radius @ height @ radius undoable: false.
! !

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 8/26/1999 12:28'!
createActorFor: aMesh
	^self createActorFor: aMesh suffix:''! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:03'!
doImageTexture: node
	| attr texFileName tex |
	"Create a new texture."
	(attr _ node attributeValueNamed: 'url') notNil
		ifTrue:[
			texFileName _ (FileDirectory on: (FileDirectory dirPathFor: scene fileURL)) fullNameFor: attr first.
			(FileDirectory default fileExists: texFileName)
				ifTrue:[tex _ myWonderland makeTextureFrom: texFileName].
			attributes at: #currentTexture put: tex].
! !

!VRMLWonderlandBuilder methodsFor: 'initialize' stamp: 'ar 8/18/1999 19:46'!
on: aVRMLScene
	scene _ aVRMLScene! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 18:59'!
doCylinder: node
	| radius height doSide doBottom doTop cylinder |
	radius _ (node attributeValueNamed:'radius').
	height _ (node attributeValueNamed:'height').
	doSide _ (node attributeValueNamed:'side').
	doBottom _ (node attributeValueNamed:'bottom').
	doTop _ (node attributeValueNamed:'top').
	cylinder _ B3DIndexedMesh vrml97Cylinder: doSide bottom: doBottom top: doTop.
	self createActorFor: cylinder defaultName:'cylinder'.
	currentActor resizeRightNow: radius @ height @ radius undoable: false.
! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:04'!
doViewpoint: node
	| attr pos r fov name m camera |
	attr _ node attributeValueNamed: 'position'.
	attr notNil ifTrue:[pos _ self positionFrom: attr].
	attr _ node attributeValueNamed: 'orientation'.
	attr notNil ifTrue:[r _ self rotationFrom: attr].
	attr _ node attributeValueNamed: 'fieldOfView'.
	attr notNil ifTrue:[fov _ attr radiansToDegrees].
	attr _ node attributeValueNamed: 'description'.
	attr notNil ifTrue:[name _ attr].
	m _ B3DMatrix4x4 identity.
	m _ m composedWithGlobal: (r asMatrix4x4).
	m _ m composedWithGlobal: (B3DMatrix4x4 withOffset: pos).
	(currentActor == baseActor and:[defaultCamera ~~ nil]) ifTrue:[
		camera _ defaultCamera.
		defaultCamera _ nil.
	] ifFalse:[
		camera _ myWonderland makeCameraNamed: (name ifNil:['camera']) parent: currentActor.
	].
	camera setComposite: m.
	camera setFieldOfView: fov.! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 18:58'!
doBox: node
	| size box |
	size _ (node attributeValueNamed:'size').
	box _ B3DIndexedMesh vrml97Box.
	self createActorFor: box defaultName:'box'.
	currentActor resizeRightNow: size first @ size second @ size third undoable: false.
! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:03'!
doTextureCoordinate: node

	| attr points |
	(attr _ node attributeValueNamed: 'point') notNil
		ifTrue:[points _ self mfVec2fFrom: attr].
	points == nil
		ifTrue:[attributes removeKey: #currentTexCoords]
		ifFalse:[attributes at: #currentTexCoords put: points].! !

!VRMLWonderlandBuilder methodsFor: 'converting' stamp: 'ar 9/15/1999 17:33'!
mfVec3fFrom: anArray
	"Convert the given array into a B3DVector3Array"
	| pts |
	pts _ WriteStream on: (B3DVector3Array new: anArray size).
	anArray do:[:spec|
		pts nextPut: (B3DVector3 x: (spec at: 1) y: (spec at: 2) z: 0.0 - (spec at: 3)).
	].
	^pts contents! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:03'!
doShape: node
	| attr shape |
	attributes at: #currentTexture put: nil.
	attributes at: #currentMaterial put: nil.
	attributes at: #currentShape put: nil.
	(attr _ node attributeValueNamed: 'appearance') notNil
		ifTrue:[attr doWith: self].
	(attr _ node attributeValueNamed: 'geometry') notNil
		ifTrue:[attr doWith: self].
	shape _ attributes at: #currentShape.
	shape == nil ifFalse:[
		shape setMaterial: (attributes at: #currentMaterial ifAbsent:[nil]).
		shape setTexturePointer: (attributes at: #currentTexture ifAbsent:[nil])].! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:02'!
doAppearance: node
	| attr |
	(attr _ node attributeValueNamed: 'material') notNil
		ifTrue:[attr doWith: self].
	(attr _ node attributeValueNamed: 'texture') notNil
		ifTrue:[attr doWith: self].
	(attr _ node attributeValueNamed: 'textureTransform') notNil
		ifTrue:[attr doWith: self].
! !

!VRMLWonderlandBuilder methodsFor: 'converting' stamp: 'ar 9/15/1999 17:33'!
positionFrom: attr
	"Fix up coord systems"
	^B3DVector3 x: (attr at: 1) y: (attr at: 2) z: 0.0 - (attr at: 3).
! !

!VRMLWonderlandBuilder methodsFor: 'building meshes' stamp: 'ar 9/14/1999 23:04'!
facesFrom: anArray for: vertices
	"Create and return a set of faces from the given index array"
	| faces vtx |
	vtx _ WriteStream on: (Array new: 4).
	faces _ WriteStream on: (Array new: anArray size // 4).
	anArray do:[:idx|
		idx = -1 ifTrue:[
			vtx position > 2 
				ifTrue:[faces nextPut: (B3DSimpleMeshFace withAll: vtx contents)].
			vtx reset.
		] ifFalse:[
			vtx nextPut: (B3DSimpleMeshVertex new position: (vertices at: idx+1)).
		].
	].
	^faces contents! !

!VRMLWonderlandBuilder methodsFor: 'building meshes' stamp: 'ar 8/19/1999 13:11'!
indexedFacesFrom: anArray
	"Create and return a set of faces from the given index array"
	| faces index idx0 idx1 idx2 |
	faces _ WriteStream on: (B3DIndexedTriangleArray new: anArray size // 4).
	index _ 0.
	[index < anArray size] whileTrue:[
		idx0 _ anArray at: (index _ index + 1).
		idx1 _ anArray at: (index _ index + 1).
		[idx2 _ anArray at: (index _ index + 1).
		faces nextPut: (B3DIndexedTriangle with: idx0+1 with: idx1+1 with: idx2+1).
		idx1 _ idx2.
		index < anArray size
			ifTrue:[idx2 _ anArray at: (index_index+1)]
			ifFalse:[idx2 _ -1].
		idx2 = -1] whileFalse.
	].
	^faces contents! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 18:59'!
doDirectionalLight: node
	| ambientIntensity intensity color on material light dir |
	ambientIntensity _ (node attributeValueNamed:'ambientIntensity').
	intensity _ (node attributeValueNamed: 'intensity').
	color _ (node attributeValueNamed: 'color').
	dir _ self positionFrom: (node attributeValueNamed:'direction').
	on _ (node attributeValueNamed:'on').
	material _ B3DMaterialColor new.
	material ambientPart: (color * ambientIntensity).
	material diffusePart: (color * intensity).
	light _ myWonderland makeLight: (Wonderland wonderlandConstants at: 'directional').
	light lightColor: material.
	light setDirection: dir.
	currentActor ifNotNil:[light reparentTo: currentActor].! !

!VRMLWonderlandBuilder methodsFor: 'building actors' stamp: 'ar 9/14/1999 23:57'!
baseActor
	^baseActor! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 8/31/2000 21:39'!
doIndexedFaceSet: node
	| attr faces points mesh ipPoints meshList map |
	attributes at: #currentPoints put: nil.
	(attr _ node attributeValueNamed: 'coord') notNil
		ifTrue:[attr doWith: self].
	points _ attributes at: #currentPoints.
	ipPoints _ attributes at: #interpolatorPoints ifAbsent:[nil].
	points ifNil:[^self].
	(attr _ node attributeValueNamed: 'coordIndex') notNil
		ifTrue:[faces _ self facesFrom: attr for: points].
	faces ifNil:[^self].
	mesh _ B3DSimpleMesh withAll: faces.
	self assignIndexedNormals: node in: mesh.
	self assignIndexedTexCoords: node in: mesh.
	self assignIndexedColors: node in: mesh.
	mesh _ mesh asIndexedTriangleMesh.
	ipPoints == nil ifFalse:[
		"Re-map points for interpolated meshes"
		map _ Dictionary new: points size * 2.
		points doWithIndex:[:pt :index| map at: pt put: index].
		meshList _ ipPoints collect:[:pts|
			mesh clone vertices: (mesh vertices collect:[:pt| pts at: (map at: pt)])].
		mesh _ B3DInterpolatedMesh new initialize meshes: meshList.
	].
	"Try to recycle the actors"
	(currentActor == nil or:[currentActor hasMesh])
		ifTrue:[self createActorFor: mesh]
		ifFalse:[	currentActor setMesh: mesh.
				attributes at: #currentShape put: currentActor].! !

!VRMLWonderlandBuilder methodsFor: 'enumerating nodes' stamp: 'ar 5/4/2000 19:03'!
doMaterial: node
	| ambientIntensity diffuseColor emissiveColor shininess specularColor material |
	ambientIntensity _ (node attributeValueNamed: 'ambientIntensity').
	diffuseColor _ (node attributeValueNamed: 'diffuseColor').
	emissiveColor _ (node attributeValueNamed:'emissiveColor').
	shininess _ (node attributeValueNamed: 'shininess').
	specularColor _ (node attributeValueNamed: 'specularColor').
	"transparency _ (node attributeValueNamed:'transparency')."
	material _ B3DMaterial new.
	material ambientPart: (diffuseColor * ambientIntensity).
	material diffusePart: diffuseColor.
	material specularPart: specularColor.
	material emission: emissiveColor.
	material shininess: shininess.
	attributes at: #currentMaterial put: material.! !

ImageMorph subclass: #WonderlandCameraControls
	instanceVariableNames: 'myCamera myScheduler myUndoStack moveAction myMoveScale myRotationScale'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandCameraControls commentStamp: '<historical>' prior: 0!
This class implements the controls for manipulating the viewpoint of a camera using the mouse.

Currently clicking and dragging on the bitmap moves the camera forward/back and turns it left/right.

Shift-clicking and dragging moves the camera left/right and up/down.

Control-clicking and dragging turns the camera left/right.

Shift-Control-clicking and dragging turns the camera up/down.

!

!WonderlandCameraControls methodsFor: 'event handling' stamp: 'ar 6/1/2001 13:50'!
mouseDown: evt
	"Handle a mouse down event."

	myUndoStack push: (UndoPOVChange for: myCamera from: (myCamera getPointOfView)).

	moveAction _ myCamera doEachFrame: [self moveCamera: World primaryHand lastEvent].
! !

!WonderlandCameraControls methodsFor: 'accessing' stamp: 'ar 11/6/2000 14:48'!
setCenter: aPoint
	aPoint == nil
		ifTrue:[self removeProperty: #center]
		ifFalse:[self setProperty: #center toValue: aPoint].! !

!WonderlandCameraControls methodsFor: 'properties' stamp: 'ar 9/16/1999 16:36'!
getRotationScale
	^myRotationScale! !

!WonderlandCameraControls methodsFor: 'event handling' stamp: 'jsp 3/30/1999 10:24'!
handlesMouseDown: evt

	^ true.
! !

!WonderlandCameraControls methodsFor: 'properties' stamp: 'ar 9/16/1999 16:37'!
setRotationScale: aNumber
	myRotationScale _ aNumber.! !

!WonderlandCameraControls class methodsFor: 'instance creation' stamp: 'jsp 3/30/1999 09:57'!
newFor: aCamera
	"Initialize the camera controls and assign the camera they're controlling"

	^ self new initializeWith: aCamera.
! !

!WonderlandCameraControls methodsFor: 'accessing' stamp: 'jsp 3/30/1999 10:15'!
extent: aPoint

	bounds extent = aPoint ifFalse: [
		self changed.
		bounds _ bounds topLeft extent: aPoint.
		self layoutChanged.
		self changed].! !

!WonderlandCameraControls methodsFor: 'initialization' stamp: 'jsp 4/9/1999 16:24'!
initializeWith: aCamera
	"Initialize this instance"

	| b |

	myCamera _ aCamera.
	myScheduler _ (aCamera getWonderland) getScheduler.
	myUndoStack _ (aCamera getWonderland) getUndoStack.

	image _ camControlsBMP.

	self extent: 59@60.
	b _ (aCamera getMorph) bounds.
	self position: ((b center x) - 30) @ (b bottom).
	self openInWorld.
! !

!WonderlandCameraControls methodsFor: 'properties' stamp: 'ar 9/16/1999 16:36'!
setMoveScale: aNumber
	myMoveScale _ aNumber.! !

!WonderlandCameraControls methodsFor: 'event handling' stamp: 'jsp 3/30/1999 10:52'!
mouseUp: evt
	"Handle a mouse up event."

	moveAction stop.
! !

!WonderlandCameraControls methodsFor: 'properties' stamp: 'ar 9/16/1999 16:36'!
getMoveScale
	^myMoveScale! !

!WonderlandCameraControls methodsFor: 'camera control' stamp: 'ar 6/1/2001 10:14'!
moveCamera: evt
	"Move the camera an amount and direction determined by the current position of the mouse and which modifier keys are held down."
	| offset dt |
	myMoveScale == nil ifTrue:[myMoveScale _ 0.1].
	myRotationScale == nil ifTrue:[myRotationScale _ 0.01].
	dt _ myScheduler getElapsedTime.
	offset _ (self getCenter) - (0@8) - (evt cursorPoint).

	(evt shiftPressed)
		ifTrue: [ (evt controlKeyPressed)
					ifTrue: [
						myCamera turnRightNow: up numberOfTurns: (dt * (offset y) * myRotationScale)
										undoable: false.
							]
					ifFalse: [
						myCamera moveRightNow: up distance: (dt * (offset y) * myMoveScale)
										undoable: false.
						myCamera moveRightNow: left distance: (dt * (offset x) * myMoveScale)
										undoable: false.
							].
				]
		ifFalse: [ (evt controlKeyPressed)
					ifTrue: [
						myCamera turnRightNow: left numberOfTurns: (dt * (offset x) * myRotationScale)
										undoable: false.
							]
					ifFalse: [
						myCamera moveRightNow: forward distance: (dt * (offset y) * myMoveScale)
										undoable: false.
						myCamera turnRightNow: left numberOfTurns: (dt * (offset x) * myRotationScale)
										undoable: false.
							].
				].
! !

!WonderlandCameraControls methodsFor: 'accessing' stamp: 'ar 11/6/2000 14:48'!
getCenter
	^self valueOfProperty: #center ifAbsent:[self center]! !

!Morph methodsFor: '*Balloon3D-Wonderland' stamp: 'ar 11/16/2002 19:42'!
installAsWonderlandTextureOn: anActor
	"Make the receiver a texture for the given actor"
	^anActor setTexturePointer: self! !

Morph subclass: #WonderlandWrapperMorph
	instanceVariableNames: 'myWonderland myActor'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandWrapperMorph commentStamp: '<historical>' prior: 0!
A WonderlandWrapperMorph is temporarily created by Cmd-Clicking on an object in a 3D world. The wrappers map the behavior of morphs into 3D worlds so that 2D interactions result in appropriate changes in the 3D world (e.g., dragging, resizing) and state changes as well (e.g., color).

Wrappers are created as a hierarchical morph list on the object they represent. This is necessary to allow accesses to higher levels in the 3D hierarchy. Note that there are currently several inconsistent behaviors for 3D objects. This is related to the ridiculous attempt of HandMorph to completely understand the interaction hierarchy, e.g., the HandMorph dispatches all events to what it *thinks* the receiver should be. Naturally, this fails completely for 3D worlds where there is no hierarchy of morphs but which may nevertheless use input events.
!

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 5/26/2001 16:11'!
renameTo: aName
	myActor renameTo: aName.! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:30'!
rotateStartFromHalo: evt with: handle! !

!WonderlandWrapperMorph class methodsFor: 'instance creation' stamp: 'ar 11/7/1999 19:03'!
on: anActor
	^self new setActor: anActor! !

!WonderlandWrapperMorph methodsFor: 'accessing' stamp: 'ar 5/8/2001 23:51'!
hasHalo: aBool
	super hasHalo: aBool.
	aBool ifFalse:[
		(self hasProperty: #surviveHaloLoss) ifFalse:[
			"Need to get rid of wrappers on top of us, so..."
			owner ifNotNil:[owner hasHalo: owner hasHalo].
			self delete].
		self removeProperty: #surviveHaloLoss].
! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:30'!
rotateEndFromHalo: evt with: handle! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:29'!
dragEndFromHalo: evt with: handle
	self dragPositionOffset: nil.
	self halo endInteraction.! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 5/8/2001 20:33'!
openViewerForArgument
	myActor beViewed.! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/10/1999 15:16'!
dupStartFromHalo: evt with: handle
	| newActor |
	self removeAllHandlesBut: handle.
	newActor _ myWonderland duplicateActor: myActor.
	myActor _ newActor.
	self dragPositionOffset: handle center - self positionInWorld.! !

!WonderlandWrapperMorph methodsFor: 'menu' stamp: 'ar 11/8/1999 13:17'!
setUserPointOfView
	| morph |
	morph _ self getCameraMorph.
	morph == nil ifTrue:[^self].
	myActor setUserPointOfView: (morph getCamera getPointOfView: myActor).! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 8/15/2001 13:26'!
tearOffTile
	^ myActor tearOffTileForSelf! !

!WonderlandWrapperMorph methodsFor: 'menu' stamp: 'ar 6/9/2000 19:17'!
addCustomMenuItems: aCustomMenu hand: aHand
	super addCustomMenuItems: aCustomMenu hand: aHand.
	aCustomMenu addLine.
	aCustomMenu addUpdating: #getHQTextureState action: #toggleHQTextureState.
	aCustomMenu addUpdating: #getActiveTextureState action: #toggleActiveTextureState.
	aCustomMenu addUpdating: #getTextureAdjust action: #toggleTextureAdjust.
	aCustomMenu add:'set user point of view' action:#setUserPointOfView.! !

!WonderlandWrapperMorph methodsFor: 'accessing' stamp: 'ar 11/7/1999 19:15'!
getCameraMorph
	| morph |
	morph _ owner.
	[morph == nil] whileFalse:[
		(morph isKindOf: WonderlandCameraMorph) ifTrue:[^morph].
		morph _ morph owner].
	^nil! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'sw 12/21/1999 16:36'!
addSimpleHandlesTo: aHaloMorph box: aBox
	self flag: #noteToAndreas. 
		 "May want to reimplement this for wonderland morphs.  For the moment, this basically disallows simple handles in the wonderland case."
	^ aHaloMorph addCircleHandles! !

!WonderlandWrapperMorph methodsFor: 'accessing' stamp: 'ar 11/7/1999 21:30'!
color: aColor
	"Change the color of the actor"
	self undoOnTop: UndoColorChange using:[myActor getColor].
	myActor setColorVector: aColor asB3DColor! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:20'!
growPositionOffset
	^self valueOfProperty: #growPositionOffset.! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 8/15/2001 13:26'!
addHandlesTo: aHaloMorph box: box
	| dismissHandle s |
	s _ aHaloMorph handleSize.
	myActor getTexturePointer == nil ifFalse:[
		aHaloMorph addHandleAt: box rightCenter color: Color lightGray on: #mouseDown send: #extractTexture: to: aHaloMorph target].

	aHaloMorph addHandleAt: box topLeft color: Color red icon: 'Halo-Menu'
		on: #mouseDown send: #doMenu:with: to: aHaloMorph.

	aHaloMorph addHandleAt: (box leftCenter + (0 @ (s+2)) min: box leftCenter + box bottomLeft // 2)
		color: Color lightBrown icon: 'Halo-Tile'
		on: #mouseDown send: #tearOffTile to: self.

	dismissHandle _ aHaloMorph addHandleAt: (box topLeft + ((s+2)@0) min: box topLeft + box topCenter // 2)
		color: Color red muchLighter icon: 'Halo-Dismiss'
		on: #mouseDown send: #mouseDownInDimissHandle:with: to: aHaloMorph.
	dismissHandle on: #mouseUp send: #maybeDismiss:with: to: aHaloMorph.
	dismissHandle on: #mouseDown send: #setDismissColor:with: to: aHaloMorph.
	dismissHandle on: #mouseMove send: #setDismissColor:with: to: aHaloMorph.

	aHaloMorph addHandleAt: box leftCenter color: Color cyan icon: 'Halo-View'
		on: #mouseDown send: #openViewerForArgument to: self.

	aHaloMorph addHandleAt: box topCenter color: Color black icon: 'Halo-Grab'
		on: #mouseDown send: #grabFromHalo:with: to: self.

	(aHaloMorph addHandleAt: (box topCenter + ((s+2)@0) min: box topCenter + box topRight // 2)
		color: Color brown icon: 'Halo-Drag'
		on: #mouseDown send: #dragStartFromHalo:with: to: self)
		on: #mouseMove send: #dragMoveFromHalo:with: to: self;
		on: #mouseUp send: #dragEndFromHalo:with: to: self.

	(aHaloMorph addHandleAt: box topRight color: Color green icon: 'Halo-Dup'
		on: #mouseDown send: #dupStartFromHalo:with: to: self)
		on: #mouseMove send: #dupMoveFromHalo:with: to: self;
		on: #mouseUp send: #dupEndFromHalo:with: to: self.

	Preferences showDebugHaloHandle ifTrue:
		[aHaloMorph addHandleAt: ((box topRight + box rightCenter) // 2)
			color: Color blue veryMuchLighter icon: 'Halo-Debug'
			on: #mouseDown send: #doDebug:with: to: aHaloMorph].

	(aHaloMorph addHandleAt: box bottomLeft color: Color blue icon: 'Halo-Rotate'
		on: #mouseDown send: #rotateStartFromHalo:with: to: self)
		on: #mouseMove send: #rotateMoveFromHalo:with: to: self;
		on: #mouseUp send: #rotateEndFromHalo:with: to: self.

	(aHaloMorph addHandleAt: box bottomRight color: Color yellow icon: 'Halo-Scale'
		on: #mouseDown send: #growStartFromHalo:with: to: self)
		on: #mouseMove send: #growMoveFromHalo:with: to: self;
		on: #mouseUp send: #growEndFromHalo:with: to: self.

	myActor isHandmade
		ifTrue: [
			(aHaloMorph addHandleAt: box center color: Color white icon: 'Halo-Paint'
				on: #mouseUp send: #paintTexture to: self)]

! !

!WonderlandWrapperMorph methodsFor: 'accessing' stamp: 'ar 2/18/2001 22:32'!
transferHalo: event from: formerHaloOwner
	"Progressively transfer the halo to the next likely recipient"
	| localEvt |
	formerHaloOwner == self ifTrue:[self setProperty: #surviveHaloLoss toValue: true].
	"Never transfer halo to top-most world"
	(self wantsHaloFromClick and:[formerHaloOwner ~~ self]) 
		ifTrue:[^self addHalo: event from: formerHaloOwner].

	event shiftPressed ifTrue:[
		"Pass it outwards"
		^owner ifNotNil:[owner transferHalo: event from: formerHaloOwner]].
	self submorphsDo:[:m|
		localEvt _ event transformedBy: (m transformedFrom: self).
		(m fullContainsPoint: localEvt position) 
			ifTrue:[^m transferHalo: event from: formerHaloOwner].
	].
	"We're at the bottom most level; throw the event back up to the root to find recipient"
	^self getCameraMorph transferHalo: event from: formerHaloOwner.! !

!WonderlandWrapperMorph methodsFor: 'accessing' stamp: 'ar 11/7/1999 19:30'!
knownName
	^myActor getName! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:17'!
removeAllHandlesBut: aHandle
	| halo |
	halo _ self halo.
	halo == nil ifFalse:[halo removeAllHandlesBut: aHandle].! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 5/22/2001 18:24'!
balloonHelpTextForHandle: aHandle
	|  itsSelector |
	itsSelector _ aHandle eventHandler firstMouseSelector.
	#(	(rotateStartFromHalo:with:				'Rotate')
		(growStartFromHalo:with:				'Change size') 
		(dupStartFromHalo:with:					'Duplicate')
		(grabFromHalo:with:						'Pick up')
		(dragStartFromHalo:with:				'Move')
		(extractTexture:							'Extract texture')
		(paintTexture							'Paint on surface')
	) do:
		[:pair | itsSelector == pair first ifTrue: [^ pair last]].
	^super balloonHelpTextForHandle: aHandle! !

!WonderlandWrapperMorph methodsFor: 'menu' stamp: 'ar 11/7/1999 22:35'!
toggleActiveTextureState
	(myActor getProperty: #activeTexture) == true
		ifTrue:[myActor setProperty: #activeTexture toValue: false]
		ifFalse:[myActor setProperty: #activeTexture toValue: true]! !

!WonderlandWrapperMorph methodsFor: 'private' stamp: 'ar 11/8/1999 14:10'!
computeFullBounds: cameraMorph
	self submorphsDo:[:m| m computeFullBounds: cameraMorph].
	self computeBounds: cameraMorph.
	fullBounds _ nil.! !

!WonderlandWrapperMorph methodsFor: 'drawing' stamp: 'ar 11/7/1999 19:12'!
fullDrawOn: aCanvas
	super fullDrawOn: aCanvas.
	fullBounds _ nil. "Force recomputation"! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:16'!
dragPositionOffset
	^self valueOfProperty: #dragPositionOffset.! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:30'!
growEndFromHalo: evt with: handle
	self growPositionOffset: nil.
	self halo endInteraction.! !

!WonderlandWrapperMorph methodsFor: 'menu' stamp: 'ar 11/7/1999 22:38'!
getTextureAdjust
	(myActor getProperty: #adjustToTexture) == true
		ifTrue:[^'do not adjust to texture']
		ifFalse:[^'auto adjust to texture']! !

!WonderlandWrapperMorph methodsFor: 'private' stamp: 'ar 11/8/1999 15:18'!
undo: anUndoClass using: aValue
	"Check if we need to add an undo action."
	myWonderland getUndoStack push: (anUndoClass for: myActor from: aValue).! !

!WonderlandWrapperMorph methodsFor: 'menu' stamp: 'ar 6/9/2000 19:31'!
toggleHQTextureState
	| tex |
	tex _ myActor getTexturePointer.
	tex isMorph ifFalse:[^self].
	(tex valueOfProperty: #highQualityTexture) == true
		ifTrue:[tex setProperty: #highQualityTexture toValue: false]
		ifFalse:[tex setProperty: #highQualityTexture toValue: true].
	tex changed.! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:16'!
dragPositionOffset: aPoint
	aPoint == nil
		ifTrue:[self removeProperty: #dragPositionOffset]
		ifFalse:[self setProperty: #dragPositionOffset toValue: aPoint].! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 10/10/2000 22:14'!
dismissViaHalo
	myActor destroy.
	self delete.! !

!WonderlandWrapperMorph methodsFor: 'private' stamp: 'ar 4/20/2001 03:37'!
computeBounds: morph
	| box |
	box _ myActor getFullBoundsFor: morph getCamera.
	box == nil ifFalse:[bounds _ box].! !

!WonderlandWrapperMorph methodsFor: 'accessing' stamp: 'ar 11/7/1999 21:36'!
color
	^myActor getColorVector asColor! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:30'!
rotateMoveFromHalo: evt with: handle! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/7/1999 20:34'!
extractTexture: evt
	evt hand attachMorph: myActor getTexturePointer asMorph! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:17'!
dragStartFromHalo: evt with: handle
	self removeAllHandlesBut: handle.
	self dragPositionOffset: handle center - self positionInWorld.
	self undo: UndoPOVChange using: myActor getPointOfView.! !

!WonderlandWrapperMorph methodsFor: 'drawing' stamp: 'ar 11/8/1999 13:45'!
drawOn: aCanvas
	| morph |
	morph _ self getCameraMorph.
	morph == nil
		ifTrue:[super drawOn: aCanvas]
		ifFalse:[self computeBounds: morph. "Update bounds from camera"
			false ifTrue:["Show a rectangle for the wrappers"
				aCanvas frameRectangle: bounds color: Color white]]! !

!WonderlandWrapperMorph methodsFor: 'menu' stamp: 'ar 11/7/1999 22:36'!
toggleTextureAdjust
	(myActor getProperty: #adjustToTexture) == true
		ifTrue:[myActor setProperty: #adjustToTexture toValue: false]
		ifFalse:[myActor setProperty: #adjustToTexture toValue: true]! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:22'!
growStartFromHalo: evt with: handle

	self removeAllHandlesBut: handle.
	self growPositionOffset: evt cursorPoint - (self pointInWorld: self bottomRight).
	self undo: UndoSizeChange using: myActor getSize.! !

!WonderlandWrapperMorph methodsFor: 'private' stamp: 'ar 11/7/1999 21:29'!
undoOnTop: anUndoClass using: aBlock
	"Check if we need to add an undo action."
	| stack top |
	stack _ myWonderland getUndoStack.
	top _ stack top.
	((top isKindOf: anUndoClass) and:[top getActor == myActor]) ifFalse:[
		stack push: (anUndoClass for: myActor from: aBlock value)].
! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:39'!
growMoveFromHalo: evt with: handle
	| newExtent scale |
	newExtent _ (self pointFromWorld: (evt cursorPoint - self growPositionOffset)) - self topLeft.
	newExtent _ newExtent max: 1@1.
	scale _ newExtent r / bounds extent r.
	evt shiftPressed
		ifTrue:[scale _ B3DVector3 x: scale y: scale z: 1.0]
		ifFalse:[scale _ B3DVector3 x: scale y: scale z: scale].
 	myActor resizeRightNow: scale undoable: false.
	handle position: evt cursorPoint - (handle extent // 2).! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/10/1999 15:16'!
dupMoveFromHalo: evt with: handle
	^self dragMoveFromHalo: evt with: handle! !

!WonderlandWrapperMorph methodsFor: 'menu' stamp: 'ar 6/9/2000 19:30'!
getHQTextureState
	| tex |
	tex _ myActor getTexturePointer.
	(tex isMorph and:[(tex valueOfProperty: #highQualityTexture) == true])
		ifTrue:[^'disable high quality texture']
		ifFalse:[^'enable high quality texture']! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:25'!
grabFromHalo: evt with: handle! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/10/1999 15:32'!
dupEndFromHalo: evt with: handle
	^self dragEndFromHalo: evt with: handle! !

!WonderlandWrapperMorph methodsFor: 'menu' stamp: 'ar 5/22/2001 18:24'!
paintTexture
	self getCameraMorph paintActor: myActor.! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 16:42'!
dragMoveFromHalo: evt with: handle
	| morph delta screenPos scenePos |
	morph _ self getCameraMorph.
	morph == nil ifTrue:[^self].
	delta _ (self pointFromWorld: evt cursorPoint - self dragPositionOffset) - self position.
	screenPos _ (myActor getPositionInPixels: morph getCamera) + delta.
	scenePos _ morph getCamera transformScreenPointToScenePoint: screenPos atDepthOf: myActor.
	myActor moveToRightNow: scenePos asSeenBy: (myWonderland getScene) undoable: false.! !

!WonderlandWrapperMorph methodsFor: 'initialization' stamp: 'ar 11/7/1999 19:07'!
createHierarchy
	"Create a wrapper hierarchy representing the owners of the receiver"
	| actor wrapper |
	actor _ myActor getParent.
	actor == myWonderland getScene ifTrue:[^self]. "At top"
	"Create new wrapper"
	wrapper _ self class on: actor.
	wrapper addMorphFront: self.
	"And continue there"
	^wrapper createHierarchy! !

!WonderlandWrapperMorph methodsFor: 'menu' stamp: 'ar 11/7/1999 22:35'!
getActiveTextureState
	(myActor getProperty: #activeTexture) == true
		ifTrue:[^'disable active texture']
		ifFalse:[^'enable active texture']! !

!WonderlandWrapperMorph methodsFor: 'initialization' stamp: 'ar 11/7/1999 19:03'!
setActor: anActor
	myActor _ anActor.
	myWonderland _ anActor getWonderland.
! !

!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'ar 11/8/1999 15:20'!
growPositionOffset: aPoint
	aPoint == nil
		ifTrue:[self removeProperty: #growPositionOffset]
		ifFalse:[self setProperty: #growPositionOffset toValue: aPoint].! !

Morph subclass: #WonderlandActorViewer
	instanceVariableNames: 'thumbnailMorph thumbnailCamera selectedActor dataMorph updatingIsOn updatingAction'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandActorViewer methodsFor: 'initialization' stamp: 'jsp 7/8/1999 15:58'!
initializeWith: aWonderland
	"Do the Wonderland specific setup for this viewer."

	thumbnailCamera _ WonderlandStillCamera createFor: aWonderland.
	thumbnailCamera turnBackgroundOff.

	thumbnailMorph _ thumbnailCamera getMorph.
	thumbnailMorph topLeft: 10@10.
	self addMorph: thumbnailMorph.
! !

!WonderlandActorViewer methodsFor: 'updating' stamp: 'jsp 7/8/1999 22:49'!
setSelectedActor: anActor
	"Update the actor viewer to reflect the information for this actor"

	selectedActor _ anActor.

	thumbnailCamera setFocusObject: anActor.
	dataMorph setActor: anActor.


! !

!WonderlandActorViewer methodsFor: 'initialization' stamp: 'jsp 7/8/1999 21:56'!
initialize
	"Initialize this morph"

	super initialize.

	updatingIsOn _ false.

	self name: 'Actor Info'.
	self extent: 500@350.
	self color: (Color r: 0.815 g: 0.972 b: 0.878).

	dataMorph _ WonderlandActorData new.
	self addMorph: dataMorph.
	dataMorph topLeft: 200@10.
! !

!WonderlandActorViewer methodsFor: 'updating' stamp: 'jsp 7/8/1999 22:49'!
turnUpdatingOff
	"Set the actor viewer to stop updating the selected actor's data."

	updatingIsOn _ false.
! !

!WonderlandActorViewer methodsFor: 'stepping' stamp: 'jsp 7/8/1999 22:52'!
wantsSteps

	^ true.
! !

!WonderlandActorViewer methodsFor: 'stepping' stamp: 'jsp 7/8/1999 23:08'!
step

	(updatingIsOn and: [ selectedActor notNil ] )
		ifTrue: [
				dataMorph setActor: selectedActor.
				thumbnailCamera setFocusObject: selectedActor.
				].

! !

!WonderlandActorViewer methodsFor: 'updating' stamp: 'jsp 7/8/1999 22:49'!
turnUpdatingOn
	"Set the actor viewer to constantly update the selected actor's data."

	updatingIsOn _ true.

! !

!WonderlandActorViewer methodsFor: 'stepping' stamp: 'jsp 7/8/1999 22:52'!
stepTime

	^ 1000.
! !

!Morph methodsFor: '*Balloon3D-Wonderland' stamp: 'ar 11/16/2002 19:42'!
mapPrimitiveVertex: aPrimitiveVertex
	"Map the given primitive vertex into 2D space."
	| pt |
	pt _ aPrimitiveVertex texCoordS @ aPrimitiveVertex texCoordT.
	^(self extent * pt) asIntegerPoint + self position.! !

!Morph methodsFor: '*Balloon3D-Wonderland' stamp: 'ar 11/16/2002 19:42'!
asTexture
	"Represent the receiver as a Wonderland texture."
	| canvas texture hqTexture texExtent tempForm valid |
	hqTexture _ self valueOfProperty: #highQualityTexture ifAbsent:[false].
	valid _ self isValidWonderlandTexture.
	texture _ self wonderlandTexture.
	"Determine the size we expect the texture to be.
	Note: This size must never be less than the receiver's since
	this will lead to (unwanted) clipping."
	texExtent _ self extent.
	hqTexture ifTrue:[
		"High quality textures round up to next power of two"
		texExtent _ texExtent x asLargerPowerOfTwo @ texExtent y asLargerPowerOfTwo.
	].
	(texture == nil or:[texture extent ~= texExtent]) ifTrue:[
		self removeProperty: #wonderlandTexture.
		texture _ nil. "Clean up for GC"
		texture _ B3DTexture extent: texExtent depth: 32.
		texture interpolate: false.
		texture wrap: false.
		texture envMode: 0.
		valid _ false].
	valid ifFalse:[
		canvas _ texture getCanvas.
		canvas translateBy: self topLeft negated
			during:[:tempCanvas| tempCanvas fullDrawMorph: self].
		self isValidWonderlandTexture: true.
		"High quality textures need an extra pass here"
		(texExtent ~= self extent) ifTrue:[
			tempForm _ texture contentsOfArea: (0@0 extent: self extent).
			tempForm displayInterpolatedOn: texture].
	].
	self setProperty: #wonderlandTexture toValue: texture.
	^ texture! !

Morph subclass: #WonderlandMorph
	instanceVariableNames: 'myWonderland'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandMorph methodsFor: 'morph depth' stamp: 'jsp 4/12/1999 15:45'!
sendToBack
	"Bring this morph to the back"

	self owner addMorphBack: self.
! !

!WonderlandMorph methodsFor: 'morph depth' stamp: 'jsp 4/12/1999 15:45'!
sendToFront
	"Bring this morph to the front"

	self owner addMorphFront: self.
! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/12/1999 15:01'!
move: aDirection distance: aDistance duration: aDuration style: aStyle
	"Move the actor the specified distance in the specified direction, taking the specified duration and using the specified animation style."

	| anim aPoint tmpPoint |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyDirection: aDirection ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the direction to move the morph because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyNumber: aDistance ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the distance to move the morph because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for moving the morph because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to move the morph because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ RelativeAnimation new.
	tmpPoint _ 0@0.

	(aDirection = left) ifTrue: [ aPoint _ (aDistance negated)@0 ]
		ifFalse: [ (aDirection = right) ifTrue: [ aPoint _ aDistance@0 ]
					ifFalse: [ (aDirection = up) ifTrue: [ aPoint _ 0@(aDistance negated) ]
								ifFalse: [ (aDirection = down) ifTrue: [ aPoint _ 0@aDistance ]
										ifFalse: [ myWonderland reportErrorToUser: 'Squeak does not know how to move the morph in that direction.'.
												^ nil ].
										].
							].
				].

	anim object: self
			update: [:tPos | self position: ((self position) + ((tPos rounded) - tmpPoint)).
							tmpPoint _ tPos rounded]
			getStartState: [tmpPoint _ 0@0. Interpolateable value: 0@0. ]
			getEndState: [ aPoint ]
			getReverseState: [ aPoint negated ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

WonderlandMorph subclass: #WonderlandCameraMorph
	instanceVariableNames: 'myCamera myControls eventFocus lastCursorPoint mouseUpButton firstPersonControls mode outline currentActor currentPosition currentCanvas currentColor currentNib lastPosition paintMode palette restore backup pickedPoint pickedActor myRenderer'
	classVariableNames: 'CachedNibs'
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandCameraMorph commentStamp: '<historical>' prior: 0!
This class implements the morph that provides a view into the 3D world.  Each instance has a camera assigned to it that determines the viewpoint and parameters of the view.  Each instance also has the camera controls for that camera attached to it, and is responsible for showing or hiding them.

!

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 5/28/2001 17:21'!
getAAPaintState
	^self aaPaintingEnabled
		ifTrue:['<on>antialiased texture painting']
		ifFalse:['<off>antialiased texture painting']
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 14:18'!
mouseDownEllipse: newEvent
	self mouseDownLine: newEvent
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/27/2000 15:50'!
polygon: newEvent
! !

!WonderlandCameraMorph methodsFor: 'change reporting' stamp: 'jsp 3/4/1999 14:40'!
layoutChanged
	"Process the changed layout and then change the aspect ratio of the camera to match"

	super layoutChanged.

	myCamera setAspectRatio: ((self width) / (self height)) asFloat.
! !

!WonderlandCameraMorph methodsFor: 'drawing' stamp: 'ar 5/22/2001 20:50'!
drawAcceleratedOn: aCanvas
	| myRect |
	myRect _ (self bounds: bounds in: nil) intersect: (0@0 extent: DisplayScreen actualScreenSize).
	(myRenderer notNil and:[myRenderer isAccelerated]) ifFalse:[
		myRenderer ifNotNil:[myRenderer destroy].
		myRenderer _ nil.
	].
	myRenderer ifNotNil:[
		myRenderer _ myRenderer bufferRect: myRect.
	].
	myRenderer ifNil:[
		myRenderer _ B3DHardwareEngine newIn: myRect.
		myRenderer ifNil:[^self drawSimulatedOn: aCanvas].
	] ifNotNil:[
		myRenderer reset.
	].
	myRenderer viewportOffset: aCanvas origin.
	myRenderer clipRect: aCanvas clipRect.
	myCamera drawSceneBackground ifFalse:[
		myRenderer restoreMorphicBackground: myRect under: self.
	].
	self renderOn: myRenderer.
	outline ifNotNil:[
		self drawAcceleratedOutlineOn: myRenderer.
	].
	myRenderer restoreMorphicForeground: myRect above: self.
	Display addExtraRegion: myRect for: self.! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 9/15/2000 23:25'!
mouseEnter: evt
	"When the mouse enters our window give it the keyboard focus"

	evt hand newKeyboardFocus: self.

	mode = #paint ifTrue: [evt hand showTemporaryCursor: palette actionCursor].
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 16:00'!
storeTexture
	restore _ currentCanvas form deepCopy.
! !

!WonderlandCameraMorph methodsFor: 'controls' stamp: 'jsp 3/30/1999 15:00'!
hideCameraControls
	"Hide the camera controls for the camera rendering into this window"

	myControls ifNotNil: [ myControls hide ].

! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/15/2001 14:00'!
fogRangeStart
	^myCamera getFogRangeStart! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 5/9/2001 14:39'!
showControlsString
	^(myControls notNil)
		ifTrue:[^'<on>camera controls'] 
		ifFalse:['<off>camera controls'].! !

!WonderlandCameraMorph methodsFor: 'accessing' stamp: 'ar 5/22/2001 13:20'!
outline: pointList
	outline _ pointList! !

!WonderlandCameraMorph methodsFor: 'debug' stamp: 'ar 5/25/2000 18:02'!
debugDraw
	Display deferUpdates: false.
	self fullDrawOn: (Display getCanvas).
	Display deferUpdates: false.
	Display forceToScreen: bounds.! !

!WonderlandCameraMorph methodsFor: 'drawing' stamp: 'ar 5/14/2001 23:09'!
forceToScreen
	"When using hardware acceleration, this method forces the receiver to swap its buffers."
	myRenderer ifNotNil:[
		myRenderer swapBuffers ifFalse:[
			"Something went wrong"
			myRenderer destroy.
			myRenderer _ nil]].! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 11/6/2000 23:38'!
containsPoint: aPoint event: anEvent
	(anEvent isMouseOver or:[firstPersonControls == true]) ifTrue:[
		"Approximate for mouseOver; they're just sent too often"
		^self bounds containsPoint: aPoint].
	^super containsPoint: aPoint event: anEvent! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/26/2001 16:39'!
mouseUpFirstPersonControl: evt
	myControls mouseUp: evt.
	myControls setCenter: nil.
	evt hand needsToBeDrawn ifFalse:[Cursor normal show].
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'ar 5/27/2001 16:32'!
roundNibOfSize: extent
	"Create a round nib for antialiased painting"
	| largeForm smallForm warp colorForm |
	CachedNibs ifNil:[CachedNibs _ Dictionary new].
	colorForm _ CachedNibs at: extent ifAbsent:[nil].
	colorForm ifNotNil:[^colorForm].
	largeForm _ Form extent: extent * 4 depth: 32.
	largeForm getCanvas fillOval: largeForm boundingBox color: Color white.
	smallForm _ Form extent: extent depth: 32.
	warp _ WarpBlt toForm: smallForm.
	warp sourceForm: largeForm destRect: smallForm boundingBox.
	warp cellSize: 4. "oh yes"
	warp warpBits.
	largeForm _ nil.
	colorForm _ smallForm asGrayScale.
	CachedNibs at: extent put: colorForm.
	^colorForm! !

!WonderlandCameraMorph methodsFor: 'hardware acceleration' stamp: 'rww 10/1/2001 01:18'!
accelerationEnabled: aBool
	"Enable or disable hardware acceleration"
	| aWorld |
	"no dangling renderers please"
	myRenderer ifNotNil:[myRenderer destroy].
	myRenderer _ nil.
	aBool
		ifTrue:[self setProperty: #accelerationEnabled toValue: aBool]
		ifFalse:[self removeProperty: #accelerationEnabled].
	"For now make sure we're registered with the world in case this guy came from the disk"
	(aWorld _ self world) ifNotNil:[
		aWorld removeActionsWithReceiver: self.
		aWorld when: #aboutToLeaveWorld send: #suspendAcceleration to: self.
		aWorld when: #aboutToEnterWorld send: #restoreAcceleration to: self.
	].! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/15/2001 13:59'!
fogColor
	^myCamera getFogColor! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/15/2001 14:01'!
fogType
	^(#(none linear exp exp2) indexOf: myCamera getFogType) - 1! !

!WonderlandCameraMorph methodsFor: 'initialization' stamp: 'ar 4/20/2001 02:17'!
intoWorld: aWorld
	"The receiver is showing in the given world"
	aWorld ifNil:[^self].
	super intoWorld: aWorld.
	aWorld when: #aboutToLeaveWorld send: #suspendAcceleration to: self.
	aWorld when: #aboutToEnterWorld send: #restoreAcceleration to: self.
	self restoreAcceleration.! !

!WonderlandCameraMorph methodsFor: 'hardware acceleration' stamp: 'ar 4/20/2001 02:15'!
accelerationSuspended
	"Return true if hardware acceleration is temporarily suspended"
	^self valueOfProperty: #accelerationSuspended ifAbsent:[false]! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'laza 3/24/2000 16:32'!
clearPainting: aPaintBoxMorph
	"Clear the image after user issued #clear in aPaintBoxMorph"
	^self clear! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/27/2001 16:36'!
mouseMovePaint: evt 
	| newEvent |
	newEvent _ self convertEvent: evt.
	newEvent ifNil:[^self].
	newEvent getVertex ifNil:[^self].
	^self perform: palette action with: newEvent! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'laza 3/24/2000 16:32'!
paletteAttached: aPaintBoxMorph
	"A new palette has been attached to the receiver.
	Don't know what to do here..."! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/15/2001 14:00'!
fogColor: aColor
	myCamera setFogColor: aColor! !

!WonderlandCameraMorph methodsFor: 'controls' stamp: 'jsp 3/30/1999 15:02'!
showCameraControls
	"Show the camera controls for the camera rendering into this window"

	myControls ifNil: [
			myControls _ WonderlandCameraControls newFor: myCamera.
			self addMorph: myControls.
					  ].

	myControls show.
! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'laza 3/30/2000 11:03'!
save
	(self getWonderland getUndoStack) push: (UndoTextureChange for: currentActor from: backup).
	palette delete.
	mode _ nil
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/27/2000 16:43'!
mouseUpRect: newEvent

! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/15/2001 14:12'!
fogDensity: aNumber
	myCamera setFogDensity: (aNumber max: 0.0)! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/30/2000 10:02'!
restoreTexture
	self restoreTextureFrom: restore! !

!WonderlandCameraMorph methodsFor: 'initialization' stamp: 'rww 10/1/2001 01:18'!
outOfWorld: aWorld
	"The receiver is leaving the given world"
	aWorld ifNil:[^self].
	self suspendAcceleration.
	aWorld removeActionsWithReceiver: self.
	super outOfWorld: aWorld.! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/30/2000 10:02'!
restoreTextureFrom: aForm
	| texture |
	texture _ aForm deepCopy.
	currentActor setTexturePointer: texture.
	currentCanvas setForm: texture.! !

!WonderlandCameraMorph class methodsFor: 'initialize-release' stamp: 'laza 3/24/2000 14:23'!
initialize
	"WonderlandCameraMorph initialize"
	self installPoohIcon! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'ar 5/28/2001 17:52'!
paint: newEvent
	| bb newPosition dx firstY lastY dy py midX |
	newPosition _ newEvent getVertex texCoords * currentCanvas extent - (currentNib extent // 2).
	"Watch out for wrapping in y"
	(newPosition y - currentPosition y) abs > (currentCanvas form height // 2) ifTrue:[
		"Go the other (shorter) way around"
		dx _ newPosition x - currentPosition x.
		newPosition y > currentPosition y ifTrue:[
			firstY _ 0. lastY _ currentCanvas form height.
			dy _ currentCanvas form height - newPosition y + currentPosition y.
			py _ currentPosition y.
		] ifFalse:[
			firstY _ currentCanvas form height. lastY _ 0.
			dy _ currentCanvas form height - currentPosition y + newPosition y.
			py _ currentCanvas form height - currentPosition y.
		].
		midX _ currentPosition x + (dx * (py / dy)).
		midX _ midX asInteger.
	].
	bb _ BitBlt toForm: currentCanvas form.
	bb 
		sourceForm: currentNib; 
		sourceRect: currentNib boundingBox;
		colorMap: (currentNib colormapIfNeededFor: bb destForm);
		combinationRule: Form blend.
	midX == nil ifTrue:[
		bb drawFrom: currentPosition asIntegerPoint to: newPosition asIntegerPoint.
	] ifFalse:[
		bb 
			drawFrom: currentPosition asIntegerPoint to: midX @ firstY;
			drawFrom: midX @ lastY to: newPosition asIntegerPoint.
	].
	currentPosition _ newPosition.! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/27/2000 13:42'!
line: newEvent
	| type |
	type _ newEvent getMorphicEvent type.
	self perform: (type, 'Line:') asSymbol with: newEvent
! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 9/16/1999 16:39'!
firstPersonKeystroke: evt
	"Handle a keyboard event"
	| keyValue |
	lastCursorPoint _ evt cursorPoint.
	keyValue _ evt keyValue.
	evt shiftPressed ifTrue:[
		keyValue = 28 ifTrue:[^myCamera move: left distance: 0.25].
		keyValue = 29 ifTrue:[^myCamera move: right distance: 0.25].
		keyValue = 30 ifTrue:[^myCamera move: forward distance: 0.25].
		keyValue = 31 ifTrue:[^myCamera move: backward distance: 0.25].
	] ifFalse:[
		keyValue = 28 ifTrue:[^myCamera turn: left turns:  0.0625 duration: 0.25 style: abruptly].
		keyValue = 29 ifTrue:[^myCamera turn: right turns:  0.0625 duration: 0.25 style: abruptly].
		keyValue = 30 ifTrue:[^myCamera move: forward].
		keyValue = 31 ifTrue:[^myCamera move: backward].
	].! !

!WonderlandCameraMorph methodsFor: 'accessing' stamp: 'ar 5/22/2001 13:20'!
outline
	^ outline ifNil:[outline _ ReadWriteStream on: #()]! !

!WonderlandCameraMorph methodsFor: 'halos and balloon help' stamp: 'laza 3/24/2000 13:56'!
balloonHelpTextForHandle: aHandle
	|  itsSelector |
	itsSelector _ aHandle eventHandler firstMouseSelector.
	itsSelector == #strokeMode ifTrue: [^'Create Pooh Object'].
	^ super balloonHelpTextForHandle: aHandle! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/9/2001 14:36'!
doFirstPersonControl
	^firstPersonControls == true! !

!WonderlandCameraMorph methodsFor: 'wrappers' stamp: 'ar 11/7/1999 22:00'!
removeAllWrappers
	submorphs do:[:m|
		(m isKindOf: WonderlandWrapperMorph) ifTrue:[m delete]].! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'bf 5/9/2001 18:07'!
loadActor
	myWonderland loadActor! !

!WonderlandCameraMorph methodsFor: 'pooh' stamp: 'ar 5/22/2001 13:21'!
clearStroke
	self outline: nil! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'ar 5/28/2001 17:12'!
clear
	currentActor ifNotNil: [
		currentActor getTexturePointer fillColor: Color white.
	].! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 11/7/1999 23:27'!
keyStroke: evt
	"Handle a keyboard event"

	| newEvent reactions |
	firstPersonControls == true "For existing camera morphs"
		ifTrue:[^self firstPersonKeystroke: evt].

	newEvent _ self convertEvent: evt.
	newEvent ifNil:[^self].
	"Check for active textures"
	newEvent getActor hasActiveTexture 
		ifTrue:[^newEvent getActor morphicKeyPress: newEvent].
	"Route to actor"
	reactions _ newEvent getActor getReactionsTo: keyPress.
	reactions ifNotNil:[ 
		reactions do: [:aReaction | aReaction reactTo: newEvent ]].
! !

!WonderlandCameraMorph class methodsFor: 'scripting' stamp: 'sw 9/27/2001 17:48'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((fog (
			(slot fogColor 'The color of fog being applied' Color readWrite 
				Player getFogColor Player setFogColor:)
			(slot fogType 'The type of fog being applied' Number readWrite 
				Player getFogType Player setFogType:)
			(slot fogDensity 'The density of fog being applied' Number readWrite 
				Player getFogDensity Player setFogDensity:)
			(slot fogRangeStart 'The range start of fog being applied' Number readWrite 
				Player getFogRangeStart Player setFogRangeStart:)
			(slot fogRangeEnd 'The range start of fog being applied' Number readWrite 
				Player getFogRangeEnd Player setFogRangeEnd:)
)))! !

!WonderlandCameraMorph class methodsFor: 'pooh stuff' stamp: ''!
poohTape	^'36 mouseMove 59 236 0 0
589 mouseMove 60 233 0 0
34 mouseMove 60 232 0 0
73 mouseMove 60 231 0 0
35 mouseMove 61 231 0 0
28 mouseMove 62 230 0 0
35 mouseMove 62 228 0 0
62 mouseMove 63 226 0 0
39 mouseMove 65 224 0 0
35 mouseMove 66 222 0 0
37 mouseMove 67 220 0 0
28 mouseMove 69 217 0 0
35 mouseMove 70 215 0 0
28 mouseMove 70 214 0 0
36 mouseMove 71 213 0 0
82 mouseMove 72 212 0 0
29 mouseMove 72 210 0 0
34 mouseMove 73 209 0 0
28 mouseMove 73 208 0 0
36 mouseMove 74 206 0 0
30 mouseMove 74 204 0 0
41 mouseMove 74 202 0 0
36 mouseMove 75 200 0 0
28 mouseMove 75 198 0 0
35 mouseMove 75 197 0 0
63 mouseMove 76 195 0 0
38 mouseMove 77 191 0 0
36 mouseMove 79 186 0 0
35 mouseMove 81 182 0 0
35 mouseMove 82 179 0 0
28 mouseMove 83 177 0 0
37 mouseMove 84 173 0 0
28 mouseMove 85 171 0 0
43 mouseMove 87 164 0 0
35 mouseMove 88 160 0 0
29 mouseMove 89 156 0 0
36 mouseMove 90 149 0 0
28 mouseMove 91 144 0 0
37 mouseMove 91 139 0 0
28 mouseMove 91 138 0 0
51 mouseMove 92 132 0 0
29 mouseMove 92 129 0 0
35 mouseMove 92 124 0 0
28 mouseMove 92 120 0 0
36 mouseMove 92 115 0 0
28 mouseMove 92 111 0 0
54 mouseMove 92 107 0 0
36 mouseMove 92 105 0 0
28 mouseMove 92 103 0 0
36 mouseMove 92 99 0 0
28 mouseMove 92 96 0 0
36 mouseMove 92 92 0 0
28 mouseMove 92 89 0 0
48 mouseMove 92 85 0 0
27 mouseMove 92 83 0 0
35 mouseMove 92 81 0 0
28 mouseMove 92 79 0 0
35 mouseMove 92 76 0 0
36 mouseMove 92 74 0 0
32 mouseMove 92 72 0 0
45 mouseMove 92 69 0 0
29 mouseMove 92 66 0 0
35 mouseMove 92 64 0 0
3339 mouseDown 92 64 1 0
138 mouseMove 92 64 1 0
59 mouseUp 92 64 0 0
34 mouseMove 92 64 0 0
753 mouseMove 95 65 0 0
29 mouseMove 101 68 0 0
38 mouseMove 114 76 0 0
29 mouseMove 116 77 0 0
67 mouseMove 117 78 0 0
28 mouseMove 118 78 0 0
51 mouseMove 119 79 0 0
28 mouseMove 120 80 0 0
38 mouseMove 122 81 0 0
29 mouseMove 124 82 0 0
35 mouseMove 125 82 0 0
77 mouseMove 128 84 0 0
36 mouseMove 130 84 0 0
28 mouseMove 131 85 0 0
37 mouseMove 134 86 0 0
30 mouseMove 137 87 0 0
37 mouseMove 143 88 0 0
35 mouseMove 145 88 0 0
46 mouseMove 148 89 0 0
37 mouseMove 151 90 0 0
28 mouseMove 153 90 0 0
36 mouseMove 156 90 0 0
30 mouseMove 159 91 0 0
36 mouseMove 163 91 0 0
40 mouseMove 168 92 0 0
36 mouseMove 171 92 0 0
29 mouseMove 173 92 0 0
34 mouseMove 176 93 0 0
37 mouseMove 179 94 0 0
29 mouseMove 179 95 0 0
37 mouseMove 180 95 0 0
45 mouseMove 181 96 0 0
1953 mouseDown 181 96 4 0
118 mouseUp 181 96 0 0
28 mouseMove 181 96 0 0
584 mouseMove 178 93 0 0
39 mouseMove 175 91 0 0
34 mouseMove 174 90 0 0
64 mouseMove 170 89 0 0
28 mouseMove 168 87 0 0
34 mouseMove 162 85 0 0
46 mouseMove 157 83 0 0
29 mouseMove 156 82 0 0
35 mouseMove 154 81 0 0
28 mouseMove 152 79 0 0
35 mouseMove 151 78 0 0
28 mouseMove 150 77 0 0
50 mouseMove 148 75 0 0
28 mouseMove 145 74 0 0
36 mouseMove 141 72 0 0
30 mouseMove 138 70 0 0
35 mouseMove 136 69 0 0
28 mouseMove 134 67 0 0
35 mouseMove 130 65 0 0
46 mouseMove 126 64 0 0
28 mouseMove 125 63 0 0
36 mouseMove 122 62 0 0
28 mouseMove 120 62 0 0
35 mouseMove 118 61 0 0
28 mouseMove 116 61 0 0
53 mouseMove 113 60 0 0
27 mouseMove 112 59 0 0
62 mouseMove 111 59 0 0
35 mouseMove 109 58 0 0
60 mouseMove 106 57 0 0
35 mouseMove 105 57 0 0
39 mouseMove 104 56 0 0
183 mouseDown 104 56 4 0
83 mouseMove 103 56 4 0
45 mouseMove 101 56 4 0
50 mouseMove 98 55 4 0
45 mouseMove 96 55 4 0
62 mouseMove 92 55 4 0
47 mouseMove 89 55 4 0
45 mouseMove 87 55 4 0
46 mouseMove 83 55 4 0
53 mouseMove 80 55 4 0
73 mouseMove 77 55 4 0
48 mouseMove 74 55 4 0
46 mouseMove 71 56 4 0
47 mouseMove 68 58 4 0
65 mouseMove 64 61 4 0
47 mouseMove 62 63 4 0
47 mouseMove 60 65 4 0
47 mouseMove 58 66 4 0
65 mouseMove 54 70 4 0
54 mouseMove 51 73 4 0
47 mouseMove 49 76 4 0
49 mouseMove 47 80 4 0
56 mouseMove 46 83 4 0
47 mouseMove 46 86 4 0
51 mouseMove 46 90 4 0
47 mouseMove 46 92 4 0
47 mouseMove 46 96 4 0
63 mouseMove 46 100 4 0
56 mouseMove 46 106 4 0
49 mouseMove 48 112 4 0
52 mouseMove 50 116 4 0
62 mouseMove 54 121 4 0
48 mouseMove 56 123 4 0
52 mouseMove 59 126 4 0
48 mouseMove 61 128 4 0
59 mouseMove 64 130 4 0
49 mouseMove 68 133 4 0
56 mouseMove 73 136 4 0
48 mouseMove 77 137 4 0
59 mouseMove 84 138 4 0
49 mouseMove 88 139 4 0
58 mouseMove 96 140 4 0
50 mouseMove 101 140 4 0
48 mouseMove 106 140 4 0
52 mouseMove 108 140 4 0
52 mouseMove 112 140 4 0
51 mouseMove 116 139 4 0
48 mouseMove 120 137 4 0
49 mouseMove 124 135 4 0
67 mouseMove 131 131 4 0
49 mouseMove 134 127 4 0
51 mouseMove 138 124 4 0
49 mouseMove 140 121 4 0
60 mouseMove 143 116 4 0
55 mouseMove 145 110 4 0
49 mouseMove 147 106 4 0
56 mouseMove 148 100 4 0
59 mouseMove 148 95 4 0
57 mouseMove 148 89 4 0
50 mouseMove 148 85 4 0
48 mouseMove 148 82 4 0
70 mouseMove 148 79 4 0
55 mouseMove 148 77 4 0
49 mouseMove 148 74 4 0
48 mouseMove 148 72 4 0
50 mouseMove 147 69 4 0
67 mouseMove 145 65 4 0
50 mouseMove 143 62 4 0
56 mouseMove 141 60 4 0
58 mouseMove 140 59 4 0
102 mouseMove 139 59 4 0
48 mouseMove 139 58 4 0
110 mouseUp 139 58 0 0
1457 mouseMove 139 58 0 0
3329 mouseMove 140 59 0 0
84 mouseMove 141 60 0 0
57 mouseMove 142 60 0 0
63 mouseMove 145 60 0 0
64 mouseMove 152 61 0 0
71 mouseMove 161 62 0 0
65 mouseMove 169 63 0 0
62 mouseMove 179 64 0 0
76 mouseMove 225 72 0 0
60 mouseMove 240 75 0 0
68 mouseMove 243 76 0 0
71 mouseMove 248 78 0 0
121 mouseMove 248 81 0 0
61 mouseMove 248 87 0 0
80 mouseMove 248 91 0 0
66 mouseMove 248 94 0 0
59 mouseMove 248 98 0 0
80 mouseMove 248 107 0 0
61 mouseMove 247 111 0 0
64 mouseMove 246 115 0 0
69 mouseMove 244 121 0 0
65 mouseMove 243 124 0 0
268 mouseDown 243 124 4 0
198 mouseUp 243 124 0 0
56 mouseMove 243 124 0 0
1710 mouseDown 243 124 2 0
100 mouseMove 243 124 2 0
85 mouseUp 243 124 0 0
71 mouseMove 243 124 0 0
634 mouseMove 250 125 0 0
83 mouseMove 253 125 0 0
58 mouseMove 254 127 0 0
83 mouseMove 254 133 0 0
78 mouseMove 254 135 0 0
58 mouseMove 254 136 0 0
84 mouseMove 254 137 0 0
454 mouseDown 254 137 4 0
84 mouseUp 254 137 0 0
76 mouseMove 254 137 0 0
4817 mouseMove 249 137 0 0
58 mouseMove 245 137 0 0
66 mouseMove 240 136 0 0
76 mouseMove 236 133 0 0
60 mouseMove 230 130 0 0
66 mouseMove 221 125 0 0
71 mouseMove 215 122 0 0
73 mouseMove 208 118 0 0
67 mouseMove 203 116 0 0
61 mouseMove 195 113 0 0
77 mouseMove 186 110 0 0
60 mouseMove 178 107 0 0
66 mouseMove 170 104 0 0
70 mouseMove 164 102 0 0
68 mouseMove 154 99 0 0
67 mouseMove 147 98 0 0
60 mouseMove 141 96 0 0
71 mouseMove 132 93 0 0
61 mouseMove 122 91 0 0
66 mouseMove 116 90 0 0
72 mouseMove 110 89 0 0
66 mouseMove 103 88 0 0
74 mouseMove 100 88 0 0
70 mouseMove 96 87 0 0
371 mouseDown 96 87 1 0
199 mouseUp 96 87 0 0
85 mouseMove 96 87 0 0
504 mouseMove 95 88 0 0
69 mouseMove 93 90 0 0
62 mouseMove 93 91 0 0
77 mouseMove 92 93 0 0
68 mouseMove 92 94 0 0
1548 mouseDown 92 94 4 0
104 mouseUp 92 94 0 0
287 mouseMove 92 94 0 0
492 mouseMove 93 95 0 0
74 mouseMove 98 95 0 0
84 mouseMove 114 95 0 0
74 mouseMove 132 92 0 0
69 mouseMove 142 91 0 0
82 mouseMove 153 90 0 0
73 mouseMove 163 90 0 0
70 mouseMove 178 90 0 0
83 mouseMove 182 90 0 0
69 mouseMove 191 90 0 0
72 mouseMove 195 93 0 0
82 mouseMove 200 98 0 0
69 mouseMove 203 104 0 0
66 mouseMove 205 110 0 0
88 mouseMove 206 117 0 0
71 mouseMove 207 123 0 0
73 mouseMove 207 128 0 0
84 mouseMove 207 132 0 0
63 mouseMove 209 135 0 0
69 mouseMove 211 140 0 0
156 mouseMove 221 141 0 0
63 mouseMove 223 141 0 0
144 mouseMove 229 141 0 0
69 mouseMove 239 138 0 0
86 mouseMove 241 136 0 0
776 mouseMove 235 136 0 0
71 mouseMove 227 136 0 0
87 mouseMove 217 136 0 0
68 mouseMove 211 125 0 0
64 mouseMove 203 101 0 0
86 mouseMove 202 98 0 0
279 mouseMove 192 95 0 0
69 mouseMove 187 93 0 0
70 mouseMove 149 81 0 0
82 mouseMove 124 72 0 0
66 mouseMove 101 64 0 0
66 mouseMove 98 63 0 0
80 mouseMove 90 63 0 0
69 mouseMove 88 63 0 0
79 mouseMove 81 64 0 0
95 mouseMove 72 65 0 0
69 mouseMove 62 65 0 0
69 mouseMove 58 65 0 0
433 mouseDown 58 65 4 0
143 mouseUp 58 65 0 0
81 mouseMove 58 65 0 0
343 mouseMove 64 66 0 0
62 mouseMove 78 66 0 0
63 mouseMove 81 66 0 0
140 mouseMove 77 66 0 0
64 mouseMove 75 66 0 0
61 mouseMove 69 66 0 0
76 mouseMove 59 66 0 0
65 mouseMove 57 66 0 0
58 mouseMove 56 66 0 0
84 mouseMove 54 66 0 0
59 mouseMove 53 66 0 0
528 mouseMove 54 66 0 0
210 mouseMove 55 66 0 0
259 mouseDown 55 66 4 0
126 mouseMove 55 66 4 0
187 mouseMove 57 67 4 0
82 mouseMove 58 68 4 0
80 mouseMove 59 68 4 0
94 mouseMove 60 68 4 0
89 mouseMove 61 68 4 0
103 mouseMove 62 68 4 0
81 mouseMove 63 68 4 0
369 mouseMove 63 67 4 0
81 mouseMove 63 66 4 0
93 mouseMove 62 65 4 0
103 mouseMove 60 64 4 0
195 mouseMove 59 64 4 0
82 mouseMove 58 64 4 0
189 mouseUp 63 64 0 0
86 mouseMove 78 64 0 0
77 mouseMove 94 64 0 0
61 mouseMove 101 64 0 0
64 mouseMove 106 64 0 0
131 mouseMove 111 64 0 0
66 mouseMove 127 66 0 0
59 mouseMove 129 66 0 0
73 mouseMove 130 66 0 0
60 mouseMove 132 66 0 0
210 mouseMove 132 64 0 0
65 mouseMove 131 63 0 0
201 mouseDown 131 63 4 0
126 mouseMove 128 63 4 0
83 mouseMove 125 63 4 0
81 mouseMove 124 63 4 0
95 mouseMove 123 63 4 0
184 mouseMove 122 64 4 0
93 mouseMove 122 67 4 0
92 mouseMove 122 68 4 0
84 mouseMove 123 69 4 0
86 mouseMove 126 70 4 0
100 mouseMove 127 70 4 0
82 mouseMove 128 70 4 0
105 mouseMove 129 70 4 0
81 mouseMove 130 70 4 0
83 mouseMove 131 69 4 0
106 mouseMove 131 67 4 0
81 mouseMove 132 66 4 0
83 mouseMove 132 64 4 0
112 mouseMove 131 63 4 0
83 mouseMove 130 62 4 0
104 mouseMove 128 62 4 0
85 mouseMove 127 62 4 0
81 mouseMove 126 62 4 0
94 mouseMove 124 64 4 0
85 mouseMove 124 65 4 0
173 mouseMove 124 66 4 0
87 mouseMove 124 67 4 0
103 mouseMove 125 67 4 0
87 mouseMove 126 67 4 0
178 mouseUp 126 67 0 0
80 mouseMove 126 67 0 0
527 mouseMove 126 68 0 0
60 mouseMove 142 71 0 0
65 mouseMove 160 76 0 0
79 mouseMove 166 86 0 0
69 mouseMove 178 101 0 0
66 mouseMove 186 112 0 0
88 mouseMove 192 121 0 0
61 mouseMove 193 127 0 0
65 mouseMove 198 144 0 0
65 mouseMove 200 149 0 0
71 mouseMove 200 153 0 0
63 mouseMove 200 154 0 0
60 mouseMove 200 159 0 0
75 mouseMove 200 169 0 0
62 mouseMove 200 177 0 0
140 mouseMove 200 196 0 0
80 mouseMove 200 201 0 0
60 mouseMove 200 204 0 0
70 mouseMove 200 210 0 0
76 mouseMove 196 218 0 0
61 mouseMove 189 221 0 0
66 mouseMove 162 228 0 0
72 mouseMove 134 235 0 0
67 mouseMove 126 236 0 0
63 mouseMove 117 236 0 0
81 mouseMove 111 235 0 0
66 mouseMove 105 234 0 0
60 mouseMove 101 233 0 0
75 mouseMove 97 233 0 0
60 mouseMove 92 233 0 0
205 mouseMove 91 234 0 0
70 mouseMove 91 236 0 0
59 mouseMove 91 237 0 0
202 mouseDown 91 237 4 0
328 mouseMove 91 237 4 0
206 mouseUp 91 237 0 0
191 mouseMove 91 237 0 0
71 mouseMove 91 239 0 0
60 mouseMove 90 242 0 0
64 mouseMove 90 245 0 0
80 mouseMove 90 247 0 0
194 mouseMove 91 247 0 0
67 mouseMove 111 242 0 0
61 mouseMove 160 240 0 0
66 mouseMove 170 240 0 0
78 mouseMove 181 239 0 0
61 mouseMove 186 239 0 0
66 mouseMove 191 237 0 0
78 mouseMove 216 233 0 0
67 mouseMove 247 228 0 0
164 mouseMove 248 226 0 0
119 mouseMove 248 225 0 0
599 mouseDown 248 225 4 0
69 mouseMove 248 225 4 0
139 mouseUp 248 225 0 0
91 mouseMove 248 225 0 0
67 mouseMove 244 200 0 0
68 mouseMove 232 163 0 0
60 mouseMove 230 153 0 0
81 mouseMove 225 141 0 0
65 mouseMove 222 137 0 0
60 mouseMove 221 136 0 0
121 mouseDown 221 136 4 0
70 mouseMove 221 136 4 0
65 mouseUp 221 136 0 0
101 mouseMove 221 136 0 0
190 mouseMove 219 136 0 0
65 mouseMove 135 122 0 0
67 mouseMove 91 106 0 0
61 mouseMove 89 104 0 0
81 mouseMove 82 101 0 0
65 mouseMove 80 97 0 0
58 mouseMove 79 95 0 0
77 mouseMove 78 93 0 0
60 mouseMove 76 90 0 0
65 mouseMove 75 87 0 0
69 mouseMove 74 85 0 0
64 mouseMove 73 83 0 0
63 mouseMove 73 82 0 0
60 mouseMove 71 78 0 0
77 mouseMove 68 74 0 0
68 mouseMove 67 73 0 0
64 mouseMove 66 72 0 0
68 mouseMove 65 70 0 0
720 mouseDown 66 70 4 0
114 mouseMove 66 70 4 0
117 mouseMove 68 72 4 0
103 mouseMove 70 74 4 0
115 mouseMove 72 76 4 0
99 mouseMove 73 78 4 0
101 mouseMove 74 81 4 0
93 mouseMove 75 82 4 0
119 mouseMove 77 85 4 0
92 mouseMove 80 89 4 0
93 mouseMove 82 91 4 0
100 mouseMove 84 93 4 0
98 mouseMove 91 98 4 0
115 mouseMove 98 103 4 0
93 mouseMove 103 106 4 0
113 mouseMove 106 108 4 0
98 mouseMove 108 108 4 0
100 mouseMove 109 108 4 0
300 mouseMove 113 110 4 0
101 mouseMove 116 111 4 0
100 mouseMove 119 113 4 0
102 mouseMove 120 113 4 0
708 mouseUp 120 113 0 0
114 mouseMove 120 113 0 0
452 mouseMove 123 113 0 0
60 mouseMove 128 113 0 0
152 mouseMove 131 113 0 0
61 mouseMove 138 113 0 0
66 mouseMove 145 113 0 0
70 mouseMove 149 114 0 0
66 mouseMove 154 114 0 0
61 mouseMove 159 115 0 0
64 mouseMove 165 115 0 0
83 mouseMove 169 116 0 0
69 mouseMove 173 116 0 0
72 mouseMove 184 119 0 0
102 mouseMove 211 126 0 0
61 mouseMove 216 128 0 0
79 mouseMove 222 133 0 0
66 mouseMove 229 146 0 0
60 mouseMove 236 166 0 0
78 mouseMove 240 175 0 0
59 mouseMove 241 176 0 0
64 mouseMove 241 178 0 0
194 mouseMove 241 181 0 0
142 mouseMove 241 182 0 0
66 mouseMove 241 184 0 0
323 mouseMove 242 184 0 0
739 mouseDown 242 184 4 0
65 mouseMove 242 184 4 0
655 mouseUp 242 184 0 0
77 mouseMove 242 184 0 0
470 mouseMove 242 186 0 0
81 mouseMove 237 212 0 0
60 mouseMove 234 222 0 0
66 mouseMove 232 226 0 0
64 mouseMove 227 232 0 0
69 mouseMove 219 238 0 0
167 mouseMove 216 244 0 0
67 mouseMove 216 246 0 0
525 mouseMove 215 246 0 0
205 mouseDown 215 246 4 0
328 mouseMove 215 246 4 0
193 mouseUp 215 246 0 0
215 mouseMove 215 246 0 0
134 mouseMove 215 243 0 0
67 mouseMove 220 233 0 0
72 mouseMove 230 224 0 0
141 mouseMove 224 161 0 0
76 mouseMove 154 123 0 0
64 mouseMove 144 121 0 0
60 mouseMove 140 122 0 0
66 mouseMove 124 123 0 0
72 mouseMove 110 122 0 0
68 mouseMove 85 114 0 0
68 mouseMove 74 111 0 0
76 mouseMove 66 109 0 0
64 mouseMove 63 107 0 0
62 mouseMove 57 104 0 0
65 mouseMove 53 102 0 0
80 mouseMove 52 101 0 0
66 mouseMove 50 99 0 0
66 mouseMove 48 98 0 0
397 mouseMove 47 97 0 0
213 mouseMove 47 96 0 0
844 mouseMove 45 96 0 0
66 mouseMove 44 96 0 0
71 mouseMove 42 95 0 0
77 mouseMove 39 94 0 0
123 mouseDown 39 94 4 0
142 mouseMove 40 99 4 0
115 mouseMove 43 103 4 0
96 mouseMove 45 108 4 0
91 mouseMove 47 111 4 0
111 mouseMove 50 116 4 0
93 mouseMove 52 119 4 0
110 mouseMove 57 122 4 0
98 mouseMove 58 125 4 0
103 mouseMove 66 130 4 0
102 mouseMove 70 131 4 0
201 mouseMove 73 132 4 0
407 mouseUp 73 132 0 0
87 mouseMove 73 132 0 0
153 mouseDown 73 132 4 0
131 mouseMove 83 133 4 0
108 mouseMove 94 133 4 0
96 mouseMove 100 133 4 0
103 mouseMove 104 133 4 0
100 mouseMove 107 133 4 0
100 mouseMove 112 132 4 0
206 mouseMove 113 132 4 0
511 mouseUp 113 132 0 0
91 mouseMove 113 132 0 0
272 mouseDown 113 132 4 0
126 mouseMove 122 127 4 0
125 mouseMove 130 120 4 0
99 mouseMove 133 118 4 0
101 mouseMove 139 113 4 0
101 mouseMove 142 109 4 0
106 mouseMove 143 106 4 0
95 mouseMove 146 100 4 0
100 mouseMove 147 94 4 0
200 mouseMove 147 93 4 0
610 mouseUp 147 93 0 0
112 mouseMove 147 93 0 0
2788 mouseMove 168 93 0 0
69 mouseMove 191 95 0 0
226 mouseMove 191 96 0 0
72 mouseMove 195 104 0 0
61 mouseMove 195 110 0 0
67 mouseMove 195 124 0 0
77 mouseMove 193 153 0 0
61 mouseMove 193 205 0 0
162 mouseMove 193 239 0 0
67 mouseMove 194 251 0 0
69 mouseMove 200 275 0 0
78 mouseMove 206 315 0 0
144 mouseMove 202 352 0 0
71 mouseMove 202 370 0 0
67 mouseMove 202 383 0 0
61 mouseMove 202 391 0 0
67 mouseMove 202 399 0 0
73 mouseMove 202 404 0 0
60 mouseMove 205 409 0 0
63 mouseMove 205 410 0 0
69 mouseMove 207 411 0 0
669 mouseMove 208 411 0 0
1192 mouseDown 208 411 4 0
72 mouseMove 208 411 4 0
79 mouseUp 208 411 0 0
158 mouseMove 208 411 0 0
474 mouseMove 208 410 0 0
79 mouseMove 208 408 0 0
66 mouseMove 211 396 0 0
69 mouseMove 223 381 0 0
65 mouseMove 232 368 0 0
66 mouseMove 247 340 0 0
60 mouseMove 258 306 0 0
81 mouseMove 264 290 0 0
60 mouseMove 268 282 0 0
66 mouseMove 271 271 0 0
85 mouseMove 273 250 0 0
60 mouseMove 274 236 0 0
66 mouseMove 271 217 0 0
61 mouseMove 271 205 0 0
83 mouseMove 269 189 0 0
68 mouseMove 265 178 0 0
60 mouseMove 265 167 0 0
77 mouseMove 265 154 0 0
61 mouseMove 265 143 0 0
65 mouseMove 265 139 0 0
68 mouseMove 265 138 0 0
65 mouseMove 265 135 0 0
67 mouseMove 265 127 0 0
58 mouseMove 265 125 0 0
79 mouseMove 265 124 0 0
530 mouseDown 265 124 2 0
106 mouseMove 265 124 2 0
75 mouseUp 265 124 0 0
74 mouseMove 265 124 0 0
493 mouseMove 267 125 0 0
98 mouseMove 268 128 0 0
77 mouseMove 268 131 0 0
76 mouseMove 268 132 0 0
58 mouseMove 268 133 0 0
77 mouseMove 269 134 0 0
63 mouseMove 270 135 0 0
202 mouseDown 270 135 4 0
109 mouseUp 270 135 0 0
85 mouseMove 270 135 0 0
2924 mouseMove 270 136 0 0
2716 mouseMove 278 133 0 0
84 mouseMove 281 130 0 0
60 mouseMove 283 128 0 0
66 mouseMove 291 124 0 0
61 mouseMove 299 119 0 0
70 mouseMove 311 110 0 0
70 mouseMove 315 105 0 0
61 mouseMove 324 101 0 0
80 mouseMove 333 97 0 0
60 mouseMove 337 94 0 0
67 mouseMove 343 91 0 0
113 mouseMove 354 88 0 0
59 mouseMove 357 86 0 0
64 mouseMove 359 85 0 0
1280 keystroke 359 85 0 112
194 mouseMove 359 85 0 0
64 keystroke 359 85 0 111
100 mouseMove 359 85 0 0
149 keystroke 359 85 0 111
110 mouseMove 359 85 0 0
183 keystroke 359 85 0 104
125 mouseMove 359 85 0 0
263 keystroke 359 85 0 32
109 mouseMove 359 85 0 0
474 keystroke 359 85 0 116
98 mouseMove 359 85 0 0
268 keystroke 359 85 0 117
117 mouseMove 359 85 0 0
122 keystroke 359 85 0 114
115 mouseMove 359 85 0 0
64 keystroke 359 85 0 110
127 mouseMove 359 85 0 0
403 mouseMove 359 85 8 0
1 keystroke 359 85 8 58
98 mouseMove 359 85 8 0
204 mouseMove 359 85 0 0
1 keystroke 359 85 0 32
116 mouseMove 359 85 0 0
185 keystroke 359 85 0 108
113 mouseMove 359 85 0 0
59 keystroke 359 85 0 101
118 mouseMove 359 85 0 0
63 keystroke 359 85 0 102
99 mouseMove 359 85 0 0
78 keystroke 359 85 0 116
94 mouseMove 359 85 0 0
865 keystroke 359 85 0 32
117 mouseMove 359 85 0 0
465 keystroke 359 85 0 116
99 mouseMove 359 85 0 0
262 keystroke 359 85 0 117
131 mouseMove 359 85 0 0
60 keystroke 359 85 0 114
103 mouseMove 359 85 0 0
193 keystroke 359 85 0 110
124 mouseMove 359 85 0 0
65 keystroke 359 85 0 115
120 mouseMove 359 85 0 0
1172 mouseMove 359 85 8 0
1 keystroke 359 85 8 58
97 mouseMove 359 85 8 0
336 mouseMove 359 85 0 0
1 keystroke 359 85 0 32
108 mouseMove 359 85 0 0
346 keystroke 359 85 0 49
98 mouseMove 359 85 0 0
252 keystroke 359 85 0 48
126 mouseMove 359 85 0 0
63 keystroke 359 85 0 48
107 mouseMove 359 85 0 0
726 keystroke 359 85 0 32
100 mouseMove 359 85 0 0
411 keystroke 359 85 0 100
107 mouseMove 359 85 0 0
64 keystroke 359 85 0 117
105 mouseMove 359 85 0 0
142 keystroke 359 85 0 114
119 mouseMove 359 85 0 0
0 keystroke 359 85 0 97
100 mouseMove 359 85 0 0
63 keystroke 359 85 0 116
119 mouseMove 359 85 0 0
212 keystroke 359 85 0 105
101 mouseMove 359 85 0 0
60 keystroke 359 85 0 111
97 mouseMove 359 85 0 0
252 keystroke 359 85 0 110
119 mouseMove 359 85 0 0
1399 mouseMove 359 85 8 0
1 keystroke 359 85 8 58
100 mouseMove 359 85 8 0
203 mouseMove 359 85 0 0
1 keystroke 359 85 0 32
114 mouseMove 359 85 0 0
263 keystroke 359 85 0 53
100 mouseMove 359 85 0 0
287 keystroke 359 85 0 48
97 mouseMove 359 85 0 0
143 keystroke 359 85 0 48
98 mouseMove 359 85 0 0
266 keystroke 359 85 0 32
111 mouseMove 359 85 0 0
121 keystroke 359 85 0 115
119 mouseMove 359 85 0 0
1 keystroke 359 85 0 116
99 mouseMove 359 85 0 0
143 keystroke 359 85 0 121
115 mouseMove 359 85 0 0
255 keystroke 359 85 0 108
99 mouseMove 359 85 0 0
1 keystroke 359 85 0 101
119 mouseMove 359 85 0 0
866 mouseMove 359 85 8 0
1 keystroke 359 85 8 58
102 mouseMove 359 85 8 0
128 mouseMove 359 85 0 0
1 keystroke 359 85 0 32
107 mouseMove 359 85 0 0
63 keystroke 359 85 0 97
111 mouseMove 359 85 0 0
60 keystroke 359 85 0 98
100 mouseMove 359 85 0 0
913 keystroke 359 85 0 114
106 mouseMove 359 85 0 0
590 keystroke 359 85 0 117
118 mouseMove 359 85 0 0
199 keystroke 359 85 0 112
120 mouseMove 359 85 0 0
262 keystroke 359 85 0 116
101 mouseMove 359 85 0 0
263 keystroke 359 85 0 108
128 mouseMove 359 85 0 0
323 keystroke 359 85 0 121
126 mouseMove 359 85 0 0
1912 mouseMove 359 85 64 0
1 keystroke 359 85 64 100
219 mouseMove 359 85 64 0
3744 mouseMove 307 85 64 0
84 mouseMove 202 85 64 0
60 mouseMove 190 85 64 0
54 mouseMove 178 85 64 0
61 mouseMove 135 83 64 0
65 mouseMove 126 83 64 0
59 mouseMove 120 83 64 0
62 mouseMove 113 82 64 0
72 mouseMove 112 82 64 0
62 mouseMove 107 82 64 0
53 mouseMove 106 82 64 0
58 mouseMove 104 82 64 0
87 mouseMove 102 82 64 0
54 mouseMove 101 82 64 0
184 mouseMove 99 82 64 0
60 mouseMove 97 82 64 0
128 mouseMove 96 82 64 0
65 mouseMove 94 82 64 0
370 mouseMove 94 81 64 0
83 mouseDown 94 81 1 0
221 mouseUp 94 81 0 0
105 mouseMove 94 81 0 0
544 mouseMove 86 91 0 0
325 mouseMove 94 91 0 0
111 mouseMove 97 90 0 0
227 mouseDown 97 90 4 0
104 mouseUp 97 90 0 0
294 mouseMove 97 90 0 0
595 mouseMove 100 90 0 0
112 mouseMove 117 97 0 0
116 mouseMove 133 96 0 0
107 mouseMove 142 96 0 0
110 mouseMove 158 97 0 0
105 mouseMove 168 101 0 0
118 mouseMove 180 108 0 0
111 mouseMove 188 117 0 0
116 mouseMove 191 128 0 0
102 mouseMove 194 138 0 0
114 mouseMove 194 144 0 0
106 mouseMove 194 156 0 0
121 mouseMove 194 158 0 0
228 mouseDown 194 158 4 0
100 mouseUp 194 158 0 0
105 mouseMove 194 158 0 0
853 mouseMove 215 185 0 0
139 mouseMove 220 203 0 0
62 mouseMove 220 205 0 0
119 mouseMove 219 205 0 0
60 mouseMove 211 209 0 0
67 mouseMove 209 210 0 0
160 mouseMove 210 212 0 0
67 mouseMove 211 213 0 0
296 mouseDown 211 213 4 0
303 mouseUp 211 213 0 0
223 mouseMove 211 221 0 0
70 mouseMove 211 228 0 0
62 mouseMove 211 235 0 0
63 mouseMove 211 236 0 0
83 mouseMove 211 239 0 0
123 mouseMove 211 241 0 0
58 mouseMove 211 249 0 0
62 mouseMove 211 253 0 0
175 mouseDown 211 253 4 0
335 mouseUp 211 253 0 0
209 mouseMove 211 253 0 0
226 mouseMove 211 252 0 0
60 mouseMove 211 242 0 0
60 mouseMove 211 237 0 0
120 mouseMove 211 236 0 0
180 mouseMove 205 224 0 0
57 mouseMove 187 192 0 0
57 mouseMove 187 190 0 0
133 mouseMove 184 182 0 0
63 mouseMove 180 171 0 0
63 mouseMove 166 150 0 0
149 mouseMove 151 136 0 0
58 mouseMove 134 117 0 0
73 mouseMove 110 103 0 0
56 mouseMove 106 101 0 0
60 mouseMove 84 88 0 0
63 mouseMove 79 87 0 0
79 mouseMove 77 86 0 0
65 mouseMove 74 85 0 0
66 mouseMove 73 84 0 0
78 mouseMove 72 84 0 0
129 mouseMove 72 83 0 0
512 mouseDown 72 83 4 0
113 mouseMove 72 83 4 0
7709 mouseUp 72 83 0 0
78 mouseMove 72 83 0 0
425 mouseMove 76 83 0 0
54 mouseMove 78 83 0 0
128 mouseMove 80 83 0 0
57 mouseMove 94 87 0 0
72 mouseMove 110 93 0 0
64 mouseMove 122 101 0 0
57 mouseMove 133 109 0 0
63 mouseMove 138 113 0 0
76 mouseMove 144 117 0 0
58 mouseMove 149 121 0 0
62 mouseMove 154 125 0 0
67 mouseMove 160 131 0 0
65 mouseMove 164 137 0 0
69 mouseMove 166 142 0 0
62 mouseMove 171 151 0 0
78 mouseMove 175 162 0 0
75 mouseMove 176 167 0 0
64 mouseMove 181 175 0 0
66 mouseMove 184 179 0 0
141 mouseMove 192 190 0 0
78 mouseMove 197 199 0 0
60 mouseMove 198 204 0 0
67 mouseMove 201 212 0 0
87 mouseMove 201 218 0 0
55 mouseMove 201 222 0 0
58 mouseMove 200 227 0 0
62 mouseMove 198 233 0 0
74 mouseMove 197 235 0 0
176 mouseMove 191 235 0 0
59 mouseMove 188 234 0 0
55 mouseMove 187 231 0 0
56 mouseMove 186 229 0 0
75 mouseMove 184 226 0 0
54 mouseMove 179 225 0 0
60 mouseMove 174 225 0 0
69 mouseMove 168 226 0 0
63 mouseMove 160 227 0 0
62 mouseMove 153 227 0 0
63 mouseMove 139 225 0 0
71 mouseMove 131 221 0 0
66 mouseMove 127 219 0 0
67 mouseMove 120 218 0 0
71 mouseMove 110 218 0 0
62 mouseMove 107 219 0 0
198 mouseMove 111 219 0 0
58 mouseMove 112 219 0 0
135 mouseDown 112 219 4 0
317 mouseMove 112 219 4 0
413 mouseUp 112 219 0 0
205 mouseMove 112 219 0 0
353 mouseMove 116 222 0 0
69 mouseMove 116 224 0 0
60 mouseMove 117 226 0 0
60 mouseMove 117 229 0 0
71 mouseMove 118 229 0 0
56 mouseMove 118 230 0 0
60 mouseMove 118 232 0 0
55 mouseMove 118 233 0 0
60 mouseMove 118 235 0 0
60 mouseMove 119 235 0 0
56 mouseDown 119 235 4 0
340 mouseMove 119 235 4 0
406 mouseUp 119 235 0 0
194 mouseMove 119 235 0 0
197 mouseMove 119 219 0 0
57 mouseMove 116 200 0 0
75 mouseMove 115 187 0 0
62 mouseMove 114 178 0 0
57 mouseMove 112 169 0 0
151 mouseMove 111 151 0 0
61 mouseMove 110 144 0 0
73 mouseMove 110 137 0 0
54 mouseMove 109 133 0 0
60 mouseMove 108 128 0 0
53 mouseMove 107 123 0 0
71 mouseMove 105 117 0 0
59 mouseMove 103 112 0 0
53 mouseMove 103 111 0 0
61 mouseMove 102 108 0 0
72 mouseMove 101 104 0 0
58 mouseMove 100 99 0 0
61 mouseMove 100 96 0 0
64 mouseMove 99 93 0 0
63 mouseMove 99 92 0 0
63 mouseMove 99 91 0 0
197 mouseMove 99 90 0 0
523 mouseMove 98 93 0 0
75 mouseMove 98 95 0 0
60 mouseMove 98 96 0 0
62 mouseMove 98 99 0 0
54 mouseMove 98 100 0 0
80 mouseMove 98 101 0 0
54 mouseMove 98 102 0 0
189 mouseDown 98 102 4 0
103 mouseMove 98 102 4 0
5163 mouseUp 98 102 0 0
83 mouseMove 98 102 0 0
237 mouseMove 105 103 0 0
59 mouseMove 113 103 0 0
131 mouseMove 119 103 0 0
60 mouseMove 127 104 0 0
66 mouseMove 138 106 0 0
61 mouseMove 143 106 0 0
57 mouseMove 149 107 0 0
63 mouseMove 154 107 0 0
72 mouseMove 160 107 0 0
56 mouseMove 167 107 0 0
70 mouseMove 179 107 0 0
82 mouseMove 180 107 0 0
76 mouseMove 186 107 0 0
55 mouseMove 190 107 0 0
60 mouseMove 192 108 0 0
67 mouseMove 210 122 0 0
63 mouseMove 214 132 0 0
66 mouseMove 216 141 0 0
59 mouseMove 217 142 0 0
85 mouseMove 220 154 0 0
67 mouseMove 224 169 0 0
55 mouseMove 224 173 0 0
73 mouseMove 226 187 0 0
61 mouseMove 229 200 0 0
63 mouseMove 239 228 0 0
87 mouseMove 242 251 0 0
61 mouseMove 242 284 0 0
64 mouseMove 242 304 0 0
62 mouseMove 242 341 0 0
67 mouseMove 242 353 0 0
61 mouseMove 239 368 0 0
57 mouseMove 237 376 0 0
76 mouseMove 234 382 0 0
60 mouseMove 231 388 0 0
54 mouseMove 223 400 0 0
61 mouseMove 219 405 0 0
63 mouseMove 215 410 0 0
53 mouseMove 213 413 0 0
548 mouseMove 212 413 0 0
197 mouseDown 212 413 4 0
84 mouseMove 212 413 4 0
63 mouseUp 212 413 0 0
149 mouseMove 212 413 0 0
267 mouseMove 304 305 0 0
57 mouseMove 460 181 0 0
109 mouseMove 461 138 0 0
62 mouseMove 452 133 0 0
61 mouseMove 448 132 0 0
55 mouseMove 442 131 0 0
72 mouseMove 440 132 0 0
348 mouseDown 440 132 4 0
86 mouseUp 440 132 0 0
66 mouseMove 440 132 0 0
945 keystroke 440 132 0 13
129 mouseMove 440 132 0 0
307 keystroke 440 132 0 112
131 mouseMove 440 132 0 0
120 keystroke 440 132 0 111
116 mouseMove 440 132 0 0
1 keystroke 440 132 0 111
109 mouseMove 440 132 0 0
410 keystroke 440 132 0 104
119 mouseMove 440 132 0 0
137 keystroke 440 132 0 32
124 mouseMove 440 132 0 0
236 keystroke 440 132 0 115
106 mouseMove 440 132 0 0
1 keystroke 440 132 0 116
112 mouseMove 440 132 0 0
177 keystroke 440 132 0 111
114 mouseMove 440 132 0 0
65 keystroke 440 132 0 112
117 mouseMove 440 132 0 0
1081 mouseMove 440 132 64 0
0 keystroke 440 132 64 100
223 mouseMove 440 132 64 0
1271 mouseMove 408 132 64 0
55 mouseMove 372 132 64 0
82 mouseMove 367 131 64 0
62 mouseMove 347 131 64 0
59 mouseMove 306 129 64 0
73 mouseMove 273 129 64 0
72 mouseMove 245 127 64 0
55 mouseMove 234 127 64 0
61 mouseMove 220 127 64 0
66 mouseMove 209 127 64 0
63 mouseMove 197 127 64 0
55 mouseMove 189 127 64 0
61 mouseMove 173 131 64 0
73 mouseMove 154 139 64 0
55 mouseMove 134 154 64 0
62 mouseMove 122 161 64 0
72 mouseMove 114 167 64 0
60 mouseMove 110 171 64 0
55 mouseMove 105 174 64 0
69 mouseMove 99 179 64 0
65 mouseMove 94 187 64 0
61 mouseMove 89 195 64 0
64 mouseMove 86 203 64 0
74 mouseMove 85 208 64 0
61 mouseMove 84 211 64 0
55 mouseMove 84 216 64 0
62 mouseMove 84 219 64 0
74 mouseMove 84 222 64 0
60 mouseMove 84 226 64 0
53 mouseMove 84 228 64 0
71 mouseMove 84 229 64 0
53 mouseDown 84 229 4 0
61 mouseMove 84 229 4 0
132 mouseUp 84 229 0 0
'! !

!WonderlandCameraMorph methodsFor: 'drawing' stamp: 'ar 5/22/2001 13:23'!
sketchOn: aCanvas
	| vtx prev next |
	vtx _ self outline.
	vtx reset.
	vtx atEnd ifTrue:[^self].
	prev _ vtx next.
	[vtx atEnd] whileFalse: [
		next _ vtx next.
		aCanvas line: prev to: next width: 5 color: Color red.
		prev _ next]! !

!WonderlandCameraMorph methodsFor: 'picking points' stamp: 'jsp 3/29/1999 11:46'!
pickPointOnBounds: boundary
	"Chooses a random point so that an edge of this morph lies on the specified bounds"

	| side pos |

	"First choose which side to move the morph to"
	side _ (1 to: 4) atRandom.

	"Now choose where on that side to move to"
	((side = 1) or: [ side = 3 ])
			ifTrue: [ pos _ (0 to: ((boundary height) - (bounds height))) atRandom]
			ifFalse: [ pos _ (0 to: ((boundary width) - (bounds width))) atRandom ].

	"Now assemble the point)"
	"left"
	(side = 1) ifTrue: [ ^ (boundary left)@((boundary top) + pos) ].

	"top"
	(side = 2) ifTrue: [ ^ ((boundary left) + pos) @ (boundary top) ].

	"right"
	(side = 3) ifTrue: [ ^ ((boundary right) - (bounds width)) @ ((boundary top) + pos) ].

	"bottom"
	^ ((boundary left) + pos) @ ((boundary bottom) - (bounds height)).
! !

!WonderlandCameraMorph methodsFor: 'drawing' stamp: 'ar 5/22/2001 13:21'!
drawSimulatedOn: aCanvas
	"Draw the receiver using the builtin software renderer"
	(myRenderer notNil and:[myRenderer isOverlayRenderer]) ifTrue:[
		"Dump it. We may just being dragged around by the hand."
		myRenderer destroy.
		myRenderer _ nil].
	aCanvas asBalloonCanvas render: self.
	outline ifNotNil:[self sketchOn: aCanvas].! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/27/1999 15:44'!
doFirstPersonControl: aBoolean
	firstPersonControls _ aBoolean! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 6/2/1999 13:45'!
getEventFocus
	"Return the actor that all events should go to. If not explicitly specified, choose the actor under the mouse cursor.  If no actor is under the mouse cursor, return nil.
	ar 6/2/1999: This method is more or less obsolete. For event handling, #convertEvent: should be used."
	self flag: #obsolete.

	eventFocus ifNotNil: [ ^ eventFocus ].

	^ myCamera pickAt: (Sensor cursorPoint).
! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 5/9/2001 14:37'!
toggleFirstPersonControl
	self doFirstPersonControl: self doFirstPersonControl not.! !

!WonderlandCameraMorph methodsFor: 'hardware acceleration' stamp: 'ar 4/20/2001 05:08'!
suspendAcceleration
	"Temporarily suspend acceleration"
	self accelerationSuspended: true.
	self changed.! !

!WonderlandCameraMorph methodsFor: 'accessing' stamp: 'laza 2/2/2000 16:10'!
mode: aSymbol
	mode _ aSymbol! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 5/28/2001 17:20'!
addCustomMenuItems: aCustomMenu hand: aHandMorph 
	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	aCustomMenu addLine.
	aCustomMenu addUpdating: #firstPersonControlString action: #toggleFirstPersonControl.
	aCustomMenu addUpdating: #showControlsString action: #toggleCameraControls.
	aCustomMenu addUpdating: #getAcceleratorState action: #toggleAcceleratorState.
	aCustomMenu addLine.
	aCustomMenu addUpdating: #getAAPaintState action: #toggleAAPaintState.
	aCustomMenu addLine.
	aCustomMenu add: 'Stop/Go button' action: #attachStopGoButtons.
	aCustomMenu add: 'Load actor' action: #loadActor.
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'ar 5/28/2001 17:19'!
aaPaintingEnabled: aBool
	aBool
		ifTrue:[self removeProperty: #aaPaintingEnabled]
		ifFalse:[self setProperty: #aaPaintingEnabled toValue: aBool]! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/26/2001 16:38'!
firstPersonMouseUp: evt
	myControls mouseUp: evt.
	myControls setCenter: nil.
	evt hand needsToBeDrawn ifFalse:[Cursor normal show].
! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'laza 3/30/2000 10:03'!
undo
	self restoreTextureFrom: backup! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/21/2001 15:31'!
fogRangeEnd: aNumber
	myCamera setFogRangeEnd: aNumber! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'jsp 4/19/1999 17:42'!
handlesMouseOver: evt
	"We want to handle mouse enter/leave events, so return true."

	^ true.

! !

!WonderlandCameraMorph methodsFor: 'pooh' stamp: 'ar 6/3/2001 14:00'!
paintActor: anActor
	self world prepareToPaint: false.
	self mode: #paint.
	backup _ anActor getTexturePointer deepCopy.
	palette _ self world paintBox.
	palette position: self position + ((self extent x + 100) @ 0); focusMorph: self.! !

!WonderlandCameraMorph methodsFor: 'picking points' stamp: 'jsp 3/29/1999 11:51'!
pickPointInBounds: boundary
	"Chooses a random point so that this morph lies within the specified bounds"

	| xpos ypos |

	xpos _ (boundary left) + (0 to: ((boundary width) - (bounds width))) atRandom.
	ypos _ (boundary top) + (0 to: ((boundary height) - (bounds height))) atRandom.

	^ xpos@ypos.

! !

!WonderlandCameraMorph methodsFor: 'stepping' stamp: 'jsp 3/22/1999 10:25'!
stepTime
	
	^ 1.
! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 6/2/1999 14:33'!
convertEvent: evt
	"Create a Wonderland event that will be dispatched later on from the given MorphicEvent."
	| newEvent temp clickedActor clickedVertex |
	eventFocus 
		ifNil:[ "Find the clicked actor under the cursor"
			temp _ myCamera pickObjectAndVertexAt: evt cursorPoint.
			clickedActor _ temp key.
			clickedVertex _ temp value]
		ifNotNil: [ "Route all events to the object having the focus"
			clickedActor _ eventFocus.
			clickedVertex _ nil].
	clickedActor ifNotNil:[
		newEvent _ WonderlandEvent new.
		newEvent setMorphicEvent: evt.
		newEvent setCameraMorph: self.
		newEvent setActor: clickedActor.
		newEvent setCamera: myCamera.
		newEvent setCursorDelta: ((evt cursorPoint) - lastCursorPoint).
		newEvent setCursorPoint: (lastCursorPoint _ evt cursorPoint).
		newEvent setVertex: clickedVertex.
		evt isKeystroke ifTrue:[newEvent setKey: (evt keyCharacter)].
	].
	^newEvent! !

!WonderlandCameraMorph methodsFor: 'pooh' stamp: 'laza 3/24/2000 15:31'!
drawOnTextureOf: anActor from: texStart to: texEnd
	"texStart and texEnd are Coordinates on a Texture"
	| texture range start end |
	texture _ anActor getTexturePointer.
	range _ texture extent.
	start _ range * texStart.
	end _ range * texEnd.
	texture drawLine: (Form dotOfSize: 3) from: start to: end clippingBox: (1@1 extent: range) rule: Form over fillColor: nil 

! !

!WonderlandCameraMorph methodsFor: 'drawing' stamp: 'ar 4/19/2001 05:51'!
drawOn: aCanvas 
	(aCanvas form == Display and:[self accelerationEnabled and:[self isFlexed not]])
		ifTrue:[self drawAcceleratedOn: aCanvas]
		ifFalse:[self drawSimulatedOn: aCanvas].! !

!WonderlandCameraMorph methodsFor: 'drawing' stamp: 'ar 5/22/2001 23:30'!
drawAcceleratedOutlineOn: aRenderer
	"Draw a pooh outline on an accelerated renderer."
	| vtxList scale out pt vtx offset z |
	"NOTE: The test below captures two distinct cases.
		#1: The software renderer (which does not support lines)
		#2: The D3D renderer (which does not support line attributes)."
	myRenderer hasFrameBufferAccess ifTrue:[
		^myRenderer provideOverlayCanvasDuring:[:cc| self sketchOn: cc].
	].
	z _ 0.5.
	vtxList _ self outline contents.
	vtxList size < 2 ifTrue:[^self].
	out _ WriteStream on: (B3DVector3Array new: vtxList size * 2).
	out nextPut: vtxList first @ z.
	2 to: vtxList size-1 do:[:i|
		pt _ vtxList at: i.
		vtx _ B3DVector3 x: pt x y: pt y z: z.
		out nextPut: vtx; nextPut: vtx].
	out nextPut: vtxList last @ z.
	out _ out contents.

	offset _ bounds origin + bounds corner * 0.5 @ 0.
	scale _ (2.0 / bounds extent x) @ (-2.0 / bounds extent y) @ 1.

	myRenderer reset. "get rid of everything"
	myRenderer material: (B3DMaterial new emission: Color red).
	myRenderer scaleBy: scale.
	myRenderer translateBy: offset negated.
	myRenderer lineWidth: 5.
	myRenderer drawLines: out normals: nil colors: nil texCoords: nil.
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/27/2000 14:20'!
mouseUpLine: newEvent

! !

!WonderlandCameraMorph methodsFor: 'pooh' stamp: 'ar 6/3/2001 13:55'!
createPoohActor
	| actor pointList box scale center subdivision mesh tex |
	pointList _ self outline.
	pointList reset.
	pointList _ pointList contents.
	pointList size < 2 ifTrue:[
		self clearStroke.
		^errorSound play].

	pointList _ self simplify: pointList.
	pointList _ self smoothen: pointList length: 10.
	pointList _ self regularize: pointList.
	box _ Rectangle encompassing: pointList.
	scale _ bounds extent y * 0.5.
	scale _ 1.0 / (scale @ scale negated).
	center _ box origin + box corner * 0.5.
	pointList _ pointList collect:[:each|  each - center * scale].
	subdivision _ PoohSubdivision constraintOutline: pointList.
	mesh _ subdivision build3DObject.

	mesh ifNil:[
		errorSound play.
	] ifNotNil:[
		actor _ self getWonderland makeActorNamed: 'sketch'.
		actor setProperty: #handmade toValue: true;
			setBackfaceCulling: #ccw;
			setMesh: mesh;
			setColor: gray.
		Preferences twoSidedPoohTextures
			ifTrue:[tex _ (Form extent: 256@512 depth: 32) asTexture fillColor: Color white]
			ifFalse:[tex _ (Form extent: 256@256 depth: 32) asTexture fillColor: Color white].
		actor setTexturePointer: tex.
		actor setComposite: (myCamera getMatrixFromRoot composedWithLocal: (B3DMatrix4x4 withOffset: 0@0@2)).
		actor scaleByMatrix: (B3DRotation axis: 0@1@0 angle: 90) asMatrix4x4.
		actor rotateByMatrix: (B3DRotation axis: 0@1@0 angle:-90) asMatrix4x4.
	].

	self clearStroke.
	self mode: nil.
	Cursor normal show.
! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/26/2001 16:41'!
mouseMoveFirstPersonControl: evt ! !

!WonderlandCameraMorph class methodsFor: 'pooh stuff' stamp: 'laza 4/4/2000 16:02'!
poohAboutImage
 ^  (ColorForm
	extent: 105@109
	depth: 4
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 273 0 0 0 0 0 0 0 0 0 0 0 0 0 4369 286261248 0 0 0 0 0 0 0 0 0 0 0 0 1118481 286330880 0 0 0 0 0 0 0 0 0 0 0 0 17895698 554766336 0 0 0 0 0 0 0 0 0 0 0 1 286331426 571543552 0 0 0 0 0 0 0 0 0 0 0 17 286401058 572592145 268435456 1118480 0 0 0 0 0 0 0 0 0 273 287449634 572592401 285212672 1118481 285212672 0 0 0 0 0 0 0 0 4369 287449634 572592401 286326784 286331153 286261248 0 0 0 0 0 0 0 17895697 286331153 287449634 571543825 286331137 286331409 286330880 0 0 0 0 0 0 69632 286331153 286335505 287449634 571543842 554766609 286335522 286330880 0 0 0 0 0 0 69905 286331153 286335249 287449634 571543842 572592401 286335522 286330880 0 0 0 0 0 0 285217041 286331426 572662033 287449634 571543842 572662305 286401058 554766592 0 0 0 0 0 1 286265617 286335522 572596497 287449634 554766882 572662305 287449634 554766592 0 0 0 0 0 1 286331153 303112738 554770705 287449634 554766882 572662306 287449634 571543552 0 0 0 0 0 1 286331426 572662306 286335505 287449634 286335522 572662306 287449634 554766336 0 0 0 0 0 17 286331426 572662305 286401057 287449634 554836514 572662306 555885090 286330880 0 0 0 0 0 4369 286401058 572662305 287449634 287449634 572662306 572662306 572662306 286330880 0 0 0 0 0 1118481 287449634 572662289 304226850 572662306 572662306 572662306 571544098 286326784 0 0 0 0 0 1118481 287449634 572662289 304226850 572662306 572662306 572662306 571544097 286261248 0 0 0 0 0 1118481 304226850 572662289 304226850 572662306 572662306 572662306 572592657 286261248 0 0 0 0 0 17895698 572662306 572662289 572662306 572662306 572662306 572662306 572592401 268435456 0 0 0 0 0 286331426 572662306 572662289 572662306 572662306 572662306 572662306 571543825 0 0 0 0 0 1 286331426 572662306 572662289 572662306 572662306 572662306 572662306 571543825 0 0 0 0 0 17 287449634 572662306 572662290 554770978 572662306 572662306 572662306 571543552 0 0 0 0 0 273 304226850 572662306 572662289 286335522 572662306 572662306 572662306 571543552 0 0 0 0 0 273 304226850 572662306 572662289 286401058 572662306 572662306 572662306 571543552 0 0 0 0 0 69905 572662306 572662306 572662289 287449634 572662306 572662306 572662306 571543552 0 0 0 0 0 69905 572662306 572662306 572662289 304226850 572662306 572662306 572662306 571543552 0 0 0 0 0 1118482 572662306 572662306 572662289 287449634 572662306 572662306 572662306 554766336 0 0 0 0 0 286331170 572662305 286331154 572662033 303112738 572662306 572662306 572662306 286326784 0 0 0 0 0 286331170 572662305 286331153 572662033 303112738 572662306 572662306 572662306 286326784 0 0 0 0 17 286331426 572662306 286331153 572657937 286335522 572662306 572662306 572662306 286261248 0 0 0 0 273 286335522 572662306 572592401 572657937 554836514 572662306 572662306 572662305 286261248 0 0 0 0 273 286401058 572662306 572657953 304222481 555885090 572662306 572662306 572662305 286261248 0 0 0 0 273 286401058 572662306 572662033 304222482 555885090 572662306 572662306 572662289 285212672 0 0 0 0 273 286401058 572662306 572662033 287445266 287449634 572662306 572662306 572662289 285212672 0 0 0 0 273 287449634 572662306 572662033 304222482 304226850 572662306 572662306 572662033 285212672 0 0 0 0 4369 287449634 572662306 572662289 287449362 304226850 572662306 572662306 572662033 285212672 0 0 0 0 1118481 304226850 572662306 572662305 287449362 287449634 572662306 572662306 572657937 268435456 0 0 0 0 1118481 572662306 572662306 572662305 286400785 287449634 572662306 572662306 572657937 0 0 0 0 0 1118481 572662306 572662306 572662306 286335249 287449634 572662306 572662306 572592400 0 0 0 0 0 4370 572662306 554770978 572662306 286331153 554836514 572662306 572592401 286331136 0 0 0 0 0 69906 572662306 554770978 572662306 286331409 286335522 571613457 286331153 286331152 0 0 0 0 0 1118482 571548194 554770978 572662306 286331153 286335522 554766609 286331153 286331153 0 0 0 0 0 17895970 554832146 554770978 572662306 571543825 554766609 286331153 286470963 858993425 268435456 0 0 0 0 286331170 286396690 554770978 572662306 554766609 286331153 286331187 858993457 286339857 285212672 0 0 0 0 286331426 286400785 286335522 572662306 554766609 286396689 286339891 286331153 286331153 286261248 0 0 0 1 286343441 304226833 286335522 572662306 572657937 286331153 286331153 286331153 288559377 286330880 0 0 0 1 286331153 304226834 286331426 572662306 572657937 286331153 288428305 286331153 286331153 286330880 0 0 0 1 286331169 572662289 286331170 572662306 572662033 286331155 286331153 286331155 856895763 304156928 0 0 0 0 286331426 572657938 286331154 572662306 572662306 554766609 286331699 856756497 286331153 287379728 0 0 0 0 286335522 572657953 287445265 572662306 572662306 554770705 286331153 286331155 858984721 304222480 0 0 0 1 286335522 572662289 304222481 287449634 572662306 572596770 554766609 858993459 858984721 572657936 0 0 0 0 286331154 572662306 554766609 286401058 572662306 572662306 572662033 286331153 288559378 572662033 0 0 0 0 286331170 572662306 554770961 286401058 572662306 572662306 572662289 286331699 858984738 572662033 0 0 0 0 286331426 572662306 572657953 286401058 572662306 572662306 572662305 319886097 286331170 572662033 0 0 0 1 286335522 572662306 572657937 286331426 572662306 572662306 572662306 286331699 856756770 572657936 0 0 0 1 286335522 572662306 572657937 554766609 572662306 572662306 572662306 288428305 286331170 572657936 0 0 0 1 286335522 572662306 572662306 554766609 286401058 572662306 572662306 286331153 286331170 572657936 0 0 0 17 286335522 572662306 572657937 572662033 286331153 286401058 572662306 286331153 286331154 572592384 0 0 0 17 286335522 572662306 572662306 571543825 286331153 286335521 304222498 286331155 858853649 571543552 0 0 0 17 286335522 572662306 572657937 286331153 286331665 286331170 287445537 286331699 858984721 286330880 0 0 0 17 286331426 572662306 572657937 304226850 286331153 286331153 286331153 286470417 286331153 286330880 0 0 0 17 286331426 572662306 572662306 572662305 286331155 288428305 286331153 286462227 286331697 286330880 17825792 0 0 273 286331426 572662306 572662306 572662289 286331153 858992913 286331153 286331153 322122547 858853377 286326784 0 0 273 286335522 572662306 572662306 572662306 286331153 286331153 288559411 823202065 286462227 858853393 286331136 0 0 273 286335522 572662306 572662306 572662306 554836497 286331155 856756531 286331697 286462227 858853649 571543808 0 0 273 286335522 572662306 572662306 572662306 572662289 286339891 286331187 858993459 858993459 856756497 572592400 0 0 273 286335522 572662306 572657937 286335522 571543825 286331153 286331153 286331153 286331153 286331170 287379729 0 0 273 286335522 572662306 304222481 286331425 286331153 286331153 288568081 286331153 322122547 856756514 287445265 0 0 17 286335522 572662305 287379729 286331153 287449617 286331155 858992913 554775347 858993459 856756513 287445265 0 0 17 286335522 572662033 287449634 572662289 287445265 286470961 286331170 571543857 286331153 286331169 572662033 0 0 17 303112482 572657937 287449634 572662289 286331153 286462225 321982754 571543825 286462771 856756513 287383825 0 0 17 303108386 572592401 287449634 572662306 286400785 286331153 286331170 571543859 858993459 858853666 304156945 0 0 1 286331169 572592401 572662306 572662306 286331153 286462225 286331426 571543825 286331153 286331170 287379728 0 0 1 286331169 572596497 572662306 572662306 287379730 554766609 823202338 571609361 858993459 856756498 304156944 0 0 1 286396705 572592402 572662306 572662306 572592418 286331153 286331426 572657939 858984721 286331154 304156944 0 0 0 286331409 304156962 572662306 572662306 572662033 286339857 286335522 571609361 286462771 856756753 304222480 0 0 0 286331153 287379746 572662306 572662306 572662289 286331155 286331425 303178001 858993459 823202081 572657936 0 0 0 17895697 286331170 572662306 572662306 572662305 286331155 286331425 572662035 856756497 286335266 572657936 0 0 0 69905 571543842 572662306 572662306 572662306 572592401 554770977 287383825 322122547 856760609 572657920 0 0 0 69905 572592402 572662306 572662306 572662306 571740434 554836497 304161043 858984721 286331153 304156928 0 0 0 69905 304156945 572662306 572662306 572662306 572596770 554836497 572596499 823202065 319885585 572592384 0 0 0 4369 286396690 286331154 572662306 572592674 571548193 336732433 572592401 322122545 288428306 572592384 0 0 0 273 286331153 554766609 304161314 572592674 554836513 287449361 572592403 858853649 288493858 304156928 0 0 0 273 286331153 287445265 286331425 572592674 304226849 287449362 572592401 286331153 823202338 304156928 0 0 0 0 286331153 286331153 286331153 287445282 304226849 304226578 572592435 825299219 286331425 571543808 0 0 0 0 286331153 286331154 287445266 287445282 286401057 304222482 572592401 286470963 287449633 571543552 0 0 0 0 268505361 286331153 286331153 286331426 554836513 304222497 554766611 858993459 824320273 286326784 0 0 0 0 273 286331153 286331153 286331153 286335249 287379985 571543825 286331153 286396689 286261248 0 0 0 0 0 17895697 286331153 286331153 286335250 287449361 554766609 286331153 286331153 268435456 0 0 0 0 0 4353 286331153 286396689 286335506 287449362 286331153 286331153 286330880 0 0 0 0 0 0 0 0 286331153 286331153 286331154 286331153 0 0 0 0 0 0 0 0 0 0 4369 286331153 286331153 286261248 0 0 0 0 0 0 0 0 0 0 17 286331153 286331169 286261248 0 0 0 0 0 0 0 0 0 0 0 1118481 286331153 268435456 0 0 0 0 0 0 0 0 0 0 0 4369 286331153 0 0 0 0 0 0 0 0 0 0 0 0 0 1118481 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)
	colorsFromArray: #(#( ) #(0.0 0.0 0.0) #(1.0 1.0 0.333) #(0.501 0.0 0.0) #(0.753 0.862 0.753) #( ) #( ) #( ) #( ) #( ) #( ) #( ) #( ) #( ) #( ) #( )  )! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'ar 5/28/2001 13:10'!
fill: newEvent
	currentCanvas form floodFill: currentColor at: currentPosition asIntegerPoint! !

!WonderlandCameraMorph methodsFor: 'geometry' stamp: 'ar 9/10/2000 16:06'!
containsPoint: aPoint
	"Perform a full picking op if the camera does not draw the background"
	myCamera drawSceneBackground
		ifTrue:[^super containsPoint: aPoint].
	"*** hack ***"
	pickedPoint = aPoint 
		ifFalse:[pickedActor _ myCamera pickAt: aPoint].
	pickedPoint _ aPoint.
	^pickedActor notNil! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/26/2001 16:42'!
mouseMoveDefault: evt 

	| newEvent reactions |
	newEvent _ self convertEvent: evt.
	newEvent ifNil:[^self].

	newEvent getActor hasActiveTexture
		ifTrue: [^ newEvent getActor morphicMouseMove: newEvent].

	reactions _ newEvent getActor getReactionsTo: mouseMove.
	reactions ifNotNil: [reactions do: [:aReaction | aReaction reactTo: newEvent]].! !

!WonderlandCameraMorph methodsFor: 'initialization' stamp: 'ar 5/26/2001 15:55'!
initialize
	super initialize.
	lastCursorPoint _ 0 @ 0.
	CurrentProjectRefactoring currentAddGuard: self.

	"Teddy"
	self mode: nil.
	self outline: nil.! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 9/15/2000 23:00'!
handlesKeyboard: evt
	^true! !

!WonderlandCameraMorph class methodsFor: 'pooh stuff' stamp: 'laza 4/5/2000 10:13'!
poohPlayTape
	"| tape source | 
	tape _ FileStream fileNamed: 'pooh.tape'.
	source _ 'poohTape	^''', tape contentsOfEntireFile, ''''.
	(WonderlandCameraMorph class methodsFor: 'pooh stuff') scanFrom: (ReadStream on: source)"
	"WonderlandCameraMorph poohPlayTape"
	| stream player |
	stream _ ReadStream on: self poohTape.
	player _ EventRecorderMorph readFrom: stream.
	player openInWorld.
	Wonderland new.
	player play.
	player delete.! !

!WonderlandCameraMorph methodsFor: 'pooh' stamp: 'ar 5/23/2001 21:35'!
regularize: pointCollection
	"Make the pointList non-intersecting, e.g., insert points at intersections and have the outline include those points"
	| pointList segments last intersections map pts |
	pointList _ pointCollection collect:[:pt| pt asIntegerPoint].
	segments _ WriteStream on: (Array new: pointList size).
	last _ pointList last.
	pointList do:[:next|
		segments nextPut: (LineSegment from: last to: next).
		last _ next.
	].
	segments _ segments contents.
	intersections _ LineIntersections of: segments.
	map _ IdentityDictionary new: segments size.
	intersections do:[:is|
		(map at: is second ifAbsentPut:[WriteStream on: (Array new: 2)]) nextPut: is first.
		(map at: is third ifAbsentPut:[WriteStream on: (Array new: 2)]) nextPut: is first.
	].
	pts _ WriteStream on: (Array new: pointList size).
	segments do:[:seg|
		intersections _ (map at: seg) contents.
		intersections _ intersections sort:
			[:p1 :p2|  (p1 squaredDistanceTo: seg start) <= (p2 squaredDistanceTo: seg start)].
		last _ intersections at: 1.
		pts nextPut: last.
		intersections do:[:next|
			(next = last and:[next = seg end]) ifFalse:[
				pts nextPut: next.
				last _ next]].
	].
	^pts contents! !

!WonderlandCameraMorph methodsFor: 'controls' stamp: 'kfr 8/7/2000 16:40'!
deleteCameraControls
	"Hide the camera controls for the camera rendering into this window"
	(self submorphOfClass: WonderlandCameraControls)
		ifNotNil: [myControls delete.
			myControls _ nil]! !

!WonderlandCameraMorph methodsFor: 'pooh' stamp: 'ar 6/3/2001 14:00'!
strokeMode
	self world prepareToPaint: false.
	self clearStroke.
	self mode: #stroke.
	Cursor crossHair beCursor.
	self removeHalo.! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 5/28/2001 17:21'!
toggleAAPaintState
	self aaPaintingEnabled: self aaPaintingEnabled not.! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 14:18'!
ellipse: newEvent
	| type |
	type _ newEvent getMorphicEvent type.
	self perform: (type, 'Ellipse:') asSymbol with: newEvent
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 09:27'!
star: newEvent
	"verts _ (0 to: 350 by: 36) collect: [:angle |
		(Point r: (oldR _ oldR = ext ifTrue: [ext*5//12] ifFalse: [ext]) degrees: angle + pt degrees)
			+ strokeOrigin].
	
	poly _ PolygonMorph new addHandles.
	poly color: currentColor; borderWidth: currentNib extent x; borderColor: Color black.
	poly privateOwner: self.
	poly bounds: (strokeOrigin extent: ext).
	poly setVertices: verts.
	poly drawOn: formCanvas."! !

!WonderlandCameraMorph class methodsFor: 'pooh stuff' stamp: 'laza 4/5/2000 10:19'!
openPoohAboutWindow
	"WonderlandCameraMorph openPoohAboutWindow"
	| window image text button extent |

	extent _ Display extent // 3 * (1.5@1.5).

	window _ SystemWindow labelled: 'About Pooh'.

	image _ (ImageMorph new image: self poohAboutImage).
	text _ TextMorph new contentsWrapped: self poohAboutText; occlusionsOnOff.
	button _ SimpleButtonMorph new label: 'Ok'; target: window; actionSelector: #delete.

	window addMorph: text frame: (0.01@0.01 corner: 0.99@0.84).
	window addMorph: image frame: (0.01@0.01 corner: 0.99@0.84).
	window addMorph: button frame: (0.45@0.85 corner: 0.55@0.99).

	window setWindowColor: Color white;
		color: Color white.

	window openInWorldExtent: extent.
! !

!WonderlandCameraMorph methodsFor: 'drag and drop' stamp: 'sma 4/30/2000 10:19'!
acceptDroppingMorph: aMorph event: evt
	"This message is sent when a morph is dropped onto a morph that has agreed to accept the dropped morph by responding 'true' to the wantsDroppedMorph:Event: message. This default implementation just adds the given morph to the receiver."
	| actor |
	actor _ myCamera pickAt: evt cursorPoint.
	actor == nil ifTrue:[^self addMorphFront: aMorph]. "This should never happen."
	myWonderland getUndoStack push: (UndoTextureChange for: actor from: actor getTexturePointer).
	aMorph installAsWonderlandTextureOn: actor.
	self world abandonAllHalos.
	"AAAARRRRGGGGGHHHH!!!!!!!!"
	aMorph owner ifNotNil:[aMorph owner privateRemoveMorph: aMorph].
	aMorph privateOwner: self.
	"Note: The above makes aMorph invisible to stupid HandMorph but keeps it in the world so it can continue stepping"
	actor adjustToTextureIfNecessary.! !

!WonderlandCameraMorph methodsFor: 'controls' stamp: 'ar 9/16/1999 16:38'!
getCameraControls
	myControls == nil ifTrue:[
		self showCameraControls.
		self hideCameraControls.
	].
	^myControls! !

!WonderlandCameraMorph methodsFor: 'hardware acceleration' stamp: 'ar 4/20/2001 05:08'!
accelerationSuspended: aBool
	"Temporarily suspend hardware acceleration"
	myRenderer ifNotNil:[myRenderer destroy].
	myRenderer _ nil.
	aBool
		ifTrue:[self setProperty: #accelerationSuspended toValue: aBool]
		ifFalse:[self removeProperty: #accelerationSuspended]! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/26/2001 16:41'!
mouseMoveStroke: evt 
	^self recordStroke: evt cursorPoint! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/27/2000 16:40'!
mouseDownRect: newEvent
	self mouseDownLine: newEvent
! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/26/2001 16:31'!
handleMouseDown: anEvent
	"System level event handling."
	anEvent wasHandled ifTrue:[^self]. "not interested"
	anEvent hand removePendingBalloonFor: self.
	anEvent hand removePendingHaloFor: self.
	anEvent wasHandled: true.
	"anEvent controlKeyPressed ifTrue:[^self invokeMetaMenu: anEvent]."
	"Make me modal during mouse transitions"
	anEvent hand newMouseFocus: self event: anEvent.
	anEvent blueButtonChanged ifTrue:[^self blueButtonDown: anEvent].
	(self allowsGestureStart: anEvent)
		ifTrue: [^ self gestureStart: anEvent].
	self mouseDown: anEvent.
	anEvent hand removeHaloFromClick: anEvent on: self.
	(self handlesMouseStillDown: anEvent) ifTrue:[
		self startStepping: #handleMouseStillDown: 
			at: Time millisecondClockValue + self mouseStillDownThreshold
			arguments: {anEvent copy resetHandlerFields}
			stepTime: 1].
! !

!WonderlandCameraMorph methodsFor: 'pooh' stamp: 'ar 5/22/2001 13:21'!
recordStroke: p
	self outline nextPut: p! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'bf 6/25/2001 12:24'!
mouseDown: evt 
	"When the user clicks in a camera window, determine which actor the    
	         user clicked on and have that actor respond to the event"
	| newEvent reactions |
	mode == #view ifTrue:[mode _ nil].
	"ar 5/28/2001 - remove the above at some point; it's compensating for old wonderlands"

	self on: #mouseMove send: #mouseMoveDefault: to: self.
	self on: #mouseUp send: #mouseUpDefault: to: self.
	(evt redButtonPressed and:[evt anyModifierKeyPressed not]) ifTrue:[
		(mode == nil and:[firstPersonControls == true]) ifTrue:[
			self getCameraControls. 		"create if necessary"
			myControls setCenter: evt position.
			myControls mouseDown: evt.
			evt hand needsToBeDrawn ifFalse:[Cursor crossHair show].
			self on: #mouseMove send: #mouseMoveFirstPersonControl: to: self.
			self on: #mouseUp send: #mouseUpFirstPersonControl: to: self.
			^self
		].

		mode == #stroke ifTrue:[
			self on: #mouseMove send: #mouseMoveStroke: to: self.
			self on: #mouseUp send: #mouseUpStroke: to: self.
			^self recordStroke: evt cursorPoint].

		newEvent _ self convertEvent: evt.
		newEvent ifNil:[^self].

		mode == #paint ifTrue:[
			newEvent getVertex ifNil:[^self].
			self prepareAction: newEvent.
			self on: #mouseMove send: #mouseMovePaint: to: self.
			self on: #mouseUp send: #mouseUpPaint: to: self.
			^self perform: palette action with: newEvent].
	] ifFalse:[
		newEvent _ self convertEvent: evt.
		newEvent ifNil:[^self].
	].
	newEvent getActor hasActiveTexture
		ifTrue: [^ newEvent getActor morphicMouseDown: newEvent].

	(evt redButtonPressed or:[evt controlKeyPressed]) ifTrue:[
		reactions _ newEvent getActor getReactionsTo: leftMouseDown.
		mouseUpButton _ leftMouseUp.
	] ifFalse: [evt yellowButtonPressed ifTrue:[ 
		reactions _ newEvent getActor getReactionsTo: rightMouseDown.
		mouseUpButton _ rightMouseUp.
	] ifFalse: [reactions _ nil]].
	reactions ifNotNil: [reactions do: [:aReaction | aReaction reactTo: newEvent]].
! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 9/15/2000 23:31'!
mouseLeave: evt
	"When the mouse leaves our window release the keyboard focus"

	evt hand releaseKeyboardFocus: self.

	mode = #paint ifTrue: [evt hand showTemporaryCursor: nil]
! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'laza 2/4/2001 10:06'!
cancelPainting: aPaintBoxMorph evt: evt
	^self cancel! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 16:06'!
mouseUpEllipse: newEvent

! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/15/2001 14:00'!
fogDensity
	^myCamera getFogDensity! !

!WonderlandCameraMorph methodsFor: 'accessing' stamp: 'laza 2/2/2000 16:10'!
mode
	^ mode
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 16:00'!
mouseDownLine: newEvent
	self storeTexture.
	lastPosition _ currentPosition
	
! !

!WonderlandCameraMorph methodsFor: 'hardware acceleration' stamp: 'ar 4/20/2001 02:15'!
accelerationEnabled
	"Return true if hardware acceleration is enabled"
	self accelerationSuspended ifTrue:[^false].
	^self valueOfProperty: #accelerationEnabled ifAbsent:[false]! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 16:02'!
mouseMoveRect: newEvent
	| rect |
	self restoreTexture.
	rect _ Rectangle encompassing: { lastPosition. currentPosition }.
	currentCanvas frameAndFillRectangle: rect fillColor: currentColor borderWidth: currentNib extent x borderColor: Color black

! !

!WonderlandCameraMorph methodsFor: 'accessing' stamp: 'ar 11/7/1999 19:18'!
getCamera
	^myCamera! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/27/2001 16:36'!
mouseUpPaint: evt
	| newEvent |
	newEvent _ self convertEvent: evt.
	newEvent ifNil: [^ self].
	newEvent getVertex ifNil:[^self].
	^ self perform: palette action with: newEvent! !

!WonderlandCameraMorph methodsFor: 'stepping' stamp: 'jsp 3/22/1999 10:25'!
wantsSteps

	^ true.
! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'laza 2/4/2001 10:07'!
undoPainting: pbox evt: evt
	^self undo! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/15/2001 14:00'!
fogRangeEnd
	^myCamera getFogRangeEnd! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'ar 5/28/2001 17:17'!
prepareAction: newEvent
	|  range  palNib |
	currentColor _ palette getColor.
	palNib _ palette getNib.
	(currentNib notNil 
		and:[currentNib extent = palNib extent 
		and:[currentNib isColorForm 
		and:[currentNib depth = 8 
		and:[currentNib colors last = currentColor]]]]) ifFalse:[
			currentNib _ self roundNibOfSize: (palette getNib extent).
			self aaPaintingEnabled
				ifTrue:[currentNib colors:((0 to: 255) collect:[:i| currentColor alpha: i / 255.0])]
				ifFalse:[currentNib colors: ((Array new: 256) at: 1 put: Color transparent; from: 2 to: 256 put: currentColor; yourself)].
		].
	currentActor _ newEvent getActor.
	currentCanvas _ (currentActor getTexturePointer getCanvas).
	range _ currentCanvas extent.
	currentPosition _ newEvent getVertex texCoords * range - (currentNib extent // 2).! !

!WonderlandCameraMorph methodsFor: 'drawing' stamp: 'ar 4/20/2001 02:47'!
renderOn: aRenderer
	"Tell our camera to draw"
	myCamera render: aRenderer.
! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'laza 3/30/2000 10:04'!
cancel
	mode _ nil.
	self restoreTextureFrom: backup.
	palette delete! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/26/2001 16:40'!
mouseUpStroke: evt
	^self createPoohActor! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/27/2000 15:50'!
stamp: newEvent
! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 4/20/2001 05:05'!
toggleAcceleratorState
	self accelerationEnabled: self accelerationEnabled not.! !

!WonderlandCameraMorph methodsFor: 'hardware acceleration' stamp: 'ar 4/20/2001 02:14'!
restoreAcceleration
	"Restore temporarily suspend acceleration"
	self accelerationSuspended: false.! !

!WonderlandCameraMorph class methodsFor: 'pooh stuff' stamp: 'laza 4/4/2000 16:44'!
poohAboutText
	| about url font pattern start end |
	about _ '     Pooh

This software is based on the work of Takeo Igarashi, Satoshi Matsuoka and Hidehiko Tanaka, which is described in "Teddy: A Sketching Interface for 3D Freeform Design".

To play around with Pooh just open a Wonderland and bring up the halos on the CameraMorph. Click on the Pooh halo and start drawing an outline. After that you can bring up the halos on the generated object and click on the Paint halo to draw on the surface. (Or have a look at this tape)' asText.


	font _ TextFontChange new fontNumber: 3.

	pattern _ 'Pooh'.
	start _ about string findString: pattern.
	end _ start + pattern size.
	about addAttribute: font from: start to: end.
	about makeBoldFrom: start to: end.

	pattern _ 'Takeo Igarashi'.
	start _ about string findString: pattern.
	end _ start + pattern size.
	url _ TextURL new url: 'http://www.mtl.t.u-tokyo.ac.jp/~takeo/teddy/teddy.htm'.
	about addAttribute: url from: start to: end.

	pattern _ 'open a Wonderland'.
	start _ about string findString: pattern.
	end _ start + pattern size.
	url _ TextDoIt new evalString: 'Wonderland new'.
	about addAttribute: url from: start to: end.

	pattern _ 'tape'.
	start _ about string findString: pattern.
	end _ start + pattern size.
	url _ TextDoIt new evalString: 'WonderlandCameraMorph poohPlayTape'.
	about addAttribute: url from: start to: end.

	^ about! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/15/2001 14:21'!
fogType: aNumber
	myCamera setFogType: (#(none linear exp exp2) atWrap: aNumber + 1)! !

!WonderlandCameraMorph methodsFor: 'pooh' stamp: 'ar 5/22/2001 13:32'!
smoothen: pointList length: unitLength
	| prevPt curPt nextPt out prevMid nextMid segment length steps deltaT |
	out _ WriteStream on: (Array new: pointList size).
	prevPt _ pointList at: pointList size-1.
	curPt _ pointList last.
	prevMid _ (curPt + prevPt) * 0.5.
	1 to: pointList size do:[:i|
		nextPt _ pointList at: i.
		nextMid _ (nextPt + curPt) * 0.5.
		segment _ Bezier2Segment from: prevMid to: nextMid via: curPt.
		length _ segment length.
		steps _ (length / unitLength) asInteger.
		steps < 1 ifTrue:[steps _ 1].
		deltaT _ 1.0 / steps.
		1 to: steps-1 do:[:k|
			out nextPut: (segment valueAt: deltaT * k)].
		out nextPut: nextMid.
		prevPt _ curPt.
		curPt _ nextPt.
		prevMid _ nextMid.
	].
	^out contents! !

!WonderlandCameraMorph class methodsFor: 'pooh stuff' stamp: 'laza 4/4/2000 14:14'!
installPoohIcon
	ScriptingSystem saveForm:
			((ColorForm
			extent: 16 @ 16
			depth: 1
			fromArray: #(0 0 0 473432064 536346624 229638144 267386880 106954752
							62914560 132120576 267386880 536346624 511180800 0 0 0 )
			offset: 0 @ 0)
			colorsFromArray: #((1.0 1.0 1.0 ) (0.0 0.0 0.004 ) ))
		atKey: 'Halo-Pooh'! !

WonderlandCameraMorph subclass: #WonderlandStillCameraMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandStillCameraMorph methodsFor: 'event handling' stamp: 'jsp 6/29/1999 01:08'!
handlesMouseDown: evt
	"We don't want to handle mouse up and down events, so return false"

	^ false.
! !

!WonderlandStillCameraMorph methodsFor: 'event handling' stamp: 'jsp 6/29/1999 01:08'!
handlesMouseOver: evt
	"We don't want to handle mouse enter/leave events, so return false."

	^ false.

! !

!WonderlandStillCameraMorph methodsFor: 'stepping' stamp: 'jsp 6/29/1999 01:07'!
wantsSteps

	^ false.
! !

!WonderlandStillCameraMorph methodsFor: 'controls' stamp: 'jsp 6/29/1999 01:20'!
showCameraControls
	"Override the parent method to do nothing."

! !

!WonderlandStillCameraMorph methodsFor: 'controls' stamp: 'jsp 6/29/1999 01:20'!
hideCameraControls
	"Override our parent method to do nothing."



! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/27/2000 15:50'!
clear: newEvent
! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 2/23/2001 17:03'!
attachStopGoButtons
	| goButton stopButton wrapper |
	goButton _  ThreePhaseButtonMorph new.
	goButton image:  (ScriptingSystem formAtKey: 'GoPicOn');
			offImage: (ScriptingSystem formAtKey: 'GoPic');
			pressedImage: (ScriptingSystem formAtKey: 'GoPicOn');
			actionSelector: #setProperty:toValue:; 
			arguments: (Array with: #keepStepping with: true);
			actWhen: #buttonUp;
			target: self;
			setNameTo: 'Go Button'.
	stopButton _ ThreePhaseButtonMorph new.
	stopButton
		image:  (ScriptingSystem formAtKey: 'StopPic');
		offImage: (ScriptingSystem formAtKey: 'StopPic');
		pressedImage:  (ScriptingSystem formAtKey: 'StopPicOn');
		actionSelector: #removeProperty:; 
		arguments: (Array with: #keepStepping);
		actWhen: #buttonUp;
		target: self;
		setNameTo: 'Stop Button'. 
	wrapper _ AlignmentMorph newRow setNameTo: 'wonderland controls'.
	wrapper vResizing: #shrinkWrap.
	wrapper hResizing: #shrinkWrap.
	wrapper beTransparent.
	wrapper addMorphBack: goButton; addMorphBack: stopButton.
	self world primaryHand attachMorph: wrapper.! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/27/2000 16:41'!
rect: newEvent
	| type |
	type _ newEvent getMorphicEvent type.
	self perform: (type, 'Rect:') asSymbol with: newEvent
! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'laza 3/30/2000 11:08'!
paletteDetached: aPaintBoxMorph
	self cancel	! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'ar 5/26/2001 16:39'!
mouseUpDefault: evt
	| newEvent reactions |
	newEvent _ self convertEvent: evt.
	newEvent ifNil: [^ self].

	newEvent getActor hasActiveTexture 
		ifTrue: [^ newEvent getActor morphicMouseUp: newEvent].

	reactions _ newEvent getActor getReactionsTo: mouseUpButton.
	reactions ifNotNil: [reactions do: [:aReaction | aReaction reactTo: newEvent]]! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'jsp 4/19/1999 12:53'!
handlesMouseDown: evt
	"We want to handle mouse up and down events, so return true"

	^ true.
! !

!WonderlandCameraMorph methodsFor: 'drawing' stamp: 'ar 5/16/2001 17:09'!
areasRemainingToFill: aRectangle
	^(myCamera drawSceneBackground)
		ifTrue:[aRectangle areasOutside: self bounds]
		ifFalse:[Array with: aRectangle]! !

!WonderlandCameraMorph methodsFor: 'event handling' stamp: 'jsp 4/19/1999 13:58'!
setEventFocus: anActor
	"Set the actor that all events should go to"

	eventFocus _ anActor.
! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 5/9/2001 14:37'!
toggleCameraControls
	myControls == nil
		ifTrue:[self showCameraControls]
		ifFalse:[self deleteCameraControls].! !

!WonderlandCameraMorph methodsFor: 'pooh-palette' stamp: 'laza 2/4/2001 10:07'!
savePainting: aPaintBoxMorph evt: evt
	"Save the image after user issued #keep in aPaintBoxMorph"
	^self save! !

!WonderlandCameraMorph methodsFor: 'pooh' stamp: 'ar 5/22/2001 13:31'!
simplify: pointList
	| pts |
	pts _ StrokeSimplifier new.
	pointList do:[:aPoint| pts add: aPoint].
	pts closeStroke.
	^pts finalStroke.
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 16:06'!
mouseMoveEllipse: newEvent
	| rect |
	self restoreTexture.
	rect _ Rectangle encompassing: { lastPosition. currentPosition }.
	currentCanvas fillOval: rect color: currentColor borderWidth: currentNib extent x borderColor: Color black

! !

!WonderlandCameraMorph methodsFor: 'fog properties' stamp: 'ar 5/21/2001 15:31'!
fogRangeStart: aNumber
	myCamera setFogRangeStart: aNumber! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 5/9/2001 14:36'!
firstPersonControlString
	^(self doFirstPersonControl) 
		ifTrue:[^'<on>first person control'] 
		ifFalse:['<off>first person control'].! !

!WonderlandCameraMorph methodsFor: 'initialization' stamp: 'ar 4/19/1999 04:31'!
okayToEnterProject
	"Check if it is okay to enter the project containing the receiver"
	B3DPrimitiveEngine isAvailable ifFalse:[
		^(self confirm:'You are about to enter a project requiring 3D primitive support.
This support is currently NOT available.
Do you really want to enter the project?')].
	^true! !

!WonderlandCameraMorph methodsFor: 'stepping' stamp: 'ar 5/10/2001 22:51'!
step

	"*** hack ***"
	pickedPoint _ nil.
	"*** hack ***"
	myWonderland getEditor isInWorld
		ifTrue:[self changed]
		ifFalse:[(self hasProperty: #keepStepping) ifTrue:[
					self changed.
					myWonderland getScheduler tick]]! !

!WonderlandCameraMorph methodsFor: 'wrappers' stamp: 'ar 2/18/2001 22:16'!
addHalo: evt
	"Add a halo to an object"
	| actor wrapper root |
	self removeAllWrappers. "Get rid of them"
	evt == nil ifTrue:[^super addHalo: evt].
	actor _ myCamera pickAt: evt cursorPoint.
	actor == nil ifTrue:[^super addHalo: evt]. "Nothing hit"
	wrapper _ WonderlandWrapperMorph on: actor.
	root _ wrapper createHierarchy.
	root computeFullBounds: self.
	self addMorphFront: root.
	"pass on the event as if it had been there"
	root processEvent: evt copy resetHandlerFields.
! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 15:42'!
erase: newEvent
	currentCanvas fillRectangle: (currentPosition extent: currentNib extent) color: Color white

! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'laza 3/29/2000 15:59'!
mouseMoveLine: newEvent
	self restoreTexture.
	currentCanvas line: lastPosition to: currentPosition width: currentNib extent x color: currentColor

! !

!WonderlandCameraMorph methodsFor: 'wrappers' stamp: 'ar 11/7/1999 21:57'!
addHalo: evt from: formerHaloOwner
	"Special case if the formerHaloOwner was a wrapper within the receiver"
	self removeAllWrappers. "Get rid of them"
	(formerHaloOwner isKindOf: WonderlandWrapperMorph)
		ifTrue:[^super addHalo: evt] "Add a halo to me"
		ifFalse:[^self addHalo: evt] "Add a halo from pick"! !

!WonderlandCameraMorph methodsFor: 'drag and drop' stamp: 'ar 11/7/1999 21:09'!
wantsDroppedMorph: aMorph event: evt
	"Return true if the receiver wishes to accept the given morph, which is being dropped by a hand in response to the given event. The default implementation returns false.
NOTE: the event is assumed to be in global (world) coordinates."
	| pt |
	(super wantsDroppedMorph: aMorph event: evt) ifFalse:[^false].
	pt _ self transformFromWorld globalPointToLocal: evt cursorPoint.
	(myCamera pickAt: pt) == nil ifTrue:[^false].
	^true! !

!WonderlandCameraMorph methodsFor: 'menu' stamp: 'ar 4/20/2001 04:10'!
getAcceleratorState
	^self accelerationEnabled
		ifTrue:['<on>hardware acceleration']
		ifFalse:['<off>hardware acceleration']
! !

!WonderlandCameraMorph methodsFor: 'initialization' stamp: 'jsp 4/23/1999 12:06'!
initializeWithCamera: aCamera
	"Initialize this instance and hand it a camera"

	myCamera _ aCamera.
	myWonderland _ aCamera getWonderland.
	self name: (myCamera getName).

	"Use a transparent background"
	color _ (Color transparent).

	self bounds: (20@20 corner: 170@170)! !

!WonderlandCameraMorph methodsFor: 'pooh-actions' stamp: 'ar 5/28/2001 17:18'!
aaPaintingEnabled
	^self valueOfProperty: #aaPaintingEnabled ifAbsent:[true]! !

!WonderlandMorph methodsFor: 'morph depth' stamp: 'jsp 4/20/1999 16:07'!
sendBehind: aMorph
	"Put this morph behind the specified morph"

	((self owner) = (aMorph owner))
		ifFalse: [ myWonderland reportErrorToUser: 'The morphs need to have the same owner to put one behind the other.'.
				  ^ nil. ].

	(self owner) addMorph: self behind: aMorph.
! !

!WonderlandMorph methodsFor: 'accessing' stamp: 'jsp 4/12/1999 15:02'!
getWonderland

	^ myWonderland.
! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/12/1999 14:56'!
move: aDirection distance: aDistance
	"Move the morph the specified distance in the specified direction"

	^ self move: aDirection distance: aDistance duration: 1.0 style: gently.
! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/12/1999 14:58'!
moveTo: aPoint duration: aDuration style: aStyle
	"Moves the object to the specified position in its parent's coordinate system using the specified style over the specified duration."

	| anim |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyPoint: aPoint ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to move the morph to because ', msg.
			^ nil ].


	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for moving the morph because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to move the morph because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ AbsoluteAnimation new.

	anim object: self
			update: [:tPos | self position: (tPos rounded)]
			getStartState: [self position]
			getEndState: [ aPoint ]
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/20/1999 14:29'!
place: aLocation morph: aMorph
	"Place this WonderlandMorph in the specified location relative to the specified morph over 1 second using the gently animation style."

	^ self place: aLocation morph: aMorph duration: 1.0 style: gently.
! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/12/1999 14:57'!
moveTo: aPoint duration: aDuration
	"Moves the object to the specified position in its parent's coordinate system using the specified style over the specified duration."

	^ self moveTo: aPoint duration: aDuration style: gently.
! !

!WonderlandMorph methodsFor: 'accessing' stamp: 'jsp 4/12/1999 15:02'!
setWonderland: aWonderland

	myWonderland _ aWonderland.
! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/20/1999 16:20'!
place: aLocation morph: aMorph duration: aDuration style: aStyle
	"Place this WonderlandMorph in the specified location relative to the specified morph."

	| anim endStateFunc |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyLocation: aLocation ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to place the morph because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for placing the morph because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyStyle: aStyle ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine the style to use to place the morph because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ AbsoluteAnimation new.

	(aLocation = toLeftOf)
		ifTrue: [ endStateFunc _ [ ((aMorph left) - (self width)) @
										((aMorph center y) - ((self height) // 2)) ] ]
		ifFalse: [

	(aLocation = toRightOf)
		ifTrue: [ endStateFunc _ [ (aMorph right) @
										((aMorph center y) - ((self height) // 2)) ] ]
		ifFalse: [

	((aLocation = onTopOf) or: [ aLocation = above ])
		ifTrue: [ endStateFunc _ [ ((aMorph center x) - ((self width) // 2)) @
										((aMorph top) - (self height)) ] ]
		ifFalse: [

	(((aLocation = below) or: [ aLocation = beneath ]) or: [ aLocation = onBottomOf ])
		ifTrue: [ endStateFunc _ [ ((aMorph center x) - ((self width) // 2)) @
										(aMorph bottom) ] ]
		ifFalse: [

	(aLocation = onCeilingOf)
		ifTrue: [ endStateFunc _ [ ((aMorph center x) - ((self width) // 2)) @
										(aMorph top) ] ]
		ifFalse: [

	(aLocation = onFloorOf)
		ifTrue: [ endStateFunc _ [ ((aMorph center x) - ((self width) // 2)) @
										((aMorph bottom) - (self height)) ] ]
		ifFalse: [

	(aLocation = inFrontOf)
		ifTrue: [ endStateFunc _ [ self sendInFrontOf: aMorph.
								   self position ] ]
		ifFalse: [

	((aLocation = inBackOf) or: [ aLocation = behind ])
		ifTrue: [ endStateFunc _ [ self sendBehind: aMorph.
								   self position ] ]

				]]]]]]].

	anim object: self
			update: [:tPos | self position: (tPos rounded)]
			getStartState: [self position]
			getEndState: endStateFunc
			style: aStyle
			duration: aDuration
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

WonderlandMorph subclass: #WonderlandActorData
	instanceVariableNames: 'nameMorph positionMorph rotationMorph parentMorph'
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants'
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandActorData methodsFor: 'initialization' stamp: 'jsp 7/9/1999 16:23'!
initialize
	"Initialize this morph by setting up the various submorphs to be able to show data about a WonderlandActor"

	| aMorph |

	super initialize.
	self color: (Color r: 0.815 g: 0.972 b: 0.878).
	self bounds: (0@0 corner: 300@120).

	nameMorph _ TextMorph new.
	nameMorph contents: 'Name: (No actor selected)'.
	nameMorph topLeft: 10@10.
	self addMorph: nameMorph.

	parentMorph _ TextMorph new.
	parentMorph contents: 'Parent: '.
	parentMorph topLeft: 10@25.
	self addMorph: parentMorph.

	aMorph _ TextMorph new.
	aMorph contents: 'Position relative to parent'.
	aMorph topLeft: 10@55.
	self addMorph: aMorph.

	positionMorph _ TextMorph new.
	positionMorph contents: ((Character tab) asString) , 'Right:    Up:    Forward: '.
	positionMorph topLeft: 10@70.
	self addMorph: positionMorph.

	aMorph _ TextMorph new.
	aMorph contents: 'Rotation about'.
	aMorph topLeft: 10@100.
	self addMorph: aMorph.

	rotationMorph _ TextMorph new.
	rotationMorph contents: ((Character tab) asString) , 'Left-Right axis: ' ,
				((Character cr) asString) , ((Character tab) asString) , 'Up-Down axis: ',
				((Character cr) asString) , ((Character tab) asString) , 'Forward-Back axis: '.
	rotationMorph topLeft: 10@115.
	self addMorph: rotationMorph.

! !

!WonderlandActorData methodsFor: 'setting actor' stamp: 'jsp 7/9/1999 16:20'!
setActor: anActor
	"Set the actor whose data this morph should show and update the displayed data."

	| position rotation |

	(anActor isNil)
		ifTrue: [
				nameMorph contents: 'Name: (No actor selected)'.
				parentMorph contents: 'Parent: '.	
				positionMorph contents: ((Character tab) asString) , 'Right:    Up:    Forward: '.
				rotationMorph contents: ((Character tab) asString) , 'Left-Right axis: ' ,
					((Character cr) asString) , ((Character tab) asString) , 'Up-Down axis: ',
					((Character cr) asString) , ((Character tab) asString) , 'Forward-Back axis: '.
				]

		ifFalse: [
				nameMorph contents: ('Name: ' , (anActor getName)).

				(anActor isKindOf: WonderlandActor)
					ifTrue: [
							parentMorph contents: ('Parent: ' , (anActor getParent getName)).

							position _ anActor getPosition.
							position at: 1 put: ((((position at: 1) * 1000) rounded / 1000.0) asString).
							position at: 2 put: ((((position at: 2) * 1000) rounded / 1000.0) asString).
							position at: 3 put: ((((position at: 3) * 1000) rounded / 1000.0) asString).
							positionMorph contents: ((Character tab) asString) , ('Right: ',
								(position at: 1) , '  Up: ', (position at: 2) , '  Forward: ' ,
								(position at: 3)).

							rotation _ anActor getAngles.
							rotation at: 1 put: ((((rotation at: 1) * 1000) rounded / 1000.0) asString).
							rotation at: 2 put: ((((rotation at: 2) * 1000) rounded / 1000.0) asString).
							rotation at: 3 put: ((((rotation at: 3) * 1000) rounded / 1000.0) asString).
							rotationMorph contents: ((Character tab) asString) , 'Left-Right axis: ' ,
								(rotation at: 1) , ((Character cr) asString) ,
								((Character tab) asString) , 'Up-Down axis: ' , (rotation at: 2) , 
								((Character cr) asString) , ((Character tab) asString) ,
								'Forward-Back axis: ' , (rotation at: 3).

							]
					ifFalse: [
							parentMorph contents: 'Parent: None'.	
							positionMorph contents: ((Character tab) asString) ,
									'Right:    Up:    Forward: '.
							rotationMorph contents: ((Character tab) asString) , 'Left-Right axis: ' ,
								((Character cr) asString) , ((Character tab) asString) ,
								'Up-Down axis: ', ((Character cr) asString) ,
								((Character tab) asString) , 'Forward-Back axis: '.
							]

				].



! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/20/1999 14:29'!
place: aLocation morph: aMorph duration: aDuration
	"Place this WonderlandMorph in the specified location relative to the specified morph over 1 second using the gently animation style."

	^ self place: aLocation morph: aMorph duration: aDuration style: gently.
! !

!WonderlandMorph methodsFor: 'morph depth' stamp: 'jsp 4/20/1999 16:07'!
sendInFrontOf: aMorph
	"Put this morph in front of the specified morph"

	((self owner) = (aMorph owner))
		ifFalse: [ myWonderland reportErrorToUser: 'The morphs need to have the same owner to put one in front of the other.'.
				  ^ nil. ].

	(self owner) addMorph: self inFrontOf: aMorph.
! !

WonderlandMorph subclass: #WonderlandControls
	instanceVariableNames: 'myScheduler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandControls commentStamp: '<historical>' prior: 0!
This class implements the control panel for Wonderlands, including the Undo button and Quit button.
!

!WonderlandControls methodsFor: 'stepping' stamp: 'jsp 3/23/1999 15:03'!
stepTime
	
	^ 1.
! !

!WonderlandControls methodsFor: 'stepping' stamp: 'jsp 3/23/1999 15:04'!
step

	myScheduler tick.
! !

!WonderlandControls methodsFor: 'stepping' stamp: 'jsp 3/23/1999 15:03'!
wantsSteps

	^ true.
! !

!WonderlandControls methodsFor: 'initialization' stamp: 'jsp 4/12/1999 17:02'!
initializeWith: aWonderland
	"Initialize the instance"

	| exitButton resetButton undoButton |

	myWonderland _ aWonderland.
	myScheduler _ myWonderland getScheduler.

	"Use a white background"
	color _ (Color r: 0.784 g: 0.784 b: 0.784).

	"Now add our buttons"
	undoButton _ SimpleButtonMorph new.
	undoButton initialize.
	undoButton label: 'Undo'.
	undoButton color: (Color green).
	undoButton target: myWonderland.
	undoButton actionSelector: #undo.
	self addMorph: undoButton.

	resetButton _ SimpleButtonMorph new.
	resetButton initialize.
	resetButton label: 'Reset'.
	resetButton color: (Color yellow).
	resetButton target: myWonderland.
	resetButton actionSelector: #reset.
	self addMorph: resetButton.

	exitButton _ SimpleButtonMorph new.
	exitButton initialize.
	exitButton label: 'Quit'.
	exitButton color: (Color red).
	exitButton target: self.
	exitButton actionSelector: #quitWonderland.
	self addMorph: exitButton.

	undoButton bounds: (5@10 corner: 45@40).
	resetButton bounds: (50@10 corner: 90@40).
	exitButton bounds: (95@10 corner: 135@40).

	"Size and position the window"
	self extent: 140@50.
! !

!WonderlandControls methodsFor: 'actions' stamp: 'jsp 4/12/1999 16:46'!
quitWonderland
	"Destroy the wonderland"

	myWonderland release.
! !

!WonderlandControls class methodsFor: 'instance creation' stamp: 'jsp 3/23/1999 15:30'!
newFor: aWonderland
	"Create a new instance"

	^ self new initializeWith: aWonderland.
! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/12/1999 14:58'!
moveTo: aPoint speed: aSpeed
	"Moves the object to the specified position in its parent's coordinate system at the specified speed."

	| anim |

	"Check our arguments to make sure they're valid"
	[ WonderlandVerifier VerifyPoint: aPoint ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser: 'Squeak could not determine where to move the morph to because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyPositiveNumber: aSpeed]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the speed to move the morph because ', msg.
			^ nil ].

	"Our parameters check out, so build the animation"
	anim _ AbsoluteAnimation new.

	anim object: self
			update: [:tPos | self position: (tPos rounded)]
			getStartState: [self position]
			getEndState: [ aPoint ]
			style: abruptly
			duration: ((self position) dist: aPoint) / aSpeed
			undoable: true
			inWonderland: myWonderland.

	^ anim.
! !

WonderlandMorph subclass: #WonderlandEditor
	instanceVariableNames: 'myTabs myControls myActorBrowser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandEditor commentStamp: '<historical>' prior: 0!
This class implements the tabbed palette that includes the script editor and the actor browser.
!

!WonderlandEditor methodsFor: 'accessing' stamp: 'jsp 4/12/1999 16:23'!
resetNamespace
	"Reset the namespace used by the script editor"

	myTabs resetNamespace.
! !

!WonderlandEditor methodsFor: 'initialization' stamp: 'jsp 8/21/1999 16:52'!
initializeWith: aWonderland
	"Initialize the editor with the Wonderland."

	myWonderland _ aWonderland.

	myTabs _ (WonderlandEditorTabs newFor: aWonderland).
	self addMorph: myTabs.

	myActorBrowser _ (WonderlandActorBrowser newFor: myWonderland).
	myActorBrowser setActorViewer: (myTabs getActorViewer).
	self addMorph: (myActorBrowser getMorph).

	myControls _ WonderlandControls newFor: aWonderland.
	self addMorph: myControls.

	self position: 200@20.
	self extent: 650@375.
	self color: (Color r: 0.784 g: 0.784 b: 0.784).
	self openInWorld.
! !

!WonderlandEditor methodsFor: 'resizing' stamp: 'jsp 8/21/1999 17:25'!
extent: aPoint

	| newPoint aMorph |

	bounds extent = aPoint
		ifFalse: [
			self changed.

			newPoint _ aPoint.
			(aPoint x < 370) ifTrue: [ newPoint _ 370@(aPoint y) ].
			(aPoint y < 130) ifTrue: [ newPoint _ (newPoint x)@130 ].

			aMorph _ myActorBrowser getMorph.
			aMorph extent: 140@((newPoint y) - 52).
			aMorph position: (self position + (0@50)).

			myTabs extent: (newPoint x - 142)@(newPoint y - 4).
			myTabs position: (self position + (142@0)).

			bounds _ bounds topLeft extent: newPoint.
			self layoutChanged.
			self changed.
				].
! !

!WonderlandEditor class methodsFor: 'instance creation' stamp: 'jsp 4/1/1999 15:04'!
newFor: aWonderland
	"Create a new editor for the Wonderland"

	^ self new initializeWith: aWonderland.
! !

!WonderlandEditor methodsFor: 'accessing' stamp: 'jsp 4/1/1999 17:37'!
updateActorBrowser
	"Tells the actor browser to update the list of actors"

	myActorBrowser updateActorList.
! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/12/1999 14:57'!
moveTo: aPoint
	"Moves the object to the specified position in its parent's coordinate system using the specified style over the specified duration."

	^ self moveTo: aPoint duration: 1.0 style: gently.
! !

!WonderlandMorph methodsFor: 'primitive behaviors' stamp: 'jsp 4/12/1999 14:56'!
move: aDirection distance: aDistance duration: aDuration
	"Move the morph the specified distance in the specified direction"

	^ self move: aDirection distance: aDistance duration: aDuration style: gently.
! !

TabbedPalette subclass: #WonderlandEditorTabs
	instanceVariableNames: 'myWonderland myScriptEditor myActorViewer myQuickReference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Morphs'!

!WonderlandEditorTabs class methodsFor: 'instance creation' stamp: 'jsp 4/12/1999 16:19'!
newFor: aWonderland
	"Create new tabs for the Wonderland editor"

	^ self new initializeWith: aWonderland.
! !

!WonderlandEditorTabs methodsFor: 'initialization' stamp: 'jsp 6/29/1999 01:41'!
initializeWith: aWonderland
	"Initialize the editor with the Wonderland."

	myWonderland _ aWonderland.

	myScriptEditor _ (WonderlandScriptEditor new).
	self addPanel: (myScriptEditor getMorph).
	(self tabsMorph submorphs at: 1) tabSelected.
	myScriptEditor setBindings: (myWonderland getNamespace).

	myActorViewer _ WonderlandActorViewer new.
	myActorViewer initializeWith: aWonderland.
	self addPanel: myActorViewer.

	myQuickReference _ (WonderlandQuickReference new).
	self addPanel: (myQuickReference getMorph).
! !

!WonderlandEditorTabs methodsFor: 'initialization' stamp: 'jsp 4/12/1999 16:15'!
addPanel: aMorph
	"Adds a new panel to the editor."

	(self tabsMorph) addTab: (ReferenceMorph forMorph: aMorph).
	(self pages) add: aMorph.! !

!WonderlandEditorTabs methodsFor: 'user-interface' stamp: 'jsp 7/8/1999 22:26'!
selectTab: aTab
	"In addition to setting the tab, if it's the actor viewer let that tab know that it's open"

	super selectTab: aTab.

	(aTab morphToInstall isKindOf: WonderlandActorViewer)
		ifTrue: [ myActorViewer turnUpdatingOn ]
		ifFalse: [ myActorViewer ifNotNil: [ myActorViewer turnUpdatingOff ] ].

! !

!WonderlandEditorTabs methodsFor: 'accessing' stamp: 'jsp 8/21/1999 17:19'!
extent: aPoint

	| newPoint |

	bounds extent = aPoint
		ifFalse: [
			self changed.

			newPoint _ (aPoint x)@(aPoint y - 20).

			myScriptEditor getMorph extent: newPoint.
			myActorViewer extent: newPoint.
			myQuickReference getMorph extent: newPoint.

			bounds _ bounds topLeft extent: aPoint.
			self layoutChanged.
			self changed
				].
! !

!WonderlandEditorTabs methodsFor: 'accessing' stamp: 'jsp 7/9/1999 15:03'!
resetNamespace
	"Reset the namespace used by the script editor"

	myScriptEditor setBindings: (myWonderland getNamespace).
	myActorViewer setSelectedActor: nil.

! !

!WonderlandEditorTabs methodsFor: 'accessing' stamp: 'jsp 6/29/1999 01:53'!
getActorViewer

	^ myActorViewer.
! !

WonderlandActor subclass: #WonderlandLightSource
	instanceVariableNames: 'lightColor'
	classVariableNames: ''
	poolDictionaries: 'B3DEngineConstants'
	category: 'Balloon3D-Wonderland-Lights'!

!WonderlandLightSource methodsFor: 'private' stamp: 'jsp 4/23/1999 15:02'!
setColorVector: aColor
	"Instaneously sets the object's color/alpha vector and copies it down the object tree for all objects that are parts of this object"

	"Set our color"
	myColor _ aColor.

	lightColor ambientPart: myColor.
	lightColor diffusePart: myColor.
	lightColor specularPart: myColor.

	myMaterial ambientPart: myColor.
	myMaterial diffusePart: myColor.
	myMaterial specularPart: myColor.

	"Set the color of our parts"
	myChildren do: [:child | (child isPart) ifTrue: [child setColorVector: aColor] ].
! !

!WonderlandLightSource methodsFor: 'set property' stamp: 'jsp 4/30/1999 11:33'!
setBrightness: aNumber
	"Sets the brightness of the light between 0 and 1 by varying the color of the light."

	"The parameter checks out, so run the animation"
	self setBrightness: aNumber duration: 1.0.
! !

!WonderlandLightSource methodsFor: 'testing' stamp: 'jsp 4/23/1999 14:01'!
hasSpot
	"Return true if the receiver has a hot spot."
	^false! !

!WonderlandLightSource methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:02'!
lightColor
	^lightColor! !

!WonderlandLightSource methodsFor: 'converting' stamp: 'jsp 4/23/1999 12:10'!
transformedBy: aTransformer
	^self clone! !

!WonderlandLightSource methodsFor: 'initialization' stamp: 'ar 3/4/2004 21:10'!
initializeFor: aWonderland
	"Initialize the light"

	super initializeFor: aWonderland.

	"Set the light's mesh and texture"
	self setMesh: (WonderlandConstants default at: 'lightMesh').
	self setTexturePointer: (WonderlandConstants default at: 'lightTexture').

	lightColor _ B3DMaterialColor new.
	self setColorVector: (B3DColor4 r: 1.0 g: 1.0 b: 1.0 a: 1.0).
! !

!WonderlandLightSource methodsFor: 'shading' stamp: 'ar 8/20/1999 00:57'!
shadeVertexBuffer: vb with: aMaterial into: colorArray
	"This is the generic shading function similar to the primitive.
	Subclasses may implement optimized versions but should evaluate
	exactly to the same value as in here if they are to be converted into
	B3DPrimitiveLights."
	| color vtxArray ambientColor vtx direction distance scale cosAngle diffusePart specularPart specDir specularFactor |
	self flag: #b3dPrimitive.
	vtxArray _ vb vertexArray.
	(self hasAmbientPart and:[vb trackAmbientColor not])
		ifTrue:[ambientColor _ aMaterial ambientPart * lightColor ambientPart].
	(self hasDiffusePart and:[vb trackDiffuseColor not])
		ifTrue:[diffusePart _ aMaterial diffusePart].
	(self hasSpecularPart and:[vb trackSpecularColor not])
		ifTrue:[specularPart _ aMaterial specularPart].
	1 to: vb vertexCount do:[:i|
		vtx _ vtxArray at: i.
		color _ colorArray at: i.

		"Compute the direction and distance of light source from vertex"
		direction _ self computeDirectionTo: vtx.
		distance _ direction length.
		(distance = 0.0 or:[distance = 1.0]) ifFalse:[direction /= distance negated].

		"Compute the attenuation for the given distance"
		self isAttenuated
			ifTrue:[scale _ self computeAttenuationFor: distance]
			ifFalse:[scale _ 1.0].

		"Compute spot light factor"
		self hasSpot
			ifTrue:[scale _ scale * (self computeSpotFactor: direction)].

		"Compute ambient part"
		self hasAmbientPart ifTrue:[
			vb trackAmbientColor 
				ifTrue:[ambientColor _ vtx b3dColor * lightColor ambientPart].
			color += (ambientColor * scale).
		].

		"Compute the diffuse part of the light"
		self hasDiffusePart ifTrue:[
			"Compute angle from light->vertex to vertex normal"
			cosAngle _ vtx normal dot: direction.
			"For one-sided lighting negate cosAngle if necessary"
			(vb twoSidedLighting not and:[cosAngle < 0.0]) 
				ifTrue:[cosAngle _ 0.0 - cosAngle].
			"For two-sided lighting check if cosAngle > 0.0 meaning that it is a front face"
			cosAngle > 0.0 ifTrue:[
				vb trackDiffuseColor ifTrue:[diffusePart _ vtx b3dColor].
				color += (diffusePart * lightColor diffusePart * (cosAngle * scale)).
			].
		].

		"Compute specular part of the light"
		(self hasSpecularPart and:[aMaterial shininess > 0.0]) ifTrue:[
			vb useLocalViewer 
				ifTrue:[specDir _ direction - vtx position safelyNormalized]
				ifFalse:[specDir _ direction - (0@0@1.0)].
			cosAngle _ vtx normal dot: specDir.
			cosAngle > 0.0 ifTrue:[
				"Normalize the angle"
				cosAngle _ cosAngle / specDir length.
				"cosAngle should be somewhere between 0 and 1.
				If not, then the vertex normal was not normalized"
				cosAngle > 1.0 ifTrue:[
					specularFactor _ cosAngle raisedTo: aMaterial shininess.
				] ifFalse:[
					self flag: #TODO. "Use table lookup later"
					specularFactor _ cosAngle raisedTo: aMaterial shininess.
				].
				color += (specularPart * lightColor specularPart * specularFactor).
			].
		].
		self flag: #TODO. "Check specular part"
		colorArray at: i put: color.
	].! !

!WonderlandLightSource methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:01'!
direction
	"If the light is directional, return the NORMALIZED direction of the light"
	^B3DVector3 zero! !

!WonderlandLightSource methodsFor: 'get property' stamp: 'jsp 4/30/1999 11:32'!
getBrightness
	"Returns the brightness of the light as a value between 0 and 1."

	^ ((myColor red) max: (myColor green)) max: (myColor blue).
! !

!WonderlandLightSource methodsFor: 'shading' stamp: 'jsp 4/23/1999 14:04'!
computeDirectionTo: aB3DPrimitiveVertex
	^self subclassResponsibility! !

!WonderlandLightSource methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:02'!
hotSpotDeltaCosine
	"Return the cosine value of the delta radius of a spot light (the fall off region)"
	^self hotSpotMaxCosine - self hotSpotMinCosine! !

!WonderlandLightSource methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:02'!
hotSpotMaxCosine
	"Return the cosine value of the outer radius of a spot light (the unlit region)"
	^0.0! !

WonderlandLightSource subclass: #WonderlandPositionalLight
	instanceVariableNames: 'attenuation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Lights'!

!WonderlandPositionalLight methodsFor: 'shading' stamp: 'ar 8/19/1999 16:44'!
computeDirectionTo: aB3DPrimitiveVertex
	"Compute the lights direction to the given vertex"

	^ aB3DPrimitiveVertex  position - (self getPositionVector).
! !

!WonderlandPositionalLight methodsFor: 'converting' stamp: 'jsp 4/29/1999 11:24'!
transformedBy: aTransformer

	^(super transformedBy: aTransformer)
			setPositionVector: (aTransformer transformPosition: (self getPositionVector)).
! !

!WonderlandPositionalLight class methodsFor: 'change logging' stamp: 'jsp 7/8/1999 23:45'!
acceptsLoggingOfCompilation

	^ self == WonderlandPositionalLight.! !

WonderlandPositionalLight subclass: #WonderlandSpotLight
	instanceVariableNames: 'target minCos maxCos deltaCos direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Lights'!

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:54'!
hotSpotDeltaCosine
	^deltaCos! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:54'!
hotSpotMaxCosine
	^maxCos! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:53'!
direction
	^direction ifNil:[direction _ (target - (self getPositionVector)) safelyNormalize].! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:54'!
maxAngle: angle
	minCos _ angle degreesToRadians cos.
	maxCos ifNotNil:[deltaCos _ maxCos - minCos].! !

!WonderlandSpotLight methodsFor: 'converting' stamp: 'jsp 4/29/1999 13:52'!
asPrimitiveLight
	"Convert the receiver into a B3DPrimitiveLight"
	| primLight |
	primLight _ super asPrimitiveLight.
	primLight flags: (primLight flags bitOr: FlagHasSpot).
	primLight spotMinCos: minCos.
	primLight spotMaxCos: maxCos.
	primLight spotDeltaCos: deltaCos.
	primLight spotExponent: self spotExponent.
	primLight direction: (target - (self getPositionVector)) safelyNormalize.
	^primLight! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:55'!
minAngle
	^minCos arcCos radiansToDegrees! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:55'!
minAngle: angle
	maxCos _ angle degreesToRadians cos.
	minCos ifNotNil:[deltaCos _ maxCos - minCos].! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:55'!
target
	^target! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:55'!
target: aVector
	target _ aVector! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:54'!
maxAngle
	^maxCos arcCos radiansToDegrees! !

!WonderlandSpotLight methodsFor: 'converting' stamp: 'jsp 4/29/1999 15:46'!
transformedBy: aTransformer
	^(super transformedBy: aTransformer) 
		target: (aTransformer transformPosition: target).
! !

!WonderlandSpotLight class methodsFor: 'change logging' stamp: 'jsp 7/8/1999 23:46'!
wantsChangeSetLogging

	^ self == WonderlandSpotLight.

! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:54'!
direction: aVector
	direction _ aVector! !

!WonderlandSpotLight methodsFor: 'testing' stamp: 'jsp 4/29/1999 13:51'!
hasSpot

	^ true.! !

!WonderlandSpotLight class methodsFor: 'change logging' stamp: ''!
acceptsLoggingOfCompilation
	^ self == WonderlandSpotLight! !

!WonderlandSpotLight methodsFor: 'accessing' stamp: 'jsp 4/29/1999 13:54'!
hotSpotMinCosine
	^minCos! !

!WonderlandPositionalLight methodsFor: 'initialization' stamp: 'jsp 4/29/1999 11:10'!
initializeFor: aWonderland
	"Initialize this light"

	super initializeFor: aWonderland.

	attenuation _ B3DLightAttenuation constant: 1.0 linear: 0.0 squared: 0.0.! !

!WonderlandPositionalLight class methodsFor: 'change logging' stamp: 'jsp 7/8/1999 23:45'!
wantsChangeSetLogging

	^ self == WonderlandPositionalLight.

! !

!WonderlandPositionalLight methodsFor: 'initialization' stamp: 'jsp 4/23/1999 14:30'!
initialize
	"Initialize this light"

	attenuation _ B3DLightAttenuation constant: 1.0 linear: 0.0 squared: 0.0.! !

!WonderlandPositionalLight methodsFor: 'converting' stamp: 'jsp 4/23/1999 14:26'!
asPrimitiveLight
	"Convert the receiver into a B3DPrimitiveLight"
	| primLight flags |
	primLight _ B3DPrimitiveLight new.
	primLight position: (self getPositionVector).
	flags _ FlagPositional.
	self attenuation isIdentity not ifTrue:[
		primLight attenuation: self attenuation.
		flags _ flags bitOr: FlagAttenuated].
	lightColor ambientPart isZero ifFalse:[
		primLight ambientPart: lightColor ambientPart.
		flags _ flags bitOr: FlagAmbientPart].
	lightColor diffusePart isZero ifFalse:[
		primLight diffusePart: lightColor diffusePart.
		flags _ flags bitOr: FlagDiffusePart].
	lightColor specularPart isZero ifFalse:[
		primLight specularPart: lightColor specularPart.
		flags _ flags bitOr: FlagSpecularPart].
	primLight flags: flags.
	^primLight! !

!WonderlandPositionalLight methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:27'!
attenuation
	^attenuation! !

!WonderlandPositionalLight methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:28'!
position

	^ self getPositionVector.
! !

!WonderlandPositionalLight methodsFor: 'shading' stamp: 'jsp 4/23/1999 14:29'!
computeAttenuationFor: distance
	"Compute the attenuation for the given distance"
	^attenuation computeAttenuationFor: distance! !

!WonderlandPositionalLight methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:28'!
attenuation: aLightAttenuation
	attenuation _ aLightAttenuation! !

WonderlandLightSource subclass: #WonderlandAmbientLight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Lights'!

!WonderlandAmbientLight methodsFor: 'shading' stamp: 'jsp 4/23/1999 14:48'!
shadeVertexBuffer: vb with: aMaterial into: colorArray
	"Overridden for simplicity and speed"
	| color |
	false ifTrue:[^super shadeVertexBuffer: vb with: aMaterial into: colorArray].
	self flag: #b3dPrimitive.
	vb trackAmbientColor ifTrue:[
		1 to: vb vertexCount do:[:i|
			color _ (vb primitiveB3dColorAt: i) * lightColor ambientPart.
			colorArray add: color at: i.
		].
	] ifFalse:[
		color _ aMaterial ambientPart * lightColor ambientPart.
		colorArray += color.
	].! !

!WonderlandAmbientLight methodsFor: 'testing' stamp: 'jsp 4/23/1999 14:45'!
hasDiffusePart
	^false! !

!WonderlandAmbientLight methodsFor: 'converting' stamp: 'jsp 4/23/1999 14:47'!
transformedBy: aTransformer
	^self! !

!WonderlandAmbientLight methodsFor: 'converting' stamp: 'jsp 4/23/1999 14:47'!
asPrimitiveLight
	"Convert the receiver into a B3DPrimitiveLight"
	| primLight |
	primLight _ B3DPrimitiveLight new.
	primLight ambientPart: lightColor ambientPart.
	primLight flags: FlagAmbientPart.
	^primLight! !

!WonderlandAmbientLight class methodsFor: 'change logging' stamp: ''!
acceptsLoggingOfCompilation
	^ self == WonderlandAmbientLight! !

!WonderlandAmbientLight methodsFor: 'testing' stamp: 'jsp 4/23/1999 14:46'!
hasSpecularPart
	^false! !

!WonderlandAmbientLight class methodsFor: 'change logging' stamp: 'jsp 7/8/1999 23:45'!
wantsChangeSetLogging

	^ self == WonderlandAmbientLight.

! !

!WonderlandAmbientLight methodsFor: 'shading' stamp: 'jsp 4/23/1999 14:48'!
computeDirectionTo: aB3DPrimitiveVertex
	^B3DVector3 zero! !

!WonderlandAmbientLight methodsFor: 'shading' stamp: 'jsp 4/23/1999 14:47'!
computeAttenuationFor: distance
	^1.0! !

!WonderlandLightSource methodsFor: 'shading' stamp: 'jsp 4/23/1999 14:04'!
computeAttenuationFor: distance
	^self subclassResponsibility! !

!WonderlandLightSource methodsFor: 'converting' stamp: 'jsp 4/23/1999 12:10'!
asPrimitiveLight
	"Convert the receiver into a B3DPrimitiveLight that can be handled by the shader primitive directly. Light sources that cannot be represented as primitive should return nil. This will result in the callback of #shadeVertexBuffer from the shader."
	^nil! !

!WonderlandLightSource class methodsFor: 'change logging' stamp: ''!
acceptsLoggingOfCompilation
	^ true! !

!WonderlandLightSource methodsFor: 'testing' stamp: 'jsp 4/23/1999 14:00'!
hasDiffusePart
	"Return true if the receiver contains a diffuse part in its color"
	^true! !

!WonderlandLightSource methodsFor: 'testing' stamp: 'jsp 4/23/1999 14:00'!
hasSpecularPart
	"Return true if the receiver contains a specular part in its color"
	^true! !

!WonderlandLightSource class methodsFor: 'change logging' stamp: 'jsp 4/23/1999 11:49'!
wantsChangeSetLogging

	^ true.


! !

!WonderlandLightSource methodsFor: 'testing' stamp: 'jsp 4/23/1999 14:00'!
hasAmbientPart
	"Return true if the receiver contains an ambient part in its color"
	^true! !

!WonderlandLightSource methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:03'!
lightColor: aMaterialColor
	lightColor _ aMaterialColor! !

!WonderlandLightSource methodsFor: 'testing' stamp: 'jsp 4/23/1999 14:01'!
isAttenuated
	"Return true if the receiver contains an attenuation.
	If so, #computeAttenuationFor: must return the attenuation for the given distance."
	^true! !

!WonderlandLightSource methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:02'!
hotSpotMinCosine
	"Return the cosine value of the inner radius of a spot light (the fully lit region)"
	^0.0! !

!WonderlandLightSource methodsFor: 'primitive behaviors' stamp: 'jsp 4/29/1999 15:01'!
destroy: aDuration
	"Implements the animated destroy of a camera.  Unlike destroy for WonderlandActors, this method merely removes the camera from the scene and removes it render window from the world."

	| lightList childList undoActions |

	undoActions _ OrderedCollection new.
	childList _ self getAllChildren.

	childList do: [:child | 
			(child isFirstClass) ifTrue: [
				"Make the non-part children point to this actor's parent"
				undoActions addLast: (UndoParentChange newFor: child from: (child getParent)).
				child reparentTo: (self getParent).
								  		].
				].

	self removeFromScene.

	lightList _ myWonderland getLights.
	lightList remove: self.

	undoActions addLast: (UndoAction new: [ (myWonderland getScene) addChild: self.
									lightList addLast: self ]).

	(myWonderland getUndoStack) push: (UndoChangeList new setChangeList: undoActions).

				


! !

!WonderlandLightSource methodsFor: 'accessing' stamp: 'jsp 4/23/1999 14:03'!
spotExponent
	"Return the exponent to be used for the spot fall off computation"
	^1! !

!WonderlandLightSource methodsFor: 'shading' stamp: 'jsp 4/23/1999 14:04'!
computeSpotFactor: light2Vertex
	"Compute the spot factor for a spot light"
	| lightDirection cosAngle minCos deltaCos maxCos |
	lightDirection _ self direction.
	cosAngle _ (lightDirection dot: light2Vertex) negated.
	(cosAngle < (minCos _ self hotSpotMinCosine)) ifTrue:[^0.0].
	maxCos _ self hotSpotMaxCosine.
"	maxCos = 1.0 ifFalse:["
		deltaCos _ self hotSpotDeltaCosine.
		deltaCos <= 0.00001 ifTrue:[
			"No delta -- a sharp boundary between on and off.
			Since off has already been determined above, we are on"
			^1.0].
		"Scale the angle to 0/1 range"
		cosAngle _ (cosAngle - minCos) / deltaCos.
		self flag: #TODO. "Don't scale by (maxCos - minCos)"
"	]."
	self flag: #TODO. "Use table lookup for spot exponent"
	^cosAngle raisedTo: self spotExponent! !

!WonderlandLightSource methodsFor: 'set property' stamp: 'jsp 4/30/1999 11:37'!
setBrightness: aNumber duration: aDuration
	"Sets the brightness of the light between 0 and 1 by varying the color of the light."

	| r g b scale currentBrightness |

	"First verify our arguments"
	[ WonderlandVerifier Verify0To1Number: aNumber ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not the brightness to make the light because ', msg.
			^ nil ].

	[ WonderlandVerifier VerifyDuration: aDuration ]
		ifError: [ :msg :rcvr |
			myWonderland reportErrorToUser:
				'Squeak could not determine the duration to use for changing the brightness of the light because ', msg.
			^ nil ].

	currentBrightness _ self getBrightness.

	(currentBrightness = 0)
		ifTrue: [
					r _ aNumber.
					g _ aNumber.
					b _ aNumber.
				]
		ifFalse: [
					scale _ (aNumber / (self getBrightness)).
					r _ (myColor red) * scale.
					g _ (myColor green) * scale.
					b _ (myColor blue) * scale.
				].

	"Our parameters check out, so run the animation"
	self setColor: { r. g. b } duration: aDuration.
! !

WonderlandLightSource subclass: #WonderlandDirectionalLight
	instanceVariableNames: 'direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Balloon3D-Wonderland-Lights'!

!WonderlandDirectionalLight class methodsFor: 'change logging' stamp: 'jsp 7/25/1999 23:26'!
wantsChangeSetLogging

	^ self == WonderlandDirectionalLight.

! !

!WonderlandDirectionalLight methodsFor: 'converting' stamp: 'jsp 4/29/1999 13:49'!
transformedBy: aTransformer
	^(super transformedBy: aTransformer)
				setRotationVector: (aTransformer transformDirection: direction)! !

!WonderlandDirectionalLight methodsFor: 'shading' stamp: 'jsp 4/29/1999 13:45'!
computeAttenuationFor: distance
	"Since a directional light is positioned at virtual infinity, it cannot have any attenuation"

	^1.0! !

!WonderlandDirectionalLight methodsFor: 'converting' stamp: 'jsp 4/29/1999 13:48'!
asPrimitiveLight
	"Convert the receiver into a B3DPrimitiveLight"
	| primLight flags |
	primLight _ B3DPrimitiveLight new.
	primLight direction: (self getRotationVector).
	flags _ FlagDirectional.
	lightColor ambientPart isZero ifFalse:[
		primLight ambientPart: lightColor ambientPart.
		flags _ flags bitOr: FlagAmbientPart].
	lightColor diffusePart isZero ifFalse:[
		primLight diffusePart: lightColor diffusePart.
		flags _ flags bitOr: FlagDiffusePart].
	lightColor specularPart isZero ifFalse:[
		primLight specularPart: lightColor specularPart.
		flags _ flags bitOr: FlagSpecularPart].
	primLight flags: flags.
	^primLight! !

!WonderlandDirectionalLight methodsFor: 'private' stamp: 'ar 8/19/1999 16:26'!
setDirection: dir
	direction _ dir.! !

!WonderlandDirectionalLight methodsFor: 'shading' stamp: 'jsp 4/29/1999 13:48'!
computeDirectionTo: aB3DPrimitiveVertex
	"A directional light has an explicit direction regardless of the vertex position"

	^ (self getRotationVector).! !

!WonderlandDirectionalLight class methodsFor: 'change logging' stamp: ''!
acceptsLoggingOfCompilation
	^ self == WonderlandDirectionalLight! !
WonderlandQuickReference initialize!
AbstractAnimation initialize!
WonderlandStyle initialize!
Wonderland initialize!
WonderlandCameraMorph initialize!
